@@ -39,14 +39,17 @@ controller = None
 # list of active renderers
 renderers = []
 
+# donation manager singleton object
+donate_manager = None
+
 # database object
 db = None
 
-# stores ItemInfo objects so we can quickly fetch them
-item_info_cache = None
+# BulkSQLManager for the main miro database
+bulk_sql_manager = None
 
-# command line arguments for thumbnailer (linux)
-movie_data_program_info = None
+# DBInfo object for the main miro database
+db_info = None
 
 # configuration data
 config = None
@@ -69,25 +72,62 @@ device_tracker = None
 # platform/frontend specific directory watcher
 directory_watcher = None
 
+# MetadataManager for local items
+local_metadata_manager = None
+
 # signal emiters for when config data changes
 backend_config_watcher = None
 frontend_config_watcher = None
 downloader_config_watcher = None
 
-# sends MetadataProgressUpdate messages to the frontend
-metadata_progress_updater = None
+# global state managers
+download_state_manager = None
+icon_cache_updater = None
+movie_data_updater = None
 
 # debugmode adds a bunch of computation that's useful for development
 # and debugging.  initalized to None; set to True/False depending on
 # mode
 debugmode = None
 
+#
+# Frontend API class.  All the frontend should define a subclass of this and
+# implement the methods.
+#
+class Frontend(object):
+    def call_on_ui_thread(self, func, *args, **kwargs):
+        """Call a function at a later time on the UI thread."""
+        raise NotImplementedError()
+
+    def run_choice_dialog(self, title, description, buttons):
+        """Show the database error dialog and wait for a choice.
+
+        This method should block until the choice is picked.  Depending on the
+        frontend other events may still be processed or not.
+
+        :returns: button that was choosen or None if the dialog was closed.
+        """
+        raise NotImplementedError()
+
+    def quit(self):
+        """Quit Miro."""
+        raise NotImplementedError()
+
+frontend = Frontend()
+
+# name of the running frontend
+frontend_name = None
+
 # widget frontend adds these
 # --------------------------
 
 # application object
 widgetapp = None
 
+# ConnectionPoolTracker object -- Note: this object could be made thread-safe
+# pretty easily, but right now it only should be used in the frontend thread.
+connection_pools = None
+
 # handles the right-hand display
 display_manager = None
 
@@ -130,6 +170,8 @@ audio_renderer = None
 # gtk/windows item type sniffer
 get_item_type = None
 
+# Tracks ItemLists that are in-use
+item_list_pool = None
 
 # cli frontend adds these
 # -----------------------
