@@ -37,7 +37,7 @@ the default one.
 import logging
 import traceback
 
-from miro import util
+from miro import buildutils
 from miro.plat import resources
 
 class AppConfig(object):
@@ -45,7 +45,7 @@ class AppConfig(object):
         self.theme_vars = {}
 
         app_config_path = resources.path('app.config')
-        self.default_vars = util.read_simple_config_file(app_config_path)
+        self.default_vars = buildutils.read_simple_config_file(app_config_path)
 
         self.load_theme(theme)
 
@@ -54,7 +54,8 @@ class AppConfig(object):
             logging.info("Using theme %s", theme)
             theme_app_config = resources.theme_path(theme, 'app.config')
             try:
-                self.theme_vars = util.read_simple_config_file(theme_app_config)
+                self.theme_vars = buildutils.read_simple_config_file(
+                        theme_app_config)
             except EnvironmentError:
                 logging.warn("Error loading theme: %s\n%s", 
                         theme_app_config, traceback.format_exc())
