@@ -67,30 +67,23 @@ _started_up = False
 _command_line_videos = None
 _command_line_view = None
 
-def _item_exists_for_path(path):
-    # in SQLite, LIKE is case insensitive, so we can use it to only look at
-    # filenames that possibly will match
-    for item_ in item.Item.make_view('filename LIKE ?',
-            (filename_to_unicode(path),)):
-        if samefile(item_.filename, path):
-            return item_
-    return False
-
 def add_video(path, manual_feed=None):
     """Add a new video
 
     :returns: True if we create a new Item object.
     """
     path = os.path.abspath(path)
-    item_for_path = _item_exists_for_path(path)
-    if item_for_path:
-        if item_for_path.deleted:
-            item_for_path.make_undeleted()
+    if item.Item.have_item_for_path(path):
         logging.debug("Not adding duplicate video: %s",
                       path.decode('ascii', 'ignore'))
-        if _command_line_videos is not None:
-            _command_line_videos.add(item_for_path)
-        return False
+        # get the first item and undelete it
+        item_for_path = _get_item_for_path(path)
+        if item_for_path is not None:
+            if item_for_path.deleted:
+                item_for_path.make_undeleted()
+            if _command_line_videos is not None:
+                _command_line_videos.add(item_for_path)
+            return False
     if manual_feed is None:
         manual_feed = feed.Feed.get_manual_feed()
     file_item = item.FileItem(
@@ -99,18 +92,28 @@ def add_video(path, manual_feed=None):
         _command_line_videos.add(file_item)
     return True
 
+def _get_item_for_path(path):
+    """Get the first item that has a given path."""
+    try:
+        return list(item.Item.items_with_path_view(path))[0]
+    except IndexError:
+        msg = ( "have_item_for_path returned True, but "
+                "items_with_path_view returned no items."
+                "  path: %s" % (path,))
+        app.controller.failed_soft("commandline.add_video", msg)
+        return None
+
 @eventloop.idle_iterator
 def add_videos(paths):
     # filter out non-existent paths
     paths = [p for p in paths if fileutil.exists(p)]
-    for path in paths:
-        app.metadata_progress_updater.will_process_path(path)
     path_iter = iter(paths)
     finished = False
     yield # yield after doing prep work
-    while not finished:
-        finished = _add_batch_of_videos(path_iter, 0.5)
-        yield # yield after each batch
+    with app.local_metadata_manager.bulk_add():
+        while not finished:
+            finished = _add_batch_of_videos(path_iter, 0.1)
+            yield # yield after each batch
 
 def _add_batch_of_videos(path_iter, max_time):
     """Add a batch of videos for add_video()
@@ -125,9 +128,7 @@ def _add_batch_of_videos(path_iter, max_time):
     app.bulk_sql_manager.start()
     try:
         for path in path_iter:
-            if not add_video(path, manual_feed=manual_feed):
-                # video was a duplicate, undo the will_process_path() call
-                app.metadata_progress_updater.path_processed(path)
+            add_video(path, manual_feed=manual_feed)
             if time.time() - start_time > max_time:
                 return False
         return True
@@ -138,7 +139,7 @@ def add_torrent(path, torrent_info_hash):
     manual_feed = feed.Feed.get_manual_feed()
     for i in manual_feed.items:
         if ((i.downloader is not None
-             and i.downloader.status.get('infohash') == torrent_info_hash)):
+             and i.downloader.info_hash == torrent_info_hash)):
             logging.info("not downloading %s, it's already a download for %s",
                          path, i)
             if i.downloader.get_state() in ('paused', 'stopped'):
@@ -271,9 +272,9 @@ def parse_command_line_args(args):
     # to use Miro to play videos and Miro goes to play a video
     # externally, then it causes an infinite loop and dies.
     if added_videos and app.config.get(prefs.PLAY_IN_MIRO):
-        item_infos = [itemsource.DatabaseItemSource._item_info_for(i)
-                      for i in _command_line_videos]
-        messages.PlayMovie(item_infos).send_to_frontend()
+        item_ids = [i.id for i in _command_line_videos]
+        item_infos = app.db.fetch_item_infos(item_ids)
+        messages.PlayMovies(item_infos).send_to_frontend()
 
     if added_downloads:
         # FIXME - switch to downloads tab?
