@@ -55,6 +55,7 @@ from miro.gtcache import gettext as _
 from miro.fileobject import FilenameType
 from miro.plat import utils
 from miro.plat import resources
+from miro.plat.popen import Popen
 
 NON_WORD_CHARS = re.compile(r"[^a-zA-Z0-9]+")
 
@@ -67,7 +68,7 @@ def get_conversions_folder():
     root = app.config.get(prefs.MOVIES_DIRECTORY)
     target_folder = os.path.join(root, "Converted")
     if not os.path.exists(target_folder):
-        os.mkdir(target_folder)
+        os.makedirs(target_folder)
     return target_folder
 
 
@@ -255,6 +256,8 @@ class ConverterInfo(object):
             name, parser, "extension", {})
         self.screen_size = self._get_config_value(
             name, parser, "ssize", {})
+        self.bit_rate = int(self._get_config_value(
+                name, parser, "bitrate", {'bitrate': 0}))
         self.platforms = self._get_config_value(
             name, parser, "only_on", {'only_on': None})
         self.displayname = _(
@@ -314,6 +317,7 @@ class ConverterManager(object):
                 self.converters.append((defaults['name'], group_converters))
             finally:
                 definition_file.close()
+        messages.ConverterList(self.converters).send_to_frontend()
 
     def lookup_converter(self, converter_id):
         """Looks up and returns a ConverterInfo object by id.
@@ -422,9 +426,6 @@ class ConversionManager(signals.SignalEmitter):
         if converter_info.executable == 'ffmpeg':
             return FFMpegConversionTask(converter_info, item_info,
                                         target_folder, create_item)
-        elif converter_info.executable == 'ffmpeg2theora':
-            return FFMpeg2TheoraConversionTask(converter_info, item_info,
-                                               target_folder, create_item)
         return None
 
     def _check_task_loop(self):
@@ -531,7 +532,14 @@ class ConversionManager(signals.SignalEmitter):
                 logging.warn("Couldn't find task for key %s", msg['key'])
                 return
             source = task.temp_output_path
-            destination, fp = next_free_filename(task.final_output_path)
+            try:
+                destination, fp = next_free_filename(task.final_output_path)
+                fp.close()
+            except ValueError:
+                logging.warn('_process_message_queue: ' 
+                             'next_free_filename failed.  Candidate = %r',
+                             task.final_output_path)
+                return
             source_info = task.item_info
             conversion_name = task.get_display_name()
             if os.path.exists(source):
@@ -546,7 +554,6 @@ class ConversionManager(signals.SignalEmitter):
             else:
                 task.error = _("Reason unknown--check log")
                 self._notify_tasks_count()
-            fp.close()
             self.emit('task-staged', task)
 
     def _move_finished_file(self, source, destination):
@@ -667,10 +674,10 @@ def build_output_paths(item_info, target_folder, converter_info):
         target_folder = get_conversions_folder()
     else:
         use_temp_dir = False
-    input_path = item_info.video_path
+    input_path = item_info.filename
     basename = os.path.basename(input_path)
 
-    title = utils.unicode_to_filename(item_info.name, target_folder).strip()
+    title = utils.unicode_to_filename(item_info.title, target_folder).strip()
     if not title:
         title = basename
 
@@ -694,9 +701,15 @@ def build_output_paths(item_info, target_folder, converter_info):
 def round_even(num):
     """This takes a number, converts it to an integer, then makes
     sure it's even.
+
+    Additional rules: this helper always rounds down to avoid stray black
+    pixels (see bz18122).
+
+    This function makes sure that the value returned is always >= 0.
     """
     num = int(num)
-    return num + (num % 2)
+    val = num - (num % 2)
+    return val if val > 0 else 0
 
 
 def build_parameters(input_path, output_path, converter_info, media_info):
@@ -779,14 +792,11 @@ class ConversionTask(object):
                  create_item):
         self.item_info = item_info
         self.converter_info = converter_info
-        self.input_path = item_info.video_path
+        self.input_path = item_info.filename
         self.final_output_path, self.temp_output_path = build_output_paths(
             item_info, target_folder, converter_info)
         self.create_item = create_item
 
-        logging.debug("temp_output_path: [%s] final_output_path: [%s]",
-                      self.temp_output_path, self.final_output_path)
-
         self.key = "%s->%s" % (self.input_path, self.final_output_path)
         self.thread = None
         self.duration = None
@@ -803,10 +813,18 @@ class ConversionTask(object):
     def get_parameters(self):
         raise NotImplementedError()
 
+    def get_output_size_guess(self):
+        if self.item_info.duration and self.converter_info.bit_rate:
+            return self.converter_info.bit_rate * self.item_info.duration / 8
+        return self.item_info.size
+
     def get_display_name(self):
         return self.converter_info.displayname
 
     def run(self):
+        logging.debug("temp_output_path: [%s] final_output_path: [%s]",
+                      self.temp_output_path, self.final_output_path)
+
         self.progress = 0
         self.thread = threading.Thread(target=utils.thread_body,
                                        args=[self._loop],
@@ -859,12 +877,9 @@ class ConversionTask(object):
                   "stdout": subprocess.PIPE,
                   "stderr": subprocess.STDOUT,
                   "stdin": subprocess.PIPE,
-                  "startupinfo": util.no_console_startupinfo()}
-        if os.name != "nt":
-            kwargs["close_fds"] = True
-
+                  "close_fds": True}
         try:
-            self.process_handle = subprocess.Popen(args, **kwargs)
+            self.process_handle = Popen(args, **kwargs)
             self.process_output(line_reader(self.process_handle.stdout))
             self.process_handle.wait()
 
@@ -915,7 +930,7 @@ class ConversionTask(object):
                 item_id, self.converter_info.identifier))
         self.log_file = file(self.log_path, "w")
         self._log_progress("STARTING CONVERSION")
-        self._log_progress("-> Item: %s" % util.stringify(self.item_info.name))
+        self._log_progress("-> Item: %s" % util.stringify(self.item_info.title))
         self._log_progress("-> Converter used: %s" % self.converter_info.name)
         self._log_progress("-> Executable: %s" % executable)
         self._log_progress("-> Parameters: %s" % ' '.join(params))
@@ -944,8 +959,8 @@ class ConversionTask(object):
             return
         logging.warning("killing conversion task %d",
                         self.process_handle.pid)
-        self.process_handle.kill()
         try:
+            self.process_handle.kill()
             self.process_handle.wait()
         except OSError:
             logging.exception('exception while interupting process')
@@ -984,6 +999,9 @@ class CopyConversionTask(ConversionTask):
         # never actually executed
         return "copy"
 
+    def get_output_size_guess(self):
+        return self.item_info.size
+
     def get_display_name(self):
         return _("Copy")
 
@@ -1022,8 +1040,8 @@ class FFMpegConversionTask(ConversionTask):
         default_parameters = build_parameters(
             self.input_path, self.temp_output_path, self.converter_info, media_info)
         # insert -strict experimental
-        default_parameters.insert(0, 'experimental')
-        default_parameters.insert(0, '-strict')
+        default_parameters.insert(-1, 'experimental')
+        default_parameters.insert(-2, '-strict')
         return utils.customize_ffmpeg_parameters(default_parameters)
 
     def check_for_errors(self, line):
@@ -1049,69 +1067,16 @@ class FFMpegConversionTask(ConversionTask):
         else:
             match = FFMpegConversionTask.PROGRESS_RE.match(line)
             if match is not None:
-                return float(match.group(1)) / self.duration
+                t = match.group(1)
+                if ':' in t:
+                    h, m, s = t.split(':')
+                    t = float(h) * 3600 + float(m) * 60 + float(s)
+                return float(t) / self.duration
             match = FFMpegConversionTask.LAST_PROGRESS_RE.match(line)
             if match is not None:
                 return 1.0
         return self.progress
 
-
-class FFMpeg2TheoraConversionTask(ConversionTask):
-    DURATION_RE = re.compile(r'f2t ;duration: ([^;]*);')
-
-    PROGRESS_RE1 = re.compile(r'\{"duration":(.*), "position":(.*), '
-                              '"audio_kbps":.*, "video_kbps":.*, '
-                              '"remaining":.*\}')
-    RESULT_RE1 = re.compile(r'\{"result": "(.*)"\}')
-
-    PROGRESS_RE2 = re.compile(r'f2t ;position: ([^;]*);')
-    RESULT_RE2 = re.compile(r'f2t ;result: ([^;]*);')
-
-    def __init__(self, converter_info, item_info, target_folder, create_item):
-        ConversionTask.__init__(self, converter_info, item_info,
-                                target_folder, create_item)
-        self.platform = app.config.get(prefs.APP_PLATFORM)
-
-    def get_executable(self):
-        return utils.get_ffmpeg2theora_executable_path()
-
-    def get_parameters(self):
-        try:
-            media_info = get_media_info(self.input_path)
-        except ValueError:
-            media_info = {}
-
-        default_parameters = build_parameters(
-            self.input_path, self.temp_output_path, self.converter_info, media_info)
-        return utils.customize_ffmpeg2theora_parameters(default_parameters)
-
-    def check_for_errors(self, line):
-        return
-
-    def monitor_progress(self, line):
-        if line.startswith('f2t'):
-            if self.duration is None:
-                match = FFMpeg2TheoraConversionTask.DURATION_RE.match(line)
-                if match is not None:
-                    self.duration = float(match.group(1))
-            match = FFMpeg2TheoraConversionTask.PROGRESS_RE2.match(line)
-            if match is not None:
-                return float(match.group(1)) / self.duration
-            match = FFMpeg2TheoraConversionTask.RESULT_RE2.match(line)
-            if match is not None:
-                return 1.0
-        else:
-            match = FFMpeg2TheoraConversionTask.PROGRESS_RE1.match(line)
-            if match is not None:
-                if self.duration is None:
-                    self.duration = float(match.group(1))
-                return float(match.group(2)) / self.duration
-            match = FFMpeg2TheoraConversionTask.RESULT_RE1.match(line)
-            if match is not None:
-                return 1.0
-        return self.progress
-
-
 def convert(converter_id, item_info, update_last=False):
     """Given a converter and an item, this starts the conversion for
     that item.
@@ -1129,7 +1094,7 @@ def _create_item_for_conversion(filename, source_info, conversion_name):
     # should only get called in the event loop.
 
     name = _('%(original_name)s (Converted to %(format)s)',
-            {'original_name': source_info.name, 'format': conversion_name})
+            {'original_name': source_info.title, 'format': conversion_name})
 
     fp_values = item.fp_values_for_file(filename, name,
             source_info.description)
