@@ -57,7 +57,7 @@ def launch():
     override_modules()
 
     from miro.plat.utils import setup_logging, initialize_locale
-    setup_logging(in_downloader=True)
+    setup_logging(os.environ.get("DEMOCRACY_DOWNLOADER_LOG"))
     util.setup_logging()
     initialize_locale()
 
