@@ -79,18 +79,7 @@ def create_downloader(url, content_type, dlid, magnet=None):
     elif content_type ==  u'application/x-magnet':
         return BTDownloader(None, dlid, magnet=url)
     else:
-        return HTTPDownloader(url, dlid, expectedContentType=content_type)
-
-def start_new_download(url, dlid, content_type, channel_name):
-    """Creates a new downloader object.
-    """
-    check_u(url)
-    check_u(content_type)
-    if channel_name:
-        check_f(channel_name)
-    dl = create_downloader(url, content_type, dlid)
-    dl.channelName = channel_name
-    _downloads[dlid] = dl
+        return HTTPDownloader(url, dlid, expected_content_type=content_type)
 
 def pause_download(dlid):
     """Pauses a download by download id.
@@ -113,16 +102,19 @@ def info_hash_to_long(info_hash):
     """
     return long(str(info_hash), 16)
 
-def start_download(dlid):
+def start_download(url, dlid, content_type, channel_name):
     try:
         download = _downloads[dlid]
+        download.start()
     except KeyError:
-        # There is no download with this id
-        err = u"in start_download(): no downloader with id %s" % dlid
-        c = command.DownloaderErrorCommand(daemon.LAST_DAEMON, err)
-        c.send()
-        return True
-    return download.start()
+        # There is no download with this id.  This is a new download.
+        check_u(url)
+        check_u(content_type)
+        if channel_name:
+            check_f(channel_name)
+        dl = create_downloader(url, content_type, dlid)
+        dl.channel_name = channel_name
+        _downloads[dlid] = dl
 
 def stop_download(dlid, delete):
     _lock.acquire()
@@ -134,7 +126,6 @@ def stop_download(dlid, delete):
         return True
     finally:
         _lock.release()
-
     return download.stop(delete)
 
 def stop_upload(dlid):
@@ -197,6 +188,9 @@ def shutdown():
         _downloads[dlid].shutdown()
     logging.info("Shutting down torrent session...")
     TORRENT_SESSION.shutdown()
+    # Flush the status updates.
+    logging.info('flushing status updates...')
+    DOWNLOAD_UPDATER.flush_update()
     logging.info("shutdown() finished")
 
 def restore_downloader(downloader):
@@ -206,13 +200,13 @@ def restore_downloader(downloader):
         return
 
     downloader = copy(downloader)
-    dler_type = downloader.get('dlerType')
-    if dler_type == u'HTTP':
+    type_ = downloader.get('type')
+    if type_ == u'HTTP':
         dl = HTTPDownloader(restore=downloader)
-    elif dler_type == u'BitTorrent':
+    elif type_ == u'BitTorrent':
         dl = BTDownloader(restore=downloader)
     else:
-        err = u"in restore_downloader(): unknown dlerType: %s" % dler_type
+        err = u"in restore_downloader(): unknown type: %s" % type_
         c = command.DownloaderErrorCommand(daemon.LAST_DAEMON, err)
         c.send()
         return
@@ -253,6 +247,7 @@ class TorrentSession(object):
         self.set_dht()
         self.set_upload_limit()
         self.set_download_limit()
+        self.set_connection_limit()
         self.set_encryption()
         self.callback_handle = app.downloader_config_watcher.connect('changed',
                 self.on_config_changed)
@@ -421,24 +416,35 @@ class DownloadStatusUpdater(object):
 
     def __init__(self):
         self.to_update = set()
+        self.cmds_done = False
 
     def start_updates(self):
         eventloop.add_timeout(self.UPDATE_CLIENT_INTERVAL, self.do_update,
                 "Download status update")
 
-    def do_update(self):
+    def flush_update(self):
+        self.do_update(periodic=False)
+
+    def do_update(self, periodic=True):
         try:
             TORRENT_SESSION.update_torrents()
             statuses = []
             for downloader in self.to_update:
                 statuses.append(downloader.get_status())
             self.to_update = set()
-            if statuses:
+            if statuses or self.cmds_done:
                 command.BatchUpdateDownloadStatus(daemon.LAST_DAEMON,
-                        statuses).send()
+                                                  statuses,
+                                                  self.cmds_done).send()
+                self.cmds_done = False
         finally:
-            eventloop.add_timeout(self.UPDATE_CLIENT_INTERVAL, self.do_update,
-                    "Download status update")
+            if periodic:
+                eventloop.add_timeout(self.UPDATE_CLIENT_INTERVAL,
+                                      self.do_update,
+                                      "Download status update")
+
+    def set_cmds_done(self):
+        self.cmds_done = True
 
     def queue_update(self, downloader):
         self.to_update.add(downloader)
@@ -461,16 +467,16 @@ class BGDownloader(object):
     def __init__(self, url, dlid):
         self.dlid = dlid
         self.url = url
-        self.startTime = clock()
-        self.endTime = self.startTime
-        self.shortFilename = filename_from_url(url)
+        self.start_time = int(clock())
+        self.end_time = None
+        self.short_filename = filename_from_url(url)
         self.pick_initial_filename()
         self.state = u"downloading"
-        self.currentSize = 0
-        self.totalSize = -1
-        self.shortReasonFailed = self.reasonFailed = u"No Error"
-        self.retryTime = None
-        self.retryCount = -1
+        self.current_size = 0
+        self.total_size = None
+        self.short_reason_failed = self.reason_failed = u"No Error"
+        self.retry_time = None
+        self.retry_count = None
 
     def get_url(self):
         return self.url
@@ -479,29 +485,31 @@ class BGDownloader(object):
         return {'dlid': self.dlid,
             'url': self.url,
             'state': self.state,
-            'totalSize': self.totalSize,
-            'currentSize': self.currentSize,
+            'total_size': self.total_size,
+            'current_size': self.current_size,
             'eta': self.get_eta(),
             'rate': self.get_rate(),
-            'uploaded': 0,
+            'upload_size': 0,
             'filename': self.filename,
-            'startTime': self.startTime,
-            'endTime': self.endTime,
-            'shortFilename': self.shortFilename,
-            'reasonFailed': self.reasonFailed,
-            'shortReasonFailed': self.shortReasonFailed,
-            'dlerType': None,
-            'retryTime': self.retryTime,
-            'retryCount': self.retryCount,
-            'channelName': self.channelName}
+            'start_time': self.start_time,
+            'end_time': self.end_time,
+            'short_filename': self.short_filename,
+            'reason_failed': self.reason_failed,
+            'short_reason_failed': self.short_reason_failed,
+            'type': None,
+            'retry_time': self.retry_time,
+            'retry_count': self.retry_count}
 
-    def update_client(self):
-        x = command.UpdateDownloadStatus(daemon.LAST_DAEMON, self.get_status())
-        return x.send()
+    def update_client(self, now=False):
+        if not now:
+            DOWNLOAD_UPDATER.queue_update(self)
+        else:
+            command.BatchUpdateDownloadStatus(daemon.LAST_DAEMON,
+                                              [self.get_status()]).send()
 
     def pick_initial_filename(self, suffix=".part", torrent=False,
-                              is_directory=False):
-        """Pick a path to download to based on self.shortFilename.
+                              is_directory=False, exists=False):
+        """Pick a path to download to based on self.short_filename.
 
         This method sets self.filename, as well as creates any leading
         paths needed to start downloading there.
@@ -512,26 +520,33 @@ class BGDownloader(object):
             ascii and needs to be transformed into something sane.
         :param is_directory: If True, we're really creating a
             directory--not a file.
+        :param exists: If True, libtorrent has already created the
+            file/directory
         """
         download_dir = os.path.join(app.config.get(prefs.MOVIES_DIRECTORY),
                                     'Incomplete Downloads')
         # Create the download directory if it doesn't already exist.
         if not os.path.exists(download_dir):
             fileutil.makedirs(download_dir)
-        filename = self.shortFilename + suffix
+        filename = self.short_filename + suffix
         if not torrent:
             # this is an ascii filename and needs to be fixed
             filename = clean_filename(filename)
-
-        full_path = os.path.join(download_dir, filename)
-        if is_directory:
-            # if this is a torrent and it's a directory of files, then
-            # we create a temp directory to put the directory of files
-            # in.
-            new_filename = next_free_directory(full_path)
-        else:
-            new_filename, fp = next_free_filename(full_path)
-            fp.close()
+        new_filename = os.path.join(download_dir, filename)
+        if not exists or not os.path.exists(new_filename):
+            if exists:
+                # if the file/directory is supposed to exist and doesn't, just
+                # create it
+                logging.warn("f/d was supposed to exist, but does not: %s",
+                             new_filename)
+            if is_directory:
+                # if this is a torrent and it's a directory of files, then
+                # we create a temp directory to put the directory of files
+                # in.
+                new_filename = next_free_directory(new_filename)
+            else:
+                new_filename, fp = next_free_filename(new_filename)
+                fp.close()
         self.filename = new_filename
 
     def move_to_movies_directory(self):
@@ -573,8 +588,8 @@ class BGDownloader(object):
 
     def move_to_directory(self, directory):
         check_f(directory)
-        if self.channelName:
-            channel_name = filter_directory_name(self.channelName)
+        if self.channel_name:
+            channel_name = filter_directory_name(self.channel_name)
             # bug 10769: shutil and windows has problems with long
             # filenames, so we clip the directory name.
             if len(channel_name) > 80:
@@ -583,15 +598,22 @@ class BGDownloader(object):
         directory = self._ensure_directory_exists(directory)
 
         src = self.filename
-        dest = os.path.join(directory, self.shortFilename)
+        dest = os.path.join(directory, self.short_filename)
         if src == dest:
             return
 
-        if os.path.isdir(src):
-            dest = next_free_directory(dest)
-        else:
-            dest, fp = next_free_filename(dest)
-            fp.close()
+        try:
+            is_dir = os.path.isdir(src)
+            if is_dir:
+                dest = next_free_directory(dest)
+            else:
+                dest, fp = next_free_filename(dest)
+                fp.close()
+        except ValueError:
+            func = 'next_free_directory' if is_dir else 'next_free_filename'
+            logging.warn('move_to_directory: %s failed.  candidate = %r',
+                         func, dest)
+            return
 
         def callback():
             # for torrent of a directory of files, we want to remove
@@ -608,64 +630,66 @@ class BGDownloader(object):
         fileutil.migrate_file(src, dest, callback)
 
     def get_eta(self):
-        """Returns a float with the estimated number of seconds left.
+        """Returns a int with the estimated number of seconds left.
         """
-        if self.totalSize == -1:
-            return -1
+        if self.total_size is None:
+            return None
         rate = self.get_rate()
         if rate > 0:
-            return (self.totalSize - self.currentSize) / rate
+            return (self.total_size - self.current_size) // rate
         else:
             return 0
 
     def get_rate(self):
-        """Returns a float with the download rate in bytes per second
+        """Returns a int with the download rate in bytes per second
         """
-        if self.endTime != self.startTime:
-            rate = self.currentSize / (self.endTime - self.startTime)
-        else:
-            rate = self.rate
-        return rate
+        return self.rate
 
     def retry_download(self):
-        self.retryDC = None
+        self.retry_dc = None
         self.start(resume=False)
 
     def handle_temporary_error(self, short_reason, reason):
         self.state = u"offline"
-        self.endTime = self.startTime = 0
-        self.rate = 0
-        self.reasonFailed = reason
-        self.shortReasonFailed = short_reason
-        self.retryCount = self.retryCount + 1
-        if self.retryCount >= len(RETRY_TIMES):
-            self.retryCount = len(RETRY_TIMES) - 1
-        self.retryDC = eventloop.add_timeout(
-            RETRY_TIMES[self.retryCount], self.retry_download,
+        self.start_time = 0
+        self.end_time = None
+        self.rate = None
+        self.reason_failed = reason
+        self.short_reason_failed = short_reason
+        if self.retry_count is None:
+            self.retry_count = 0
+        else:
+            self.retry_count = self.retry_count + 1
+        if self.retry_count >= len(RETRY_TIMES):
+            self.retry_count = len(RETRY_TIMES) - 1
+        self.retry_dc = eventloop.add_timeout(
+            RETRY_TIMES[self.retry_count], self.retry_download,
             "Logarithmic retry")
         now = datetime.datetime.now()
-        self.retryTime = now + datetime.timedelta(
-            seconds=RETRY_TIMES[self.retryCount])
+        self.retry_time = now + datetime.timedelta(
+            seconds=RETRY_TIMES[self.retry_count])
         logging.warning("Temporary error: '%s' '%s'.  retrying at %s %s",
-                        short_reason, reason, self.retryTime, self.retryCount)
+                        short_reason, reason, self.retry_time,
+                        self.retry_count)
         self.update_client()
 
     def handle_error(self, short_reason, reason):
         self.state = u"failed"
-        self.reasonFailed = reason
-        self.shortReasonFailed = short_reason
+        self.reason_failed = reason
+        self.short_reason_failed = short_reason
         self.update_client()
 
     def handle_network_error(self, error):
         if isinstance(error, httpclient.NetworkError):
             if (isinstance(error, (httpclient.MalformedURL,
-                                   httpclient.UnknownHostError,
                                    httpclient.AuthorizationFailed,
                                    httpclient.ProxyAuthorizationFailed,
+                                   httpclient.TooManyRedirects,
+                                   httpclient.InvalidRedirect,
                                    httpclient.UnexpectedStatusCode))):
                 self.handle_error(error.getFriendlyDescription(),
                                   error.getLongDescription())
-                self.retryCount = -1 # reset retryCount
+                self.retry_count = None # reset retry_count
             else:
                 self.handle_temporary_error(error.getFriendlyDescription(),
                                             error.getLongDescription())
@@ -687,7 +711,14 @@ class BGDownloader(object):
                 size = 0
             preserved = (app.config.get(prefs.PRESERVE_X_GB_FREE) *
                          1024 * 1024 * 1024)
-            available = get_available_bytes_for_movies() - preserved
+            # below code gets the additional space we expect to be taken by
+            # in-progress downloads.  total_size is None when the download
+            # doesn't know how big it is.
+            additional_space =  sum(dl.total_size - dl.current_size
+                                    for dl in _downloads.values()
+                                    if dl.total_size is not None)
+            available = (get_available_bytes_for_movies() - preserved -
+                         additional_space)
             accept = (size <= available)
         return accept
 
@@ -695,22 +726,18 @@ class HTTPDownloader(BGDownloader):
     CHECK_STATS_TIMEOUT = 1.0
 
     def __init__(self, url=None, dlid=None, restore=None,
-                 expectedContentType=None):
-        self.retryDC = None
-        self.channelName = None
-        self.expectedContentType = expectedContentType
+                 expected_content_type=None):
+        self.retry_dc = None
+        self.channel_name = None
+        self.expected_content_type = expected_content_type
         if restore is not None:
-            if not isinstance(restore.get('totalSize', 0), int):
-                # Sometimes restoring old downloaders caused errors
-                # because their totalSize wasn't an int.  (see #3965)
-                restore['totalSize'] = int(restore['totalSize'])
             self.__dict__.update(restore)
             self.restartOnError = True
         else:
             BGDownloader.__init__(self, url, dlid)
             self.restartOnError = False
         self.client = None
-        self.rate = 0
+        self.rate = None
         if self.state == u'downloading':
             self.start_download()
         elif self.state == u'offline':
@@ -720,14 +747,14 @@ class HTTPDownloader(BGDownloader):
 
     def start_new_download(self):
         """Start a download, discarding any existing data"""
-        self.currentSize = 0
-        self.totalSize = -1
+        self.current_size = 0
+        self.total_size = None
         self.start_download(resume=False)
 
     def start_download(self, resume=True):
-        if self.retryDC:
-            self.retryDC.cancel()
-            self.retryDC = None
+        if self.retry_dc:
+            self.retry_dc.cancel()
+            self.retry_dc = None
         if resume:
             resume = self._resume_sanity_check()
 
@@ -737,9 +764,7 @@ class HTTPDownloader(BGDownloader):
             self.url, self.on_download_finished, self.on_download_error,
             header_callback=self.on_headers, write_file=self.filename,
             resume=resume)
-        self.update_client()
-        eventloop.add_timeout(self.CHECK_STATS_TIMEOUT, self.update_stats,
-                'update http downloader stats')
+        self.update_stats()
 
     def _resume_sanity_check(self):
         """Do sanity checks to test if we should try HTTP Resume.
@@ -752,15 +777,15 @@ class HTTPDownloader(BGDownloader):
         # size.  In particular, before the libcurl change, we would
         # preallocate the entire file, so we need to undo this.
         file_size = os.stat(self.filename)[stat.ST_SIZE]
-        if file_size > self.currentSize:
+        if file_size > self.current_size:
             # use logging.info rather than warn, since this is the
             # usual case from upgrading from 3.0.x to 3.1
-            logging.info("File larger than currentSize: truncating.  "
+            logging.info("File larger than current_size: truncating.  "
                          "url: %s, path: %s.", self.url, self.filename)
             f = open(self.filename, "ab")
-            f.truncate(self.currentSize)
+            f.truncate(self.current_size)
             f.close()
-        elif file_size < self.currentSize:
+        elif file_size < self.current_size:
             # Data got deleted somehow.  Let's start over.
             logging.warn("File doesn't contain enough data to resume.  "
                     "url: %s, path: %s.", self.url, self.filename)
@@ -778,9 +803,9 @@ class HTTPDownloader(BGDownloader):
             self.client.cancel(remove_file=remove_file)
             self.destroy_client()
         # if it's in a retrying state, we want to nix that, too
-        if self.retryDC:
-            self.retryDC.cancel()
-            self.retryDC = None
+        if self.retry_dc:
+            self.retry_dc.cancel()
+            self.retry_dc = None
 
     def handle_error(self, short_reason, reason):
         BGDownloader.handle_error(self, short_reason, reason)
@@ -790,47 +815,47 @@ class HTTPDownloader(BGDownloader):
                 fileutil.remove(self.filename)
             except OSError:
                 pass
-        self.currentSize = 0
-        self.totalSize = -1
+        self.current_size = 0
+        self.total_size = None
 
     def handle_temporary_error(self, short_reason, reason):
         self.cancel_request()
         BGDownloader.handle_temporary_error(self, short_reason, reason)
 
     def handle_move_error(self, error):
-        logging.exception("Error moving to movies directory")
-        logging.warn("filename: %s, shortFilename: %s, movies directory: %s",
-                self.filename, self.shortFilename,
+        logging.exception("Error moving to movies directory\n"
+                "filename: %s, short_filename: %s, movies directory: %s",
+                self.filename, self.short_filename,
             app.config.get(prefs.MOVIES_DIRECTORY))
         text = _("Error moving to movies directory")
         self.handle_generic_error(text)
 
     def on_headers(self, info):
         if 'total-size' in info:
-            self.totalSize = info['total-size']
-        if not self.accept_download_size(self.totalSize):
+            self.total_size = info['total-size']
+        if not self.accept_download_size(self.total_size):
             self.handle_error(_("Not enough disk space"),
                 _("%(amount)s MB required to store this video",
-                  {"amount": self.totalSize / (2 ** 20)}))
+                  {"amount": self.total_size / (2 ** 20)}))
             return
-        # We should successfully download the file.  Reset retryCount
+        # We should successfully download the file.  Reset retry_count
         # and accept defeat if we see an error.
         self.restartOnError = False
-        # update shortFilename based on the headers.  This will affect
+        # update short_filename based on the headers.  This will affect
         # how we move the file once the download is finished
-        self.shortFilename = clean_filename(info['filename'])
-        if self.expectedContentType is not None:
-            ext_content_type = self.expectedContentType
+        self.short_filename = clean_filename(info['filename'])
+        if self.expected_content_type is not None:
+            ext_content_type = self.expected_content_type
         else:
             ext_content_type = info.get('content-type')
-        self.shortFilename = check_filename_extension(self.shortFilename,
+        self.short_filename = check_filename_extension(self.short_filename,
                 ext_content_type)
 
     def on_download_error(self, error):
         if isinstance(error, httpclient.ResumeFailed):
             # try starting from scratch
-            self.currentSize = 0
-            self.totalSize = -1
+            self.current_size = 0
+            self.total_size = None
             self.start_new_download()
         elif isinstance(error, httpclient.AuthorizationCanceled):
             self.destroy_client()
@@ -845,16 +870,17 @@ class HTTPDownloader(BGDownloader):
     def on_download_finished(self, response):
         self.destroy_client()
         self.state = u"finished"
-        self.endTime = clock()
+        self.end_time = int(clock())
+        self.rate = None
         # bug 14131 -- if there's nothing here, treat it like a temporary
         # error
-        if self.currentSize == 0:
+        if self.current_size == 0:
             self.handle_network_error(httpclient.PossiblyTemporaryError(
                 _("no content")))
 
         else:
-            if self.totalSize == -1:
-                self.totalSize = self.currentSize
+            if self.total_size is None:
+                self.total_size = self.current_size
             try:
                 self.move_to_movies_directory()
             except (OSError, IOError), e:
@@ -863,7 +889,7 @@ class HTTPDownloader(BGDownloader):
 
     def get_status(self):
         data = BGDownloader.get_status(self)
-        data['dlerType'] = 'HTTP'
+        data['type'] = 'HTTP'
         return data
 
     def update_stats(self):
@@ -874,14 +900,14 @@ class HTTPDownloader(BGDownloader):
             return
         stats = self.client.get_stats()
         if stats.status_code in (200, 206):
-            # Only upload currentSize/rate if we are currently
+            # Only upload current_size/rate if we are currently
             # downloading something.  Don't change them before the
             # transfer starts, while we are handling redirects, etc.
-            self.currentSize = stats.downloaded + stats.initial_size
+            self.current_size = stats.downloaded + stats.initial_size
             self.rate = stats.download_rate
         eventloop.add_timeout(self.CHECK_STATS_TIMEOUT, self.update_stats,
                 'update http downloader stats')
-        DOWNLOAD_UPDATER.queue_update(self)
+        self.update_client()
 
     def pause(self):
         """Pauses the download.
@@ -908,7 +934,7 @@ class HTTPDownloader(BGDownloader):
             # Cancel the request, don't keep around partially
             # downloaded data
             self.cancel_request(remove_file=True)
-        self.currentSize = 0
+        self.current_size = 0
         self.state = u"stopped"
         self.update_client()
 
@@ -959,7 +985,7 @@ def save_fast_resume_data(info_hash, fast_resume_data):
             return
 
     try:
-        with open(fast_resume_file, "wb") as f:
+        with open(fast_resume_file, 'wb') as f:
             f.write(fast_resume_data)
     except (OSError, IOError):
         logging.exception("Error occured trying to write fast_resume_data")
@@ -1015,17 +1041,18 @@ class BTDownloader(BGDownloader):
     def __init__(self, url=None, item=None, restore=None, magnet=None):
         self.metainfo = None
         self.torrent = None
-        self.rate = self.eta = 0
-        self.upRate = self.uploaded = 0
+        self.rate = self.eta = None
+        self.upload_rate = None
+        self.upload_size = 0
         self.activity = None
         self.fast_resume_data = None
-        self.retryDC = None
-        self.channelName = None
-        self.uploadedStart = 0
+        self.retry_dc = None
+        self.channel_name = None
+        self.uploaded_start = 0
         self.restarting = False
-        self.seeders = -1
-        self.leechers = -1
-        self.connections = -1
+        self.seeders = None
+        self.leechers = None
+        self.connections = None
         self.metainfo_updated = False
         self.info_hash = None
         self.magnet = magnet
@@ -1051,22 +1078,22 @@ class BTDownloader(BGDownloader):
             else:
                 torrent_info = lt.torrent_info(lt.bdecode(self.metainfo))
                 params["ti"] = torrent_info
-                self.totalSize = torrent_info.total_size()
+                self.total_size = torrent_info.total_size()
                 duplicate = TORRENT_SESSION.find_duplicate_torrent(
                     params["ti"])
 
             if duplicate is not None:
                 c = command.DuplicateTorrent(daemon.LAST_DAEMON,
-                        duplicate.dlid, self.dlid)
+                                             duplicate.dlid, self.dlid)
                 c.send()
                 return
 
             if self.firstTime and not self.accept_download_size(
-                self.totalSize):
+                self.total_size):
                 self.handle_error(
                     _("Not enough disk space"),
                     _("%(amount)s MB required to store this video",
-                      {"amount": self.totalSize / (2 ** 20)})
+                      {"amount": self.total_size / (2 ** 20)})
                     )
                 return
 
@@ -1077,7 +1104,26 @@ class BTDownloader(BGDownloader):
             params["auto_managed"] = False
             params["paused"] = False
             params["duplicate_is_error"] = True
-            params["storage_mode"] = lt.storage_mode_t.storage_mode_allocate
+
+            # About file allocation: the default scheme that would work is
+            # using sparse file allocation.  With sparse, physical disk space
+            # is consumed on an as-needed basis and we no shuffling of the
+            # individual file chunks is required (missing ones are zero-filled
+            # on the fly and don't take up disk space).  Unfortunately not
+            # all filesystems support sparse files and on those that don't
+            # the results are less than ideal.  In particular, pause
+            # immediately after a large file is started can take a long time
+            # because the close() of the torrent file handle must wait till
+            # all bytes are zero-filled.  Using compact mode has the
+            # disadvantage of extra disk i/o because of pieces are moved
+            # on the fly rather than being placed into its logical location
+            # within the file but makes everything else usable again.
+            if self.magnet:
+                # unfortunately, compact allocation doesn't work for Magnet
+                # links, so we revert to the allocate mode
+                params["storage_mode"] = lt.storage_mode_t.storage_mode_allocate
+            else:
+                params["storage_mode"] = lt.storage_mode_t.storage_mode_compact
 
             if self.info_hash:
                 self.fast_resume_data = load_fast_resume_data(self.info_hash)
@@ -1205,36 +1251,36 @@ class BTDownloader(BGDownloader):
                       self.rate,
                       self.seeders,
                       self.leechers,
-                      self.currentSize)
+                      self.current_size)
 
     def update_status(self):
         """
         activity -- string specifying what's currently happening or None for
                 normal operations.
-        upRate -- upload rate in B/s
-        downRate -- download rate in B/s
-        upTotal -- total MB uploaded
+        upload_rate -- upload rate in B/s
+        rate -- download rate in B/s
+        upload_size -- total MB upload_size
         downTotal -- total MB downloaded
         fractionDone -- what portion of the download is completed.
         timeEst -- estimated completion time, in seconds.
-        totalSize -- total size of the torrent in bytes
+        total_size -- total size of the torrent in bytes
         seeders -- number of seeders for this torrent
         leechers -- number of leechers for this torrent
         connecting -- nummber of peers we're connected to
         """
         status = self.torrent.status()
-        self.totalSize = status.total_wanted
-        self.rate = status.download_payload_rate
-        self.upRate = status.upload_payload_rate
-        self.uploaded = status.total_payload_upload + self.uploadedStart
+        self.total_size = status.total_wanted
+        self.rate = int(status.download_payload_rate)
+        self.upload_rate = int(status.upload_payload_rate)
+        self.upload_size = status.total_payload_upload + self.uploaded_start
         self.seeders = status.num_complete
         self.leechers = status.num_incomplete
         self.connections = status.num_connections
         try:
-            self.eta = ((status.total_wanted - status.total_wanted_done) /
-                        float(status.download_payload_rate))
+            self.eta = ((status.total_wanted - status.total_wanted_done) //
+                        status.download_payload_rate)
         except ZeroDivisionError:
-            self.eta = 0
+            self.eta = None
 
         # FIXME - this needs some more examination before it's
         # enabled.
@@ -1250,7 +1296,7 @@ class BTDownloader(BGDownloader):
         else:
             self.activity = None
 
-        self.currentSize = status.total_wanted_done
+        self.current_size = status.total_wanted_done
 
         # these are useful for debugging torrent issues
         # self._debug_print_status()
@@ -1260,14 +1306,14 @@ class BTDownloader(BGDownloader):
              and status.state == lt.torrent_status.states.seeding)):
             self.move_to_movies_directory()
             self.state = u"uploading"
-            self.endTime = clock()
-            self.update_client()
-        else:
-            DOWNLOAD_UPDATER.queue_update(self)
+            self.end_time = int(clock())
+            self.rate = 0
+
+        self.update_client()
 
         if app.config.get(prefs.LIMIT_UPLOAD_RATIO):
             if status.state == lt.torrent_status.states.seeding:
-                if ((float(self.uploaded) / self.totalSize >
+                if ((float(self.upload_size) / self.total_size >
                      app.config.get(prefs.UPLOAD_RATIO))):
                     self.stop_upload()
 
@@ -1336,9 +1382,9 @@ class BTDownloader(BGDownloader):
             self.magnet = data['url']
             data['url'] = None
         self.__dict__.update(data)
-        self.rate = self.eta = 0
-        self.upRate = 0
-        self.uploadedStart = self.uploaded
+        self.rate = self.eta = None
+        self.upload_rate = 0
+        self.uploaded_start = self.upload_size
         if self.state in (u'downloading', u'uploading'):
             self.run_downloader(done=True)
         elif self.state == u'offline':
@@ -1346,13 +1392,13 @@ class BTDownloader(BGDownloader):
 
     def get_status(self):
         data = BGDownloader.get_status(self)
-        data['upRate'] = self.upRate
-        data['uploaded'] = self.uploaded
+        data['upload_rate'] = self.upload_rate
+        data['upload_size'] = self.upload_size
         if self.metainfo_updated:
             data['metainfo'] = self.metainfo
             self.metainfo_updated = False
         data['activity'] = self.activity
-        data['dlerType'] = 'BitTorrent'
+        data['type'] = 'BitTorrent'
         data['seeders'] = self.seeders
         data['leechers'] = self.leechers
         data['connections'] = self.connections
@@ -1404,9 +1450,9 @@ class BTDownloader(BGDownloader):
             return
 
         self.state = u"downloading"
-        if self.retryDC:
-            self.retryDC.cancel()
-            self.retryDC = None
+        if self.retry_dc:
+            self.retry_dc.cancel()
+            self.retry_dc = None
         self.update_client()
         self.get_metainfo()
 
@@ -1438,7 +1484,7 @@ class BTDownloader(BGDownloader):
             except (KeyError, RuntimeError):
                 self.handle_corrupt_torrent()
                 return
-            self.shortFilename = utf8_to_filename(name)
+            self.short_filename = utf8_to_filename(name)
             try:
                 self.pick_initial_filename(
                     suffix="", torrent=True, is_directory=is_directory)
@@ -1447,8 +1493,9 @@ class BTDownloader(BGDownloader):
             # exceptions.
             #
             # Not sure if this is correct but if we throw a runtime
-            # error like above it can't hurt anyone.
-            except (OSError, IOError):
+            # error like above it can't hurt anyone.  ValueError to catch
+            # next_free_filename().
+            except (ValueError, OSError, IOError):
                 raise RuntimeError
         self.update_client()
         self._resume_torrent()
@@ -1461,21 +1508,26 @@ class BTDownloader(BGDownloader):
         """
         if not self.torrent.has_metadata():
             return
-        self.shortFilename =  utf8_to_filename(
+        self.short_filename =  utf8_to_filename(
             self.torrent.get_torrent_info().name())
 
         # FIXME: we should determine whether it is a directory
         # in the same way in got_metainfo and got_delayed_metainfo
         is_directory = False
+        multiple_files = False
         for file_ in self.torrent.get_torrent_info().files():
-            if os.sep in file_.path:
+            # if there is >1 file, we'll go back through the loop and
+            # multiple_files will be True
+            if os.sep in file_.path or multiple_files:
                 is_directory = True
+                break
+            else:
+                multiple_files = True
 
-        is_directory = (is_directory or 
-                       len(self.torrent.get_torrent_info().files()) > 1)
         try:
             self.pick_initial_filename(
-                suffix="", torrent=True, is_directory=is_directory)
+                suffix="", torrent=True, is_directory=is_directory,
+                exists=True)
             # Somewhere deep it calls makedirs() which can throw
             # exceptions.
             #
@@ -1485,8 +1537,25 @@ class BTDownloader(BGDownloader):
             raise RuntimeError
         save_path = self.calc_save_path()
         self.torrent.move_storage(save_path)
+        self.metainfo = self.calc_metainfo()
+        self.metainfo_updated = True
+        self.update_client()
 
 
+    def calc_metainfo(self):
+        torrent_info = self.torrent.get_torrent_info()
+        metainfo = {
+            'info': lt.bdecode(torrent_info.metadata())
+        }
+        trackers = list(torrent_info.trackers())
+        if trackers:
+            # which tracker URL should we use?  For now, we just use the first
+            # one in the list.
+            metainfo['announce'] = trackers[0].url
+        else:
+            logging.warn("calc_metainfo(): no announce URL")
+        return lt.bencode(metainfo)
+
     def handle_corrupt_torrent(self):
         self.handle_error(
             _("Corrupt Torrent"),
