@@ -35,22 +35,21 @@ network requests and scheduling.
 TODO: handle user setting clock back
 """
 
-import threading
 import errno
+import heapq
+import logging
+import Queue
 import select
 import socket
-import heapq
-import Queue
-import logging
+import threading
 import traceback
+
 from miro import app
 from miro import config
 from miro import trapcall
 from miro import signals
 from miro import util
-
 from miro.clock import clock
-
 from miro.plat.utils import thread_body
 
 cumulative = {}
@@ -172,7 +171,7 @@ class ThreadPool(object):
     instead is call them in a separate thread and return the result in
     a callback that executes in the event loop.
     """
-    THREADS = 3
+    THREADS = 4
 
     def __init__(self, event_loop):
         self.event_loop = event_loop
@@ -227,12 +226,12 @@ class ThreadPool(object):
         # in a blocking operation which is exactly the point of having them
         # so eventloop.join() in turn blocks.  So if it doesn't clean up
         # in time let the daemon flag in the Thread() do its job.  See #16584.
-        while len(self.threads) > 0:
-            x = self.threads.pop()
+        for t in self.threads:
             try:
-                x.join(0.5)
+                t.join(0.5)
             except StandardError:
                 pass
+        self.threads = []
 
 class SimpleEventLoop(signals.SignalEmitter):
     def __init__(self):
@@ -523,6 +522,9 @@ def shutdown():
 def connect(signal, callback):
     _eventloop.connect(signal, callback)
 
+def connect_after(signal, callback):
+    _eventloop.connect_after(signal, callback)
+
 def disconnect(signal, callback):
     _eventloop.disconnect(signal, callback)
 
@@ -608,3 +610,41 @@ def idle_iterator(func):
         return idle_iterate(func, "%s() (using idle_iterator)" % func.__name__, 
                             args=args, kwargs=kwargs)
     return queuer
+
+class DelayedFunctionCaller(object):
+    """Call a function sometime in the future using add_idle()/add_timeout()
+
+    This class also tracks whether a function has been scheduled and avoids
+    scheduling it twice.
+    """
+    def __init__(self, func):
+        """Create a DelayedFunctionCaller
+
+        :param func: function to call.
+        """
+        self.dc = None
+        self.func = func
+        self.name = 'delayed call to %s' % func
+
+    def call_when_idle(self, *args, **kwargs):
+        """Call our function when we're idle."""
+        if self.dc is None:
+            self.dc = add_idle(self.call_now, self.name, args=args,
+                               kwargs=kwargs)
+
+    def call_after_timeout(self, timeout, *args, **kwargs):
+        """Call our function after a timeout."""
+        if self.dc is None:
+            self.dc = add_timeout(timeout, self.call_now, self.name,
+                                  args=args, kwargs=kwargs)
+
+    def call_now(self, *args, **kwargs):
+        """Call our function immediately."""
+        self.cancel_call()
+        self.func(*args, **kwargs)
+
+    def cancel_call(self):
+        """Cancel a timeout/idle callback, if scheduled."""
+        if self.dc is not None:
+            self.dc.cancel()
+            self.dc = None
