@@ -57,13 +57,12 @@ from miro import dialogs
 from miro import download_utils
 from miro import eventloop
 from miro import feedupdate
-from miro import flashscraper
 from miro import models
 from miro import prefs
 from miro.plat import resources
 from miro import downloader
 from miro.util import (returns_unicode, returns_filename, unicodify, check_u,
-                       check_f, quote_unicode_url, escape, to_uni,
+                       check_f, quote_unicode_url, to_uni,
                        is_url, stringify, is_magnet_uri)
 from miro import fileutil
 from miro.plat.utils import filename_to_unicode, make_url_safe, unmake_url_safe
@@ -78,8 +77,6 @@ from miro.clock import clock
 # the unittests to speed things up
 _RUN_FEED_PARSER_INLINE = False
 
-WHITESPACE_PATTERN = re.compile(r"^[ \t\r\n]*$")
-
 DEFAULT_FEED_ICON = "images/icon-podcast-small.png"
 
 @returns_unicode
@@ -101,7 +98,6 @@ class _RateLimiter(object):
         self.last_time = time.time()
 
     def check_for_sleep(self):
-        new_time = time.time()
         elapsed = self.last_time - time.time()
         if elapsed < 0.1:
             return # don't sleep until a decent of time has passed.
@@ -237,7 +233,9 @@ def run_feedparser(html, callback, errback):
         else:
             callback(rv)
     else:
-        workerprocess.run_feedparser(html, callback, errback)
+        workerprocess.send(workerprocess.FeedparserTask(html),
+                           lambda msg, result: callback(result),
+                           lambda msg, error: errback(error))
 
 # Wait X seconds before updating the feeds at startup
 INITIAL_FEED_UPDATE_DELAY = 5.0
@@ -247,7 +245,7 @@ class FeedImpl(DDBObject):
     """
     def setup_new(self, url, ufeed, title=None):
         check_u(url)
-        if title:
+        if title is not None:
             check_u(title)
         self.url = url
         self.ufeed = ufeed
@@ -303,19 +301,19 @@ class FeedImpl(DDBObject):
         if firstTriggerDelay >= 0:
             self.scheduler = eventloop.add_timeout(
                 firstTriggerDelay, self.update,
-                "Podcast update (%s)" % self.get_title())
+                "Podcast update (%s)" % self.title)
         else:
             if self.updateFreq > 0:
                 logging.debug("scheduling update in %s seconds (%s)",
                              self.updateFreq,
-                             self.get_title())
+                             self.title)
                 self.scheduler = eventloop.add_timeout(
                     self.updateFreq, self.update,
-                    "Podcast update (%s)" % self.get_title())
+                    "Podcast update (%s)" % self.title)
             else:
                 logging.debug("updateFreq is %s: skipping update (%s)",
                              self.updateFreq,
-                             self.get_title())
+                             self.title)
 
     def cancel_update_events(self):
         if hasattr(self, 'scheduler') and self.scheduler is not None:
@@ -331,21 +329,6 @@ class FeedImpl(DDBObject):
         """Get the path to our thumbnail when there isn't a downloaded icon"""
         return default_feed_icon_path()
 
-    @returns_unicode
-    def get_title(self):
-        """Returns the title of the feed
-        """
-        try:
-            title = self.title
-            if title is None or WHITESPACE_PATTERN.match(title):
-                if self.ufeed.baseTitle is not None:
-                    title = self.ufeed.baseTitle
-                else:
-                    title = self.url
-            return title
-        except AttributeError:
-            return u""
-
     @returns_unicode
     def get_url(self):
         """Returns the URL of the feed
@@ -392,7 +375,7 @@ class FeedImpl(DDBObject):
         pass
 
     def __str__(self):
-        return "%s - %s" % (self.__class__.__name__, stringify(self.get_title()))
+        return "%s - %s" % (self.__class__.__name__, stringify(self.title))
 
     def clean_old_items(self):
         """
@@ -436,17 +419,17 @@ class Feed(DDBObject, iconcache.IconCacheOwnerMixin):
         self.maxNew = 3
         self.maxOldItems = None
         self.expire = u"system"
-        self.expireTime = None
+        self.expire_timedelta = None
         self.fallBehind = -1
-        self.last_viewed = datetime.min
 
         self.baseTitle = None
-        self.origURL = url
+        self.orig_url = url
         self.errorState = False
         self.loading = True
         self._actualFeed = None
         self._set_feed_impl(FeedImpl(url, self, title))
         self.setup_new_icon_cache()
+        self.thumbnail_path = None
         self.informOnError = True
         self.folder_id = None
         self.searchTerm = search_term
@@ -483,7 +466,7 @@ class Feed(DDBObject, iconcache.IconCacheOwnerMixin):
                     pass
         # otherwise, make a new FeedImpl
         if self._actualFeed is None:
-            self._set_feed_impl(FeedImpl(self.origURL, self))
+            self._set_feed_impl(FeedImpl(self.orig_url, self))
             self.signal_change()
         return self._actualFeed
 
@@ -491,15 +474,15 @@ class Feed(DDBObject, iconcache.IconCacheOwnerMixin):
 
     @classmethod
     def get_by_url(cls, url):
-        return cls.make_view('origURL=?', (url,)).get_singleton()
+        return cls.make_view('orig_url=?', (url,)).get_singleton()
 
     @classmethod
     def get_by_url_and_search(cls, url, searchTerm):
         if searchTerm is not None:
-            view = cls.make_view('origURL=? AND searchTerm=?',
+            view = cls.make_view('orig_url=? AND searchTerm=?',
                     (url, searchTerm))
         else:
-            view = cls.make_view('origURL=? AND searchTerm IS NULL', (url,))
+            view = cls.make_view('orig_url=? AND searchTerm IS NULL', (url,))
         return view.get_singleton()
 
     @classmethod
@@ -528,7 +511,7 @@ class Feed(DDBObject, iconcache.IconCacheOwnerMixin):
 
     @classmethod
     def watched_folder_view(cls):
-        return cls.make_view("origURL LIKE 'dtv:directoryfeed:%'")
+        return cls.make_view("orig_url LIKE 'dtv:directoryfeed:%'")
 
     def on_db_insert(self):
         self.generate_feed(True)
@@ -626,26 +609,18 @@ class Feed(DDBObject, iconcache.IconCacheOwnerMixin):
                     self.auto_pending_items.count())
             return self._num_available
 
-    def get_viewed(self):
-        """Returns true iff this feed has been looked at
-        """
-        return self.last_viewed != datetime.min
-
     def mark_as_viewed(self):
         """Sets the last time the feed was viewed to now
         """
-        # get the list of available items before we reset the time
-        available_items = list(self.available_items)
-        self.last_viewed = datetime.now()
         try:
             del self._num_available
         except AttributeError:
             pass
+        for item in list(self.available_items):
+            item.unset_new()
         if self.in_folder():
             self.get_folder().signal_change()
         self.signal_change()
-        for item in available_items:
-            item.signal_change(needs_save=False)
 
     def start_manual_download(self):
         next_ = None
@@ -681,7 +656,7 @@ class Feed(DDBObject, iconcache.IconCacheOwnerMixin):
                 return []
             delta = timedelta(days=expire_after_x_days)
         else:
-            delta = self.expireTime
+            delta = self.expire_timedelta
         return models.Item.feed_expiring_view(self.id, datetime.now() - delta)
 
     def expire_items(self):
@@ -698,11 +673,12 @@ class Feed(DDBObject, iconcache.IconCacheOwnerMixin):
         """See item.get_thumbnail to figure out which items to send
         signals for.
         """
-        self.signal_change(needs_save=False)
+        self.calc_thumbnail_path()
+        self.signal_change()
         for item in self.items:
             if not item.icon_cache or not (item.icon_cache.is_valid() or
                     item.screenshot or
-                    item.isContainerItem):
+                    item.is_container_item):
                 item.signal_change(needs_save=False)
 
     def get_id(self):
@@ -717,12 +693,20 @@ class Feed(DDBObject, iconcache.IconCacheOwnerMixin):
         return self.loading or (self.actualFeed and self.actualFeed.updating)
 
     @returns_unicode
-    def get_title(self):
+    def get_title_without_search_terms(self):
         if self.userTitle is not None:
             return self.userTitle
+        elif self.actualFeed.title is not None:
+            return self.actualFeed.title
+        elif self.baseTitle is not None:
+            return self.baseTitle
+        else:
+            return self.actualFeed.url
 
-        title = self.actualFeed.get_title()
-        if self.searchTerm is not None:
+    @returns_unicode
+    def get_title(self):
+        title = self.get_title_without_search_terms()
+        if self.userTitle is None and self.searchTerm is not None:
             title = u"%s for '%s'" % (title, self.searchTerm)
         return title
 
@@ -777,7 +761,7 @@ class Feed(DDBObject, iconcache.IconCacheOwnerMixin):
         """
         self.confirm_db_thread()
         self.expire = type_
-        self.expireTime = timedelta(hours=time_)
+        self.expire_timedelta = timedelta(hours=time_)
 
         if self.expire == u"never":
             for item in self.items:
@@ -861,35 +845,36 @@ class Feed(DDBObject, iconcache.IconCacheOwnerMixin):
 
     def generate_feed(self, removeOnError=False):
         newFeed = None
-        if self.origURL == u"dtv:directoryfeed":
+        if self.orig_url == u"dtv:directoryfeed":
             newFeed = DirectoryFeedImpl(self)
             self.visible = False
-        elif (self.origURL.startswith(u"dtv:directoryfeed:")):
-            url = self.origURL[len(u"dtv:directoryfeed:"):]
+        elif (self.orig_url.startswith(u"dtv:directoryfeed:")):
+            url = self.orig_url[len(u"dtv:directoryfeed:"):]
             dir_ = unmake_url_safe(url)
             newFeed = DirectoryWatchFeedImpl(self, dir_)
-        elif self.origURL == u"dtv:search":
+        elif self.orig_url == u"dtv:search":
             newFeed = SearchFeedImpl(self)
             self.visible = False
-        elif self.origURL == u"dtv:searchDownloads":
+        elif self.orig_url == u"dtv:searchDownloads":
             newFeed = SearchDownloadsFeedImpl(self)
             self.visible = False
-        elif self.origURL == u"dtv:manualFeed":
+        elif self.orig_url == u"dtv:manualFeed":
             newFeed = ManualFeedImpl(self)
             self.visible = False
-        elif SEARCH_URL_MATCH_RE.match(self.origURL):
-            newFeed = SavedSearchFeedImpl(self.origURL, self)
+        elif SEARCH_URL_MATCH_RE.match(self.orig_url):
+            newFeed = SavedSearchFeedImpl(self.orig_url, self)
         else:
-            self.download = grab_url(self.origURL,
+            self.download = grab_url(self.orig_url,
                     lambda info: self._generate_feed_callback(info, removeOnError),
                     lambda error: self._generate_feed_errback(error, removeOnError),
                     default_mime_type=u'application/rss+xml')
-            logging.debug("added async callback to create feed %s", self.origURL)
+            logging.debug("added async callback to create feed %s",
+                          self.orig_url)
         if newFeed:
             self.finish_generate_feed(newFeed)
 
     def is_watched_folder(self):
-        return self.origURL.startswith("dtv:directoryfeed:")
+        return self.orig_url.startswith("dtv:directoryfeed:")
 
     def _handle_feed_loading_error(self, errorDescription):
         self.download = None
@@ -925,7 +910,7 @@ class Feed(DDBObject, iconcache.IconCacheOwnerMixin):
         if not self.id_exists():
             return
         logging.warning("Couldn't load podcast at %s (%s)",
-                        self.origURL, error)
+                        self.orig_url, error)
         self._handle_feed_loading_error(error.getFriendlyDescription())
 
     def _generate_feed_callback(self, info, removeOnError):
@@ -940,8 +925,8 @@ class Feed(DDBObject, iconcache.IconCacheOwnerMixin):
 
         if not self.id_exists():
             return
-        if info['updated-url'] != self.origURL and \
-                not self.origURL.startswith('dtv:'): # we got redirected
+        if info['updated-url'] != self.orig_url and \
+                not self.orig_url.startswith('dtv:'): # we got redirected
             f = lookup_feed(info['updated-url'], self.searchTerm)
             if f is not None: # already have this feed, so delete us
                 self.remove()
@@ -1129,12 +1114,12 @@ class Feed(DDBObject, iconcache.IconCacheOwnerMixin):
         """
         self.confirm_db_thread()
         expireAfterSetting = app.config.get(prefs.EXPIRE_AFTER_X_DAYS)
-        if ((self.expireTime is None or self.expire == 'never'
+        if ((self.expire_timedelta is None or self.expire == 'never'
              or (self.expire == 'system' and expireAfterSetting <= 0))):
             return 0
         else:
-            return (self.expireTime.days * 24 +
-                    self.expireTime.seconds / 3600)
+            return (self.expire_timedelta.days * 24 +
+                    self.expire_timedelta.seconds / 3600)
 
     def is_autodownloadable(self):
         """Returns true iff item is autodownloadable
@@ -1176,16 +1161,23 @@ class Feed(DDBObject, iconcache.IconCacheOwnerMixin):
             self.get_folder().signal_change()
 
     def thumbnail_valid(self):
-        return self.icon_cache and self.icon_cache.is_valid()
+        return self.thumbnail_path is not None
 
     @returns_filename
     def get_thumbnail_path(self):
-        self.confirm_db_thread()
         if self.thumbnail_valid():
-            return fileutil.expand_filename(self.icon_cache.get_filename())
+            return self.thumbnail_path
         else:
             return self.actualFeed.default_thumbnail_path()
 
+    def calc_thumbnail_path(self):
+        self.confirm_db_thread()
+        if self.icon_cache and self.icon_cache.is_valid():
+            self.thumbnail_path = fileutil.expand_filename(
+                self.icon_cache.get_filename())
+        else:
+            self.thumbnail_path = None
+
     def has_downloaded_items(self):
         return self.num_downloaded() > 0
 
@@ -1229,7 +1221,7 @@ class RSSFeedImplBase(ThrottledUpdateFeedImpl):
                     feed_id=self.ufeed.id, channel_title=channel_title)
         else:
             item = models.Item(fp_values, feed_id=self.ufeed.id,
-                    eligibleForAutoDownload=not self.initialUpdate,
+                    eligible_for_autodownload=not self.initialUpdate,
                     channel_title=channel_title)
             if not item.matches_search(self.ufeed.searchTerm):
                 item.remove()
@@ -1258,11 +1250,22 @@ class RSSFeedImplBase(ThrottledUpdateFeedImpl):
 
         if channel_title != None and self._allow_feed_to_override_title():
             self.title = channel_title
-        if (parsed.feed.has_key('image') and
-                parsed.feed.image.has_key('url') and
-                self._allow_feed_to_override_thumbnail()):
-            self.thumbURL = parsed.feed.image.url
-            self.ufeed.icon_cache.request_update(is_vital=True)
+        if parsed.feed.has_key('image'):
+            image = parsed.feed['image']
+            image_url = None
+            if isinstance(image, dict):
+                if 'url' in image:
+                    image_url = image['url']
+                elif 'href' in image:
+                    image_url = image['href']
+            elif isinstance(image, basestring):
+                image_url = image
+            else:
+                logging.warn('strange image value from %r: %r',
+                             self.url, image)
+            if image_url and self._allow_feed_to_override_thumbnail():
+                self.thumbURL = image_url
+                self.ufeed.icon_cache.request_update(is_vital=True)
 
         items_byid = {}
         items_byURLTitle = {}
@@ -1337,7 +1340,7 @@ class RSSFeedImplBase(ThrottledUpdateFeedImpl):
         if self.initialUpdate:
             self.initialUpdate = False
             for latest in models.Item.latest_in_feed_view(self.ufeed_id):
-                latest.eligibleForAutoDownload = True
+                latest.eligible_for_autodownload = True
                 latest.signal_change()
             if self.ufeed.is_autodownloadable():
                 self.ufeed.mark_as_viewed()
@@ -1373,7 +1376,7 @@ class RSSFeedImplBase(ThrottledUpdateFeedImpl):
         candidates = []
         for item in self.old_items:
             if item.downloader is None:
-                candidates.append((item.creationTime, item))
+                candidates.append((item.creation_time, item))
         candidates.sort()
         for time_, item in candidates[:extra]:
             item.remove()
@@ -1552,7 +1555,7 @@ class RSSMultiFeedBase(RSSFeedImplBase):
         self.modified = {}
         self.download_dc = {}
         self.updating = 0
-        self.urls = self.calc_urls()
+        self._urls = None
 
     def setup_restored(self):
         """Called by pickle during deserialization
@@ -1560,7 +1563,13 @@ class RSSMultiFeedBase(RSSFeedImplBase):
         RSSFeedImplBase.setup_restored(self)
         self.download_dc = {}
         self.updating = 0
-        self.urls = self.calc_urls()
+        self._urls = None
+
+    @property
+    def urls(self):
+        if self._urls is None:
+            self._urls = self.calc_urls()
+        return self._urls
 
     def calc_urls(self):
         """Calculate the list of URLs to parse.
@@ -1630,6 +1639,7 @@ class RSSMultiFeedBase(RSSFeedImplBase):
                 etag=etag, modified=modified,
                 default_mime_type=u'application/rss+xml',)
             self.updating += 1
+        self.ufeed.signal_change(needs_save=False)
 
     def _update_errback(self, error, url):
         if not self.ufeed.id_exists():
@@ -1745,7 +1755,7 @@ class ScraperFeedImpl(ThrottledUpdateFeedImpl):
             self.linkHistory[url] = self.tempHistory[url]
         self.tempHistory = {}
 
-    def get_html(self, urlList, depth=0, linkNumber=0, top=False):
+    def get_html(self, urlList, depth=0, link_number=0, top=False):
         """Grabs HTML at the given URL, then processes it
         """
         url = urlList.pop(0)
@@ -1760,7 +1770,7 @@ class ScraperFeedImpl(ThrottledUpdateFeedImpl):
                 return
             self.downloads.discard(download)
             try:
-                self.process_downloaded_html(info, urlList, depth, linkNumber,
+                self.process_downloaded_html(info, urlList, depth, link_number,
                                            top)
             finally:
                 self.check_done()
@@ -1775,7 +1785,7 @@ class ScraperFeedImpl(ThrottledUpdateFeedImpl):
                 modified=modified, default_mime_type='text/html')
         self.downloads.add(download)
 
-    def process_downloaded_html(self, info, urlList, depth, linkNumber,
+    def process_downloaded_html(self, info, urlList, depth, link_number,
                               top=False):
         self.ufeed.confirm_db_thread()
         #print "Done grabbing %s" % info['updated-url']
@@ -1793,11 +1803,11 @@ class ScraperFeedImpl(ThrottledUpdateFeedImpl):
             else:
                 subLinks = self.scrape_links(info['body'], info['redirected-url'], setTitle=top)
             if top:
-                self.process_links(subLinks, 0, linkNumber)
+                self.process_links(subLinks, 0, link_number)
             else:
-                self.process_links(subLinks, depth+1, linkNumber)
+                self.process_links(subLinks, depth+1, link_number)
         if len(urlList) > 0:
-            self.get_html(urlList, depth, linkNumber)
+            self.get_html(urlList, depth, link_number)
 
     def check_done(self):
         if len(self.downloads) == 0:
@@ -1806,7 +1816,7 @@ class ScraperFeedImpl(ThrottledUpdateFeedImpl):
             self.ufeed.signal_change()
             self.schedule_update_events(-1)
 
-    def add_video_item(self, link, dict_, linkNumber):
+    def add_video_item(self, link, dict_, link_number):
         link = unicodify(link.strip())
         if dict_.has_key('title'):
             title = dict_['title']
@@ -1828,14 +1838,14 @@ class ScraperFeedImpl(ThrottledUpdateFeedImpl):
                 'enclosures': [FeedParserDict({'url': link})]
                 })
         i = models.Item(FeedParserValues(fp_dict),
-                             linkNumber=linkNumber, feed_id=self.ufeed.id,
-                             eligibleForAutoDownload=False)
+                             link_number=link_number, feed_id=self.ufeed.id,
+                             eligible_for_autodownload=False)
         if ((self.ufeed.searchTerm is not None
              and not i.matches_search(self.ufeed.searchTerm))):
             i.remove()
             return
 
-    def process_links(self, links, depth=0, linkNumber=0):
+    def process_links(self, links, depth=0, link_number=0):
         # FIXME: compound names for titles at each depth??
         maxDepth = 2
         urls = links[0]
@@ -1846,8 +1856,8 @@ class ScraperFeedImpl(ThrottledUpdateFeedImpl):
         if depth < maxDepth:
             for link in urls:
                 if depth == 0:
-                    linkNumber += 1
-                #print "Processing %s (%d)" % (link,linkNumber)
+                    link_number += 1
+                #print "Processing %s (%d)" % (link,link_number)
 
                 # FIXME: Using file extensions totally breaks the
                 # standard and won't work with Broadcast Machine or
@@ -1878,9 +1888,9 @@ class ScraperFeedImpl(ThrottledUpdateFeedImpl):
                       mimetype == "application/ogg" or
                       mimetype == "application/x-annodex" or
                       mimetype == "application/x-bittorrent"):
-                    self.add_video_item(link, links[link], linkNumber)
+                    self.add_video_item(link, links[link], link_number)
             if len(newURLs) > 0:
-                self.get_html(newURLs, depth, linkNumber)
+                self.get_html(newURLs, depth, link_number)
 
     def on_remove(self):
         for download in self.downloads:
@@ -2093,7 +2103,11 @@ class DirectoryScannerImplBase(FeedImpl):
                     self.handle_watcher_updates,
                     "handle directory watcher updates")
 
+    @eventloop.idle_iterator
     def handle_watcher_updates(self):
+        # If we are not longer valid just return
+        if not self.ufeed.id_exists():
+            return
         # find deleted paths that we have items for
         to_remove = []
         for item in self.items:
@@ -2103,16 +2117,25 @@ class DirectoryScannerImplBase(FeedImpl):
         known_files = self.calc_known_files()
         for x in self.items:
             known_files.add_path(x.get_filename())
-        to_add = self._filter_paths(self._watcher_paths_added, known_files)
+        to_add = []
+        start = time.time()
+        for f in self._filter_paths(self._watcher_paths_added, known_files):
+            to_add.append(f)
+            if time.time() - start > 0.4:
+                yield
+                if not self.id_exists():
+                    return
+                start = time.time()
         # commit changes
-        app.bulk_sql_manager.start()
-        try:
-            for item in to_remove:
-                item.remove()
-            for path in to_add:
-                self._make_child(path)
-        finally:
-            app.bulk_sql_manager.finish()
+        with app.local_metadata_manager.bulk_add():
+            app.bulk_sql_manager.start()
+            try:
+                for item in to_remove:
+                    item.remove()
+                for path in to_add:
+                    self._make_child(path)
+            finally:
+                app.bulk_sql_manager.finish()
         # cleanup and prepare for the next change
         self._watcher_paths_deleted = set()
         self._watcher_paths_added = set()
@@ -2131,24 +2154,50 @@ class DirectoryScannerImplBase(FeedImpl):
         self._add_known_files(known_files)
         return known_files
 
+    # Subclass may override this to implement asynchronous preparation
+    # if it is long-running
+    def schedule_update(self):
+        self.do_update()
+
     def update(self):
+        self.ufeed.confirm_db_thread()
+        if not self.ufeed.id_exists():
+            return
+
         if not self.updating:
             self.updating = True
-            self.do_update()
+            self.schedule_update()
 
     @eventloop.idle_iterator
     def do_update(self):
-        self.ufeed.confirm_db_thread()
+
+        def should_halt_early():
+            """Check if we should halt before completing the entire update.
+
+            This should be called after each yield statement.
+            """
+            return not self.id_exists()
+
+        if should_halt_early():
+            return
 
         self._before_update()
 
         known_files = self.calc_known_files()
         my_files = set()
+        my_items = list(self.items)
+        # pause after doing prep work
+        yield
+        if should_halt_early():
+            return
 
         # Remove items with deleted files or that that are in feeds
         to_remove = []
         duplicate_paths = []
-        for item in self.items:
+        start = time.time()
+        for item in my_items:
+            if not item.id_exists():
+                continue
             filename = item.get_filename()
             if (filename is None or
                 not fileutil.isfile(filename) or
@@ -2159,6 +2208,11 @@ class DirectoryScannerImplBase(FeedImpl):
             else:
                 duplicate_paths.append(filename)
                 to_remove.append(item)
+            if time.time() - start > 0.4:
+                yield
+                if should_halt_early():
+                    return
+                start = time.time()
         if duplicate_paths:
             app.controller.failed_soft("scanning directory",
                 "duplicate paths in directory watcher: %s (impl: %s" %
@@ -2166,7 +2220,8 @@ class DirectoryScannerImplBase(FeedImpl):
         app.bulk_sql_manager.start()
         try:
             for item in to_remove:
-                item.remove()
+                if item.id_exists():
+                    item.remove()
         finally:
             app.bulk_sql_manager.finish()
 
@@ -2180,10 +2235,25 @@ class DirectoryScannerImplBase(FeedImpl):
         # files on the filesystem
         scan_dir = self._scan_dir()
         if fileutil.isdir(scan_dir) and not is_file_bundle(scan_dir):
-            all_files = fileutil.miro_allfiles(scan_dir)
-            to_add = self._filter_paths(all_files, known_files)
-            for path in to_add:
-                app.metadata_progress_updater.will_process_path(path)
+            all_files = []
+            start = time.time()
+            for f in fileutil.miro_allfiles(scan_dir):
+                all_files.append(f)
+                if time.time() - start > 0.4:
+                    yield
+                    if should_halt_early():
+                        return
+                    start = time.time()
+            start = time.time()
+            to_add = []
+            for path in self._filter_paths(all_files, known_files):
+                to_add.append(path)
+                if time.time() - start > 0.4:
+                    yield
+                    if should_halt_early():
+                        return
+                    start = time.time()
+
             # Keep track of the paths we will add in case we get directory
             # watcher updates.  In that case, we want these paths to be in
             # known_files.  It's very important that the next line come before
@@ -2192,15 +2262,14 @@ class DirectoryScannerImplBase(FeedImpl):
             path_iter = iter(to_add)
             finished = False
             yield # yield after doing prep work
-            try:
-                while not finished:
-                    finished = self._add_batch_of_videos(path_iter, 0.2)
-                    yield # yield after each batch
-            except ObjectNotFoundError:
-                # whoops, we disappeared! clean up and quit
-                for path in path_iter:
-                    app.metadata_progress_updater.path_processed(path)
+            if should_halt_early():
                 return
+            with app.local_metadata_manager.bulk_add():
+                while not finished:
+                    finished = self._add_batch_of_videos(path_iter, 0.1)
+                    yield # yield after each batch
+                    if should_halt_early():
+                        return
         self._after_update()
         self.updating = False
         self.pending_paths_to_add = []
@@ -2231,13 +2300,8 @@ class DirectoryScannerImplBase(FeedImpl):
         This method removes items from paths if they are in known_files or
         they are not media files
         """
-        rv = []
-        for path in paths:
-            ufile = filename_to_unicode(path)
-            if (not known_files.contains_path(path) and
-                    filetypes.is_media_filename(ufile)):
-                rv.append(path)
-        return rv
+        return (p for p in paths if not known_files.contains_path(p) and
+              filetypes.is_media_filename(filename_to_unicode(p)))
 
 class DirectoryWatchFeedImpl(DirectoryScannerImplBase):
     def setup_new(self, ufeed, directory):
@@ -2283,19 +2347,36 @@ class DirectoryFeedImpl(DirectoryScannerImplBase):
     """
     def setup_new(self, ufeed):
         DirectoryScannerImplBase.setup_new(self, url=u"dtv:directoryfeed",
-                ufeed=ufeed, title=None)
+                ufeed=ufeed, title=u"")
         self.set_update_frequency(5)
         self.schedule_update_events(0)
+        self._prepare_count = 0
         self.start_watching_directory()
 
     def setup_restored(self):
+        self._prepare_count = 0
         DirectoryScannerImplBase.setup_restored(self)
         self.start_watching_directory()
 
-    def _before_update(self):
+    def schedule_update(self):
+        self._async_prepare()
+
+    def _async_prepare(self):
         # Make sure container items have created FileItems for their contents
+        #
+        # Since find_new_children() is an idle iterator we can safely
+        # count on the call returning immediately and at the end of the loop
         for container in models.Item.containers_view():
-            container.find_new_children()
+            container.find_new_children(callback=self._async_prepare_complete)
+            self._prepare_count += 1
+
+    def _async_prepare_complete(self, unused):
+        if not self.id_exists():
+            return
+        self._prepare_count -= 1
+        if self._prepare_count == 0:
+            # All done?  Go to the do_update()
+            self.do_update()
 
     def _calc_known_files(self):
         pass
@@ -2314,9 +2395,6 @@ class DirectoryFeedImpl(DirectoryScannerImplBase):
     def _scan_dir(self):
         return app.config.get(prefs.MOVIES_DIRECTORY)
 
-    @returns_unicode
-    def get_title(self):
-        return _('Local Files')
 
 class SearchFeedImpl(RSSMultiFeedBase):
     """Search and Search Results feeds
@@ -2331,7 +2409,6 @@ class SearchFeedImpl(RSSMultiFeedBase):
         self.set_update_frequency(-1)
         self.ufeed.autoDownloadable = False
         # keeps the items from being seen as 'newly available'
-        self.ufeed.last_viewed = datetime.max
         self.ufeed.signal_change()
 
     def setup_restored(self):
@@ -2355,7 +2432,7 @@ class SearchFeedImpl(RSSMultiFeedBase):
                 item.remove()
         finally:
             app.bulk_sql_manager.finish()
-        self.urls = []
+        self._urls = []
         self.searching = False
         if set_engine is not None:
             self.engine = set_engine
@@ -2385,7 +2462,7 @@ class SearchFeedImpl(RSSMultiFeedBase):
         self.searching = True
         self.engine = engine
         self.query = query
-        self.urls = self.calc_urls()
+        self._urls = None
         self.update()
         self.ufeed.signal_change()
 
@@ -2429,34 +2506,21 @@ class SearchFeedImpl(RSSMultiFeedBase):
         # the dtv:search feed should never automatically update.
         return
 
-    @returns_unicode
-    def get_title(self):
-        return _('Search')
-
 class SearchDownloadsFeedImpl(FeedImpl):
     def setup_new(self, ufeed):
         FeedImpl.setup_new(self, url=u'dtv:searchDownloads', ufeed=ufeed,
-                title=None)
+                title=_('Search'))
         self.set_update_frequency(-1)
 
-    @returns_unicode
-    def get_title(self):
-        return _('Search')
 
 class ManualFeedImpl(FeedImpl):
     """Downloaded Videos/Torrents that have been added using by the
     user opening them with democracy.
     """
     def setup_new(self, ufeed):
-        FeedImpl.setup_new(self, url=u'dtv:manualFeed', ufeed=ufeed,
-                title=None)
+        FeedImpl.setup_new(self, url=u'dtv:manualFeed', ufeed=ufeed, title=u"")
         self.ufeed.expire = u'never'
         self.set_update_frequency(-1)
-        self.ufeed.last_viewed = datetime.max
-
-    @returns_unicode
-    def get_title(self):
-        return _('Local Files')
 
 LINK_PATTERN = re.compile("<(a|embed)\s[^>]*(href|src)\s*=\s*\"([^\"]*)\"[^>]*>(.*?)</a(.*)", re.S)
 IMG_PATTERN = re.compile(".*<img\s.*?src\s*=\s*\"(.*?)\".*?>", re.S)
