@@ -34,14 +34,15 @@ simultaniously updating at any given time.  Right now the limit is set
 to 3.
 """
 
+import collections
+
 from miro import eventloop
-from miro import datastructures
 
 MAX_UPDATES = 3
 
 class FeedUpdateQueue(object):
     def __init__(self):
-        self.update_queue = datastructures.Fifo()
+        self.update_queue = collections.deque()
         self.timeouts = {}
         self.callback_handles = {}
         self.currently_updating = set()
@@ -61,7 +62,7 @@ class FeedUpdateQueue(object):
 
     def do_update(self, feed, update_callback):
         del self.timeouts[feed.id]
-        self.update_queue.enqueue((feed, update_callback))
+        self.update_queue.append((feed, update_callback))
         self.run_update_queue()
 
     def update_finished(self, feed):
@@ -76,7 +77,7 @@ class FeedUpdateQueue(object):
     def run_update_queue(self):
         while (len(self.update_queue) > 0 and 
                len(self.currently_updating) < MAX_UPDATES):
-            feed, update_callback = self.update_queue.dequeue()
+            feed, update_callback = self.update_queue.popleft()
             if feed in self.currently_updating:
                 continue
             handle = feed.connect('update-finished', self.update_finished)
