@@ -122,6 +122,23 @@ def abspath(path):
     path = collapse_filename(path)
     return path
 
+def copy_with_progress(input_path, output_path, block_size=32*1024):
+    flags = os.O_WRONLY | os.O_CREAT
+    if hasattr(os, 'O_SYNC'):
+        flags |= os.O_SYNC
+    output_fd = os.open(output_path, flags)
+    with file(input_path, 'rb') as input:
+        with os.fdopen(output_fd, 'wb') as output:
+            data = input.read(block_size)
+            while data:
+                output.write(data)
+                result = yield len(data)
+                if result:
+                    # return True to cancel. NB: you should probably remove the
+                    # output file
+                    break
+                data = input.read(block_size)
+
 try:
     samefile = os.path.samefile
 except AttributeError:
@@ -184,7 +201,7 @@ def migrate_file(source, dest, callback, retry_after=10, retry_for=60):
     else:
         callback()
 
-class DeletesInProgressTracker(set):
+class DeletesInProgressTracker(object):
     def __init__(self):
         self.set = set()
     def normalize(self, path):
@@ -198,7 +215,7 @@ class DeletesInProgressTracker(set):
 
 deletes_in_progress = DeletesInProgressTracker()
 
-def delete(path, retry_after=10, retry_for=60):
+def delete(path, retry_after=10, retry_for=60, firsttime=True):
     """Try to delete a file or directory.  If this fails because the
     file is open, we retry deleting the file every so often This
     probably only makes a difference on Windows.
@@ -222,7 +239,14 @@ def delete(path, retry_after=10, retry_for=60):
             logging.info('Retrying delete for %s (%d)', path, retry_after)
             eventloop.add_timeout(retry_after, delete,
                     "Delete File Retry", args=(path, retry_after,
-                        retry_for - retry_after))
+                        retry_for - retry_after, False))
+            if firsttime:
+                from miro.workerprocess import _subprocess_manager
+                if _subprocess_manager.is_running:
+                    logging.debug('restarting subprocess_manager to hopefully '
+                                  'free file references')
+                    _subprocess_manager.restart(clean=True)
+
     else:
         deletes_in_progress.discard(path)
 
