@@ -33,10 +33,6 @@ level running of the Miro application.
 
 It also holds:
 
-* :class:`InfoUpdater` -- tracks channel/item updates from the backend
-  and sends the information to the frontend
-* :class:`InfoUpdaterCallbackList` -- tracks the list of callbacks for
-  info updater
 * :class:`WidgetsMessageHandler` -- frontend message handler
 * :class:`DisplayStateStore` -- stores state of each display
 """
@@ -51,6 +47,7 @@ import urllib
 from miro import app
 from miro import config
 from miro import crashreport
+from miro import data
 from miro import prefs
 from miro import feed
 from miro import startup
@@ -62,6 +59,7 @@ from miro import filetypes
 from miro.gtcache import gettext as _
 from miro.gtcache import ngettext
 from miro.frontends.widgets import dialogs
+from miro.frontends.widgets import infoupdater
 from miro.frontends.widgets import newsearchfeed
 from miro.frontends.widgets import newfeed
 from miro.frontends.widgets import newfolder
@@ -72,6 +70,7 @@ from miro.frontends.widgets import searchfilesdialog
 from miro.frontends.widgets import removefeeds
 from miro.frontends.widgets import diagnostics
 from miro.frontends.widgets import crashdialog
+from miro.frontends.widgets import itemlist
 from miro.frontends.widgets import itemlistcontroller
 from miro.frontends.widgets import prefpanel
 from miro.frontends.widgets import displays
@@ -105,7 +104,9 @@ class Application:
         self.window = None
         self.ui_initialized = False
         messages.FrontendMessage.install_handler(self.message_handler)
-        app.info_updater = InfoUpdater()
+        app.item_list_pool = itemlist.ItemListPool()
+        app.item_tracker_updater = itemlist.ItemTrackerUpdater()
+        app.info_updater = infoupdater.InfoUpdater()
         app.saved_items = set()
         app.watched_folder_manager = watchedfolders.WatchedFolderManager()
         app.store_manager = stores.StoreManager()
@@ -139,6 +140,7 @@ class Application:
         """Connects to signals, installs handlers, and calls :meth:`startup`
         from the :mod:`miro.startup` module.
         """
+        app.frontend = WidgetsFrontend()
         self.connect_to_signals()
         startup.install_movies_directory_gone_handler(self.handle_movies_directory_gone)
         startup.install_first_time_handler(self.handle_first_time)
@@ -149,6 +151,7 @@ class Application:
         requesting data from the backend.  Also sets up managers,
         initializes the ui, and displays the :class:`MiroWindow`.
         """
+        data.init()
         # Send a couple messages to the backend, when we get responses,
         # WidgetsMessageHandler() will call build_window()
         messages.TrackGuides().send_to_backend()
@@ -162,8 +165,11 @@ class Application:
         self.setup_globals()
         self.ui_initialized = True
 
-        self.window = MiroWindow(app.config.get(prefs.LONG_APP_NAME),
-                                 self.get_main_window_dimensions())
+        title = app.config.get(prefs.LONG_APP_NAME)
+        if app.debugmode:
+            title += ' (%s - %s)' % (app.config.get(prefs.APP_VERSION),
+                                     app.config.get(prefs.APP_REVISION_NUM))
+        self.window = MiroWindow(title, self.get_main_window_dimensions())
         self.window.connect_weak('key-press', self.on_key_press)
         self.window.connect_weak('on-shown', self.on_shown)
         self._window_show_callback = self.window.connect_weak('show',
@@ -172,8 +178,9 @@ class Application:
     def setup_globals(self):
         app.item_list_controller_manager = \
                 itemlistcontroller.ItemListControllerManager()
-        app.menu_manager = menus.MenuStateManager()
         app.playback_manager = playback.PlaybackManager()
+        app.menu_manager = menus.MenuManager()
+        app.menu_manager.setup_menubar(self.menubar)
         app.search_manager = search.SearchManager()
         app.inline_search_memory = search.InlineSearchMemory()
         app.tabs = tablistmanager.TabListManager()
@@ -199,6 +206,8 @@ class Application:
         This is useful for e.g. restoring a saved selection, which is overridden
         by the default first-row selection if done too early.
         """
+        app.startup_timer.log_time("window shown")
+        app.startup_timer.log_total_time()
         logging.debug('on_shown')
         app.tabs.on_shown()
 
@@ -206,67 +215,56 @@ class Application:
         if app.playback_manager.is_playing:
             return playback.handle_key_press(key, mods)
 
-    def handle_movies_directory_gone(self, continue_callback):
-        call_on_ui_thread(self._handle_movies_directory_gone, continue_callback)
+    def handle_movies_directory_gone(self, msg, movies_dir,
+                                     allow_continue=False):
+        call_on_ui_thread(self._handle_movies_directory_gone, msg, movies_dir,
+                          allow_continue)
 
-    def _handle_movies_directory_gone(self, continue_callback):
+    def _handle_movies_directory_gone(self, msg, movies_dir, allow_continue):
         # Make sure we close the upgrade dialog before showing a new one
         self.message_handler.close_upgrade_dialog()
 
-        title = _("Movies directory gone")
-        movies_directory = app.config.get(prefs.MOVIES_DIRECTORY)
-        if os.path.isdir(movies_directory):
-            # allow users to continue if the directory exists
-            description = _(
-                "%(shortappname)s can't use the primary video directory "
-                "located at:\n"
-                "\n"
-                "%(moviedirectory)s\n"
-                "\n"
-                "This may be because it is located on an external drive "
-                "that is not connected, is a directory that "
-                "%(shortappname)s does not have write permission to, or "
-                "there is something that is not a directory at that "
-                "path.\n"
-                "\n"
-                "If you continue, the primary video directory will be "
-                "reset to a location on this drive.  If you had videos "
-                "downloaded this will cause %(shortappname)s to lose "
-                "details about those videos.\n "
-                "\n"
-                "If you quit, then you can connect the drive or otherwise "
-                "fix the problem and relaunch %(shortappname)s.",
-                {"shortappname": app.config.get(prefs.SHORT_APP_NAME),
-                 "moviedirectory": movies_directory})
-            choice = dialogs.show_choice_dialog(title, description,
-                    [dialogs.BUTTON_CONTINUE, dialogs.BUTTON_QUIT])
+        title = _("Movies folder gone")
+        description = _(
+            "%(shortappname)s can't use your primary video folder, which "
+            "is currently set to:\n"
+            "\n"
+            "%(moviedirectory)s\n"
+            "\n"
+            "%(reason)s\n"
+            "\n",
+            {"shortappname": app.config.get(prefs.SHORT_APP_NAME),
+             "moviedirectory": movies_dir,
+             "reason":msg,
+            })
+        if allow_continue:
+            description += _(
+                "You may continue with the current folder, choose a new "
+                "folder or quit and try to fix the issue manually.")
+            buttons = [
+                dialogs.BUTTON_CONTINUE,
+                dialogs.BUTTON_CHOOSE_NEW_FOLDER,
+                dialogs.BUTTON_QUIT
+            ]
         else:
-            # if the directory doesn't exist, don't let the user continue
-            description = _(
-                "%(shortappname)s can't use the primary video directory "
-                "located at:\n"
-                "\n"
-                "%(moviedirectory)s\n"
-                "\n"
-                "This may be because it is located on an external drive "
-                "that is not connected, is a directory that "
-                "%(shortappname)s does not have write permission to, or "
-                "there is something that is not a directory at that "
-                "path.\n"
-                "\n"
-                "%(shortappname)s will now exit. You can connect the drive "
-                "or otherwise fix the problem and relaunch "
-                "%(shortappname)s.",
-                {"shortappname": app.config.get(prefs.SHORT_APP_NAME),
-                    "moviedirectory": movies_directory})
-            dialogs.show_message(title, description,
-                                       alert_type=dialogs.CRITICAL_MESSAGE)
-            choice = dialogs.BUTTON_QUIT
-
+            description += _(
+                "You choose a new folder or quit and try to fix the issue "
+                "manually.")
+            buttons = [
+                dialogs.BUTTON_CHOOSE_NEW_FOLDER,
+                dialogs.BUTTON_QUIT
+            ]
+        choice = dialogs.show_choice_dialog(title, description, buttons)
         if choice == dialogs.BUTTON_CONTINUE:
-            continue_callback()
-        else:
-            self.do_quit()
+            startup.fix_movies_gone(None)
+            return
+        elif choice == dialogs.BUTTON_CHOOSE_NEW_FOLDER:
+            title = _("Choose new primary video folder")
+            new_movies_dir = dialogs.ask_for_directory(title, movies_dir)
+            if new_movies_dir is not None:
+                startup.fix_movies_gone(new_movies_dir)
+                return
+        self.do_quit()
 
     def handle_first_time(self, continue_callback):
         call_on_ui_thread(lambda: self._handle_first_time(continue_callback))
@@ -279,6 +277,7 @@ class Application:
         firsttimedialog.FirstTimeDialog(continue_callback).run()
 
     def build_window(self):
+        app.startup_timer.log_time("in build_window")
         app.display_manager = displays.DisplayManager()
         app.tabs['site'].extend(self.message_handler.initial_guides)
         app.tabs['store'].extend(self.message_handler.initial_stores)
@@ -306,8 +305,6 @@ class Application:
         messages.TrackNewVideoCount().send_to_backend()
         messages.TrackNewAudioCount().send_to_backend()
         messages.TrackUnwatchedCount().send_to_backend()
-        messages.TrackDevices().send_to_backend()
-        messages.TrackSharing().send_to_backend()
 
     def get_main_window_dimensions(self):
         """Override this to provide platform-specific Main Window dimensions.
@@ -368,6 +365,20 @@ class Application:
     def resume_play_selection(self):
         app.item_list_controller_manager.resume_play_selection()
 
+    def enable_net_lookup_for_selection(self):
+        selection = app.item_list_controller_manager.get_selection()
+        id_list = [info.id for info in selection
+                   if info.downloaded and not info.net_lookup_enabled]
+        m = messages.SetNetLookupEnabled(id_list, True)
+        m.send_to_backend()
+
+    def disable_net_lookup_for_selection(self):
+        selection = app.item_list_controller_manager.get_selection()
+        id_list = [info.id for info in selection
+                   if info.downloaded and info.net_lookup_enabled]
+        m = messages.SetNetLookupEnabled(id_list, False)
+        m.send_to_backend()
+
     def on_stop_clicked(self, button=None):
         app.playback_manager.stop()
 
@@ -417,13 +428,12 @@ class Application:
 
     def toggle_column(self, name):
         current_display = app.display_manager.get_current_display()
-        app.widget_state.toggle_sort(current_display.type, current_display.id,
-            unicode(name))
-        current_display.update_columns_enabled()
+        if current_display:
+            current_display.toggle_column_enabled(unicode(name))
 
     def share_item(self, item):
-        share_items = {"file_url": item.file_url,
-                       "item_name": item.name}
+        share_items = {"file_url": item.url,
+                       "item_name": item.title}
         if item.feed_url:
             share_items["feed_url"] = item.feed_url
         query_string = "&".join([
@@ -467,7 +477,20 @@ class Application:
 
     def open_video(self):
         title = _('Open Files...')
-        filenames = dialogs.ask_for_open_pathname(title, select_multiple=True)
+        audio_extensions = [mem[1:] for mem in filetypes.AUDIO_EXTENSIONS]
+        video_extensions = [mem[1:] for mem in filetypes.VIDEO_EXTENSIONS]
+        torrent_extensions = [mem[1:] for mem in filetypes.TORRENT_EXTENSIONS]
+        all_extensions = (audio_extensions + video_extensions
+                          + torrent_extensions)
+        filenames = dialogs.ask_for_open_pathname(
+            title,
+            filters=[
+                (_('All Media Files'), all_extensions),
+                (_('Video Files'), video_extensions),
+                (_('Audio Files'), audio_extensions),
+                (_('Torrent Files'), torrent_extensions)
+                ],
+            select_multiple=True)
 
         if not filenames:
             return
@@ -524,14 +547,6 @@ class Application:
         if url is not None:
             messages.DownloadURL(url).send_to_backend()
 
-    def add_files(self):
-        # this was:
-        #addfilesdialog.AddFilesDialog().run()
-        # that module no longer exists so presumably this is unreachable
-        app.widgetapp.handle_soft_failure('Application.add_files',
-            "Application.add_files is no longer used",
-            with_exception=False)
-
     def check_version(self):
         # this gets called by the backend, so it has to send a message to
         # the frontend to open a dialog
@@ -571,8 +586,7 @@ class Application:
                     app.playback_manager.on_movie_finished()
 
         external_count = len([s for s in selection if s.is_external])
-        failed_count = len([s for s in selection if s.download_info and
-                            s.download_info.state == u'failed'])
+        failed_count = len([s for s in selection if s.is_failed_download])
         folder_count = len([s for s in selection if s.is_container_item])
         total_count = len(selection)
 
@@ -684,7 +698,7 @@ class Application:
             return
 
         title = _('Save Item As...')
-        filename = selection[0].video_path
+        filename = selection[0].filename
         filename = os.path.basename(filename)
         filename = dialogs.ask_for_save_pathname(title, filename)
 
@@ -693,6 +707,13 @@ class Application:
 
         messages.SaveItemAs(selection[0].id, filename).send_to_backend()
 
+    def set_media_kind(self, kind):
+        logging.debug('set media kind = %s', kind)
+        selection = app.item_list_controller_manager.get_selection()
+        if not selection:
+            return
+        messages.SetMediaKind(selection, kind).send_to_backend()
+
     def convert_items(self, converter_id):
         selection = app.item_list_controller_manager.get_selection()
         for item_info in selection:
@@ -708,8 +729,8 @@ class Application:
             return
 
         selection = selection[0]
-        if selection.file_url:
-            app.widgetapp.copy_text_to_clipboard(selection.file_url)
+        if selection.url:
+            app.widgetapp.copy_text_to_clipboard(selection.url)
 
     def add_new_feed(self):
         url = newfeed.run_dialog()
@@ -734,10 +755,8 @@ class Application:
 
     def import_choose_files(self):
         # opens dialog allowing you to choose files and folders
-        audio_extensions = [mem.replace(".", "")
-                            for mem in filetypes.AUDIO_EXTENSIONS]
-        video_extensions = [mem.replace(".", "")
-                            for mem in filetypes.VIDEO_EXTENSIONS]
+        audio_extensions = [mem[1:] for mem in filetypes.AUDIO_EXTENSIONS]
+        video_extensions = [mem[1:] for mem in filetypes.VIDEO_EXTENSIONS]
         files_ = dialogs.ask_for_open_pathname(
             _("Choose files to import"),
             filters=[
@@ -1004,21 +1023,10 @@ class Application:
         importing media.
         """
         app.config.set(prefs.MUSIC_TAB_CLICKED, True)
-        name, path = get_plat_media_player_name_path()
-        if path is None:
-            return
-        trans_data = {
-            'media_player': name,
-            'short_app_name': app.config.get(prefs.SHORT_APP_NAME)}
-        title = _('Import Music from %(media_player)s?', trans_data)
-        description = _("We see that you have %(media_player)s installed.  "
-                        "Would you like %(short_app_name)s to import the "
-                        "music from it?", trans_data)
-        ret = dialogs.show_choice_dialog(title, description,
-                                         [dialogs.BUTTON_YES,
-                                          dialogs.BUTTON_NO])
-        if ret == dialogs.BUTTON_YES:
-            app.watched_folder_manager.add(path)
+        dialog = firsttimedialog.MusicSetupDialog()
+        dialog.run()
+        if dialog.import_path():
+            app.watched_folder_manager.add(dialog.import_path())
 
     def quit_ui(self):
         """Quit  out of the UI event loop."""
@@ -1059,6 +1067,27 @@ class Application:
             if path is not None:
                 self.message_handler.profile_next_message(message_obj, path)
 
+    def clog_backend(self):
+        """Dev method: hog the backend to simluate the backend being
+        unresponsive.
+
+        NB: strings not translated on purpose.
+        """
+        title = 'Clog backend'
+        description = ('Make the backend busy by sleeping for a specified '
+                      'number of seconds to simulate a clogged backend.\n\n'
+                      'WARNING: use judiciously!\n\n'
+                      'Default is 0 seconds.')
+        initial_text = '0'
+        n = dialogs.ask_for_string(title, description, initial_text)
+        if n == None:
+            return
+        try:
+            n = int(n)
+        except ValueError:
+            n = 0
+        messages.ClogBackend(n).send_to_backend()
+
     def profile_redraw(self):
         """Devel method: profile time to redraw part of the interface."""
 
@@ -1093,6 +1122,9 @@ class Application:
         gc.collect()
         self._printout_memory_stats('MEMORY STATS AFTER GARBAGE COLLECTION')
 
+    def force_feedparser_processing(self):
+        messages.ForceFeedparserProcessing().send_to_backend()
+
     def _printout_memory_stats(self, title):
         # base_classes is a list of base classes that we care about.  If you
         # want to check memory usage for a different class, add it to the
@@ -1257,107 +1289,6 @@ class Application:
     def on_backend_shutdown(self, obj):
         logging.info('Shutting down...')
 
-class InfoUpdaterCallbackList(object):
-    """Tracks the list of callbacks for InfoUpdater.
-    """
-
-    def __init__(self):
-        self._callbacks = {}
-
-    def add(self, type_, id_, callback):
-        """Adds the callback to the list for ``type_`` ``id_``.
-
-        :param type_: the type of the thing (feed, site, ...)
-        :param id_: the id for the thing
-        :param callback: the callback function to add
-        """
-        key = (type_, id_)
-        self._callbacks.setdefault(key, set()).add(callback)
-
-    def remove(self, type_, id_, callback):
-        """Removes the callback from the list for ``type_`` ``id_``.
-
-        :param type_: the type of the thing (feed, site, ...)
-        :param id_: the id for the thing
-        :param callback: the callback function to remove
-        """
-        key = (type_, id_)
-        callback_set = self._callbacks[key]
-        callback_set.remove(callback)
-        if len(callback_set) == 0:
-            del self._callbacks[key]
-
-    def get(self, type_, id_):
-        """Get the list of callbacks for ``type_``, ``id_``.
-
-        :param type_: the type of the thing (feed, site, ...)
-        :param id_: the id for the thing
-        """
-        key = (type_, id_)
-        if key not in self._callbacks:
-            return []
-        else:
-            # return a new list of callbacks, so that if we iterate over the
-            # return value, we don't have to worry about callbacks being
-            # removed midway.
-            return list(self._callbacks[key])
-
-class InfoUpdater(signals.SignalEmitter):
-    """Track channel/item updates from the backend.
-
-    To track item updates, use the item_list_callbacks and
-    item_changed_callbacks attributes, both are instances of
-    InfoUpdaterCallbackList.  To track tab updates, connect to one of the
-    signals below.
-
-    Signals:
-
-    * feeds-added (self, info_list) -- New feeds were added
-    * feeds-changed (self, info_list) -- Feeds were changed
-    * feeds-removed (self, info_list) -- Feeds were removed
-    * sites-added (self, info_list) -- New sites were added
-    * sites-changed (self, info_list) -- Sites were changed
-    * sites-removed (self, info_list) -- Sites were removed
-    * playlists-added (self, info_list) -- New playlists were added
-    * playlists-changed (self, info_list) -- Playlists were changed
-    * playlists-removed (self, info_list) -- Playlists were removed
-    """
-    def __init__(self):
-        signals.SignalEmitter.__init__(self)
-        for prefix in ('feeds', 'sites', 'playlists'):
-            self.create_signal('%s-added' % prefix)
-            self.create_signal('%s-changed' % prefix)
-            self.create_signal('%s-removed' % prefix)
-
-        self.item_list_callbacks = InfoUpdaterCallbackList()
-        self.item_changed_callbacks = InfoUpdaterCallbackList()
-
-    def handle_items_changed(self, message):
-        callback_list = self.item_changed_callbacks
-        for callback in callback_list.get(message.type, message.id):
-            callback(message)
-
-    def handle_item_list(self, message):
-        callback_list = self.item_list_callbacks
-        for callback in callback_list.get(message.type, message.id):
-            callback(message)
-
-    def handle_tabs_changed(self, message):
-        if message.type == 'feed':
-            signal_start = 'feeds'
-        elif message.type == 'site':
-            signal_start = 'sites'
-        elif message.type == 'playlist':
-            signal_start = 'playlists'
-        else:
-            return
-        if message.added:
-            self.emit('%s-added' % signal_start, message.added)
-        if message.changed:
-            self.emit('%s-changed' % signal_start, message.changed)
-        if message.removed:
-            self.emit('%s-removed' % signal_start, message.removed)
-
 class WidgetsMessageHandler(messages.MessageHandler):
     """Handles frontend messages.
 
@@ -1386,6 +1317,7 @@ class WidgetsMessageHandler(messages.MessageHandler):
         self.progress_dialog = None
         self.dbupgrade_progress_dialog = None
         self._profile_info = None
+        self._startup_failure_mode = self._database_failure_mode = False
 
     def profile_next_message(self, message_obj, path):
         self._profile_info = (message_obj, path)
@@ -1408,7 +1340,12 @@ class WidgetsMessageHandler(messages.MessageHandler):
         self.handle_tabs_changed(message)
         # Now, reply to backend, and eject the share.
         if share.mount:
-            messages.SharingEject(share).send_to_backend()
+            messages.StopTrackingShare(share.share_id).send_to_backend()
+
+    def handle_downloader_sync_command_complete(self, message):
+        # We used to need this command, but with the new ItemList code it's
+        # obsolute.
+        logging.debug('DownloaderSyncCommandComplete')
 
     def handle_jettison_tabs(self, message):
         typ = message.type
@@ -1458,12 +1395,9 @@ class WidgetsMessageHandler(messages.MessageHandler):
         dialogs.show_message(title, description, dialogs.INFO_MESSAGE)
         app.tabs.select_guide()
 
-    def handle_device_eject_failed(self, message):
-        name = message.device.name
-        title = _('Eject failed')
-        description = _("Ejecting device '%(name)s' failed.\n\n"
-                        "The device is in use.", {'name': name})
-        dialogs.show_message(title, description, dialogs.WARNING_MESSAGE)
+    def handle_show_warning(self, message):
+        dialogs.show_message(message.title, message.description,
+                             dialogs.WARNING_MESSAGE)
 
     def handle_frontend_quit(self, message):
         if self.dbupgrade_progress_dialog:
@@ -1508,7 +1442,8 @@ class WidgetsMessageHandler(messages.MessageHandler):
             self.dbupgrade_progress_dialog = None
 
     def handle_startup_failure(self, message):
-        if hasattr(self, "_startup_failure_mode"):
+        if self._startup_failure_mode:
+            logging.info("already in startup failure mode--skipping")
             return
         self._startup_failure_mode = True
         # We may still have the DB upgrade dialog open.  If so, close it.
@@ -1519,7 +1454,7 @@ class WidgetsMessageHandler(messages.MessageHandler):
         app.widgetapp.do_quit()
 
     def handle_startup_database_failure(self, message):
-        if hasattr(self, "_database_failure_mode"):
+        if self._database_failure_mode:
             logging.info("already in db failure mode--skipping")
             return
         self._database_failure_mode = True
@@ -1543,6 +1478,9 @@ class WidgetsMessageHandler(messages.MessageHandler):
         else:
             app.widgetapp.do_quit()
 
+    def startup_failed(self):
+        return self._startup_failure_mode or self._database_failure_mode
+
     def handle_startup_success(self, message):
         app.widgetapp.startup_ui()
         signals.system.emit('startup-success')
@@ -1558,6 +1496,10 @@ class WidgetsMessageHandler(messages.MessageHandler):
             app.widgetapp.build_window()
 
     def call_handler(self, method, message):
+        if self.startup_failed():
+            logging.warn("skipping message: %s because startup failed",
+                         message)
+            return
         # uncomment this next line if you need frontend messages
         # logging.debug("handling frontend %s", message)
         if (self._profile_info is not None and
@@ -1572,7 +1514,7 @@ class WidgetsMessageHandler(messages.MessageHandler):
         self._profile_info = None
 
     def handle_current_search_info(self, message):
-        app.search_manager.set_search_info(message.engine, message.text)
+        app.search_manager.set_initial_search_info(message.engine, message.text)
         self._saw_pre_startup_message('search-info')
 
     def handle_tab_list(self, message):
@@ -1632,17 +1574,17 @@ class WidgetsMessageHandler(messages.MessageHandler):
             else:
                 tablist.add(info)
         tablist.model_changed()
+        app.connection_pools.on_tabs_changed(message)
         app.info_updater.handle_tabs_changed(message)
 
-    def handle_item_list(self, message):
-        app.info_updater.handle_item_list(message)
-        if app.menu_manager:
-            app.menu_manager.update_menus()
+    def handle_item_changes(self, message):
+        app.item_tracker_updater.on_item_changes(message)
 
-    def handle_items_changed(self, message):
-        app.info_updater.handle_items_changed(message)
-        if app.menu_manager:
-            app.menu_manager.update_menus()
+    def handle_device_item_changes(self, message):
+        app.item_tracker_updater.on_device_item_changes(message)
+
+    def handle_sharing_item_changes(self, message):
+        app.item_tracker_updater.on_sharing_item_changes(message)
 
     def handle_download_count_changed(self, message):
         app.widgetapp.download_count = message.count
@@ -1669,6 +1611,9 @@ class WidgetsMessageHandler(messages.MessageHandler):
         app.widgetapp.unwatched_count = message.count
         app.widgetapp.handle_unwatched_count_changed()
 
+    def handle_converter_list(self, message):
+        app.menu_manager.add_converters(message.converters)
+
     def handle_conversions_count_changed(self, message):
         library_tab_list = app.tabs['library']
         library_tab_list.update_converting_count(message.running_count,
@@ -1715,7 +1660,7 @@ class WidgetsMessageHandler(messages.MessageHandler):
         if isinstance(current_display, displays.DeviceDisplay):
             current_display.handle_device_sync_changed(message)
 
-    def handle_play_movie(self, message):
+    def handle_play_movies(self, message):
         app.playback_manager.start_with_items(message.item_infos)
 
     def handle_stop_playing(self, message):
@@ -1774,6 +1719,25 @@ class WidgetsMessageHandler(messages.MessageHandler):
         if not app.item_list_controller_manager or not app.tabs:
             return # got a metadata update before the UI opens
         app.item_list_controller_manager.update_metadata_progress(
-            message.target, message.remaining, message.eta, message.total)
+            message.target, message.finished, message.finished_local,
+            message.eta, message.total)
         app.tabs.update_metadata_progress(
-            message.target, message.remaining, message.eta, message.total)
+            message.target, message.finished_local, message.eta,
+            message.total)
+
+    def handle_set_net_lookup_enabled_finished(self, message):
+        prefpanel.enable_net_lookup_buttons()
+
+    def handle_net_lookup_counts(self, message):
+        prefpanel.update_net_lookup_counts(message.net_lookup_count,
+                                           message.total_count)
+
+class WidgetsFrontend(app.Frontend):
+    def call_on_ui_thread(self, func, *args, **kwargs):
+        call_on_ui_thread(func, *args, **kwargs)
+
+    def run_choice_dialog(self, title, description, buttons):
+        return dialogs.show_choice_dialog(title, description, buttons)
+
+    def quit(self):
+        app.widgetapp.do_quit()
