@@ -30,6 +30,7 @@
 """displays.py -- Handle switching the content on the right hand side of the
 app.
 """
+import functools
 import logging
 
 import os
@@ -39,6 +40,8 @@ from miro import messages
 from miro import signals
 from miro import prefs
 from miro import filetypes
+from miro.data import item
+from miro.data import itemtrack
 from miro.gtcache import gettext as _
 from miro.gtcache import ngettext
 from miro.frontends.widgets import browser
@@ -53,7 +56,10 @@ from miro.frontends.widgets import searchcontroller
 from miro.frontends.widgets import tabcontroller
 from miro.frontends.widgets import playlist
 from miro.frontends.widgets import widgetutil
+from miro.frontends.widgets.widgetstatestore import WidgetStateStore
+
 from miro.plat.frontends.widgets.threads import call_on_ui_thread
+from miro.plat.frontends.widgets import timer
 from miro.plat.frontends.widgets import widgetset
 
 class Display(signals.SignalEmitter):
@@ -96,6 +102,29 @@ class Display(signals.SignalEmitter):
         """
         pass
 
+    def get_column_info(self):
+        """Get info about the togglable columns for this display.
+
+        By default this returns None, which indicates that the display doesn't
+        support togglable columns.
+
+        Subclasses can override this and return the tuple (columns_enabled,
+        columns_available) which describes the togglable columns.
+        Both should be a list of column names.
+        """
+        return None
+
+    def toggle_column_enabled(self, name):
+        """Change if a column is enabled.
+
+        This method is called after the sorts menu changes.  By default we do
+        nothing.  If a display has togglable columns, it should override this
+        and update the enabled columns.
+
+        :param name: unicode identifier for the column
+        """
+        pass
+
 class TabDisplay(Display):
     """Display that displays the selection in the tab list."""
 
@@ -103,7 +132,7 @@ class TabDisplay(Display):
         raise NotImplementedError()
 
     def on_activate(self, is_push):
-        app.menu_manager.update_menus()
+        app.menu_manager.update_menus('tab-selection-changed')
 
     @staticmethod
     def should_display(tab_type, selected_tabs):
@@ -122,6 +151,8 @@ class DisplayManager(object):
     def __init__(self):
         # displays that we construct when the user clicks on them
         self.on_demand_display_classes = [
+                VideoItemsDisplay,
+                AudioItemsDisplay,
                 FeedDisplay,
                 AllFeedsDisplay,
                 PlaylistDisplay,
@@ -144,17 +175,17 @@ class DisplayManager(object):
                 # catch-all.
                 DummyDisplay,
         ]
+        # _select_display_for_tabs_args holds the arguments passed to
+        # select_display_for_tabs()
+        self._select_display_for_tabs_args = None
         # displays that we keep alive all the time
         self.permanent_displays = set()
-        self.add_permanent_display(VideoItemsDisplay())
-        self.add_permanent_display(AudioItemsDisplay())
         self.display_stack = []
         self.selected_tab_list = self.selected_tabs = None
         app.info_updater.connect('sites-removed', SiteDisplay.on_sites_removed)
 
     def add_permanent_display(self, display):
         self.permanent_displays.add(display)
-        display.on_selected()
 
     def get_current_display(self):
         try:
@@ -165,6 +196,31 @@ class DisplayManager(object):
 
     def select_display_for_tabs(self, selected_tab_list, selected_tabs):
         """Select a display to show in the right-hand side.  """
+        if self._select_display_for_tabs_args is None:
+            # First call to select_display_for_tabs(), we need to schedule
+            # _select_display_for_tabs() to be called.
+            timer.add(0.01, self._select_display_for_tabs)
+        # For all cases, we want to store these arguments in
+        # _select_display_for_tabs_args so that when
+        # _select_display_for_tabs() is called it uses them.
+        self._select_display_for_tabs_args = (selected_tab_list,
+                                              selected_tabs)
+
+    def _select_display_for_tabs(self):
+        """Call that does the work for select_display_for_tabs()
+
+        select_display_for_tabs() defers action in case the user is quickly
+        switching between tabs.  In that case, we only need to select the last
+        tab that was switched to.  This method does the actual work.
+        """
+        if self._select_display_for_tabs_args is None:
+            app.widgetapp.handle_soft_failure(
+                "_select_display_for_tabs():",
+                "_select_display_for_tabs_args is None")
+            return
+        selected_tab_list, selected_tabs = self._select_display_for_tabs_args
+        self._select_display_for_tabs_args = None
+
         if (selected_tab_list is self.selected_tab_list and
                 selected_tabs == self.selected_tabs and
                 len(self.display_stack) > 0 and
@@ -206,7 +262,13 @@ class DisplayManager(object):
         The main reason for this method is when we are playing video and the
         current tab gets removed (#16225).  In this case, we want to select a
         new tab and make a display for that tab, but not show that display
-        until video stops
+        until video stops.
+
+        Normally this is called as part of a deferred call by
+        change_non_video_displays().  We do this because loading a display
+        is a relatively expensive process and we want to be able to cancel
+        the operation if the display is going to be extremely transient,
+        e.g. during a continued keypress event as part of navigation.
         """
 
         if (len(self.display_stack) == 0 or
@@ -253,8 +315,10 @@ class DisplayManager(object):
         display = self.display_stack.pop()
         if unselect:
             self._unselect_display(display, on_top=True)
-        self.current_display.on_activate(is_push=False)
-        app.widgetapp.window.set_main_area(self.current_display.widget)
+        current_display = self.current_display
+        if current_display:
+            current_display.on_activate(is_push=False)
+            app.widgetapp.window.set_main_area(current_display.widget)
 
     def _unselect_display(self, display, on_top):
         if on_top:
@@ -266,6 +330,64 @@ class DisplayManager(object):
     def push_folder_contents_display(self, folder_info, start_playing=False):
         self.push_display(FolderContentsDisplay(folder_info, start_playing))
 
+class RecentlyActiveTracker(object):
+    """Used by GuideDisplay to track recently downloaded/played items."""
+
+    # maximum number of items to track for each of the lists
+    ITEM_LIMIT = 6
+
+    def __init__(self, guide_tab):
+        # map ItemTracker objects to the GuideTab method we should call to set
+        # the list for
+        self.trackers = {
+            self._recently_downloaded_tracker():
+                guide_tab.set_recently_downloaded,
+            self._recently_played_tracker('video'):
+                guide_tab.set_recently_watched,
+            self._recently_played_tracker('audio'):
+                guide_tab.set_recently_listened,
+        }
+
+        for (tracker, set_recent_method) in self.trackers.items():
+            list_callback = functools.partial(self.on_list_changed,
+                                              set_recent_method)
+            change_callback = functools.partial(self.on_items_changed,
+                                                set_recent_method)
+            tracker.connect('list-changed', list_callback)
+            tracker.connect('items-changed', change_callback)
+            app.item_tracker_updater.add_tracker(tracker)
+            set_recent_method(tracker.get_items())
+
+    def destroy(self):
+        for (tracker, set_recent_method) in self.trackers:
+            app.item_tracker_updater.remove_tracker(tracker)
+
+    def _recently_downloaded_tracker(self):
+        query = itemtrack.ItemTrackerQuery()
+        query.add_condition('downloaded_time', 'IS NOT', None)
+        query.add_condition('expired', '=', False)
+        query.add_condition('parent_id', 'IS', None)
+        query.add_condition('watched_time', 'IS', None)
+        query.set_order_by(['-downloaded_time'])
+        query.set_limit(self.ITEM_LIMIT)
+        return itemtrack.ItemTracker(call_on_ui_thread, query,
+                                     item.ItemSource())
+
+    def _recently_played_tracker(self, file_type):
+        query = itemtrack.ItemTrackerQuery()
+        query.add_condition('file_type', '=', file_type)
+        query.add_condition('watched_time', 'IS NOT', None)
+        query.set_order_by(['-last_watched'])
+        query.set_limit(self.ITEM_LIMIT)
+        return itemtrack.ItemTracker(call_on_ui_thread, query,
+                                     item.ItemSource())
+
+    def on_list_changed(self, set_recent_method, tracker):
+        set_recent_method(tracker.get_items())
+
+    def on_items_changed(self, set_recent_method, tracker, changed_ids):
+        set_recent_method(tracker.get_items())
+
 class GuideDisplay(TabDisplay):
     @staticmethod
     def should_display(tab_type, selected_tabs):
@@ -274,20 +396,12 @@ class GuideDisplay(TabDisplay):
     def __init__(self, tab_type, selected_tabs):
         Display.__init__(self)
         self.widget = guidecontroller.GuideTab(selected_tabs[0].browser)
-        app.info_updater.item_list_callbacks.add(u'guide-sidebar', None,
-                                                 self.on_item_list),
-        app.info_updater.item_changed_callbacks.add(u'guide-sidebar', None,
-                                                    self.on_item_changed),
-        messages.TrackItems(u'guide-sidebar', None).send_to_backend()
+        self.recently_active_tracker = RecentlyActiveTracker(self.widget)
         app.display_manager.add_permanent_display(self) # once we're loaded,
                                                         # stay loaded
 
-    def on_item_list(self, message):
-        self.widget.on_item_list(message.items)
-
-    def on_item_changed(self, message):
-        self.widget.on_item_changed(message.added, message.changed,
-                                    message.removed)
+    def cleanup(self):
+        self.recently_active_tracker.destroy()
 
 class SiteDisplay(TabDisplay):
     _open_sites = {} # maps site ids -> BrowserNav objects for them
@@ -296,9 +410,13 @@ class SiteDisplay(TabDisplay):
     def on_sites_removed(cls, info_updater, id_list):
         for id_ in id_list:
             try:
-                del cls._open_sites[id_]
+                browser = cls._open_sites[id_]
             except KeyError:
                 pass
+            else:
+                # explicitly destroy the browser.  Some platforms need this
+                # call to cleanup.
+                browser.destroy()
 
     @staticmethod
     def should_display(tab_type, selected_tabs):
@@ -315,7 +433,6 @@ class SiteDisplay(TabDisplay):
 class ItemListDisplayMixin(object):
     def on_selected(self):
         app.item_list_controller_manager.controller_created(self.controller)
-        self.controller.start_tracking()
 
     def on_activate(self, is_push):
         app.item_list_controller_manager.controller_displayed(self.controller)
@@ -333,10 +450,12 @@ class ItemListDisplayMixin(object):
                 self.controller)
 
     def cleanup(self):
-        self.controller.stop_tracking()
+        self.controller.cleanup()
         app.item_list_controller_manager.controller_destroyed(self.controller)
 
-    def update_columns_enabled(self):
+    def toggle_column_enabled(self, name):
+        app.widget_state.toggle_column_enabled(self.type, self.id,
+                self.controller.selected_view, name)
         self.controller.update_columns_enabled()
 
 class ItemListDisplay(ItemListDisplayMixin, TabDisplay):
@@ -351,6 +470,13 @@ class ItemListDisplay(ItemListDisplayMixin, TabDisplay):
     def make_controller(self, tab):
         raise NotImplementedError()
 
+    def get_column_info(self):
+        available = WidgetStateStore.get_columns_available(self.type, self.id,
+                self.controller.selected_view)
+        enabled = app.widget_state.get_columns_enabled(self.type, self.id,
+                self.controller.selected_view)
+        return enabled, available
+
 class FeedDisplay(ItemListDisplay):
     TAB_TYPE = u'feed'
     UPDATER_SIGNAL_NAME = 'feeds-changed'
@@ -377,7 +503,7 @@ class AllFeedsDisplay(FeedDisplay):
                selected_tabs[0].tab_class == u'feed')
 
     def make_controller(self, tab):
-        return feedcontroller.AllFeedsController(tab.id, True, True)
+        return feedcontroller.AllFeedsController()
 
     def cleanup(self):
         ItemListDisplay.cleanup(self)
@@ -416,12 +542,6 @@ class DeviceDisplay(DeviceDisplayMixin, TabDisplay):
                isinstance(selected_tabs[0], messages.DeviceInfo) and \
                not getattr(selected_tabs[0], 'fake', False)
 
-    def on_selected(self):
-        self.controller.start_tracking()
-
-    def cleanup(self):
-        self.controller.stop_tracking()
-
     def handle_current_sync_information(self, message):
         if not getattr(self.controller.device, 'fake', False):
             self.controller.handle_current_sync_information(message)
@@ -430,20 +550,31 @@ class DeviceDisplay(DeviceDisplayMixin, TabDisplay):
         if not getattr(self.controller.device, 'fake', False):
             self.controller.handle_device_sync_changed(message)
 
+    def on_activate(self, is_push):
+        if self.controller.device.mount:
+            message = messages.QuerySyncInformation(self.controller.device)
+            message.send_to_backend()
+
 class DeviceItemDisplay(DeviceDisplayMixin, ItemListDisplay):
     @staticmethod
     def should_display(tab_type, selected_tabs):
+        # FIXME: re-implement DeviceItemController with the new ItemList code
         return tab_type == u'device' and len(selected_tabs) == 1 and \
                isinstance(selected_tabs[0], messages.DeviceInfo) and \
                getattr(selected_tabs[0], 'fake', False)
 
 class SharingDisplay(ItemListDisplay):
+    def __init__(self, tab_type, selected_tabs):
+        # our type is always 'sharing', regardless if the tab type is
+        # 'sharing' or 'sharing-playlist'
+        ItemListDisplay.__init__(self, u'sharing', selected_tabs)
+
     @staticmethod
     def should_display(tab_type, selected_tabs):
-        return tab_type == u'sharing' and len(selected_tabs) == 1
+        return tab_type.startswith(u'sharing') and len(selected_tabs) == 1
 
     def make_controller(self, tab):
-        return sharingcontroller.SharingView(tab)
+        return sharingcontroller.SharingController(tab)
 
 class SearchDisplay(ItemListDisplay):
     @staticmethod
@@ -454,13 +585,6 @@ class SearchDisplay(ItemListDisplay):
         return searchcontroller.SearchController()
 
 class AudioVideoItemsDisplay(ItemListDisplay):
-    def __init__(self):
-        Display.__init__(self)
-        self.controller = self.make_controller()
-        self.widget = self.controller.widget
-        self.type = self.__class__.tab_type
-        self.id = self.__class__.tab_id
-
     @classmethod
     def should_display(cls, tab_type, selected_tabs):
         return (hasattr(selected_tabs[0], 'type') and selected_tabs[0].type ==
@@ -470,14 +594,14 @@ class VideoItemsDisplay(AudioVideoItemsDisplay):
     tab_type = u'videos'
     tab_id = u'videos'
 
-    def make_controller(self):
+    def make_controller(self, tab):
         return itemlistcontroller.VideoItemsController()
 
 class AudioItemsDisplay(AudioVideoItemsDisplay):
     tab_type = u'music'
     tab_id = u'music'
 
-    def make_controller(self):
+    def make_controller(self, tab):
         return itemlistcontroller.AudioItemsController()
 
 class OtherItemsDisplay(ItemListDisplay):
@@ -569,19 +693,19 @@ class CantPlayWidget(widgetset.SolidBackground):
         return label
 
     def _on_reveal(self, button):
-        app.widgetapp.reveal_file(self.video_path)
+        app.widgetapp.reveal_file(self.filename)
 
     def _on_play_externally(self, button):
-        app.widgetapp.open_file(self.video_path)
+        app.widgetapp.open_file(self.filename)
 
     def _on_skip(self, button):
         app.playback_manager.play_next_item()
 
-    def set_video_path(self, video_path):
-        self.video_path = video_path
-        self.filename_label.set_text(os.path.split(video_path)[-1])
-        self.filetype_label.set_text(os.path.splitext(video_path)[1])
-        if filetypes.is_playable_filename(video_path):
+    def set_filename(self, filename):
+        self.filename = filename
+        self.filename_label.set_text(os.path.split(filename)[-1])
+        self.filetype_label.set_text(os.path.splitext(filename)[1])
+        if filetypes.is_playable_filename(filename):
             self.play_externally_button.set_text(_('Play Externally'))
         else:
             self.play_externally_button.set_text(_('Open Externally'))
@@ -630,7 +754,7 @@ class VideoDisplay(Display):
     def setup(self, item_info, item_type, volume):
         self.show_renderer()
         self.cant_play_widget.set_remote(item_info.remote)
-        self.cant_play_widget.set_video_path(item_info.video_path)
+        self.cant_play_widget.set_filename(item_info.filename)
         self.item_info = item_info
         if item_type != 'video':
             self._open_error()
