@@ -36,6 +36,7 @@ from miro.frontends.widgets.itemlistwidgets import (
 from miro.frontends.widgets import itemcontextmenu
 from miro.frontends.widgets import prefpanel
 
+from miro import app
 from miro import messages
 from miro import downloader
 from miro import prefs
@@ -44,11 +45,11 @@ class DownloadsController(itemlistcontroller.ItemListController):
     def __init__(self):
         itemlistcontroller.ItemListController.__init__(
             self, u'downloading', u'downloading')
-        self.item_list.resort_on_update = True
         self.toolbar = None
+        self.update_buttons()
 
     def build_widget(self):
-        self.titlebar = self.make_titlebar()
+        self.titlebar.switch_to_view(self.widget.selected_view)
         self.widget.titlebar_vbox.pack_start(self.titlebar)
 
         self.status_toolbar = DownloadStatusToolbar()
@@ -70,7 +71,7 @@ class DownloadsController(itemlistcontroller.ItemListController):
         titlebar.connect("resume-all", self._on_resume_all)
         titlebar.connect("cancel-all", self._on_cancel_all)
         titlebar.connect("settings", self._on_settings)
-        titlebar.switch_to_view(self.widget.selected_view)
+        titlebar.hide_album_view_button()
         return titlebar
 
     def make_context_menu_handler(self):
@@ -82,6 +83,17 @@ class DownloadsController(itemlistcontroller.ItemListController):
     def _update_free_space(self):
         self.status_toolbar.update_free_space()
 
+    # The pause_all/resume_all and cancel_all disables the automatic sorting
+    # mechanism until the operations are complete, because even though the
+    # operation affects all items in the list the status updates for these
+    # items are not batched.
+    #
+    # When complete, the backend is expected to send a reply indicating that
+    # all operations (up to this point) are complete and at that point we can
+    # re-enable the sort again.  See DownloadSyncCommandComplete().  We hope
+    # for the best that the remote command to do whatever's needed should not
+    # take more than a second or two which should be accurate for a moderately
+    # sized download list.
     def _on_pause_all(self, widget):
         messages.PauseAllDownloads().send_to_backend()
 
@@ -94,6 +106,29 @@ class DownloadsController(itemlistcontroller.ItemListController):
     def _on_settings(self, widget):
         prefpanel.show_window("downloads")
 
-    def on_items_changed(self):
+    def handle_item_list_changes(self):
+        itemlistcontroller.ItemListController.handle_item_list_changes(self)
+        self.update_rates()
+        self.update_buttons()
+
+    def update_rates(self):
         self.status_toolbar.update_rates(
-            downloader.total_down_rate, downloader.total_up_rate)
+            app.download_state_manager.total_down_rate,
+            app.download_state_manager.total_up_rate)
+
+    def update_buttons(self):
+        items = self.item_list.get_items()
+        if len(items) == 0:
+            self.titlebar.set_button_enabled("resume", False)
+            self.titlebar.set_button_enabled("pause", False)
+            self.titlebar.set_button_enabled("cancel", False)
+        else:
+            all_paused = all_downloading = True
+            for item in items:
+                if item.is_paused:
+                    all_downloading = False
+                else:
+                    all_paused = False
+            self.titlebar.set_button_enabled("resume", not all_downloading)
+            self.titlebar.set_button_enabled("pause", not all_paused)
+            self.titlebar.set_button_enabled("cancel", True)
