@@ -35,22 +35,23 @@ from miro.gtcache import ngettext
 from miro import messages
 from miro.frontends.widgets import feedsettingspanel
 from miro.frontends.widgets import itemcontextmenu
+from miro.frontends.widgets import itemlist
 from miro.frontends.widgets import itemlistcontroller
 from miro.frontends.widgets import itemlistwidgets
-from miro.frontends.widgets import itemtrack
 from miro.frontends.widgets import itemrenderer
 from miro.frontends.widgets import widgetutil
 
-class FeedController(itemlistcontroller.ItemListController,
-                     itemlistcontroller.FilteredListMixin):
+class FeedController(itemlistcontroller.ItemListController):
     """Controller object for feeds."""
-    TYPE = u'feed'
     def __init__(self, id_, is_folder, is_directory_feed):
         self.is_folder = is_folder
         self.is_directory_feed = is_directory_feed
         self.titlebar = None
-        itemlistcontroller.ItemListController.__init__(self, self.TYPE, id_)
-        itemlistcontroller.FilteredListMixin.__init__(self)
+        if is_folder:
+            type_ = u'feed-folder'
+        else:
+            type_ = u'feed'
+        itemlistcontroller.ItemListController.__init__(self, type_, id_)
         self.show_resume_playing_button = True
 
     def make_context_menu_handler(self):
@@ -59,8 +60,7 @@ class FeedController(itemlistcontroller.ItemListController,
     def build_widget(self):
         feed_info = widgetutil.get_feed_info(self.id)
 
-        self.titlebar = self.make_titlebar(feed_info)
-        self.titlebar.connect('toggle-filter', self.on_toggle_filter)
+        self.titlebar.connect('filter-clicked', self.on_filter_clicked)
         self.titlebar.switch_to_view(self.widget.selected_view)
         self.titlebar.connect('search-changed', self._on_search_changed)
         self.widget.titlebar_vbox.pack_start(self.titlebar)
@@ -79,7 +79,8 @@ class FeedController(itemlistcontroller.ItemListController,
         return itemrenderer.ItemRenderer(display_channel=self.is_folder,
                 is_podcast=(not feed_info.is_directory_feed))
 
-    def make_titlebar(self, feed_info):
+    def make_titlebar(self):
+        feed_info = widgetutil.get_feed_info(self.id)
         if feed_info.is_directory_feed:
             titlebar = itemlistwidgets.WatchedFolderTitlebar()
         elif feed_info.is_folder:
@@ -87,6 +88,8 @@ class FeedController(itemlistcontroller.ItemListController,
         else:
             titlebar = itemlistwidgets.ChannelTitlebar()
             titlebar.connect('save-search', self._on_save_search)
+        if not self.is_directory_feed:
+            titlebar.hide_album_view_button()
         return titlebar
 
     def get_saved_search_text(self):
@@ -131,9 +134,6 @@ class FeedController(itemlistcontroller.ItemListController,
     def _on_auto_download_changed(self, widget, setting):
         messages.AutodownloadChange(self.id, setting).send_to_backend()
 
-    def on_initial_list(self):
-        self._update_counts()
-
     def on_items_changed(self):
         self._update_counts()
 
@@ -211,8 +211,15 @@ class FeedController(itemlistcontroller.ItemListController,
 
 class AllFeedsController(FeedController):
     TYPE = u'tab'
-    def build_item_tracker(self):
-        return itemtrack.ItemListTracker.create(u'feed', self.id)
 
-    def make_titlebar(self, feed_info):
+    def __init__(self):
+        FeedController.__init__(self, u'feed-base-tab', True, True)
+
+    def make_titlebar(self):
         return itemlistwidgets.AllFeedsTitlebar()
+
+    def get_item_list_grouping(self):
+        return itemlist.feed_grouping
+
+    def get_multi_row_album_mode(self):
+        return 'feed'
