@@ -31,12 +31,14 @@
 """
 
 from miro import app
+from miro import fileutil
 from miro import prefs
 from miro import util
 from miro import messages
 from miro.fileobject import FilenameType
 from miro.plat.frontends.widgets import widgetset
 from miro.plat.frontends.widgets import threads
+from miro.frontends.widgets import widgetconst
 from miro.frontends.widgets import widgetutil
 from miro.frontends.widgets import prefpanel
 from miro.frontends.widgets import dialogs
@@ -55,8 +57,24 @@ _SYSTEM_LANGUAGE = os.environ.get("LANGUAGE", "")
 WIDTH = 475
 HEIGHT = 375
 
+def get_media_player_name_path(where):
+    """Get the media player name and path from the platform.
 
-def _build_title_question(text):
+    This method wraps the platform version with a bit of error checking and
+    logging.
+
+    :param where: what piece of code calling this?
+    """
+    mp_name, mp_path = get_plat_media_player_name_path()
+    logging.debug("%s: got media player name/path: %r %r",
+                  where, mp_name, mp_path)
+    if mp_path is None or mp_name is None:
+        # If either path or name is not given, make sure that neither are
+        # set.  See #18865.
+        return None, None
+    return mp_name, mp_path
+
+def _build_title(text):
     """Builds and returns a title widget for the panes in the
     First Time Startup dialog.
     """
@@ -72,6 +90,22 @@ def _build_paragraph_text(text):
     lab.set_size_request(WIDTH - 40, -1)
     return widgetutil.align_left(lab, bottom_pad=15)
 
+def _build_checkbox_and_label(checkbox, label_text):
+    """Build a checkbox with a label right under it.
+
+    This is useful because checkboxes don't wrap properly.  Labels don't
+    wrap great either, but we can hack things to make labels wrape.
+    """
+    label = widgetset.Label(label_text)
+    label.set_size_request(WIDTH - 40, -1)
+    label.set_wrap(True)
+    vbox = widgetset.VBox(spacing=0)
+    vbox.pack_start(widgetutil.align_left(checkbox))
+    vbox.pack_start(widgetutil.align_left(label))
+    label.set_size(widgetconst.SIZE_SMALL)
+    checkbox.set_size(widgetconst.SIZE_NORMAL)
+    return vbox
+
 class FirstTimeDialog(widgetset.DialogWindow):
     def __init__(self, done_firsttime_callback, title=None):
         if title == None:
@@ -99,9 +133,8 @@ class FirstTimeDialog(widgetset.DialogWindow):
 
         self._done_firsttime_callback = done_firsttime_callback
 
-        self.mp_name, self.mp_path = get_plat_media_player_name_path()
-        self._has_media_player = (
-            self.mp_name is not None and self.mp_path is not None)
+        mp_info = get_media_player_name_path('firsttimedialog')
+        self.mp_name, self.mp_path = mp_info
 
         self._page_box = widgetset.VBox()
         self._pages = self.build_pages()
@@ -113,13 +146,10 @@ class FirstTimeDialog(widgetset.DialogWindow):
 
     def build_pages(self):
         pages = [self.build_language_page(),
-                 self.build_startup_page()]
-
-        if self._has_media_player:
-            pages.append(self.build_media_player_import_page())
-
-        pages.extend([self.build_find_files_page(),
-                      self.build_search_page()])
+                 self.build_startup_page(),
+                 self.build_music_page(),
+                 self.build_find_files_page(),
+                 self.build_search_page()]
 
         for page in pages:
             page.set_size_request(WIDTH - 40, HEIGHT - 40)
@@ -131,7 +161,8 @@ class FirstTimeDialog(widgetset.DialogWindow):
 
     def on_close(self, widget=None):
         if self.import_media_player_stuff:
-            logging.debug("firsttimedialog: adding mp_path")
+            logging.debug("firsttimedialog: adding mp_path (%r)",
+                          self.mp_path)
             app.watched_folder_manager.add(self.mp_path)
         if self.gathered_media_files:
             logging.debug("firsttimedialog: adding %d files",
@@ -182,7 +213,7 @@ class FirstTimeDialog(widgetset.DialogWindow):
                     "to help you get started.",
                     {'name': app.config.get(prefs.SHORT_APP_NAME)})))
 
-        vbox.pack_start(_build_title_question(_(
+        vbox.pack_start(_build_title(_(
                     "What language would you like %(name)s to be in?",
                     {'name': app.config.get(prefs.SHORT_APP_NAME)})))
 
@@ -200,8 +231,7 @@ class FirstTimeDialog(widgetset.DialogWindow):
             # import time, so if someone changes the language, then
             # the translations have already happened.  we reload the
             # module to force them to happen again.  bug 17515
-            if "miro.frontends.widgets.widgetconst" in sys.modules:
-                reload(sys.modules["miro.frontends.widgets.widgetconst"])
+            reload(widgetconst)
             self.this_page(rebuild=True)
 
         lang_option_menu = widgetset.OptionMenu([op[1] for op in lang_options])
@@ -249,7 +279,7 @@ class FirstTimeDialog(widgetset.DialogWindow):
                     "and update your podcasts.",
                     {'name': app.config.get(prefs.SHORT_APP_NAME)})))
 
-        vbox.pack_start(_build_title_question(_(
+        vbox.pack_start(_build_title(_(
                     "Would you like to run %(name)s on startup?",
                     {'name': app.config.get(prefs.SHORT_APP_NAME)})))
 
@@ -288,7 +318,7 @@ class FirstTimeDialog(widgetset.DialogWindow):
                     "will be copied or duplicated.",
                     {"name": app.config.get(prefs.SHORT_APP_NAME)})))
 
-        vbox.pack_start(_build_title_question(_(
+        vbox.pack_start(_build_title(_(
                     "Would you like %(name)s to search your computer "
                     "for media files?",
                     {"name": app.config.get(prefs.SHORT_APP_NAME)})))
@@ -456,7 +486,10 @@ class FirstTimeDialog(widgetset.DialogWindow):
             threads.call_on_ui_thread(self.make_search_progress)
 
         except StopIteration:
-            num_found = len(self.gathered_media_files)
+            if self.gathered_media_files:
+                num_found = len(self.gathered_media_files)
+            else:
+                num_found = 0
             self.search_complete(
                 ngettext(
                     "found %(count)s media file",
@@ -465,21 +498,6 @@ class FirstTimeDialog(widgetset.DialogWindow):
                     {"count": num_found}))
             self.finder = None
 
-        except Exception:
-            # this is here to get more data for bug #17422
-            logging.exception("exception thrown in make_search_progress")
-
-            # we want to clean up after this exception, too.
-            num_found = len(self.gathered_media_files)
-            self.search_complete(
-                ngettext(
-                    "found %(count)s media file",
-                    "found %(count)s media files",
-                    num_found,
-                    {"count": num_found}))
-            self.finder = None
-
-
     def start_search(self):
         # only start a search if we haven't gathered anything, yet.
         if self.gathered_media_files is not None:
@@ -536,50 +554,117 @@ class FirstTimeDialog(widgetset.DialogWindow):
 
         return vbox
 
-
-    def build_media_player_import_page(self):
-        vbox = widgetset.VBox(spacing=5)
-
-        vbox.pack_start(_build_title_question(_(
-                "Would you like to display your %(player)s music and "
-                "video in %(appname)s?",
-                {"player": self.mp_name,
-                 "appname": app.config.get(prefs.SHORT_APP_NAME)})))
-
-        rbg = widgetset.RadioButtonGroup()
-        yes_rb = widgetset.RadioButton(_("Yes"), rbg)
-        no_rb = widgetset.RadioButton(_("No"), rbg)
-        yes_rb.set_selected()
-
-        vbox.pack_start(widgetutil.align_left(yes_rb))
-        vbox.pack_start(widgetutil.align_left(no_rb))
-
-        lab = widgetset.Label(_(
-                "Note: %(appname)s won't move or copy any files on your "
-                "disk.  It will just add them to your %(appname)s library.",
-                {"appname": app.config.get(prefs.SHORT_APP_NAME)}))
-        lab.set_size_request(WIDTH - 40, -1)
-        lab.set_wrap(True)
-        vbox.pack_start(widgetutil.align_left(lab))
-
-        def handle_next(widget):
-            if rbg.get_selected() == yes_rb:
-                self.import_media_player_stuff = True
-            else:
-                self.import_media_player_stuff = False
-            self.next_page()
+    def build_music_page(self):
+        vbox = MusicSetupVBox(self.mp_name, _('Music Setup'))
+        if vbox.import_cbx:
+            def on_import_toggled(cbx):
+                self.import_media_player_stuff = cbx.get_checked()
+            vbox.import_cbx.connect("toggled", on_import_toggled)
 
         prev_button = widgetset.Button(_("< Previous"))
         prev_button.connect('clicked', lambda x: self.prev_page())
 
         next_button = widgetset.Button(_("Next >"))
-        next_button.connect('clicked', handle_next)
+        next_button.connect('clicked', lambda x: self.next_page())
 
         vbox.pack_start(
             widgetutil.align_bottom(widgetutil.align_right(
                     widgetutil.build_hbox((prev_button, next_button)))),
             expand=True)
 
-        vbox = widgetutil.pad(vbox)
-
         return vbox
+
+class MusicSetupVBox(widgetset.VBox):
+    """VBox for the music setup page.
+
+    This is separated out because we want to re-use it when the user first
+    clicks the music tab.
+
+    Attributes:
+    - import_cbx: Checkbox to import music from the platform player
+    - net_lookup_cbx: Checkbox to enable net lookups by default
+
+    This class handles changing the NET_LOOKUP_BY_DEFAULT pref, but the class
+    using it must handle responding import_cbx being checked.
+    """
+    def __init__(self, mp_name, title):
+        widgetset.VBox.__init__(self, spacing=5)
+
+        self.pack_start(_build_title(title))
+        if mp_name is not None:
+            self.import_cbx = widgetset.Checkbox(
+                _("Show my %(player)s music in my %(appname)s library.",
+                  {"player": mp_name,
+                   "appname": app.config.get(prefs.SHORT_APP_NAME)}))
+            controls = _build_checkbox_and_label(
+                self.import_cbx, _("(Don't worry, no files are copied.)"))
+            self.pack_start(widgetutil.pad(controls, bottom=10))
+        else:
+            self.import_cbx = None
+
+        self.net_lookup_cbx = widgetset.Checkbox(
+            _('Cleanup Song Info and Album Art'))
+
+        self.pack_start(_build_checkbox_and_label(
+            self.net_lookup_cbx, _('Miro will use Echonest and 7Digital '
+                                   'to cleanup all song titles, info, and '
+                                   'album art.  Recommended.')))
+        prefpanel.attach_boolean(self.net_lookup_cbx,
+                                 prefs.NET_LOOKUP_BY_DEFAULT)
+
+        # Give the "Note:" heading top padding to separate it from the
+        # checkboxes, but no bottom padding to keep it together with the rest
+        # of the text.
+        heading = widgetset.Label(_("Note:"))
+        heading.set_wrap(True)
+        heading.set_bold(True)
+        heading.set_size_request(WIDTH - 40, -1)
+        self.pack_start(widgetutil.pad(heading, top=15))
+        text = _("You can manually set or undo song info cleanup by "
+                 "right-clicking on a song or batch of songs.  You can "
+                 "adjust lookup settings at any time in Miro preferences."
+                 "\n\n"
+                 "Online lookup involves sending anonymized data about "
+                 "your songs to Miro and indirectly to Echonest and "
+                 "7Digital.")
+        self.pack_start(_build_paragraph_text(text))
+
+class MusicSetupDialog(dialogs.MainDialog):
+    """Ask some questions on music settings.
+
+    We pop this up the first time the user clicks on the music tab, if we
+    haven't already asked the questions in the first-time setup dialog.
+    """
+    def __init__(self):
+        dialogs.MainDialog.__init__(self, _("Music Setup"))
+        mp_info = get_media_player_name_path('musicsetupdialog')
+        self.mp_name, self.mp_path = mp_info
+        if self.already_added_media_player_path():
+            self.mp_name = self.mp_path = None
+        self.vbox = MusicSetupVBox(self.mp_name, _('First Time Music Setup'))
+        self.set_extra_widget(self.vbox)
+        self.add_button(_("Get Started"))
+
+    def already_added_media_player_path(self):
+        if self.mp_path is None:
+            return False
+        wf_model = app.watched_folder_manager.model
+        wf_iter = wf_model.first_iter()
+        while wf_iter is not None:
+            path = wf_model[wf_iter][1]
+            if fileutil.samefile(path, self.mp_path):
+                return True
+            wf_iter = wf_model.next_iter(wf_iter)
+
+    def import_path(self):
+        """Path to import music from.
+
+        The code using MusicSetupDialog should check this once the dialog is
+        closed and create a watched folder if needed.
+
+        :returns: path or None if we shouldn't import anything.
+        """
+        if self.vbox.import_cbx and self.vbox.import_cbx.get_checked():
+            return self.mp_path
+        else:
+            return None
