@@ -35,26 +35,22 @@ import gobject
 import gtk
 
 from miro import app
-from miro import player
 from miro.gtcache import gettext as _
 from miro import messages
+from miro.plat.frontends.widgets import hidemouse
 from miro.plat import resources
 from miro.plat import screensaver
+from miro.frontends.widgets.gtk import player
 from miro.frontends.widgets.gtk.window import Window, WrappedWindow
 from miro.frontends.widgets.gtk.widgetset import (
     Widget, VBox, Label, HBox, Alignment, Background, DrawingArea,
     ClickableImageButton)
-from miro.frontends.widgets.gtk.persistentwindow import PersistentWindow
+from miro.plat.frontends.widgets import videoembed
 
 BLACK = (0.0, 0.0, 0.0)
 WHITE = (1.0, 1.0, 1.0)
 GREEN = (159.0 / 255.0, 202.0 / 255.0, 120.0 / 255.0)
 
-# Global VideoWidget object.  We re-use so we can re-use our
-# PersistentWindow
-video_widget = None
-
-
 class ClickableLabel(Widget):
     """This is like a label and reimplements many of the Label things,
     but it's an EventBox with a Label child widget.
@@ -118,11 +114,6 @@ class NullRenderer:
         pass
 
 
-def make_hidden_cursor():
-    pixmap = gtk.gdk.Pixmap(None, 1, 1, 1)
-    color = gtk.gdk.Color()
-    return gtk.gdk.Cursor(pixmap, pixmap, color, color, 0, 0)
-
 
 def make_label(text, handler, visible=True):
     if visible:
@@ -213,20 +204,6 @@ class VideoOverlay(Window):
         window.move(screen_rect.x, screen_rect.y + screen_rect.height -
                 my_height)
 
-
-class VideoWidget(Widget):
-    def __init__(self, renderer):
-        Widget.__init__(self)
-        self.set_widget(PersistentWindow())
-        self._widget.set_double_buffered(False)
-        self._widget.add_events(gtk.gdk.POINTER_MOTION_MASK)
-        self._widget.add_events(gtk.gdk.BUTTON_PRESS_MASK)
-        renderer.set_widget(self._widget)
-
-    def destroy(self):
-        self._widget.destroy()
-
-
 class Divider(DrawingArea):
     def size_request(self, layout):
         return (1, 25)
@@ -238,22 +215,27 @@ class Divider(DrawingArea):
         context.rel_line_to(0, context.height)
         context.stroke()
 
-
 class VideoDetailsWidget(Background):
     def __init__(self):
         Background.__init__(self)
         self.item_info = None
         self.rebuild_video_details()
-        self._delete_link = None
-        self._delete_image = None
+        self._delete_link = self._delete_image = None
+        self._keep_link = self._keep_image = None
         self._will_play_handle = app.playback_manager.connect(
             'will-play', self.on_will_play)
+        self._info_changed_handle = app.playback_manager.connect(
+            'playing-info-changed', self.on_info_changed)
 
     def on_will_play(self, widget, duration):
         # we need to update the video details now that the file is
         # open and we know more about subtitle track info.
         self.rebuild_video_details()
 
+    def on_info_changed(self, widget, item_info):
+        self.item_info = item_info
+        self.rebuild_video_details()
+
     def rebuild_video_details(self):
         # this removes the child widget if there is one
         self.remove()
@@ -312,6 +294,12 @@ class VideoDetailsWidget(Background):
         right_side_hbox.pack_start(_align_middle(self._delete_image))
         self._delete_link = make_label(_("Delete"), self.handle_delete)
         right_side_hbox.pack_start(_align_middle(self._delete_link))
+        if self.item_info.can_be_saved: # keepable
+            self._keep_image = make_image_button(
+                'images/keep-button.png', self.handle_keep)
+            right_side_hbox.pack_start(_align_middle(self._keep_image))
+            self._keep_link = make_label(_("Keep"), self.handle_keep)
+            right_side_hbox.pack_start(_align_middle(self._keep_link))
 
         outer_hbox = HBox()
         outer_hbox.pack_start(_align_left(left_side_hbox, left_pad=10),
@@ -372,6 +360,8 @@ class VideoDetailsWidget(Background):
     def handle_keep(self, widget):
         messages.KeepVideo(self.item_info.id).send_to_backend()
         self._widget.window.set_cursor(None)
+        self.reset()
+        self.rebuild_video_details()
 
     def handle_delete(self, widget):
         item_info = self.item_info
@@ -419,8 +409,7 @@ class VideoDetailsWidget(Background):
         menu.append(sep)
 
         child = gtk.MenuItem(_("Select a Subtitles file..."))
-        sensitive = 'PlayingLocalVideo' in app.menu_manager.enabled_groups
-        child.set_sensitive(sensitive)
+        child.set_sensitive(app.playback_manager.is_playing_video)
         child.connect('activate', self.handle_select_subtitle_file)
         child.show()
         menu.append(child)
@@ -430,20 +419,20 @@ class VideoDetailsWidget(Background):
     def handle_disable_subtitles(self, widget):
         if widget.active:
             app.video_renderer.disable_subtitles()
-            app.widgetapp.window.on_playback_change(app.playback_manager)
+            app.menu_manager.update_menus('playback-changed')
             self.rebuild_video_details()
 
     def handle_subtitle_change(self, widget, index):
         if widget.active:
-            app.video_renderer.enable_subtitle_track(index)
-            app.widgetapp.window.on_playback_change(app.playback_manager)
+            app.video_renderer.set_subtitle_track(index)
+            app.menu_manager.update_menus('playback-changed')
             self.rebuild_video_details()
 
     def handle_select_subtitle_file(self, widget):
         app.playback_manager.open_subtitle_file()
 
     def handle_commentslink(self, widget, event):
-        app.widgetapp.open_url(self.item_info.commentslink)
+        app.widgetapp.open_url(self.item_info.comments_link)
 
     def handle_share(self, widget, event):
         app.widgetapp.share_item(self.item_info)
@@ -480,8 +469,12 @@ class VideoDetailsWidget(Background):
             self._delete_link.on_leave_notify(None, None)
         if self._delete_image:
             self._delete_image.on_leave_notify(None, None)
+        if self._keep_link:
+            self._keep_link.on_leave_notify(None, None)
+        if self._keep_image:
+            self._keep_image.on_leave_notify(None, None)
 
-class VideoPlayer(player.Player, VBox):
+class VideoPlayer(player.GTKPlayer, VBox):
     """Video renderer widget.
 
     Note: ``app.video_renderer`` must be initialized before
@@ -491,49 +484,33 @@ class VideoPlayer(player.Player, VBox):
     HIDE_CONTROLS_TIMEOUT = 2000
 
     def __init__(self):
-        global video_widget
-        player.Player.__init__(self)
+        player.GTKPlayer.__init__(self, app.video_renderer)
         VBox.__init__(self)
-        if app.video_renderer is not None:
-            self.renderer = app.video_renderer
-        else:
-            self.renderer = NullRenderer()
 
         self.overlay = None
         self.screensaver_manager = None
 
-        if video_widget is None:
-            video_widget = VideoWidget(self.renderer)
-        self._video_widget = video_widget
+        self._video_widget = videoembed.VideoWidget(self.renderer)
         self.pack_start(self._video_widget, expand=True)
 
         self._video_details = VideoDetailsWidget()
         self.pack_start(self._video_details)
 
         self.hide_controls_timeout = None
-        self.motion_handler = None
-        self.videobox_motion_handler = None
-        self.hidden_cursor = make_hidden_cursor()
-        # piggyback on the TrackItemsManually message that playback.py sends.
-        app.info_updater.item_changed_callbacks.add('manual', 'playback-list',
-                self._on_items_changed)
-        self._item_id = None
 
-        self._video_widget.wrapped_widget_connect(
-            'button-press-event', self.on_button_press)
+        self._video_widget.connect('double-click', self.on_double_click)
+        self._video_widget.connect('mouse-motion', self.on_mouse_motion)
 
     def teardown(self):
-        self.renderer.reset()
-        app.info_updater.item_changed_callbacks.remove('manual',
-                'playback-list', self._on_items_changed)
-        self._items_changed_callback = None
+        # remove the our embedding widget from the hierarchy
         self.remove(self._video_widget)
-
-    def _on_items_changed(self, message):
-        for item_info in message.changed:
-            if item_info.id == self._item_id:
-                self._video_details.update_info(item_info)
-                break
+        # now that we aren't showing a video widget, we can reset playback
+        self.renderer.reset()
+        self._video_widget.destroy()
+        # remove callbacks
+        self._video_widget.disconnect_all()
+        # dereference VideoWidget
+        self._video_widget = None
 
     def update_for_presentation_mode(self, mode):
         pass
@@ -541,7 +518,6 @@ class VideoPlayer(player.Player, VBox):
     def set_item(self, item_info, success_callback, error_callback):
         self._video_details.set_video_details(item_info)
         self.renderer.select_file(item_info, success_callback, error_callback)
-        self._item_id = item_info.id
 
     def get_elapsed_playback_time(self):
         return self.renderer.get_current_time()
@@ -555,12 +531,12 @@ class VideoPlayer(player.Player, VBox):
     def play(self):
         self.renderer.play()
         # do this to trigger the overlay showing up for a smidge
-        self.on_mouse_motion(None, None)
+        self.on_mouse_motion(None)
 
     def play_from_time(self, resume_time=0):
-        # Note: This overrides player.Player's version of play_from_time, but
-        # this one seeks directly rather than fiddling with
-        # total_playback_time.
+        # FIXME: this overrides the default implementation.  The reason
+        # is going through the default implementation it requires the total
+        # time and it may not be ready at this point.
         self.seek_to_time(resume_time)
         self.play()
 
@@ -590,17 +566,13 @@ class VideoPlayer(player.Player, VBox):
             self.screensaver_manager.disable()
         self.rebuild_video_details()
         self._make_overlay()
-        self.motion_handler = self.wrapped_widget_connect(
-                'motion-notify-event', self.on_mouse_motion)
-        self.videobox_motion_handler = self.overlay._window.connect(
-                'motion-notify-event', self.on_mouse_motion)
+        self.overlay._window.connect('motion-notify-event',
+                self.on_motion_notify)
         if not app.playback_manager.detached_window:
-            app.widgetapp.window.menubar.hide()
+            app.widgetapp.window.menubar._widget.hide()
         self.schedule_hide_controls(self.HIDE_CONTROLS_TIMEOUT)
-        # make sure all hide() calls go through, otherwise we get the wrong
-        # size on windows (#10810)
-        while gtk.events_pending():
-            gtk.main_iteration()
+        # Need to call set_decorated() before fullscreen.  See #10810.
+        _window().set_decorated(False)
         _window().fullscreen()
 
     def _make_overlay(self):
@@ -633,21 +605,25 @@ class VideoPlayer(player.Player, VBox):
     def prepare_switch_to_detached_playback(self):
         gobject.timeout_add(0, self.rebuild_video_details)
 
-    def on_button_press(self, widget, event):
-        if event.type == gtk.gdk._2BUTTON_PRESS:
-            app.playback_manager.toggle_fullscreen()
-            return True
-        return False
+    def on_double_click(self, widget):
+        app.playback_manager.toggle_fullscreen()
 
-    def on_mouse_motion(self, widget, event):
+    def on_motion_notify(self, widget, event):
+        self.on_mouse_motion(widget)
+
+    def on_mouse_motion(self, widget):
         if not self.overlay:
             return
         if not self.overlay.is_visible():
             show_it_all = False
 
-            if event is None:
-                show_it_all = True
-            else:
+            # NOTE: this code wasn't working when I went through the
+            # windows overhaul, so I just left it commented out.  Will says we
+            # should eventually implement this.
+            #
+            #if event is None:
+                #show_it_all = True
+            #else:
                 # figures out the monitor that miro is fullscreened on and
                 # gets the monitor geometry for that.
                 # if app.playback_manager.detached_window is not None:
@@ -664,7 +640,8 @@ class VideoPlayer(player.Player, VBox):
 
                 # Hack to fix #17213.  Eventually we should remove this and
                 # uncomment the code above to implement #8655
-                show_it_all = True
+                #show_it_all = True
+            show_it_all = True
 
             if show_it_all:
                 self.show_controls()
@@ -674,15 +651,18 @@ class VideoPlayer(player.Player, VBox):
         else:
             self.last_motion_time = time.time()
 
+    def hide_mouse(self):
+        hidemouse.hide(_window().window)
+
     def show_mouse(self):
-        _window().window.set_cursor(None)
+        hidemouse.unhide(_window().window)
 
     def show_controls(self):
         self.show_mouse()
         self.overlay.show()
 
     def hide_controls(self):
-        _window().window.set_cursor(self.hidden_cursor)
+        self.hide_mouse()
         if self.overlay and self.overlay.is_visible():
             self.overlay.close()
 
@@ -711,14 +691,15 @@ class VideoPlayer(player.Player, VBox):
         if self.screensaver_manager is not None:
             self.screensaver_manager.enable()
             self.screensaver_manager = None
-        app.widgetapp.window.menubar.show()
+        app.widgetapp.window.menubar._widget.show()
         self.rebuild_video_details()
         self._video_details.show()
         self._destroy_overlay()
         _window().unfullscreen()
-        self._widget.disconnect(self.motion_handler)
+        # Undo above call to set_decorated()
+        _window().set_decorated(True)
         self.cancel_hide_controls()
-        _window().window.set_cursor(None)
+        self.show_mouse()
 
     def select_subtitle_file(self, sub_path, handle_successful_select):
         app.video_renderer.select_subtitle_file(
@@ -728,3 +709,15 @@ class VideoPlayer(player.Player, VBox):
 
     def select_subtitle_encoding(self, encoding):
         app.video_renderer.select_subtitle_encoding(encoding)
+
+    def get_subtitle_tracks(self):
+        return self.renderer.get_subtitle_tracks()
+
+    def get_enabled_subtitle_track(self):
+        return self.renderer.get_enabled_audio_track()
+
+    def set_subtitle_track(self, track_index):
+        if track_index is not None:
+            self.renderer.set_subtitle_track(track_index)
+        else:
+            self.renderer.disable_subtitles()
