@@ -30,6 +30,9 @@
 """itemcontextmenu.py -- Handle popping up a context menu for an item
 """
 
+import functools
+
+from miro import api
 from miro import app
 from miro import displaytext
 from miro import messages
@@ -52,9 +55,12 @@ class ItemContextMenuHandler(object):
                     for iter in tableview.get_selection()]
 
         if len(selected) == 1:
-            return self._make_context_menu_single(selected[0])
+            menu = self._make_context_menu_single(selected[0])
         else:
-            return self._make_context_menu_multiple(selected)
+            menu = self._make_context_menu_multiple(selected)
+        # allow extensions to change the menu
+        api.hook_invoke('item_context_menu', selected, menu)
+        return menu
 
     def _remove_context_menu_item(self, selection):
         """Returns the appropriate remove/delete menu item.
@@ -99,7 +105,7 @@ class ItemContextMenuHandler(object):
         section = []
 
         def play_externally():
-            app.widgetapp.open_file(item.video_path)
+            app.widgetapp.open_file(item.filename)
             messages.MarkItemWatched(item).send_to_backend()
 
         # drm items seem to go in misc and are always unplayable.
@@ -128,6 +134,18 @@ class ItemContextMenuHandler(object):
                 convert_menu = self._make_convert_menu()
                 section.append((_('Convert to...'), convert_menu))
 
+            if not (item.device or item.remote):
+                section.append((_('Set media kind as...'),
+                                self._make_edit_metadata_menu()))
+            if not item.remote and not item.device:
+                if item.net_lookup_enabled:
+                    label = _("Don't Use Online Lookup Data")
+                    callback = app.widgetapp.disable_net_lookup_for_selection
+                else:
+                    label = _("Use Online Lookup Data")
+                    callback = app.widgetapp.enable_net_lookup_for_selection
+                section.append((label, callback))
+
             if section:
                 menu_sections.append(section)
                 section = []
@@ -137,8 +155,10 @@ class ItemContextMenuHandler(object):
             playing_item = app.playback_manager.get_playing_item()
             is_paused = app.playback_manager.is_paused
 
-            if item != playing_item or (item == playing_item and is_paused):
+            if item != playing_item:
                 section.append((_('Play'), app.widgetapp.play_selection))
+            elif item == playing_item and is_paused:
+                section.append((_('Play'), app.playback_manager.toggle_paused))
             else:
                 section.append((_('Pause'), app.playback_manager.pause))
             # Resume
@@ -169,44 +189,31 @@ class ItemContextMenuHandler(object):
                 menu_sections.append(section)
                 section = []
 
-            # Edit Item Details, Delete, Resume/Stop Seeding
-            # this doesn't work for device or remote items.
             if not (item.device or item.remote):
                 section.append((
                         _("Edit Item Details"), app.widgetapp.edit_items))
 
-            if not (item.device or item.remote):
-                if item.seeding_status == 'seeding':
-                    section.append((
-                            _('Stop Seeding'),
-                            messages.StopUpload(item.id).send_to_backend))
-                elif item.seeding_status == 'stopped':
-                    section.append((
-                            _('Resume Seeding'),
-                            messages.StartUpload(item.id).send_to_backend))
                 if not item.is_container_item:
                     section.append((
                             _('Add to Playlist'),
                             app.widgetapp.add_to_playlist))
 
-        elif ((item.download_info is not None and
-               item.download_info.state != 'failed')):
-            if item.download_info.state != 'finished':
-                if not menu_sections:
-                    # make sure that the default menu option isn't destructive
-                    # (re: #16715)
-                    section.append(None)
+        elif item.is_download:
+            if not menu_sections:
+                # make sure that the default menu option isn't destructive
+                # (re: #16715)
+                section.append(None)
+            section.append((
+                    _('Cancel Download'),
+                    messages.CancelDownload(item.id).send_to_backend))
+            if not item.is_paused:
                 section.append((
-                        _('Cancel Download'),
-                        messages.CancelDownload(item.id).send_to_backend))
-                if item.download_info.state != 'paused':
-                    section.append((
-                            _('Pause Download'),
-                            messages.PauseDownload(item.id).send_to_backend))
-                else:
-                    section.append((
-                            _('Resume Download'),
-                            messages.ResumeDownload(item.id).send_to_backend))
+                        _('Pause Download'),
+                        messages.PauseDownload(item.id).send_to_backend))
+            else:
+                section.append((
+                        _('Resume Download'),
+                        messages.ResumeDownload(item.id).send_to_backend))
 
         else:
             if not (item.device or item.remote):
@@ -215,8 +222,7 @@ class ItemContextMenuHandler(object):
                     section.append((
                             _('Download'),
                             messages.StartDownload(item.id).send_to_backend))
-                    if (item.download_info and
-                        item.download_info.state == u'failed'):
+                    if item.is_failed_download:
                         section.append((
                             _('Cancel Download'),
                             messages.CancelDownload(
@@ -229,6 +235,15 @@ class ItemContextMenuHandler(object):
                 # Play
                 section.append((_('Play'), app.widgetapp.play_selection))
 
+        if item.is_seeding:
+            section.append((
+                    _('Stop Seeding'),
+                    messages.StopUpload(item.id).send_to_backend))
+        elif not item.is_seeding and item.is_torrent:
+            section.append((
+                    _('Resume Seeding'),
+                    messages.StartUpload(item.id).send_to_backend))
+
         if item.downloaded and not item.remote:
             if file_navigator_name:
                 reveal_text = _('Show File in %(progname)s',
@@ -237,7 +252,7 @@ class ItemContextMenuHandler(object):
                 reveal_text = _('File on Disk')
 
             section.append((reveal_text,
-                lambda: app.widgetapp.check_then_reveal_file(item.video_path)))
+                lambda: app.widgetapp.check_then_reveal_file(item.filename)))
             remove = self._remove_context_menu_item([item])
             if remove:
                 section.append(remove)
@@ -297,6 +312,8 @@ class ItemContextMenuHandler(object):
         paused = []
         uploadable = []
         expiring = []
+        net_lookup_enabled = []
+        net_lookup_disabled = []
         editable = False
 
         # local functions
@@ -328,6 +345,10 @@ class ItemContextMenuHandler(object):
             if info.downloaded:
                 downloaded.append(info)
                 if info.is_playable:
+                    if info.net_lookup_enabled:
+                        net_lookup_enabled.append(info)
+                    else:
+                        net_lookup_disabled.append(info)
                     playable.append(info)
                     if info.device:
                         device.append(info)
@@ -343,12 +364,11 @@ class ItemContextMenuHandler(object):
                         container.append(info)
                 if not (info.device or info.remote):
                     editable = True
-            elif info.state == 'paused':
+            elif info.is_paused:
                 paused.append(info)
-            elif info.state == 'downloading':
+            elif info.is_download:
                 downloading.append(info)
-                if (info.download_info.torrent and
-                        info.download_info.state != 'uploading'):
+                if info.is_torrent and not info.is_seeding:
                     uploadable.append(info)
             else:
                 available.append(info)
@@ -383,7 +403,17 @@ class ItemContextMenuHandler(object):
                 menu.append(None)
                 convert_menu = self._make_convert_menu()
                 menu.append((_('Convert to...'), convert_menu))
-
+        menu.append((_('Set media kind as...'),
+                    self._make_edit_metadata_menu()))
+        if downloaded and not remote and not device:
+            if net_lookup_enabled:
+                label = _("Don't Use Online Lookup Data")
+                callback = app.widgetapp.disable_net_lookup_for_selection
+                menu.append((label, callback))
+            if net_lookup_disabled:
+                label = _("Use Online Lookup Data")
+                callback = app.widgetapp.enable_net_lookup_for_selection
+                menu.append((label, callback))
 
         if available:
             if len(menu) > 0:
@@ -425,17 +455,30 @@ class ItemContextMenuHandler(object):
 
         return menu
 
+    def _make_edit_metadata_menu(self):
+        # Edit metadata
+        edit_metadata_menu = []
+
+        edit_metadata_menu.append((_('Movie'),
+          lambda: app.widgetapp.set_media_kind(u'movie')))
+        edit_metadata_menu.append((_('Show'),
+          lambda: app.widgetapp.set_media_kind(u'show')))
+        edit_metadata_menu.append((_('Clip'),
+          lambda: app.widgetapp.set_media_kind(u'clip')))
+        edit_metadata_menu.append((_('Podcast'),
+          lambda: app.widgetapp.set_media_kind(u'podcast')))
+
+        return edit_metadata_menu
+
     def _make_convert_menu(self):
         convert_menu = []
-        sections = conversion_manager.get_converters()
-        for index, section in enumerate(sections):
-            for converter in section[1]:
-                def convert(converter=converter.identifier):
-                    app.widgetapp.convert_items(converter)
-                convert_menu.append((converter.displayname, convert))
-            if index+1 < len(sections):
-                convert_menu.append(None)
-        convert_menu.append(None)
+        sections = app.menu_manager.get_converters()
+        for index, converter_list in enumerate(sections):
+            for (identifier, title) in converter_list:
+                func = functools.partial(app.widgetapp.convert_items,
+                                         identifier)
+                convert_menu.append((title, func))
+            convert_menu.append(None)
         convert_menu.append((_("Show Conversion Folder"),
                              app.widgetapp.reveal_conversions_folder))
         return convert_menu
