@@ -262,7 +262,14 @@ class NumberField(Field):
             self.widget.set_width(width)
             self.widget.set_max_length(width)
         value = self.common_value or ""
-        self.widget.set_text(str(value))
+        # Ugh!  Convert to utf-8 or else decode may not work!
+        if isinstance(value, unicode):
+            value = value.encode('utf-8')
+        try:
+            value = str(value)
+        except TypeError, ValueError:
+            value = ''
+        self.widget.set_text(value)
 
     def get_value(self):
         value = self.widget.get_text()
@@ -318,7 +325,7 @@ class ThumbnailField(DialogOwnerMixin, Field):
     TITLE = _("Choose a thumbnail file")
     DIALOG = widgetset.FileOpenDialog
     def __init__(self, items, label):
-        Field.__init__(self, 'cover_art', items, label)
+        Field.__init__(self, 'cover_art_path', items, label)
         DialogOwnerMixin.__init__(self, self.DIALOG, self.TITLE)
         path = self.common_value
 
@@ -489,7 +496,7 @@ class GeneralPanel(DialogPanel):
         """Pack the left column into the middle HBox of the main VBox."""
         widget = widgetset.VBox()
         left = []
-        left.append(TextField('name', self.items, _("Name")))
+        left.append(TextField('title', self.items, _("Name")))
         left.append(TextField('artist', self.items, _("Artist")))
         left.append(TextField('album', self.items, _("Album")))
         left.append(TextField('genre', self.items, _("Genre")))
@@ -527,7 +534,7 @@ class GeneralPanel(DialogPanel):
 
     def _pack_bottom(self):
         """Pack the bottom row into the VBox."""
-        bottom = [PathField('video_path', self.items, _("Path"), readonly=True)]
+        bottom = [PathField('filename', self.items, _("Path"), readonly=True)]
         self.vbox.pack_start(widgetutil.pad(widgetset.HLine(), top=25, bottom=10,
             left=15, right=15))
         for field in bottom:
@@ -558,15 +565,10 @@ class VideoPanel(DialogPanel):
                 (u'podcast', _("Podcast")),
             ]),
         ]
-        content = widgetset.VBox()
+        self.vbox = widgetset.VBox()
         for field in self.fields:
             field.set_label_width(120)
-            content.pack_start(field.get_box(), padding=5)
-        # XXX - hack: OS X is cutting off the right side of the box in single
-        # selection mode; this seems like a bug in layout. padding the right
-        # side causes only padding to be cut off.
-        # XXX - this padding fixes 17065. 17065 is the same layout issue?
-        self.vbox = widgetutil.pad(content, right=15)
+            self.vbox.pack_start(field.get_box(), padding=5)
 
 class ToggleButtonBackground(widgetset.Background):
     """Gradiated background for an individual ToggleButton."""
@@ -750,6 +752,17 @@ class ItemEditDialog(widgetset.Dialog):
         self.panels[name] = content
         self.toggler.add_option(name, label)
 
+    def set_width_from_panels(self):
+        """Set the min-width for our panels.
+
+        We set the min-width to the width of the biggest panel, to avoid
+        things moving too much when the user switches between them
+        """
+        max_width = -1
+        for panel in self.panels.values():
+            max_width = max(max_width, panel.vbox.get_size_request()[0])
+        self.content_panel.set_size_request(max_width, -1)
+
     def on_choose_panel(self, _toggler, name):
         self.content_panel.set(self.panels[name].vbox)
 
@@ -761,6 +774,7 @@ class ItemEditDialog(widgetset.Dialog):
         self._add_panel(_("General"), 'general', GeneralPanel(self.items))
         self._add_panel(_("Video"), 'video', VideoPanel(self.items))
         self._pack_bottom()
+        self.set_width_from_panels()
         self.toggler.choose('general')
         self.content_panel.set(self.panels['general'].vbox)
         return self.vbox
