@@ -36,6 +36,7 @@ from miro import signals
 from miro import displaytext
 from miro import prefs
 from miro import util
+from miro.data import item
 from miro.gtcache import gettext as _
 from miro.frontends.widgets import cellpack
 from miro.frontends.widgets import imagepool
@@ -180,9 +181,14 @@ class ItemRendererSignals(signals.SignalEmitter):
 
     signals:
         throbber-drawn (obj, item_info) -- a progress throbber was drawn
+        item-retrying (obj, item_info) -- a download will be retried, and we
+        need to update the time
     """
     def __init__(self):
-        signals.SignalEmitter.__init__(self, 'throbber-drawn')
+        signals.SignalEmitter.__init__(self,
+                                       'throbber-drawn',
+                                       'item-retrying',
+                                      )
 
 _cached_images = {} # caches ImageSurface for get_image()
 def get_image(image_name):
@@ -207,13 +213,18 @@ def _load_image(image_name):
     path = os.path.join('images', filename)
     return imagepool.get_surface(resources.path(path))
 
-class ItemRendererBase(widgetset.InfoListRenderer):
+class ItemRenderer(widgetset.ItemListRenderer):
     MIN_WIDTH = 600
     HEIGHT = 147
 
-    def __init__(self, wide_image=False):
-        widgetset.InfoListRenderer.__init__(self)
+    def __init__(self, display_channel=True, is_podcast=False,
+                 wide_image=False):
+        widgetset.ItemListRenderer.__init__(self)
         self.canvas = ItemRendererCanvas(wide_image)
+        self.signals = ItemRendererSignals()
+        self.display_channel = display_channel
+        self.is_podcast = is_podcast
+        self.setup_torrent_folder_description()
 
     def get_size(self, style, layout_manager):
         return self.MIN_WIDTH, self.HEIGHT
@@ -242,32 +253,19 @@ class ItemRendererBase(widgetset.InfoListRenderer):
                 context.height, selected, hotspot)
         layout.draw(context)
 
-    def layout_all(self, layout_manager, width, height, selected, hotspot):
-        """Create a ItemRendererLayout object for our cell."""
-        raise NotImplementedError()
-
-class ItemRenderer(ItemRendererBase):
-    def __init__(self, display_channel=True, is_podcast=False,
-            wide_image=False):
-        ItemRendererBase.__init__(self, wide_image)
-        self.signals = ItemRendererSignals()
-        self.display_channel = display_channel
-        self.is_podcast = is_podcast
-        self.setup_torrent_folder_description()
-
     def setup_torrent_folder_description(self):
         text = (u'<a href="#show-torrent-contents">%s</a>' %
                 SHOW_CONTENTS_TEXT)
         self.torrent_folder_description = util.HTMLStripper().strip(text)
 
     def layout_all(self, layout_manager, width, height, selected, hotspot):
-        download_mode = (self.info.state in ('downloading', 'paused'))
+        download_mode = self.info.is_download
         self.canvas.start_new_cell(layout_manager, width, height, selected,
                 hotspot, download_mode)
         # add elements that are always present
         self.canvas.add_thumbnail(self.info.thumbnail,
                 self.calc_thumbnail_hotspot())
-        self.canvas.add_text(self.info.name, ITEM_TITLE_COLOR,
+        self.canvas.add_text(self.info.title, ITEM_TITLE_COLOR,
                 self.calc_description(), self.calc_extra_info())
         # add elements for download-mode or non-download-mode
         if download_mode:
@@ -278,6 +276,8 @@ class ItemRenderer(ItemRendererBase):
 
     def add_normal_mode_elements(self):
         """Add elements when we aren't in download mode."""
+        if isinstance(self.info, item.DBErrorItemInfo):
+            return
         self.add_main_button()
         self.add_emblem()
         self.add_secondary_button()
@@ -327,8 +327,8 @@ class ItemRenderer(ItemRendererBase):
             self.canvas.add_remove_button(*self.remove_button_info())
 
         if self.info.expiration_date:
-            text = displaytext.expiration_date(self.info.expiration_date)
-            self.canvas.add_keep_button('keep', text)
+            self.canvas.add_keep_button('keep',
+                                        self.info.expiration_date_text)
         elif self.info.id in app.saved_items:
             self.canvas.add_saved_emblem()
 
@@ -339,48 +339,40 @@ class ItemRenderer(ItemRendererBase):
         self.canvas.add_menu_button_download_mode()
 
     def add_progress_bar(self):
-        dl_info = self.info.download_info
-        if dl_info.state == 'paused':
+        if self.info.is_paused:
             pause_button_mode = 'resume'
         else:
             pause_button_mode = 'pause'
-        if dl_info.downloaded_size == 0:
-            # show empty bar before we start up
-            self.canvas.add_progress_bar(0.0, pause_button_mode)
-        elif dl_info.total_size <= 0:
-            # show throbber once we've started, but still don't know the
-            # total_size
+        progress = self.info.download_progress
+        if progress is None:
+            # show throbber
             throbber_index = self.attrs.get('throbber-value', 0) % 10
             self.canvas.add_progress_throbber(throbber_index,
                     pause_button_mode)
             self.signals.emit('throbber-drawn', self.info)
         else:
-            # show regular bar otherwise
-            amount = float(dl_info.downloaded_size) / dl_info.total_size
-            self.canvas.add_progress_bar(amount, pause_button_mode)
+            # show regular bar
+            self.canvas.add_progress_bar(progress, pause_button_mode)
 
     def add_download_info(self):
-        dl_info = self.info.download_info
-        if self.info.state == 'paused':
-            eta = down_rate = 0
-        else:
-            eta = dl_info.eta
-            down_rate = dl_info.rate
         download_info = ItemDownloadInfo()
-        download_info.add_line('time-left',
-                displaytext.time_string_0_blank(eta), None)
+        download_info.add_line('time-left', self.info.eta_text, None)
         download_info.add_line('dl-speed',
-                displaytext.download_rate(down_rate),
-                displaytext.size_string(dl_info.downloaded_size))
-        if dl_info.torrent:
+                               self.info.download_rate_text,
+                               self.info.downloaded_size_text)
+        if self.info.is_torrent:
             download_info.add_line('ul-speed',
-                displaytext.download_rate(self.info.up_rate),
-                displaytext.size_string(self.info.up_total))
+                                   self.info.upload_rate_text,
+                                   self.info.upload_size_text)
         self.canvas.add_download_info(download_info)
-        if self.info.download_info.rate == 0:
+        if self.info.is_paused:
+            pass
+        elif self.info.rate is None:
             self.canvas.add_startup_info(
-                self.info.download_info.startup_activity)
-        elif dl_info.torrent and dl_info.state != 'paused':
+                self.info.startup_activity)
+            if self.info.is_retrying:
+                self.signals.emit('item-retrying', self.info)
+        elif self.info.is_torrent:
             self.add_torrent_info()
 
     def add_torrent_info(self):
@@ -388,7 +380,7 @@ class ItemRenderer(ItemRendererBase):
                 (_('PEERS'), str(self.info.connections)),
                 (_('SEEDS'), str(self.info.seeders)),
                 (_('LEECH'), str(self.info.leechers)),
-                (_('SHARE'), "%.2f" % self.info.up_down_ratio),
+                (_('SHARE'), "%.2f" % self.info.upload_ratio),
         )
         self.canvas.add_torrent_info(lines)
 
@@ -402,8 +394,7 @@ class ItemRenderer(ItemRendererBase):
             return None
 
     def calc_description(self):
-        if (self.info.download_info and self.info.download_info.torrent and
-                self.info.children):
+        if self.info.is_torrent_folder:
             text, links = self.torrent_folder_description
         else:
             text, links = self.info.description_stripped
@@ -412,22 +403,24 @@ class ItemRenderer(ItemRendererBase):
                 preface_color)
 
     def calc_description_preface(self):
-        if (self.display_channel and self.info.feed_name and
+        if (self.display_channel and self.info.parent_title and
                 not self.info.is_external):
-            return ("%s: " % self.info.feed_name, FEED_NAME_COLOR)
+            return ("%s: " % self.info.parent_title, FEED_NAME_COLOR)
         return '', widgetutil.WHITE
 
     def calc_extra_info(self):
-        return (self.info.display_date, self.info.display_duration,
-                self.info.display_size, self.info.file_format)
+        display_date = displaytext.date_slashes(self.info.release_date)
+        display_duration = displaytext.duration(self.info.duration)
+        display_size = displaytext.size_string(self.info.size)
+        return (display_date, display_duration, display_size,
+                self.info.file_format)
 
     def calc_extra_button(self):
         """Calculate the button to put to the right of the emblem.
 
         :returns: (text, hotspot_name) tuple, or None
         """
-        if (self.info.download_info and
-                self.info.download_info.state == 'uploading'):
+        if self.info.is_seeding:
             return (STOP_SEEDING_TEXT, 'stop_seeding')
         elif self.info.pending_auto_dl:
             return (CANCEL_TEXT, 'cancel_auto_download')
@@ -448,12 +441,11 @@ class ItemRenderer(ItemRendererBase):
         if self.info.has_drm:
             visuals = EMBLEM_VISUALS_DRM
             text = _('DRM locked')
-        elif (self.info.download_info
-                and self.info.download_info.state == 'failed'):
+        elif self.info.is_failed_download:
             visuals = EMBLEM_VISUALS_FAILED
             image = get_image('status-icon-alert')
             text = u"%s-%s" % (ERROR_TEXT,
-                    self.info.download_info.short_reason_failed)
+                    self.info.short_reason_failed)
         elif self.info.pending_auto_dl:
             visuals = EMBLEM_VISUALS_QUEUED
             text = QUEUED_TEXT
@@ -472,7 +464,7 @@ class ItemRenderer(ItemRendererBase):
             text = _("Resume at %(resumetime)s",
                      {"resumetime": displaytext.short_time_string(
                          self.info.resume_time)})
-        elif not self.info.item_viewed and self.info.state == "new":
+        elif self.info.new:
             visuals = EMBLEM_VISUALS_NEWLY_AVAILABLE
             text = NEWLY_AVAILABLE_TEXT
         else:
@@ -1093,7 +1085,9 @@ class ItemRendererCanvas(object):
         context.fill()
 
     def draw_thumbnail(self, context, x, y, width, height):
-        icon = imagepool.get_surface(self.thumbnail, (width, height))
+        icon = imagepool.get_surface(self.thumbnail, (width, height),
+                                     invalidator=util.mtime_invalidator(
+                self.thumbnail))
         icon_x = x + (width - icon.width) // 2
         icon_y = y + (height - icon.height) // 2
         # if our thumbnail is far enough to the left, we need to set a clip
@@ -1269,15 +1263,15 @@ class EmblemDrawer(object):
                 cap_image.height)
 
 class PlaylistItemRenderer(ItemRenderer):
-    def __init__(self, playlist_sorter):
+    def __init__(self, playlist_order):
         ItemRenderer.__init__(self, display_channel=False)
-        self.playlist_sorter = playlist_sorter
+        self.playlist_order = playlist_order
 
     def remove_button_info(self):
         return ('remove-playlist', 'remove')
 
     def calc_description_preface(self):
-        order_number = self.playlist_sorter.sort_key(self.info) + 1
+        order_number = self.playlist_order.item_position(self.info)
         if self.info.description_stripped[0]:
             text = "%s - " % order_number
         else:
@@ -1293,17 +1287,30 @@ class DeviceItemRenderer(ItemRenderer):
     def calc_extra_button(self):
         return DOWNLOAD_TO_MY_MIRO_TEXT, 'download-device-item'
 
-class ConversionItemRenderer(ItemRendererBase):
+class ConversionItemRenderer(widgetset.CustomCellRenderer):
     """Class to draw conversion items
 
     This one is substantially different from ItemRenderer because it deals
     with ConversionTaskInfo objects.
     """
     def __init__(self):
-        ItemRendererBase.__init__(self, wide_image=True)
+        widgetset.CustomCellRenderer.__init__(self)
+        self.canvas = ItemRendererCanvas(wide_image=True)
         self.canvas.set_progress_bar_images('conversion-progress-left',
                 'conversion-progress-middle', 'conversion-progress-right')
 
+    def get_size(self, style, layout_manager):
+        return ItemRenderer.MIN_WIDTH, ItemRenderer.HEIGHT
+
+    def hotspot_test(self, style, layout_manager, x, y, width, height):
+        layout = self.layout_all(layout_manager, width, height, False, None)
+        return layout.find_hotspot_name(x, y)
+
+    def render(self, context, layout_manager, selected, hotspot, hover):
+        layout = self.layout_all(layout_manager, context.width,
+                context.height, selected, hotspot)
+        layout.draw(context)
+
     def layout_all(self, layout_manager, width, height, selected, hotspot):
         download_mode = (self.info.state == 'running')
         self.canvas.start_new_cell(layout_manager, width, height, selected,
