@@ -54,7 +54,7 @@ from miro import app
 from miro import messages
 from miro import prefs
 from miro.plat.frontends.widgets import widgetset
-from miro.frontends.widgets import menus
+from miro.frontends.widgets import keyboard
 from miro.frontends.widgets import widgetutil
 from miro.frontends.widgets import dialogs
 from miro.frontends.widgets import widgetconst
@@ -64,7 +64,7 @@ from miro.plat import resources
 from miro.plat.utils import filename_to_unicode, get_logical_cpu_count
 from miro.plat.frontends.widgets.bonjour import install_bonjour
 from miro.plat.frontends.widgets.threads import call_on_ui_thread
-from miro.gtcache import gettext as _
+from miro.gtcache import gettext as _, gettext_lazy
 from miro import gtcache
 
 # Note: we do an additional import from prefpanelset half way down the file.
@@ -473,8 +473,23 @@ def pack_extras(vbox, panel):
         [vbox.pack_start(mem) for mem in extras[1:]]
 
 class GeneralPanel(PanelBuilder):
+    def __init__(self):
+        PanelBuilder.__init__(self)
+        self.disable_net_lookup_buttons = False
+
     def build_widget(self):
         v = widgetset.VBox(8)
+        self.pack_startup_shutdown_section(v)
+        self.pack_sidebar_section(v)
+        self.pack_language_section(v)
+        self.pack_echonest_section(v)
+        pack_extras(v, "general")
+        return v
+
+    def pack_startup_shutdown_section(self, v):
+        startup_shutdown_heading = dialogwidgets.heading(
+                                       _('Startup / Shutdown'))
+        v.pack_start(startup_shutdown_heading)
 
         run_at_startup_cbx = widgetset.Checkbox(_(
             "Automatically run %(appname)s when I log in.",
@@ -501,6 +516,21 @@ class GeneralPanel(PanelBuilder):
                        prefs.WARN_IF_CONVERTING_ON_QUIT)
         v.pack_start(warn_if_converting_cbx)
 
+    def pack_sidebar_section(self, v):
+        sidebar_heading = dialogwidgets.heading(_('Sidebar'))
+        v.pack_start(sidebar_heading)
+
+        cbx = widgetset.Checkbox(_('Show videos from podcasts in the Videos '
+                                   'section.'))
+        attach_boolean(cbx, prefs.SHOW_PODCASTS_IN_VIDEO)
+        v.pack_start(cbx)
+
+        cbx = widgetset.Checkbox(_('Show audio from podcasts in the Music '
+                                   'section.'))
+        attach_boolean(cbx, prefs.SHOW_PODCASTS_IN_MUSIC)
+        v.pack_start(cbx)
+
+    def pack_language_section(self, v):
         # FIXME - need to automatically generate list of available
         # languages in correct language
         lang_options = gtcache.get_languages()
@@ -509,9 +539,9 @@ class GeneralPanel(PanelBuilder):
         lang_option_menu = widgetset.OptionMenu([op[1] for op in lang_options])
         attach_combo(lang_option_menu, prefs.LANGUAGE,
                      [op[0] for op in lang_options])
-        v.pack_start(widgetutil.align_left(
-            widgetutil.build_control_line((
-                        widgetset.Label(_("Language:")), lang_option_menu))))
+
+        language_heading = dialogwidgets.heading(_("Language"))
+        v.pack_start(language_heading)
 
         v.pack_start(widgetutil.align_left(
             dialogwidgets.note(
@@ -519,26 +549,105 @@ class GeneralPanel(PanelBuilder):
                       "restart %(appname)s.)",
                       {"appname": app.config.get(prefs.SHORT_APP_NAME)}))))
 
-        pack_extras(v, "general")
+        v.pack_start(widgetutil.align_left(
+            widgetutil.build_control_line((
+                        widgetset.Label(_('Display in:')), lang_option_menu))))
+
+    def pack_echonest_section(self, v):
+        heading = dialogwidgets.heading(
+            _("Music Album Art and Info Cleanup"))
+        v.pack_start(heading)
+
+        help_text = _('Online lookup can find matching album art and song '
+                      'info for all your audio files')
+        v.pack_start(widgetset.Label(help_text))
+
+        # Link to the data providers for net lookup.  This method doesn't look
+        # the best, but hopefully it can be translated somewhat decently.
+        provider_links = widgetset.HBox()
+        provider_links.pack_start(widgetset.Label(_("Via:")))
+        provider_links.pack_start(widgetset.Label(" "))
+        echonest_link = widgetutil.LinkButton(_('Echonest'))
+        seven_digital_link = widgetutil.LinkButton(_('7Digital'))
+        provider_links.pack_start(echonest_link)
+        provider_links.pack_start(widgetset.Label(" "))
+        provider_links.pack_start(seven_digital_link)
+        def on_echonest_click(button):
+            app.widgetapp.open_url("http://the.echonest.com/")
+        echonest_link.connect("clicked", on_echonest_click)
+        def on_7digital_click(button):
+            app.widgetapp.open_url("http://www.7digital.com/")
+        seven_digital_link.connect("clicked", on_7digital_click)
+        v.pack_start(widgetutil.align_center(provider_links))
+
+        self.run_net_lookup_button = widgetset.Button(
+            _('Run Online Lookup for All Current Music'))
+        self.run_net_lookup_button.connect(
+            'clicked', self.net_lookup_button_clicked, True)
+        self.remove_net_lookup_button = widgetset.Button(
+            _("Remove All Lookup data"))
+        self.remove_net_lookup_button.connect(
+            'clicked', self.net_lookup_button_clicked, False)
+
+        v.pack_start(widgetutil.build_control_line(
+            (self.run_net_lookup_button, self.remove_net_lookup_button)))
+
+        self.net_lookup_label = widgetset.Label(self._net_lookup_label_text())
+        v.pack_start(self.net_lookup_label)
+
+        cb_text = _('Automatically run online lookup for any new audio files '
+                    'that I add to Miro.')
+        cbx = widgetset.Checkbox(cb_text)
+        attach_boolean(cbx, prefs.NET_LOOKUP_BY_DEFAULT)
+        v.pack_start(cbx)
+        self._update_net_lookup_butons()
+
+    def net_lookup_button_clicked(self, button, value):
+        messages.SetNetLookupEnabled(None, value).send_to_backend()
+        self.disable_net_lookup_buttons = True
+        self._update_net_lookup_butons()
+
+    def enable_net_lookup_buttons(self):
+        self.disable_net_lookup_buttons = False
+        self._update_net_lookup_butons()
+
+    def update_net_lookup_counts(self):
+        self.net_lookup_label.set_text(self._net_lookup_label_text())
+        self._update_net_lookup_butons()
+
+    def _net_lookup_label_text(self):
+        global _net_lookup_counts
+
+        return _("Using lookup data for %(net_lookup_count)s "
+                 "out of %(total_count)s items", {
+                     'net_lookup_count': _net_lookup_counts[0],
+                     'total_count': _net_lookup_counts[1],
+                 })
+
+    def _update_net_lookup_butons(self):
+        """Enable/Disable the buttons for echonest lookup."""
+        global _net_lookup_counts
+
+        net_lookup_count, total_count = _net_lookup_counts
+
+        if self.disable_net_lookup_buttons:
+            self.run_net_lookup_button.disable()
+            self.remove_net_lookup_button.disable()
+        elif net_lookup_count == total_count:
+            self.run_net_lookup_button.disable()
+            self.remove_net_lookup_button.enable()
+        elif net_lookup_count == 0:
+            self.run_net_lookup_button.enable()
+            self.remove_net_lookup_button.disable()
+        else:
+            self.run_net_lookup_button.enable()
+            self.remove_net_lookup_button.enable()
 
-        return v
 
 class PodcastsPanel(PanelBuilder):
     def build_widget(self):
         grid = dialogwidgets.ControlGrid()
 
-        cbx = widgetset.Checkbox(_('Show videos from podcasts in the Videos '
-                                   'section.'))
-        attach_boolean(cbx, prefs.SHOW_PODCASTS_IN_VIDEO)
-        grid.pack(cbx)
-        grid.end_line(spacing=2)
-
-        cbx = widgetset.Checkbox(_('Show audio from podcasts in the Music '
-                                   'section.'))
-        attach_boolean(cbx, prefs.SHOW_PODCASTS_IN_MUSIC)
-        grid.pack(cbx)
-        grid.end_line(spacing=12)
-
         cc_options = [(1440, _("Every day")),
                       (60, _("Every hour")),
                       (30, _("Every 30 minutes")),
@@ -564,8 +673,10 @@ class PodcastsPanel(PanelBuilder):
         attach_combo(max_option_menu, prefs.MAX_OLD_ITEMS_DEFAULT,
             [op[0] for op in max_options])
 
-        view_options = [(WidgetStateStore.STANDARD_VIEW, _("Standard view")),
-                      (WidgetStateStore.LIST_VIEW, _("List view"))]
+        view_options = [
+                (WidgetStateStore.STANDARD_VIEW, _("Standard view")),
+                (WidgetStateStore.LIST_VIEW, _("List view")),
+            ]
         view_option_menu = widgetset.OptionMenu([op[1] for op in view_options])
         attach_combo(view_option_menu, prefs.PODCASTS_DEFAULT_VIEW,
             [op[0] for op in view_options])
@@ -884,7 +995,7 @@ class FoldersPanel(PanelBuilder):
         grid.end_line(spacing=18)
         grid.pack_label(
             _('Watch for new video and audio items in these folders '
-              'and include them in library:'), span=2)
+              'and include them in library:'), span=2, width=500)
         grid.end_line()
         grid.pack(self.watched_folder_helper.folder_list, pad_right=12)
         grid.pack(self.watched_folder_helper.button_box)
@@ -1050,32 +1161,34 @@ class PlaybackPanel(PanelBuilder):
         subtitles_cbx = widgetset.Checkbox(
             _('Automatically enable movie subtitles when available.'))
 
-        playback_heading = dialogwidgets.heading(_("Continuous Playback"))
-
-        rbg = widgetset.RadioButtonGroup()
-        play_rb = widgetset.RadioButton(
-            _("Play video and audio items one after another"), rbg)
-        stop_rb = widgetset.RadioButton(
-            _("Stop after each video or audio item"), rbg)
-
-        resume_heading = dialogwidgets.heading(_("Resume Playback"))
-
+        videos_heading = dialogwidgets.heading(_("Video Playback"))
         resume_videos_cbx = widgetset.Checkbox(
-            _('Continue playing videos from where they were last stopped.'))
+            _('Resume playback from where the video was last stopped.'))
+        continue_videos_cbx = widgetset.Checkbox(
+               _('Play continuously - when one item finishes, play the next item in the list.'))
+
+        music_heading = dialogwidgets.heading(_("Music Playback"))
         resume_music_cbx = widgetset.Checkbox(
-            _('Continue playing music files from '
-              'where they were last stopped.'))
+            _('Resume playback from where the song was last stopped.'))
+        continue_music_cbx = widgetset.Checkbox(
+               _('Play continuously - when one item finishes, play the next item in the list.'))
+
+        podcasts_heading = dialogwidgets.heading(_("Podcast Playback"))
         resume_podcasts_cbx = widgetset.Checkbox(
-            _('Continue playing podcast files from '
-              'where they were last stopped.'))
+            _('Resume playback from where the podcast was last stopped.'))
+        continue_podcasts_cbx = widgetset.Checkbox(
+               _('Play continuously - when one item finishes, play the next item in the list.'))
 
         attach_boolean(miro_cbx, prefs.PLAY_IN_MIRO,
-                       (separate_cbx, resume_heading,
+                       (separate_cbx, videos_heading,
+                        music_heading, podcasts_heading,
                         resume_videos_cbx,
+                        continue_videos_cbx,
                         resume_music_cbx,
+                        continue_music_cbx,
                         resume_podcasts_cbx,
-                        subtitles_cbx, playback_heading,
-                        play_rb, stop_rb))
+                        continue_podcasts_cbx,
+                        subtitles_cbx))
 
         v.pack_start(widgetutil.align_left(miro_cbx, bottom_pad=6))
 
@@ -1085,23 +1198,26 @@ class PlaybackPanel(PanelBuilder):
         attach_boolean(subtitles_cbx, prefs.ENABLE_SUBTITLES)
         v.pack_start(widgetutil.align_left(subtitles_cbx, bottom_pad=6))
 
-        v.pack_start(widgetutil.align_left(playback_heading,
-                     left_pad=3, top_pad=6 , bottom_pad=6))
-
-        attach_radio([(stop_rb, True), (play_rb, False)],
-                     prefs.SINGLE_VIDEO_PLAYBACK_MODE)
-        v.pack_start(widgetutil.align_left(play_rb), padding=2)
-        v.pack_start(widgetutil.align_left(stop_rb))
-
-        v.pack_start(widgetutil.align_left(resume_heading,
+        v.pack_start(widgetutil.align_left(videos_heading,
                      left_pad=3, top_pad=12 , bottom_pad=6))
-
         attach_boolean(resume_videos_cbx, prefs.RESUME_VIDEOS_MODE)
-        attach_boolean(resume_music_cbx, prefs.RESUME_MUSIC_MODE)
-        attach_boolean(resume_podcasts_cbx, prefs.RESUME_PODCASTS_MODE)
         v.pack_start(widgetutil.align_left(resume_videos_cbx, bottom_pad=6))
+        attach_boolean(continue_videos_cbx, prefs.CONTINUOUS_VIDEO_PLAYBACK_MODE)
+        v.pack_start(widgetutil.align_left(continue_videos_cbx, bottom_pad=6))
+
+        v.pack_start(widgetutil.align_left(music_heading,
+                     left_pad=3, top_pad=12 , bottom_pad=6))
+        attach_boolean(resume_music_cbx, prefs.RESUME_MUSIC_MODE)
         v.pack_start(widgetutil.align_left(resume_music_cbx, bottom_pad=6))
+        attach_boolean(continue_music_cbx, prefs.CONTINUOUS_MUSIC_PLAYBACK_MODE)
+        v.pack_start(widgetutil.align_left(continue_music_cbx, bottom_pad=6))
+
+        v.pack_start(widgetutil.align_left(podcasts_heading,
+                     left_pad=3, top_pad=12 , bottom_pad=6))
+        attach_boolean(resume_podcasts_cbx, prefs.RESUME_PODCASTS_MODE)
         v.pack_start(widgetutil.align_left(resume_podcasts_cbx, bottom_pad=6))
+        attach_boolean(continue_podcasts_cbx, prefs.CONTINUOUS_PODCAST_PLAYBACK_MODE)
+        v.pack_start(widgetutil.align_left(continue_podcasts_cbx, bottom_pad=6))
 
         pack_extras(v, "playback")
 
@@ -1339,29 +1455,34 @@ class ExtensionsPanel(PanelBuilder):
         self.extensions_helper.load()
 
 # Add the initial panels
-add_panel("general", _("General"), GeneralPanel, 'images/pref_tab_general.png')
-add_panel("feeds", _("Podcasts"), PodcastsPanel, 'images/pref_tab_feeds.png')
-add_panel("downloads", _("Downloads"), DownloadsPanel,
+add_panel("general", gettext_lazy("General"), GeneralPanel,
+          'images/pref_tab_general.png')
+add_panel("feeds", gettext_lazy("Podcasts"), PodcastsPanel,
+          'images/pref_tab_feeds.png')
+add_panel("downloads", gettext_lazy("Downloads"), DownloadsPanel,
           'images/pref_tab_downloads.png')
-add_panel("folders", _("Folders"), FoldersPanel, 'images/pref_tab_folders.png')
-add_panel("disk_space", _("Disk space"), DiskSpacePanel,
+add_panel("folders", gettext_lazy("Folders"), FoldersPanel,
+          'images/pref_tab_folders.png')
+add_panel("disk_space", gettext_lazy("Disk space"), DiskSpacePanel,
           'images/pref_tab_disk_space.png')
-add_panel("playback", _("Playback"), PlaybackPanel,
+add_panel("playback", gettext_lazy("Playback"), PlaybackPanel,
           'images/pref_tab_playback.png')
-add_panel("sharing", _("Sharing"), SharingPanel, 'images/pref_tab_sharing.png')
-add_panel("conversions", _("Conversions"), ConversionsPanel,
+add_panel("sharing", gettext_lazy("Sharing"), SharingPanel,
+          'images/pref_tab_sharing.png')
+add_panel("conversions", gettext_lazy("Conversions"), ConversionsPanel,
           'images/pref_tab_conversions.png')
-add_panel("stores", _("Stores"), StoresPanel, 'images/pref_tab_stores.png')
-add_panel("extensions", _("Extensions"), ExtensionsPanel,
+add_panel("stores", gettext_lazy("Stores"), StoresPanel,
+          'images/pref_tab_stores.png')
+add_panel("extensions", gettext_lazy("Extensions"), ExtensionsPanel,
           'images/pref_tab_extensions.png')
 
 class PreferencesWindow(widgetset.PreferencesWindow):
     def __init__(self):
         widgetset.PreferencesWindow.__init__(self, _("Preferences"))
-        self.panel_builders = []
+        self.panel_builders = {}
         for name, title, image_name, panel_builder_class in _PANEL:
             panel_builder = panel_builder_class()
-            self.panel_builders.append(panel_builder)
+            self.panel_builders[name] = panel_builder
             panel = panel_builder.build_widget()
             alignment = widgetset.Alignment(xalign=0.5, yalign=0.5)
             alignment.set_padding(20, 20, 20, 20)
@@ -1370,6 +1491,9 @@ class PreferencesWindow(widgetset.PreferencesWindow):
 
         self.finish_panels()
 
+    def get_panel(self, name):
+        return self.panel_builders[name]
+
     def select_panel(self, selection):
         if selection is None:
             widgetset.PreferencesWindow.select_panel(self, 0)
@@ -1380,17 +1504,17 @@ class PreferencesWindow(widgetset.PreferencesWindow):
                     break
 
     def do_key_press(self, key, mods):
-        if key == menus.ESCAPE:
+        if key == keyboard.ESCAPE:
             self.close()
             return True
         return False
 
     def do_show(self):
-        for panel_builder in self.panel_builders:
+        for panel_builder in self.panel_builders.values():
             panel_builder.on_window_open()
 
     def do_hide(self):
-        for panel_builder in self.panel_builders:
+        for panel_builder in self.panel_builders.values():
             panel_builder.on_window_closed()
 
 _pref_window = None
@@ -1398,6 +1522,7 @@ _pref_window = None
 def show_window(selection=None):
     """Displays the preferences window."""
     global _pref_window
+
     if _pref_window is None:
         _pref_window = PreferencesWindow()
     _pref_window.select_panel(selection)
@@ -1408,3 +1533,17 @@ def is_window_shown():
 
 def hide_window():
     _pref_window.close()
+
+def enable_net_lookup_buttons():
+    global _pref_window
+    if _pref_window is not None:
+        _pref_window.get_panel('general').enable_net_lookup_buttons()
+
+_net_lookup_counts = (0, 0)
+def update_net_lookup_counts(net_lookup_count, total_count):
+    global _pref_window
+    global _net_lookup_counts
+
+    _net_lookup_counts = (net_lookup_count, total_count)
+    if _pref_window is not None:
+        _pref_window.get_panel('general').update_net_lookup_counts()
