@@ -49,15 +49,19 @@ class SearchManager(signals.SignalEmitter):
         signals.SignalEmitter.__init__(self)
         self.create_signal('search-started')
         self.create_signal('search-complete')
-        self.engine = searchengines.get_last_engine().name
+        self.engine = u'all'
         self.text = ''
         self.searching = False
 
+    def set_initial_search_info(self, engine, text):
+        self.engine = engine
+        self.text = text
+
     def set_search_info(self, engine, text):
         if not searchengines.get_engine_for_name(engine):
-            logging.warn('Manager asked to set engine to non-existent %s',
+            logging.info('Manager asked to set engine to non-existent %s.  '
+                         'Probably engine has been removed.',
                          engine)
-            self.perform_search(searchengines.get_last_engine().name, '')
             return
         self.engine = engine
         self.text = text
