@@ -29,9 +29,12 @@
 
 """Constants that define the look-and-feel."""
 
+import logging
 import math
 
+from miro import app
 from miro import displaytext
+from miro import util
 from miro.gtcache import gettext as _
 from miro.frontends.widgets import cellpack
 from miro.frontends.widgets import imagepool
@@ -291,7 +294,7 @@ class ConnectTabRenderer(TabRenderer):
             return hotspot_info[0]
 
 # Renderers for the list view
-class ListViewRendererText(widgetset.InfoListRendererText):
+class ListViewRendererText(widgetset.ItemListRendererText):
     """Renderer for list view columns that are just plain text"""
 
     bold = False
@@ -301,7 +304,7 @@ class ListViewRendererText(widgetset.InfoListRendererText):
     right_aligned = False
 
     def __init__(self):
-        widgetset.InfoListRendererText.__init__(self)
+        widgetset.ItemListRendererText.__init__(self)
         self.set_bold(self.bold)
         self.set_color(self.color)
         self.set_font_scale(self.font_size)
@@ -316,28 +319,57 @@ class DescriptionRenderer(ListViewRendererText):
     attr_name = 'description_oneline'
 
 class FeedNameRenderer(ListViewRendererText):
-    attr_name = 'feed_name'
+    attr_name = 'parent_title'
 
 class DateRenderer(ListViewRendererText):
-    attr_name = 'display_date'
+    min_width = 80
+
+    def get_value(self, info):
+        return displaytext.date_slashes(info.release_date)
 
 class LengthRenderer(ListViewRendererText):
-    attr_name = 'display_duration_short'
+    def get_value(self, info):
+        return displaytext.short_time_string(info.duration)
 
 class ETARenderer(ListViewRendererText):
     right_aligned = True
-    attr_name = 'display_eta'
+
+    def get_value(self, info):
+        return info.eta_text
 
 class TorrentDetailsRenderer(ListViewRendererText):
-    attr_name = 'display_torrent_details'
+    def get_value(self, info):
+        if not info.is_torrent:
+            return ''
+
+        details = _(
+            "S: %(seeders)s  |  "
+            "L: %(leechers)s  |  "
+            "UR: %(up_rate)s  |  "
+            "UT: %(up_total)s  |  "
+            "DR: %(down_rate)s  |  "
+            "DT: %(down_total)s  |  "
+            "R: %(upload_ratio)s",
+            {"seeders": info.seeders,
+             "leechers": info.leechers,
+             "up_rate": info.upload_rate_text,
+             "up_total": info.upload_size_text,
+             "down_rate": info.download_rate_text,
+             "down_total": info.downloaded_size_text,
+             "ratio": info.upload_ratio_text})
+        return details
 
 class DownloadRateRenderer(ListViewRendererText):
     right_aligned = True
-    attr_name = 'display_rate'
+
+    def get_value(self, info):
+        return info.download_rate_text
 
 class SizeRenderer(ListViewRendererText):
     right_aligned = True
-    attr_name = 'display_size'
+    min_width = 60
+    def get_value(self, info):
+        return displaytext.size_string(info.size)
 
 class ArtistRenderer(ListViewRendererText):
     attr_name = 'artist'
@@ -346,22 +378,28 @@ class AlbumRenderer(ListViewRendererText):
     attr_name = 'album'
 
 class TrackRenderer(ListViewRendererText):
-    attr_name = 'display_track'
+    def get_value(self, info):
+        return displaytext.integer(info.track)
 
 class YearRenderer(ListViewRendererText):
-    attr_name = 'display_year'
+    def get_value(self, info):
+        return displaytext.integer(info.year)
 
 class GenreRenderer(ListViewRendererText):
     attr_name = 'genre'
 
 class DateAddedRenderer(ListViewRendererText):
-    attr_name = 'display_date_added'
+    min_width = 90
+    def get_value(self, info):
+        return displaytext.date_slashes(info.date_added)
 
 class LastPlayedRenderer(ListViewRendererText):
-    attr_name = 'display_last_played'
+    def get_value(self, info):
+        return displaytext.date_slashes(info.last_played)
 
 class DRMRenderer(ListViewRendererText):
-    attr_name = 'display_drm'
+    def get_value(self, info):
+        return _("Locked") if info.has_drm else u""
 
 class FileTypeRenderer(ListViewRendererText):
     attr_name = 'file_format'
@@ -370,19 +408,29 @@ class ShowRenderer(ListViewRendererText):
     attr_name = 'show'
 
 class KindRenderer(ListViewRendererText):
-    attr_name = 'display_kind'
+    def get_value(self, info):
+        if info.kind == 'movie':
+            return _("Movie")
+        elif info.kind == 'show':
+            return _("Show")
+        elif info.kind == 'clip':
+            return _("Clip")
+        elif info.kind == 'podcast':
+            return _("Podcast")
+        else:
+            return None
 
 class PlaylistOrderRenderer(ListViewRendererText):
     """Displays the order an item is in a particular playlist.
     """
-    def __init__(self, playlist_sorter):
+    def __init__(self, playlist_order):
         ListViewRendererText.__init__(self)
-        self.playlist_sorter = playlist_sorter
+        self.playlist_order = playlist_order
 
     def get_value(self, info):
-        return str(self.playlist_sorter.sort_key(info) + 1)
+        return str(self.playlist_order.item_position(info))
 
-class ListViewRenderer(widgetset.InfoListRenderer):
+class ListViewRenderer(widgetset.ItemListRenderer):
     """Renderer for more complex list view columns.
 
     This class is useful for renderers that use the cellpack.Layout class.
@@ -441,7 +489,7 @@ class NameRenderer(ListViewRenderer):
         # make a Layout Object
         layout = cellpack.Layout()
         # add the text
-        textbox = layout_manager.textbox(self.info.name)
+        textbox = layout_manager.textbox(self.info.title)
         textbox.set_wrap_style('truncated-char')
         # 4px here is half of ListView.COLUMN_PADDING - 2px for luck
         layout.add_text_line(textbox, 4, 0, width)
@@ -467,8 +515,7 @@ class StatusRenderer(ListViewRenderer):
     def layout_all(self, layout_manager, width, height, selected):
         # add the button, if needed
         if self.should_show_download_button():
-            layout = cellpack.Layout()
-
+            layout = self.layout_text(layout_manager, width, height)
             button = self.make_button(layout_manager)
             button_x = width - button.get_size()[0]
             layout.add_image(button, button_x, 0, hotspot='download')
@@ -476,8 +523,7 @@ class StatusRenderer(ListViewRenderer):
             layout.center_y(top=0, bottom=height)
             return layout
 
-        if (self.info.state in ('downloading', 'paused') and
-            self.info.download_info.state != 'pending'):
+        if self.info.is_download and not self.info.pending_manual_download:
             return self.layout_progress(layout_manager, width, height)
         else:
             return self.layout_text(layout_manager, width, height)
@@ -493,16 +539,16 @@ class StatusRenderer(ListViewRenderer):
         return button
 
     def should_show_download_button(self):
-        nonlocal = self.info.device or self.info.remote
-        return ((not self.info.downloaded and
-                self.info.state not in ('downloading', 'paused')) or nonlocal)
+        can_sync = self.info.device or self.info.remote
+        can_download = not self.info.downloaded and not self.info.is_download
+        return can_sync or can_download
 
     def layout_progress(self, layout_manager, width, height):
         """Handle layout when we should display a progress bar """
 
         layout = cellpack.Layout()
         # add left button
-        if self.info.state == 'downloading':
+        if not self.info.is_paused:
             left_button = 'pause'
         else:
             left_button = 'resume'
@@ -549,19 +595,15 @@ class StatusRenderer(ListViewRenderer):
                     text = displaytext.expiration_date_short(
                             self.info.expiration_date)
                     return (text, EXPIRING_TEXT_COLOR)
-        elif (self.info.download_info and
-                self.info.download_info.rate == 0):
-            if self.info.download_info.state == 'paused':
-                return (_('paused'), DOWNLOADING_COLOR)
-            elif self.info.download_info.state == 'pending':
-                return (_('queued'), DOWNLOADING_COLOR)
-            elif self.info.download_info.state == 'failed':
-                return (self.info.download_info.short_reason_failed,
-                        DOWNLOADING_COLOR)
-            else:
-                return (self.info.download_info.startup_activity,
-                        DOWNLOADING_COLOR)
-        elif not self.info.item_viewed:
+        elif self.info.is_paused:
+            return (_('paused'), DOWNLOADING_COLOR)
+        elif self.info.pending_manual_download:
+            return (_('queued'), DOWNLOADING_COLOR)
+        elif self.info.is_failed_download:
+            return (self.info.short_reason_failed, ERROR_COLOR)
+        elif self.info.is_download and self.info.rate is None:
+            return (self.info.startup_activity, DOWNLOADING_COLOR)
+        elif self.info.new:
             return (_('Newly Available'), AVAILABLE_COLOR)
         return ('', self.default_text_color)
 
@@ -570,8 +612,7 @@ class StatusRenderer(ListViewRenderer):
 
         if self.info.expiration_date:
             button_name = 'keep'
-        elif (self.info.state == 'downloading' and
-              self.info.download_info.state == 'pending'):
+        elif self.info.pending_manual_download:
             button_name = 'cancel'
         else:
             return
@@ -579,7 +620,7 @@ class StatusRenderer(ListViewRenderer):
         button_x = width - button.width # right-align
         layout.add_image(button, button_x, 0, hotspot=button_name)
 
-class RatingRenderer(widgetset.InfoListRenderer):
+class RatingRenderer(widgetset.ItemListRenderer):
     """Render ratings column
 
     This cell supports updating based on hover states and rates items based on
@@ -594,7 +635,7 @@ class RatingRenderer(widgetset.InfoListRenderer):
     ICON_COUNT = 5
 
     def __init__(self):
-        widgetset.InfoListRenderer.__init__(self)
+        widgetset.ItemListRenderer.__init__(self)
         self.want_hover = True
         self.icon = {}
         # TODO: to support scaling, we need not to check min_height until after
@@ -606,7 +647,9 @@ class RatingRenderer(widgetset.InfoListRenderer):
             path = resources.path('images/star-%s.png' % state)
             self.icon[state] = imagepool.get_surface(path,
                                (self.icon_width, self.icon_height))
-        self.min_width = self.width = int(self.icon_width * self.ICON_COUNT)
+        self.min_width = self.width = int((self.icon_width +
+                                           self.ICON_HORIZONTAL_SPACING) *
+                                          self.ICON_COUNT)
         self.hover = None
 
     def hotspot_test(self, style, layout_manager, x, y, width, height):
@@ -679,7 +722,7 @@ class RatingRenderer(widgetset.InfoListRenderer):
                 state = 'unset'
         return self.icon[state]
 
-class StateCircleRenderer(widgetset.InfoListRenderer):
+class StateCircleRenderer(widgetset.ItemListRenderer):
     """Renderer for the state circle column."""
 
     # NOTE: we don't inherit from ListViewRenderer because we handle
@@ -691,25 +734,11 @@ class StateCircleRenderer(widgetset.InfoListRenderer):
     min_height = 9
 
     def __init__(self):
-        widgetset.InfoListRenderer.__init__(self)
+        widgetset.ItemListRenderer.__init__(self)
         self.icon = {}
-        self.setup_size = (-1, -1)
-
-    def setup_icons(self, width, height):
-        """Create icons that will fill our allocated area correctly. """
-        if (width, height) == self.setup_size:
-            return
-
-        icon_width = int(height / 2.0)
-        icon_height = int((icon_width / self.ICON_PROPORTIONS) + 0.5)
-        # FIXME: by the time min_width is set below, it doesn't matter --Kaz
-        self.width = self.min_width = icon_width
-        self.height = icon_height
-        icon_dimensions = (icon_width, icon_height)
         for state in StateCircleRenderer.ICON_STATES:
             path = resources.path('images/status-icon-%s.png' % state)
-            self.icon[state] = imagepool.get_surface(path, icon_dimensions)
-        self.setup_size = (width, height)
+            self.icon[state] = imagepool.get_surface(path)
 
     def get_size(self, style, layout_manager):
         return self.min_width, self.min_height
@@ -718,34 +747,366 @@ class StateCircleRenderer(widgetset.InfoListRenderer):
         return None
 
     def render(self, context, layout_manager, selected, hotspot, hover):
-        self.setup_icons(context.width, context.height)
         icon = self.calc_icon()
+        if icon is None:
+            return
         # center icon vertically and horizontally
-        x = int((context.width - self.width) / 2)
-        y = int((context.height - self.height) / 2)
-        if icon:
-            icon.draw(context, x, y, icon.width, icon.height)
+        x = (context.width - icon.width) // 2
+        y = (context.height - icon.height) // 2
+        icon.draw(context, x + 1, y, icon.width, icon.height)
 
     def calc_icon(self):
         """Get the icon we should show.
 
         :returns: ImageSurface to display
         """
-        if self.info.state == 'downloading':
+        if self.info.is_download:
             return self.icon['downloading']
-        elif self.info.is_playing:
+        elif app.playback_manager.is_playing_item(self.info):
             return self.icon['playing']
-        elif self.info.state == 'newly-downloaded':
+        elif self.info.new:
+            return self.icon['new']
+        elif self.info.downloaded and not self.info.video_watched:
             return self.icon['unplayed']
-        elif (self.info.downloaded and self.info.is_playable and
-              not self.info.video_watched):
-            return self.icon['new']
-        elif (not self.info.item_viewed and not self.info.expiration_date and
-                not self.info.is_external and not self.info.downloaded):
-            return self.icon['new']
         else:
             return None
 
+class _MultiRowAlbumRenderStrategy(object):
+    """Utility class that controls what we render in MultiRowAlbumRenderer
+
+    We subclass for each of the different modes that we use (standard, feed,
+    videos, etc).
+
+    This class is just used internally.
+    """
+
+    def get_image_path(self, item_info, first_info):
+        """Get a path to the image we should draw."""
+        raise NotImplementedError()
+
+    def get_album(self, item_info, first_info):
+        """Get album name to render."""
+        raise NotImplementedError()
+
+    def get_artist(self, item_info, first_info):
+        """Get artist name to render."""
+        raise NotImplementedError()
+
+    def get_track_number(self, item_info, first_info):
+        """Track number to show"""
+        raise NotImplementedError()
+
+class _StandardRenderStrategy(_MultiRowAlbumRenderStrategy):
+    def get_image_path(self, item_info, first_info):
+        if first_info.cover_art_path is not None:
+            return first_info.cover_art_path
+        else:
+            return first_info.thumbnail
+
+    def get_album(self, item_info, first_info):
+        return item_info.album
+
+    def get_artist(self, item_info, first_info):
+        return item_info.artist
+
+    def get_track_number(self, item_info, first_info):
+        if item_info.track is not None:
+            return item_info.track
+        else:
+            return ''
+
+class _FeedRenderStrategy(_MultiRowAlbumRenderStrategy):
+    def get_image_path(self, item_info, first_info):
+        try:
+            feed_info = widgetutil.get_feed_info(item_info.feed_id)
+        except KeyError:
+            return first_info.thumbnail
+        else:
+            return feed_info.thumbnail
+
+    def get_album(self, item_info, first_info):
+        return ''
+
+    def get_artist(self, item_info, first_info):
+        return item_info.parent_title
+
+    def get_track_number(self, item_info, first_info):
+        return ''
+
+class _VideoRenderStrategy(_MultiRowAlbumRenderStrategy):
+    def get_image_path(self, item_info, first_info):
+        return first_info.thumbnail
+
+    def get_album(self, item_info, first_info):
+        if item_info.show:
+            return item_info.show
+        elif item_info.parent_title:
+            return item_info.parent_title
+        else:
+            return None
+
+    def get_artist(self, item_info, first_info):
+        return None
+
+    def get_track_number(self, item_info, first_info):
+        return ''
+
+class MultiRowAlbumRenderer(widgetset.ItemListRenderer):
+    """Renderer for album view."""
+
+    IGNORE_PADDING = True
+    DRAW_BACKGROUND = False
+
+    IMAGE_MARGIN_TOP = 4
+    IMAGE_MARGIN_BOTTOM = 3
+    IMAGE_MARGIN_LEFT = 7
+    IMAGE_MARGIN_RIGHT = 6
+
+    MIN_TEXT_WIDTH = 78
+    TEXT_PADDING_RIGHT = 6
+    TRACK_NUMBER_MARGIN_RIGHT = 13
+
+    BACKGROUND_COLOR = widgetutil.WHITE
+    TEXT_COLOR = widgetutil.BLACK
+    TRACK_TEXT_COLOR = widgetutil.css_to_color('#969696')
+    BOTTOM_LINE_COLOR = widgetutil.css_to_color('#dddddd')
+    FONT_SIZE = widgetutil.font_scale_from_osx_points(11)
+
+    min_width = 260
+
+    def __init__(self):
+        widgetset.ItemListRenderer.__init__(self)
+        self._render_strategy = _StandardRenderStrategy()
+        self._setup_default_image_map()
+
+    def _setup_default_image_map(self):
+        """Setup the _default_image_map attribute.
+
+        _default_image_map maps the default images for things to a default
+        image that looks better in album view.
+        """
+        # check if we're using one of the default image files and switch to an
+        # album-view-specific default file in that case
+        mappings = [
+            ('thumb-default-audio.png', 'album-view-default-audio.png'),
+            ('thumb-default-video.png', 'album-view-default-video.png'),
+            ('icon-podcast-small.png', 'album-view-default-podcast.png'),
+            ('icon-watched-folder.png', 'album-view-watched-folder.png'),
+        ]
+        self._default_image_map = {}
+        for src, dest in mappings:
+            src_path = resources.path('images/%s' % src)
+            dest_path = resources.path('images/%s' % dest)
+            self._default_image_map[src_path] = dest_path
+
+    def get_image_path(self):
+        image_path = self._render_strategy.get_image_path(
+            self.info, self.get_first_info())
+        if image_path in self._default_image_map:
+            return self._default_image_map[image_path]
+        else:
+            return image_path
+
+    def get_album(self):
+        return self._render_strategy.get_album(self.info,
+                                               self.get_first_info())
+
+    def get_artist(self):
+        return self._render_strategy.get_artist(self.info,
+                                                self.get_first_info())
+
+    def get_track_number(self):
+        return self._render_strategy.get_track_number(self.info,
+                                                      self.get_first_info())
+
+    def get_current_row(self):
+        return self.group_info[0]
+
+    def get_total_rows(self):
+        return self.group_info[1]
+
+    def get_first_info(self):
+        return self.group_info[2]
+
+    def switch_mode(self, new_mode):
+        """Switch which mode we use to render the album art.
+
+        Currently there are 3 modes:
+
+        - 'standard' -- standard view of the data
+        - 'feed' -- use feed info instead of album info
+        - 'video' -- mode for the all videos tab
+        """
+        if new_mode == 'standard':
+            self._render_strategy = _StandardRenderStrategy()
+        elif new_mode == 'feed':
+            self._render_strategy = _FeedRenderStrategy()
+        elif new_mode == 'video':
+            self._render_strategy = _VideoRenderStrategy()
+        else:
+            raise ValueError("Unknown mode: %s" % new_mode)
+
+    def get_size(self, style, layout_manager):
+        # return 0 for height because we render to multiple columns.  We let
+        # the other columns determin the row height
+        return self.min_width, 0
+
+    def hotspot_test(self, style, layout_manager, x, y, width, height):
+        return 'album-click'
+
+    def render(self, context, layout_manager, selected, hotspot, hover):
+        if not self.sanity_check_before_render(context):
+            return
+
+        # draw our background color behind everything.  We need this in case
+        # there's transparency in our album art
+        context.set_color(self.BACKGROUND_COLOR)
+        context.rectangle(0, 0, context.width, context.height)
+        context.fill()
+
+        self.calc_album_art_size(context)
+        self.render_album_art(context)
+        self.render_track_number(context, layout_manager)
+        self.render_album_or_artist(context, layout_manager)
+
+    def sanity_check_before_render(self, context):
+        """Do some sanity checking before starting to render things.
+
+        Returns True if we're okay to render, False if we should bail
+        """
+        if self.group_info is None:
+            # we can't render if group_info isn't set
+            logging.warn("group_info is None in MultiRowAlbumRenderer")
+            return False
+        if context.height == 0:
+            # not sure how this would happen, but we need to avoid
+            # divide-by-zero errors if it does
+            logging.warn("row height is 0 in MultiRowAlbumRenderer")
+            return False
+        return True
+
+    def calc_album_art_size(self, context):
+        """Calculate how big we are going to draw album art.
+
+        This is currently big enough so it fits in 6 rows with the top/bottom
+        padding.
+        """
+        self.album_art_size = context.height * 6
+        self.album_art_size -= (self.IMAGE_MARGIN_TOP +
+                self.IMAGE_MARGIN_BOTTOM)
+
+    def make_album_art(self, context):
+        """Make an image to draw as album art.
+
+        Returns ImageSurface to draw or None if we don't have anything
+        """
+        if self.get_total_rows() < 6:
+            # don't draw album art if we have less than 6 items in the group
+            return None
+
+        album_art_path = self.get_image_path()
+        if album_art_path is None:
+            return None
+        return imagepool.get_surface(album_art_path,
+                size=(self.album_art_size, self.album_art_size),
+                                     invalidator=util.mtime_invalidator(
+                album_art_path))
+
+    def render_album_art(self, context):
+        album_art = self.make_album_art(context)
+        if (album_art is not None and
+                self.cell_contains_album_art(context, album_art)):
+            self.render_album_art_slice(context, album_art)
+
+    def cell_contains_album_art(self, context, album_art):
+        """Does this cell contain a portion of the album art?
+        """
+        album_art_bottom = album_art.height + self.IMAGE_MARGIN_TOP
+        cell_top = self.get_current_row() * context.height
+        cell_bottom = cell_top + context.height
+        return (cell_bottom > self.IMAGE_MARGIN_TOP and
+                cell_top < album_art_bottom)
+
+    def render_album_art_slice(self, context, image):
+        """Render the slice of the album art for this cell."""
+
+        if context.width < image.width:
+            # not enough width to draw
+            return
+
+        # setup variables to track where we are copying from and to
+
+        dest_x = self.IMAGE_MARGIN_LEFT
+        width = image.width
+
+        dest_y = 0
+        height = context.height
+
+        src_x = 0
+        src_y = self.get_current_row() * context.height - self.IMAGE_MARGIN_TOP
+
+        if src_y < 0:
+            # The cell is contains the top padding for our image.
+            # move dest_y and src_y down
+            dest_y -= src_y
+            src_y = 0
+            # descrease height
+            height -= dest_y
+        src_y_bottom = src_y + height
+        if src_y_bottom > image.height:
+            # The cell is contains the bottom padding for our image.
+            # decrease height
+            extra_space = src_y_bottom - image.height
+            height -= extra_space
+        # draw our image slice
+        if height > 0:
+            image.draw_rect(context, dest_x, dest_y, src_x, src_y,
+                    width, height)
+
+    def render_album_or_artist(self, context, layout_manager):
+        x = (self.album_art_size + self.IMAGE_MARGIN_LEFT +
+                self.IMAGE_MARGIN_RIGHT)
+        if self.get_current_row() == 0:
+            text = self.get_artist()
+            bold = True
+        elif self.get_current_row() == 1:
+            text = self.get_album()
+            bold = False
+        else:
+            return
+
+        width = self.album_artist_text_end - x
+        if width < 10:
+            # don't try to render if we have a really small, or negative
+            # amount of space
+            return
+        # setup a textbox for the text
+        layout_manager.set_font(self.FONT_SIZE, bold=bold)
+        layout_manager.set_text_color(self.TEXT_COLOR)
+        textbox = layout_manager.textbox(text)
+        # truncate the textbox to the area we have in a cell.
+        textbox.set_wrap_style('truncated-char')
+        textbox.set_width(width)
+        # middle-align the text to line-up with the other cells
+        line_height = textbox.font.line_height()
+        y = (context.height - line_height) / 2.0
+        # okay, ready to draw
+        textbox.draw(context, x, y, width, line_height)
+
+    def render_track_number(self, context, layout_manager):
+        # setup a textbox for the text
+        layout_manager.set_font(self.FONT_SIZE)
+        layout_manager.set_text_color(self.TRACK_TEXT_COLOR)
+        textbox = layout_manager.textbox(str(self.get_track_number()))
+        # place the text on the right-side of the cell
+        text_width, text_height = textbox.get_size()
+        x = context.width - self.TEXT_PADDING_RIGHT - text_width
+        # middle-align the text to line-up with the other cells
+        y = (context.height - text_height) // 2
+        # okay, ready to draw
+        textbox.draw(context, x, y, text_width, text_height)
+        self.album_artist_text_end = x - self.TEXT_PADDING_RIGHT
+
 class ProgressBarColorSet(object):
     PROGRESS_BASE_TOP = (0.92, 0.53, 0.21)
     PROGRESS_BASE_BOTTOM = (0.90, 0.45, 0.08)
@@ -902,9 +1263,5 @@ class ProgressBarDrawer(cellpack.Packer):
 
 class ItemProgressBarDrawer(ProgressBarDrawer):
     def __init__(self, info):
-        ProgressBarDrawer.__init__(self, 0, ProgressBarColorSet)
-        if info.download_info and info.size > 0.0:
-            self.progress_ratio = (float(info.download_info.downloaded_size) /
-                    info.size)
-        else:
-            self.progress_ratio = 0.0
+        ProgressBarDrawer.__init__(self, info.download_progress,
+                                   ProgressBarColorSet)
