@@ -29,7 +29,7 @@
 
 """Displays the list of tabs on the left-hand side of the app."""
 
-from __future__ import with_statement # python2.5
+import collections
 import itertools
 from hashlib import md5
 try:
@@ -47,12 +47,12 @@ from miro import messages
 from miro import errors
 from miro.gtcache import gettext as _
 from miro.plat import resources
-from miro.frontends.widgets import playback
-from miro.frontends.widgets import style
 from miro.frontends.widgets import imagepool
+from miro.frontends.widgets import keyboard
+from miro.frontends.widgets import playback
 from miro.frontends.widgets import statictabs
+from miro.frontends.widgets import style
 from miro.frontends.widgets import widgetutil
-from miro.frontends.widgets import menus
 from miro.frontends.widgets.tablistdnd import (FeedListDragHandler,
      FeedListDropHandler, PlaylistListDragHandler, PlaylistListDropHandler,
      MediaTypeDropHandler, DeviceDropHandler)
@@ -181,7 +181,7 @@ class TabUpdaterMixin(object):
             del self.updating_animations[id_]
             return
         self.view.pulse_updating_image(iter_)
-        timer_id = timer.add(0.1, self.pulse_updating_animation, id_)
+        timer_id = timer.add(0.5, self.pulse_updating_animation, id_)
         self.updating_animations[id_] = timer_id
 
 class TabBlinkerMixin(object):
@@ -323,14 +323,14 @@ class TabList(signals.SignalEmitter):
         self.view.connect_weak('row-activated', self.on_row_activated)
 
     def on_key_press(self, view, key, mods):
-        if key == menus.DOWN_ARROW and len(mods) == 0:
+        if key == keyboard.DOWN_ARROW and len(mods) == 0:
             # Test if the user is trying to move down past the last row in the
             # table, if so, select the next tablist.
             if view.is_selected(_last_iter(view, view.model)):
                 if self._move_to_next_tablist():
                     return True
             return False
-        elif key == menus.UP_ARROW and len(mods) == 0:
+        elif key == keyboard.UP_ARROW and len(mods) == 0:
             # Test if the user is trying to move up past the first row in the
             # table, if so, select the next tablist.
             if view.is_selected(view.model.first_iter()):
@@ -612,9 +612,12 @@ class HideableTabList(TabList):
         self._after_change(True)
 
     def on_key_press(self, view, key, mods):
-        if key == menus.DELETE or key == menus.BKSPACE:
+        if key == keyboard.DELETE or key == keyboard.BKSPACE:
             self.on_delete_key_pressed()
             return True
+        elif key == keyboard.F5:
+            app.widgetapp.update_selected_feeds()
+            return True
         return TabList.on_key_press(self, view, key, mods)
 
     def on_delete_key_pressed(self):
@@ -626,7 +629,7 @@ class HideableTabList(TabList):
                 self.expand(self.info.id)
                 self.view.model_changed()
 
-    def on_row_expanded_change(self, view, iter_, path, expanded):
+    def on_row_expanded_change(self, view, iter_, expanded):
         info = self.view.model[iter_][0]
         if info == self.info:
             message = messages.TabExpandedChange(self.type, expanded)
@@ -673,7 +676,7 @@ class HideableTabList(TabList):
             self.emit('tab-name-changed', old_name, info.name)
 
     def remove(self, id_list):
-        deleted_ids = []
+        deleted_ids = set(id_list)
         with self.removing():
             for id_ in id_list:
                 try:
@@ -682,9 +685,9 @@ class HideableTabList(TabList):
                     # child of a tab we already deleted
                     continue
                 # override default of nil
-                deleted_ids = self.forget_child_iters(iter_)
+                deleted_ids.update(self.forget_child_iters(iter_))
                 self.view.model.remove(iter_)
-        if set(deleted_ids + id_list).intersection(set(app.tabs.selected_ids)):
+        if deleted_ids.intersection(set(app.tabs.selected_ids)):
             # hack for 17653: on OS X, deleting the selected tab doesn't
             # send selection-changed - so if the tab(s) we've 
             # deleted is selected, explicitly change the selection to
@@ -695,16 +698,16 @@ class HideableTabList(TabList):
     def forget_child_iters(self, parent_iter):
         model = self.view.model
         iter_ = model.child_iter(parent_iter)
-        deleted_ids = []
+        deleted_ids = set()
         while iter_ is not None:
             # No need to wrap these around a try...except.  If we can get
             # the child iter (from above) then this must exist.  If it doesn't
             # then there is some sort of internal inconsistency.
             id_ = model[iter_][0].id
             del self.iter_map[id_]
-            deleted_ids += self.forget_child_iters(iter_)
+            deleted_ids.update(self.forget_child_iters(iter_))
             iter_ = self.view.model.remove(iter_)
-            deleted_ids.append(id_)
+            deleted_ids.add(id_)
         return deleted_ids
 
     def model_changed(self):
@@ -728,6 +731,8 @@ class HideableTabList(TabList):
 class DeviceTabListHandler(object):
     def __init__(self, tablist):
         self.tablist = tablist
+        # map device ids to the fake tabs that we created for them.
+        self.fake_tabs_created = {}
 
     def _fake_info(self, info, typ, name):
         new_data = {
@@ -753,17 +758,29 @@ class DeviceTabListHandler(object):
                 self._fake_info(info, 'audio', _('Music'))]
 
     def _add_fake_tabs(self, info):
+        if info.id in self.fake_tabs_created:
+            # fake tabs already added
+            return
+
+        fake_tabs = self._get_fake_infos(info)
+        self.fake_tabs_created[info.id] = fake_tabs
         with self.tablist.adding():
-            for fake in self._get_fake_infos(info):
-                HideableTabList.add(self.tablist,
-                                    fake,
-                                    info.id)
+            for fake in fake_tabs:
+                HideableTabList.add(self.tablist, fake, info.id)
             try:
                 self.tablist.expand(info.id)
             except errors.WidgetActionError:
                 pass # if the Connect Tab isn't open, we can't expand the tab
 
-    def add(self, info):
+    def _remove_fake_tabs(self, info):
+        if info.id not in self.fake_tabs_created:
+            # fake tabs already removed
+            return
+        fake_tabs = self.fake_tabs_created[info.id]
+        self.tablist.remove([fake_tab.id for fake_tab in fake_tabs])
+        del self.fake_tabs_created[info.id]
+
+    def add(self, info, parent_id):
         HideableTabList.add(self.tablist, info)
         if info.mount and not info.info.has_multiple_devices:
             self._add_fake_tabs(info)
@@ -816,18 +833,130 @@ class DeviceTabListHandler(object):
 
     def on_hotspot_clicked(self, view, hotspot, iter_):
         if hotspot == 'eject-device':
-            info = view.model[iter_][0]
-            messages.DeviceEject(info).send_to_backend()
+            self.eject_device(view.model[iter_][0])
+
+    def eject_device(self, device_info):
+        # stop playback from the device
+        currently_playing = app.playback_manager.get_playing_item()
+        if currently_playing and currently_playing.device == device_info:
+            app.playback_manager.stop()
+        # navigate away from the audio/video tabs and remove them
+        self._remove_fake_tabs(device_info)
+        messages.DeviceEject(device_info).send_to_backend()
+
+class FakeSharingInfo(object):
+    """TabInfo that we use for the "fake" tabs under a share
+
+    This includes the audio, video, podcast, and playlist tabs.
+    """
+    def __init__(self, share_id, tab_name, label):
+        self.id = u"sharing-%s-%s" % (share_id, tab_name)
+        self.share_id = share_id
+        self.name = label
+        self.type = u'sharing-fake-playlist'
+        self.icon = widgetutil.make_surface("icon-%s" % tab_name)
+        self.active_icon = widgetutil.make_surface("icon-%s_active" % tab_name)
 
 class SharingTabListHandler(object):
+    """Handles all of the sharing tabs """
     def __init__(self, tablist):
         self.tablist = tablist
+        # map share ids to the fake tabs for them
+        self.fake_tabs_created = {}
+        # before the fake tabs are created, we can't add any playlist tabs.
+        # pending_playlist_tabs maps share ids to those playlist tabs.
+        self.pending_playlist_tabs = collections.defaultdict(list)
+
+    def _make_fake_tabs(self, info):
+        return [
+            FakeSharingInfo(info.share_id, u'video', _('Video')),
+            FakeSharingInfo(info.share_id, u'audio', _('Audio')),
+            FakeSharingInfo(info.share_id, u'playlist', _('Playlists')),
+            FakeSharingInfo(info.share_id, u'podcast', _('Podcasts')),
+        ]
+
+    def _add_fake_tabs(self, info):
+        """Add the psuedo-tabs below the share tabs.
+
+        This includes the audio, video, playlists, and podcasts tabs.
+        """
+        if info.share_id in self.fake_tabs_created:
+            # fake tabs already added
+            return
+        self.fake_tabs_created[info.share_id] = []
+        with self.tablist.adding():
+            for tab in self._make_fake_tabs(info):
+                self.tablist.add(tab, info.id)
+                self.fake_tabs_created[info.share_id].append(tab)
+        self._add_pending_playlist_tabs(info.share_id)
+        try:
+            self.tablist.expand(info.id)
+        except errors.WidgetActionError:
+            pass # if the Connect Tab isn't open, we can't expand the tab
+
+    def _remove_fake_tabs(self, info):
+        if info.share_id not in self.fake_tabs_created:
+            # fake tabs already removed
+            return
+        fake_tabs = self.fake_tabs_created[info.share_id]
+        self.tablist.remove([fake_tab.id for fake_tab in fake_tabs])
+        del self.fake_tabs_created[info.share_id]
+
+    def _add_pending_playlist_tabs(self, share_id):
+        if share_id not in self.pending_playlist_tabs:
+            return
+
+        pending_tabs = self.pending_playlist_tabs.pop(share_id)
+        with self.tablist.adding():
+            for tab in pending_tabs:
+                self.add_playlist_tab(tab)
+
+    def add(self, info, parent_id):
+        # Need to avoid calling ConnectList.add since that will result in an
+        # infinite loop.
+        if isinstance(info, messages.SharingInfo):
+            self.add_sharing_tab(info)
+        else:
+            self.add_playlist_tab(info)
+
+    def add_sharing_tab(self, info):
+        HideableTabList.add(self.tablist, info)
+        self._handle_sharing_info_change(info)
+
+    def add_playlist_tab(self, info):
+        if info.share_id in self.fake_tabs_created:
+            # fake tabs are created, we can add this info now
+            parent_id = info.share_id
+            if info.podcast:
+                parent_id = "sharing-%s-podcast" % (info.share_id,)
+            else:
+                parent_id = "sharing-%s-playlist" % (info.share_id,)
+            HideableTabList.add(self.tablist, info, parent_id)
+        else:
+            # fake tabs not created yet, wait to add the info
+            self.pending_playlist_tabs[info.share_id].append(info)
+
+    def update(self, info):
+        # Need to avoid calling ConnectList.update since that will result in an
+        # infinite loop.
+        HideableTabList.update(self.tablist, info)
+        if isinstance(info, messages.SharingInfo):
+            self._handle_sharing_info_change(info)
+
+    def _handle_sharing_info_change(self, info):
+        if info.is_updating:
+            self.tablist.start_updating(info.id)
+        else:
+            self.tablist.stop_updating(info.id)
+        if info.mount:
+            self._add_fake_tabs(info)
+        else:
+            self._remove_fake_tabs(info)
 
     def on_hotspot_clicked(self, view, hotspot, iter_):
         if hotspot == 'eject-device':
             # Don't track this tab anymore for music.
             info = view.model[iter_][0]
-            info.mount = False
             # We must stop the playback if we are playing from the same
             # share that we are ejecting from.
             host = info.host
@@ -843,49 +972,42 @@ class SharingTabListHandler(object):
             if typ == u'connect' and (info == selected_tabs[0] or
               getattr(selected_tabs[0], 'parent_id', None) == info.id):
                 app.tabs.select_guide()
-            messages.SharingEject(info).send_to_backend()
-
-    def update(self, info):
-        if info.is_updating:
-            self.tablist.start_updating(info.id)
-        else:
-            self.tablist.stop_updating(info.id)
-        HideableTabList.update(self.tablist, info)
+            messages.StopTrackingShare(info.share_id).send_to_backend()
 
     def init_info(self, info):
-        info.type = u'sharing'
         info.unwatched = info.available = 0
-        active = None
-        if info.is_folder and info.playlist_id is None:
-            thumb_path = resources.path('images/sharing.png')
-        # Checking the name instead of a supposedly unique id is ok for now
-        # because
-        elif info.playlist_id == u'video':
-            thumb_path = resources.path('images/icon-video.png')
-            active = resources.path('images/icon-video_active.png')
-            info.name = _('Video')
-        elif info.playlist_id == u'audio':
-            thumb_path = resources.path('images/icon-audio.png')
-            active = resources.path('images/icon-audio_active.png')
-            info.name = _('Music')
-        elif info.playlist_id == u'playlist':
-            thumb_path = resources.path('images/icon-playlist.png')
-            active = resources.path('images/icon-playlist_active.png')
-            info.name = _('Playlists')
-        elif info.playlist_id == u'podcast':
-            thumb_path = resources.path('images/icon-podcast.png')
-            active = resources.path('images/icon-podcast_active.png')
-            info.name = _('Podcasts')
+        if isinstance(info, messages.SharingInfo):
+            self.init_share_info(info)
         else:
-            if info.podcast:
-                thumb_path = resources.path('images/icon-podcast-small.png')
-                active = resources.path('images/icon-podcast-small_active.png')
-            else:
-                thumb_path = resources.path('images/icon-playlist-small.png')
-                active = resources.path('images/icon-playlist-small_active.png')
+            self.init_playlist_info(info)
+
+    def init_share_info(self, info):
+        info.type = u'sharing'
+        info.icon = imagepool.get_surface(
+            resources.path('images/sharing.png'))
+
+    def init_playlist_info(self, info):
+        info.type = u'sharing-playlist'
+        if info.podcast:
+            thumb_path = resources.path('images/icon-podcast-small.png')
+            active = resources.path('images/icon-podcast-small_active.png')
+        else:
+            thumb_path = resources.path('images/icon-playlist-small.png')
+            active = resources.path('images/icon-playlist-small_active.png')
         info.icon = imagepool.get_surface(thumb_path)
-        if active:
-            info.active_icon = imagepool.get_surface(active)
+        info.active_icon = imagepool.get_surface(active)
+
+class SharingPlaylistTabListHandler(object):
+    def __init__(self, tablist):
+        self.tablist = tablist
+
+    def add(self, info, parent_id):
+        parent_id = info.share_id
+        if info.podcast:
+            parent_id = "sharing-%s-podcast" % (info.share_id,)
+        else:
+            parent_id = "sharing-%s-playlist" % (info.share_id,)
+        HideableTabList.add(self.tablist, info, parent_id)
 
 class ConnectList(TabUpdaterMixin, HideableTabList):
     name = _('Connect')
@@ -900,18 +1022,21 @@ class ConnectList(TabUpdaterMixin, HideableTabList):
         HideableTabList.__init__(self)
         TabUpdaterMixin.__init__(self)
         self._set_up = True # setup_list is never called?
+
+        sharing_tab_list_handler = SharingTabListHandler(self)
         self.info_class_map = {
             messages.DeviceInfo: DeviceTabListHandler(self),
-            messages.SharingInfo: SharingTabListHandler(self),
+            messages.SharingInfo: sharing_tab_list_handler,
+            messages.SharingPlaylistInfo: sharing_tab_list_handler,
             TabInfo: None,
             }
         self.view.connect_weak('hotspot-clicked', self.on_hotspot_clicked)
         self.view.set_drag_dest(DeviceDropHandler(self))
 
-    def on_row_expanded_change(self, view, iter_, path, expanded):
+    def on_row_expanded_change(self, view, iter_, expanded):
         info = self.view.model[iter_][0]
         if info is self.info:
-            HideableTabList.on_row_expanded_change(self, view, iter_, path,
+            HideableTabList.on_row_expanded_change(self, view, iter_,
                                                    expanded)
 
     def on_delete_key_pressed(self):
@@ -924,24 +1049,26 @@ class ConnectList(TabUpdaterMixin, HideableTabList):
 
     def on_hotspot_clicked(self, view, hotspot, iter_):
         info = self.view.model[iter_][0]
-        handler = self.info_class_map[type(info)]
-        return handler.on_hotspot_clicked(view, hotspot, iter_)
+        handler = self.info_class_map.get(type(info))
+        if hasattr(handler, 'on_hotspot_clicked'):
+            return handler.on_hotspot_clicked(view, hotspot, iter_)
 
     def init_info(self, info):
         if info is self.info:
             return
-        handler = self.info_class_map[type(info)]
-        return handler.init_info(info)
+        handler = self.info_class_map.get(type(info))
+        if hasattr(handler, 'init_info'):
+            handler.init_info(info)
 
     def add(self, info, parent_id=None):
-        handler = self.info_class_map[type(info)]
+        handler = self.info_class_map.get(type(info))
         if hasattr(handler, 'add'):
-            handler.add(info) # device doesn't use the parent_id
+            handler.add(info, parent_id) # device doesn't use the parent_id
         else:
             HideableTabList.add(self, info, parent_id)
 
     def update(self, info):
-        handler = self.info_class_map[type(info)]
+        handler = self.info_class_map.get(type(info))
         if hasattr(handler, 'update'):
             handler.update(info)
         else:
