@@ -41,6 +41,7 @@ class WidgetStateStore(object):
     # supposed to be. The CUSTOM_VIEW has to be caught in widgetstatestore 
     # and replaced with the proper view for the given situation.
     CUSTOM_VIEW = 2 
+    ALBUM_VIEW = 3
     LIST_VIEW = 1
     STANDARD_VIEW = 0
     DEFAULT_VIEW_TYPE = {
@@ -49,6 +50,7 @@ class WidgetStateStore(object):
         u'device-video': STANDARD_VIEW,
         u'downloading': STANDARD_VIEW,
         u'feed': CUSTOM_VIEW,
+        u'feed-folder': CUSTOM_VIEW,
         u'folder-contents': STANDARD_VIEW,
         u'music': LIST_VIEW,
         u'others': LIST_VIEW,
@@ -57,17 +59,7 @@ class WidgetStateStore(object):
         u'sharing': LIST_VIEW,
         u'videos': STANDARD_VIEW,
      }
-    FILTER_VIEW_ALL = 0
-    FILTER_UNWATCHED = 1
-    FILTER_NONFEED = 2
-    FILTER_DOWNLOADED = 4
-    FILTER_VIEW_VIDEO = 8
-    FILTER_VIEW_AUDIO = 16
-    FILTER_VIEW_MOVIES = 32
-    FILTER_VIEW_SHOWS = 64
-    FILTER_VIEW_CLIPS = 128
-    FILTER_VIEW_PODCASTS = 256
-    DEFAULT_DISPLAY_FILTERS = FILTER_VIEW_ALL
+    DEFAULT_DISPLAY_FILTERS = ['all']
     DEFAULT_COLUMN_WIDTHS = {
         u'album': 100,
         u'artist': 110,
@@ -78,6 +70,7 @@ class WidgetStateStore(object):
         u'feed-name': 70,
         u'file-type': 70,
         u'genre': 65,
+        u'multi-row-album': 200,
         u'kind': 70,
         u'length': 60,
         u'name': 200,
@@ -97,6 +90,7 @@ class WidgetStateStore(object):
         u'tab': u'feed-name', # all-feeds
         u'downloading': u'name',
         u'feed': u'-date',
+        u'feed-folder': u'-date',
         u'folder-contents': u'artist',
         u'music': u'artist',
         u'others': u'name',
@@ -107,6 +101,8 @@ class WidgetStateStore(object):
     DEFAULT_SORT_COLUMN[u'device-audio'] = DEFAULT_SORT_COLUMN[u'music']
     DEFAULT_SORT_COLUMN[u'device-video'] = DEFAULT_SORT_COLUMN[u'videos']
     DEFAULT_SORT_COLUMN[u'sharing'] = DEFAULT_SORT_COLUMN[u'videos']
+    # DEFAULT_COLUMNS stores the default columns when using list view for
+    # different display types.  We tweak it in _calc_default_columns()
     DEFAULT_COLUMNS = {
         u'videos':
             [u'state', u'name', u'length', u'date-added', u'feed-name',
@@ -124,6 +120,8 @@ class WidgetStateStore(object):
             u'size', u'date', u'status'],
         u'feed':
             [u'state', u'name', u'length', u'size', u'date', u'status'],
+        u'feed-folder':
+            [u'state', u'name', u'length', u'size', u'date', u'status'],
         u'search':
             [u'state', u'name', u'description', u'status', u'file-type',
             u'feed-name', u'date'],
@@ -149,7 +147,7 @@ class WidgetStateStore(object):
         u'file-type'])
     AVAILABLE_COLUMNS['search'] |= set([u'size', u'rating'])
     AVAILABLE_COLUMNS['videos'] |= set([u'description', u'date', u'rating',
-        u'file-type', u'show', u'kind', u'status'])
+        u'file-type', u'show', u'kind', u'status', u'genre'])
     AVAILABLE_COLUMNS[u'device-audio'] = AVAILABLE_COLUMNS[u'music'].copy()
     AVAILABLE_COLUMNS[u'device-video'] = AVAILABLE_COLUMNS[u'videos'].copy()
     AVAILABLE_COLUMNS[u'feed'] = ((AVAILABLE_COLUMNS['music'] |
@@ -165,7 +163,7 @@ class WidgetStateStore(object):
     # This needs to be handled here, though analagous properties (e.g.
     # NO_RESIZE_COLUMNS) are handled in widgetconst. #16783 refers to fixing the
     # WSS mess - these shouldn't be sets of strings anyway.
-    MANDATORY_SORTERS = frozenset([u'name'])
+    MANDATORY_SORTERS = frozenset([u'name', u'multi-row-album'])
 
     def __init__(self):
         self.displays = {}
@@ -223,7 +221,7 @@ class WidgetStateStore(object):
         else:
             view = WidgetStateStore.DEFAULT_VIEW_TYPE[display_type]
             if view == WidgetStateStore.CUSTOM_VIEW:
-                if display_type == u'feed':
+                if display_type in (u'feed', u'feed-folder'):
                     view = app.config.get(prefs.PODCASTS_DEFAULT_VIEW)
                 else:
                     app.widgetapp.handle_soft_failure("Getting default view",
@@ -239,16 +237,22 @@ class WidgetStateStore(object):
         self._save_display_state(display_type, display_id)
 
     def get_filters(self, display_type, display_id):
+        """Get the active filters
+
+        :returns: set of active filter strings
+        """
         display = self._get_display(display_type, display_id)
         if display.active_filters is None:
-            return WidgetStateStore.DEFAULT_DISPLAY_FILTERS
+            return set(WidgetStateStore.DEFAULT_DISPLAY_FILTERS)
         return display.active_filters
 
-    def toggle_filters(self, display_type, display_id, filter_):
-        filters = self.get_filters(display_type, display_id)
-        filters = WidgetStateStore.toggle_filter(filters, filter_)
+    def set_filters(self, display_type, display_id, active_filters):
+        """Set the active filters
+
+        active_filters should be a set of filter key strings.
+        """
         display = self._get_display(display_type, display_id)
-        display.active_filters = filters
+        display.active_filters = active_filters
         self._save_display_state(display_type, display_id)
 
     def set_shuffle(self, display_type, display_id, shuffle):
@@ -311,58 +315,81 @@ class WidgetStateStore(object):
         display.last_played_item_id = id_
         self._save_display_state(display_type, display_id)
 
-    def get_sorts_enabled(self, display_type, display_id):
-        display = self._get_display(display_type, display_id)
-        columns = display.list_view_columns
-        if columns is None:
-            columns = WidgetStateStore.DEFAULT_COLUMNS[display_type]
-        available = WidgetStateStore.AVAILABLE_COLUMNS[display_type]
-        # If a column used to be disableable for a display but now is not,
-        # re-add it at the end of the list:
-        mandatory = available & WidgetStateStore.MANDATORY_SORTERS
-        columns.extend(mandatory.difference(columns))
-        # If a column used to be enableable for a display but now is not,
+# ViewState properties for list and album view
+
+    def get_columns_enabled(self, display_type, display_id, view_type):
+        view = self._get_view(display_type, display_id, view_type)
+        # Copy the column list.  We may modify it in _add_manditory_columns()
+        # and that shouldn't change the source value.
+        if view.columns_enabled is not None:
+            columns = list(view.columns_enabled)
+        else:
+            columns = self._calc_default_columns(display_type, view_type)
+        available = WidgetStateStore.get_columns_available(display_type,
+                display_id, view_type)
+        self._add_manditory_columns(view_type, columns)
+        # If a column used to be enableable for a view but now is not,
         # filter it out:
         return [x for x in columns if x in available]
 
-    def set_sorts_enabled(self, display_type, display_id, enabled):
-        display = self._get_display(display_type, display_id)
-        display.list_view_columns = enabled
-        self._save_display_state(display_type, display_id)
+    def _calc_default_columns(self, display_type, view_type):
+        columns = list(WidgetStateStore.DEFAULT_COLUMNS[display_type])
+        if view_type == self.get_album_view_type():
+            # Remove columns that contain info in the album/artist column.
+            filter_out = (u'artist', u'album', u'track', u'feed-name')
+            columns = [n for n in columns if n not in filter_out]
+        return columns
 
-    def toggle_sort(self, display_type, display_id, column):
-        columns = self.get_sorts_enabled(display_type, display_id)
+    def _add_manditory_columns(self, view_type, columns):
+        """Add manditory columns to the list of columns enabled."""
+        # We currently handle name and multi-row-album.  Add an assertion so
+        # that if MANDATORY_SORTERS changes without this code changing, we'll
+        # see a crash report.
+
+        assert (WidgetStateStore.MANDATORY_SORTERS ==
+                set((u'name', u'multi-row-album')))
+        if u'name' not in columns:
+            columns.append(u'name')
+        if (view_type == self.get_album_view_type() and
+                u'multi-row-album' not in columns):
+            columns.insert(0, u'multi-row-album')
+
+    def set_columns_enabled(self, display_type, display_id, view_type,
+            enabled):
+        view = self._get_view(display_type, display_id, view_type)
+        view.columns_enabled = enabled
+        self._save_view_state(display_type, display_id, view_type)
+
+    def toggle_column_enabled(self, display_type, display_id, view_type,
+            column):
+        columns = self.get_columns_enabled(display_type, display_id, view_type)
         if column in columns:
             columns.remove(column)
         else:
             columns.append(column)
-        self.set_sorts_enabled(display_type, display_id, columns)
-
-# ViewState properties that are only valid for specific view_types:
+        self.set_columns_enabled(display_type, display_id, view_type, columns)
 
     def get_column_widths(self, display_type, display_id, view_type):
-        if WidgetStateStore.is_list_view(view_type):
-            display = self._get_display(display_type, display_id)
-            column_widths = display.list_view_widths or {}
-            columns = self.get_sorts_enabled(display_type, display_id)
-            for name in columns:
-                default = WidgetStateStore.DEFAULT_COLUMN_WIDTHS[name]
-                column_widths.setdefault(name, default)
-            return column_widths.copy()
-        else:
-            raise ValueError()
+        # fetch dict containing the widths from the DisplayInfo
+        view_info = self._get_view(display_type, display_id, view_type)
+        column_widths = view_info.column_widths
+        if column_widths is None:
+            column_widths = {}
+        # get widths for each enabled column
+        columns = self.get_columns_enabled(display_type, display_id,
+                view_type)
+        for name in columns:
+            default = WidgetStateStore.DEFAULT_COLUMN_WIDTHS[name]
+            column_widths.setdefault(name, default)
+        return column_widths.copy()
 
     def update_column_widths(self, display_type, display_id, view_type,
                              widths):
-        if WidgetStateStore.is_list_view(view_type):
-            display = self._get_display(display_type, display_id)
-            if display.list_view_widths is None:
-                display.list_view_widths = self.get_column_widths(
-                                        display_type, display_id, view_type)
-            display.list_view_widths.update(widths)
-            self._save_display_state(display_type, display_id)
-        else:
-            raise ValueError()
+        view_info = self._get_view(display_type, display_id, view_type)
+        if view_info.column_widths is None:
+            view_info.column_widths = {}
+        view_info.column_widths.update(widths)
+        self._save_view_state(display_type, display_id, view_type)
 
 # ViewState properties that are global to the whole frontend
     def get_item_details_expanded(self, view_type):
@@ -405,8 +432,12 @@ class WidgetStateStore(object):
 # static properties of a display_type:
 
     @staticmethod
-    def get_columns_available(display_type):
-        return WidgetStateStore.AVAILABLE_COLUMNS[display_type]
+    def get_columns_available(display_type, display_id, view_type):
+        available = WidgetStateStore.AVAILABLE_COLUMNS[display_type].copy()
+        # copy the set, since we may modify it before returning it
+        if view_type == WidgetStateStore.get_album_view_type():
+            available.add(u'multi-row-album')
+        return available
 
 # static properties of a view_type:
 
@@ -418,79 +449,9 @@ class WidgetStateStore(object):
     def is_standard_view(view_type):
         return view_type == WidgetStateStore.STANDARD_VIEW
 
-# manipulate a filter set:
-
     @staticmethod
-    def toggle_filter(filters, filter_):
-        if filter_ == WidgetStateStore.FILTER_VIEW_ALL:
-            return filter_
-        elif filter_ in (WidgetStateStore.FILTER_VIEW_VIDEO,
-                         WidgetStateStore.FILTER_VIEW_AUDIO):
-            exclude = ~(WidgetStateStore.FILTER_VIEW_VIDEO |
-                       WidgetStateStore.FILTER_VIEW_AUDIO)
-            if not ((filters & WidgetStateStore.FILTER_DOWNLOADED) or
-                    (filters & WidgetStateStore.FILTER_UNWATCHED)):
-                # only downloaded items have a file type
-                filters = filters | WidgetStateStore.FILTER_DOWNLOADED
-            return (filters & exclude) | filter_
-        elif filter_ in (WidgetStateStore.FILTER_UNWATCHED,
-                         WidgetStateStore.FILTER_DOWNLOADED):
-            exclude = ~(WidgetStateStore.FILTER_UNWATCHED |
-                         WidgetStateStore.FILTER_DOWNLOADED)
-            return (filters & exclude) | filter_
-        elif filter_ in (WidgetStateStore.FILTER_VIEW_MOVIES,
-                         WidgetStateStore.FILTER_VIEW_SHOWS,
-                         WidgetStateStore.FILTER_VIEW_CLIPS,
-                         WidgetStateStore.FILTER_VIEW_PODCASTS):
-            exclude = ~(WidgetStateStore.FILTER_VIEW_MOVIES |
-                         WidgetStateStore.FILTER_VIEW_SHOWS |
-                         WidgetStateStore.FILTER_VIEW_CLIPS |
-                         WidgetStateStore.FILTER_VIEW_PODCASTS)
-            return (filters & exclude) | filter_
-        else:
-            return filters | filter_
-
-# static properties of a filter combination:
-
-    @staticmethod
-    def is_view_all_filter(filters):
-        return filters == WidgetStateStore.FILTER_VIEW_ALL
-
-    @staticmethod
-    def is_view_video_filter(filters):
-        return bool(filters & WidgetStateStore.FILTER_VIEW_VIDEO)
-
-    @staticmethod
-    def is_view_audio_filter(filters):
-        return bool(filters & WidgetStateStore.FILTER_VIEW_AUDIO)
-
-    @staticmethod
-    def is_view_movies_filter(filters):
-        return bool(filters & WidgetStateStore.FILTER_VIEW_MOVIES)
-
-    @staticmethod
-    def is_view_shows_filter(filters):
-        return bool(filters & WidgetStateStore.FILTER_VIEW_SHOWS)
-
-    @staticmethod
-    def is_view_clips_filter(filters):
-        return bool(filters & WidgetStateStore.FILTER_VIEW_CLIPS)
-
-    @staticmethod
-    def is_view_podcasts_filter(filters):
-        return bool(filters & WidgetStateStore.FILTER_VIEW_PODCASTS)
-
-    @staticmethod
-    def has_unwatched_filter(filters):
-        return bool(filters & WidgetStateStore.FILTER_UNWATCHED)
-
-    @staticmethod
-    def has_non_feed_filter(filters):
-        return bool(filters & WidgetStateStore.FILTER_NONFEED)
-
-    @staticmethod
-    def has_downloaded_filter(filters):
-        return bool(filters & WidgetStateStore.FILTER_DOWNLOADED)
+    def is_album_view(view_type):
+        return view_type == WidgetStateStore.ALBUM_VIEW
 
 # static properties:
 
@@ -530,44 +491,12 @@ class WidgetStateStore(object):
     def get_standard_view_type():
         return WidgetStateStore.STANDARD_VIEW
 
-    # filters:
+    @staticmethod
+    def get_album_view_type():
+        return WidgetStateStore.ALBUM_VIEW
 
     @staticmethod
-    def get_view_all_filter():
-        return WidgetStateStore.FILTER_VIEW_ALL
-
-    @staticmethod
-    def get_view_video_filter():
-        return WidgetStateStore.FILTER_VIEW_VIDEO
-
-    @staticmethod
-    def get_view_audio_filter():
-        return WidgetStateStore.FILTER_VIEW_AUDIO
-
-    @staticmethod
-    def get_view_movies_filter():
-        return WidgetStateStore.FILTER_VIEW_MOVIES
-
-    @staticmethod
-    def get_view_shows_filter():
-        return WidgetStateStore.FILTER_VIEW_SHOWS
-
-    @staticmethod
-    def get_view_clips_filter():
-        return WidgetStateStore.FILTER_VIEW_CLIPS
-
-    @staticmethod
-    def get_view_podcasts_filter():
-        return WidgetStateStore.FILTER_VIEW_PODCASTS
-
-    @staticmethod
-    def get_unwatched_filter():
-        return WidgetStateStore.FILTER_UNWATCHED
-
-    @staticmethod
-    def get_non_feed_filter():
-        return WidgetStateStore.FILTER_NONFEED
-
-    @staticmethod
-    def get_downloaded_filter():
-        return WidgetStateStore.FILTER_DOWNLOADED
+    def get_all_view_types():
+        return (WidgetStateStore.LIST_VIEW,
+                WidgetStateStore.STANDARD_VIEW,
+                WidgetStateStore.ALBUM_VIEW)
