@@ -34,7 +34,10 @@ classes.
 """
 
 import base64
-import json
+try:
+    import simplejson as json
+except ImportError:
+    import json
 import logging
 import os
 import re
