@@ -164,6 +164,11 @@ class MalformedURL(NetworkError):
         NetworkError.__init__(self, _('Invalid URL'),
                 _('"%(url)s" is not a valid URL', {"url": url}))
 
+class InvalidRedirect(NetworkError):
+    def __init__(self, url):
+        NetworkError.__init__(self, _('Invalid Redirect'),
+                _('"%(url)s" is not a valid redirect URL', {"url": url}))
+
 class UnknownHostError(NetworkError):
     """A file: URL doesn't exist"""
     def __init__(self, host):
@@ -203,10 +208,12 @@ class TransferOptions(object):
     """
 
     def __init__(self, url, etag=None, modified=None, resume=False,
-            post_vars=None, post_files=None, write_file=None):
+            post_vars=None, post_files=None, write_file=None,
+                 extra_headers=None):
         self.url = url
         self.etag = etag
         self.modified = modified
+        self.extra_headers = extra_headers
         self.resume = resume
         self.post_vars = post_vars
         self.post_files = post_files
@@ -245,6 +252,8 @@ class TransferOptions(object):
             out_headers['etag'] = self.etag
         if self.modified is not None:
             out_headers['If-Modified-Since'] = self.modified
+        if self.extra_headers is not None:
+            out_headers.update(self.extra_headers)
 
         handle = self._init_handle()
         self._setup_post(handle, out_headers)
@@ -466,15 +475,14 @@ class CurlTransfer(object):
         elif self.options.write_file is not None:
             if not self.saw_head_success:
                 # try a HEAD request first to see if the request will work.
-                # This avoids writing error responses to our file.  It also
-                # avoids the issue of RESUME_FROM being applied to the error
-                # response.
+                # It avoids the issue of RESUME_FROM being applied to the 
+                # error response.
                 self.handle.setopt(pycurl.NOBODY, 1)
                 self.trying_head_request = True
             else:
                 self.handle.setopt(pycurl.URL, self.last_url)
                 self._open_file()
-                self.handle.setopt(pycurl.WRITEDATA, self._filehandle)
+                self.handle.setopt(pycurl.WRITEFUNCTION, self._write_file)
         elif self.content_check_callback is not None:
             self.handle.setopt(pycurl.WRITEFUNCTION, self._call_content_check)
         else:
@@ -485,6 +493,10 @@ class CurlTransfer(object):
             self.handle.setopt(pycurl.VERBOSE, 1)
             self.handle.setopt(pycurl.DEBUGFUNCTION, self.debug_func)
 
+    def _write_file(self, buf):
+        if self.check_response_code(self.status_code):
+            self._filehandle.write(buf)
+
     def _lookup_auth(self):
         """Lookup existing HTTP passwords to use.
 
@@ -552,7 +564,7 @@ class CurlTransfer(object):
         return False
 
 
-    def debug_func(self, type, msg):
+    def debug_func(self, typ, msg):
         type_map = {
                 pycurl.INFOTYPE_HEADER_IN: 'header-in',
                 pycurl.INFOTYPE_HEADER_OUT: 'header-out',
@@ -560,7 +572,7 @@ class CurlTransfer(object):
                 pycurl.INFOTYPE_DATA_OUT: 'data-out',
                 pycurl.INFOTYPE_TEXT: 'text',
         }
-        type_str = type_map.get(type, type)
+        type_str = type_map.get(typ, typ)
         logging.warn("libcurl debug (%s) %r", type_str, msg)
 
 
@@ -589,6 +601,15 @@ class CurlTransfer(object):
                 self.status_code = None
             elif 'location' in self.headers:
                 # doing a redirect, clear out the headers
+                redirect_url = self.headers['location']
+                scheme, _, _, _ = download_utils.parse_url(redirect_url)
+                if not scheme:
+                    logging.warn("%s: Non-absolute redirect URL: %s",
+                                 self.options.url, redirect_url)
+                elif scheme not in ('http', 'https'):
+                    self.cancel(remove_file=True)
+                    self.call_errback(InvalidRedirect(redirect_url))
+                    return
                 self.headers = {}
             elif not self.headers_finished:
                 curl_manager.call_after_perform(self.on_headers_finished)
@@ -677,16 +698,28 @@ class CurlTransfer(object):
                 self.saw_head_success = True
         elif info['status'] == 401:
             self.handle_http_auth()
-        elif info['status'] in (405, 501):
-            if self.trying_head_request:
-                # server didn't like the HEAD request, so just try the GET
-                self._send_new_request()
-                self.saw_head_success = True
-            else:
-                self.call_errback(UnexpectedStatusCode(info['status']))
         elif info['status'] == 407:
             self.handle_proxy_auth()
-        elif info['status'] >= 500 and info['status'] < 600:
+        elif self.trying_head_request:
+            # The response code wasn't what we expected, but we are doing
+            # a HEAD request.
+            #
+            # Servers have "inventive" strategies for dealing with HEAD
+            # requests.  I don't really feel like handling them one by
+            # one since you never know what comes through, so just assume
+            # things are okay and let the actual download handler deal with
+            # failure.
+            #
+            # We handle this before all of the error handling to make sure
+            # we have a chance to catch this.
+            self._send_new_request()
+            self.saw_head_success = True
+        elif ((info['status'] >= 500 and info['status'] < 600) or
+              (info['status'] == 404 and
+               self.last_url.startswith(('http://vimeo.com',
+                                         'http://www.vimeo.com')))):
+            # 500 errors are hopefully temporary, as are 404s from Vimeo
+            # (#19066)
             logging.info("httpclient: possibly temporary http error: HTTP %s",
                          info['status'])
             self.call_errback(PossiblyTemporaryError(info['status']))
@@ -774,13 +807,13 @@ class CurlTransfer(object):
 
     def call_callback(self, info):
         self._cleanup_filehandle()
-        eventloop.add_idle(self.callback, 'curl transfer callback',
-                args=(info,))
+        msg = 'curl transfer callback: %s' % (self.callback,)
+        eventloop.add_idle(self.callback, msg, args=(info,))
 
     def call_errback(self, error):
         self._cleanup_filehandle()
-        eventloop.add_idle(self.errback, 'curl transfer errback',
-                           args=(error,))
+        msg = 'curl transfer errback: %s' % (self.errback,)
+        eventloop.add_idle(self.errback, msg, args=(error,))
 
     def _cleanup_filehandle(self):
         if self._filehandle is not None:
@@ -947,12 +980,12 @@ class LibCURLManager(eventloop.SimpleEventLoop):
             try:
                 self.pop_transfer(handle).on_finished()
             except StandardError:
-                logging.stacktrace("Error calling on_finished()")
+                logging.warning("Error calling on_finished()", exc_info=True)
         for handle, code, message in errors:
             try:
                 self.pop_transfer(handle).on_error(code, handle)
             except StandardError:
-                logging.stacktrace("Error calling on_error()")
+                logging.warning("Error calling on_error()", exc_info=True)
 
     def pop_transfer(self, handle):
         transfer = self.transfer_map.pop(handle)
@@ -991,7 +1024,7 @@ def sanitize_url(url):
 def grab_url(url, callback, errback, header_callback=None,
         content_check_callback=None, write_file=None, etag=None, modified=None,
         default_mime_type=None, resume=False, post_vars=None,
-        post_files=None):
+        post_files=None, extra_headers=None):
     """Quick way to download a network resource
 
     grab_url is a simple interface to the HTTPClient class.
@@ -1012,6 +1045,7 @@ def grab_url(url, callback, errback, header_callback=None,
     :param post_vars: dictionary of variables to send as POST data
     :param post_files: files to send as POST data (see
         xhtmltools.multipart_encode for the format)
+    :param extra_headers: an option dictionary of extra headers to send
 
     The callback will be passed a dictionary that contains all the HTTP
     headers, as well as the following keys:
@@ -1035,7 +1069,7 @@ def grab_url(url, callback, errback, header_callback=None,
         return _grab_file_url(url, callback, errback, default_mime_type)
     else:
         options = TransferOptions(url, etag, modified, resume, post_vars,
-                post_files, write_file)
+                post_files, write_file, extra_headers)
         transfer = CurlTransfer(options, callback, errback, header_callback,
                 content_check_callback)
         transfer.start()
