@@ -43,7 +43,6 @@ import BaseHTTPServer
 import SocketServer
 import threading
 import httplib
-from httplib import _CS_IDLE, _CS_REQ_STARTED, _CS_REQ_SENT, _UNKNOWN
 import gzip
 try:
     from cStringIO import StringIO
@@ -382,7 +381,7 @@ class DaapHttpRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
                     seekend = 0
                 rc = DAAP_PARTIAL_CONTENT
         generation = threading.current_thread().generation
-        file_obj = self.server.backend.get_file(item_id, generation, ext,
+        file_obj, hint = self.server.backend.get_file(item_id, generation, ext,
                                                 self.get_session(),
                                                 self.get_request_path,
                                                 offset=seekpos, chunk=chunk)
@@ -390,7 +389,7 @@ class DaapHttpRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
             return (DAAP_FILENOTFOUND, [], extra_headers)
         self.log_message('daap server: streaming with filobj %s', file_obj)
         # Return a special response, the encode_reponse() will handle correctly
-        return (rc, [(file_obj, seekpos, seekend)], extra_headers)
+        return (rc, [(file_obj, hint, seekpos, seekend)], extra_headers)
 
     def get_request_path(self, itemid, enclosure):
         # XXX
@@ -863,6 +862,9 @@ class DaapClient(object):
         # We've been disconnected, or server gave incorrect response?
         except (IOError, ValueError):
             self.disconnect()
+        except AttributeError:
+            logging.debug('AttributeError caught; probably during shutdown. '
+                          'Ignoring.')
 
     # Generic check for http response.  ValueError() on unexpected response.
     def check_reply(self, response, http_code=httplib.OK, callback=None,
