@@ -39,14 +39,15 @@ two.  These messages are handled asynchronously.
 This module defines the messages that are passed between the two threads.
 """
 
+import copy
 import logging
 
 from miro.gtcache import gettext as _
-from miro.folder import ChannelFolder, PlaylistFolder
 from miro.messagetools import Message, MessageHandler
 from miro.plat import resources
 from miro import app
 from miro import displaytext
+from miro import models
 from miro import guide
 from miro import search
 from miro import prefs
@@ -128,52 +129,6 @@ class StopTrackingGuides(BackendMessage):
     """
     pass
 
-class TrackItems(BackendMessage):
-    """Begin tracking items for a feed
-
-    After this message is sent, the backend will send back a ItemList message,
-    then it will send ItemsChanged messages for items in the feed.
-
-    type is the type of object that we are tracking items for.  It can be one
-    of the following:
-
-    * feed -- Items in a feed
-    * playlist -- Items in a playlist
-    * new -- Items that haven't been watched
-    * downloading -- Items being downloaded
-    * library -- All items
-
-    id should be the id of a feed/playlist. For new, downloading and library
-    it is ignored.
-    """
-    def __init__(self, typ, id_):
-        self.type = typ
-        self.id = id_
-
-class TrackItemsManually(BackendMessage):
-    """Track a manually specified list of items.
-
-    TrackItemsManually can only be used to track database items.
-
-    No ItemList message will be sent, since the sender is providing the inital
-    list of items.  Instead, if the infos_to_track is out of date of date,
-    then an ItemsChanged message will be sent with the changes.
-
-    ItemsChanged messages will have "manual" as the type and will use the id
-    specified in the constructed.
-    """
-    def __init__(self, id_, infos_to_track):
-        self.id = id_
-        self.infos_to_track = infos_to_track
-        self.type = 'manual'
-
-class StopTrackingItems(BackendMessage):
-    """Stop tracking items for a feed.
-    """
-    def __init__(self, typ, id_):
-        self.type = typ
-        self.id = id_
-
 class TrackDownloadCount(BackendMessage):
     """Start tracking the number of downloading items.  After this message is
     received the backend will send a corresponding DownloadCountChanged
@@ -265,15 +220,17 @@ class StopTrackingWatchedFolders(BackendMessage):
     """
     pass
 
-class TrackSharing(BackendMessage):
-    """Start tracking media shares.
+class TrackShare(BackendMessage):
+    """Start tracking a media share
     """
-    pass
+    def __init__(self, share_id):
+        self.share_id = share_id
 
-class TrackDevices(BackendMessage):
-    """Start tracking devices.
+class StopTrackingShare(BackendMessage):
+    """Stop tracking a media share
     """
-    pass
+    def __init__(self, share_id):
+        self.share_id = share_id
 
 class SetFeedExpire(BackendMessage):
     """Sets the expiration for a feed.
@@ -646,6 +603,13 @@ class KeepVideo(BackendMessage):
     def __repr__(self):
         return BackendMessage.__repr__(self) + (", id: %s" % self.id)
 
+class SetMediaKind(BackendMessage):
+    """Set the media kind of the list of items to be the specified kind.
+    """
+    def __init__(self, item_infos, kind):
+        self.item_infos = item_infos
+        self.kind = kind
+
 class SaveItemAs(BackendMessage):
     """Saves an item in the dark clutches of Miro to somewhere else.
     """
@@ -893,7 +857,46 @@ class RateItem(BackendMessage):
         self.info = info
         self.rating = rating
 
+class SetNetLookupEnabled(BackendMessage):
+    """Enable/disable internet lookups for a set of items.  """
+    def __init__(self, item_ids, enabled):
+        """Create a new message
+
+        :param item_ids: list of item ids or None for all current items
+        :param boolean enabled: enable/disable flag
+        """
+        self.item_ids = item_ids
+        self.enabled = enabled
+
+class ClogBackend(BackendMessage):
+    """Dev message: intentionally clog the backend for a specified number of 
+    seconds.
+    """
+    def __init__(self, n=0):
+        self.n = n
+
+class ForceFeedparserProcessing(BackendMessage):
+    """Force the backend to do a bunch of feedparser updates
+    """
+    pass
+
+class ForceDBSaveError(BackendMessage):
+    """Simulate an error running an INSERT/UPDATE statement on the main DB.
+    """
+
+class ForceDeviceDBSaveError(BackendMessage):
+    """Simulate an error running an INSERT/UPDATE statement on a device DB.
+    """
+    def __init__(self, device_info):
+        self.device_info = device_info
+
 # Frontend Messages
+class DownloaderSyncCommandComplete(FrontendMessage):
+    """Tell the frontend that the pause/resume all command are complete,
+    so that we only sort once.  This saves time sorting and also prevents
+    UI clog when items are updated and gets sorted one by one.
+    """
+    pass
 
 class JettisonTabs(FrontendMessage):
     """Tell the frontend to remove certain sidebar tabs from its model.  Done
@@ -917,10 +920,11 @@ class SharingDisappeared(FrontendMessage):
     def __init__(self, share):
         self.share = share
 
-class DeviceEjectFailed(FrontendMessage):
-    """Tell the frontend that the device eject failed."""
-    def __init__(self, device):
-        self.device = device
+class ShowWarning(FrontendMessage):
+    """Tell the frontend to show a warning."""
+    def __init__(self, title, description):
+        self.title = title
+        self.description = description
 
 class FrontendQuit(FrontendMessage):
     """The frontend should exit."""
@@ -1012,7 +1016,7 @@ class ChannelInfo(object):
             self.search_term = channel_obj.searchTerm
         else:
             self.search_term = None
-        if not isinstance(channel_obj, ChannelFolder):
+        if not isinstance(channel_obj, models.ChannelFolder):
             self.has_original_title = channel_obj.has_original_title()
             self.is_updating = channel_obj.is_updating()
             self.parent_id = channel_obj.folder_id
@@ -1054,7 +1058,7 @@ class PlaylistInfo(object):
     def __init__(self, playlist_obj):
         self.name = playlist_obj.get_title()
         self.id = playlist_obj.id
-        self.is_folder = isinstance(playlist_obj, PlaylistFolder)
+        self.is_folder = isinstance(playlist_obj, models.PlaylistFolder)
         if self.is_folder:
             self.parent_id = None
         else:
@@ -1087,218 +1091,6 @@ class GuideInfo(object):
     def __repr__(self):
         return '<miro.messages.GuideInfo(%i) "%s">' % (self.id, self.name)
 
-class ItemInfo(object):
-    """Tracks the state of an item
-
-    :param name: name of the item
-    :param id: object id
-    :param source: the ItemSource this ItemInfo was generated from
-    :param feed_id: id for the items feed
-    :param feed_name: name of the feed item is attached to
-    :param feed_url: URL of the feed item is attached to 
-    :param description: longer description for the item (HTML)
-    :param state: see Item.get_state()
-    :param release_date: datetime object when the item was published
-    :param size: size of the item in bytes
-    :param duration: length of the video in seconds
-    :param resume_time: time at which playback should restart
-    :param permalink: URL to a permalink to the item (or None)
-    :param commentslink: URL to a comments page for the item (or None)
-    :param payment_link: URL of the payment page associated with the item
-                         (or empty string)
-    :param has_shareable_url: does this item have a shareable URL?
-    :param can_be_saved: is this an expiring downloaded item?
-    :param downloaded: has the item been downloaded?
-    :param is_external: is this item external (true) or from a channel
-                        (false)?
-    :param expiration_date: datetime object for when the item will expire
-                            (or None)
-    :param item_viewed: has the user ever seen the item?
-    :param video_watched: has the user watched the video for the item?
-    :param video_path: the file path to the video for this item (or None)
-    :param file_type: type of the downloaded file (video/audio/other)
-    :param subtitle_encoding: encoding for subtitle display
-    :param media_type_checked: has the movie data util checked file_type?
-    :param seeding_status: Torrent seeding status ('seeding', 'stopped',
-                           or None)
-    :param thumbnail: path to the thumbnail for this file
-    :param thumbnail_url: URL for the item's thumbnail (or None)
-    :param file_format: User-facing format description.  Possibly the
-                        file type,  pulled from the mime_type, or more
-                        generic, like "audio"
-    :param remote: is this item from a media share or local?
-    :param host: machine hosting the item, only valid if remote is set
-    :param port: port to connect to for item, only valid if remote is set
-    :param license: this file's license, if known.
-    :param mime_type: mime-type of the enclosure that would be downloaded
-    :param album_artist: the album artist of the album
-    :param artist: the primary artist of the track
-    :param album: the track's album of origin
-    :param track: the track number within the album
-    :param year: the track's year of release
-    :param genre: the track's genre
-    :param rating: the user's rating of the track
-    :param date_added: when the item became part of the user's db
-    :param last_played: the date/time the item was last played
-    :param file_url: URL of the enclosure that would be downloaded
-    :param download_info: DownloadInfo object containing info about the
-                          download (or None)
-    :param is_container_item: whether or not this item is actually a
-                              collection of files as opposed to an
-                              individual item
-    :param children: for container items the children of the item.
-    :param is_playable: is this item a audio/video file, or a container that
-                        contains audio/video files inside.
-    :param is_playing: Whether item is the currently playing (or paused) item
-    :param leechers: (Torrent only) number of leeching clients
-    :param seeders: (Torrent only) number of seeding clients
-    :param up_rate: (Torrent only) how fast we're uploading data
-    :param down_rate: (Torrent only) how fast we're downloading data
-    :param up_total: (Torrent only) total amount we've uploaded
-    :param down_total: (Torrent only) total amount we've downloaded
-    :param up_down_ratio: (Torrent only) ratio of uploaded to downloaded
-    :param has_drm: True/False if known; None if unknown (usually means no)
-    """
-
-    html_stripper = util.HTMLStripper()
-
-    def __repr__(self):
-        return "<ItemInfo %r>" % self.id
-
-    def __getstate__(self):
-        d = self.__dict__.copy()
-        d['device'] = None
-        del d['description_stripped']
-        del d['search_terms']
-        return d
-
-    def __setstate__(self, d):
-        self.__dict__.update(d)
-        self.description_stripped = ItemInfo.html_stripper.strip(
-                self.description)
-        self.search_terms = search.calc_search_terms(self)
-
-    def __init__(self, id_, **kwargs):
-        self.id = id_
-
-        self.__dict__.update(kwargs) # we're just a thin wrapper around some
-                                     # data
-
-        # stuff we can calculate from other attributes
-        if not hasattr(self, 'description_stripped'):
-            self.description_stripped = ItemInfo.html_stripper.strip(
-                self.description)
-        if not hasattr(self, 'search_terms'):
-            self.search_terms = search.calc_search_terms(self)
-        self.name_sort_key = util.name_sort_key(self.name)
-        self.album_sort_key = util.name_sort_key(self.album)
-        self.artist_sort_key = util.name_sort_key(self.artist)
-        if self.album_artist:
-            self.album_artist_sort_key = util.name_sort_key(self.album_artist)
-        else:
-            self.album_artist_sort_key = self.artist_sort_key
-        # pre-calculate things that get displayed in list view
-        self.description_oneline = (
-                self.description_stripped[0].replace('\n', '$'))
-        self.display_date = displaytext.date_slashes(self.release_date)
-        self.display_duration = displaytext.duration(self.duration)
-        self.display_duration_short = displaytext.short_time_string(
-                self.duration)
-        self.display_size = displaytext.size_string(self.size)
-        self.display_date_added = displaytext.date_slashes(self.date_added)
-        self.display_last_played = displaytext.date_slashes(self.last_played)
-        self.display_track = displaytext.integer(self.track)
-        self.display_year = displaytext.integer(self.year)
-        self.display_torrent_details = self.calc_torrent_details()
-        self.display_drm = self.has_drm and _("Locked") or u""
-        # FIXME: display_kind changes here need also be applied in itemedit
-        if self.kind == 'movie':
-            self.display_kind = _("Movie")
-        elif self.kind == 'show':
-            self.display_kind = _("Show")
-        elif self.kind == 'clip':
-            self.display_kind = _("Clip")
-        elif self.kind == 'podcast':
-            self.display_kind = _("Podcast")
-        else:
-            self.display_kind = None
-
-        if self.state == 'downloading':
-            dl_info = self.download_info
-            if dl_info.eta > 0:
-                self.display_eta = displaytext.time_string(dl_info.eta)
-            else:
-                self.display_eta = ''
-            self.display_rate = displaytext.download_rate(dl_info.rate)
-        else:
-            self.display_rate = self.display_eta = ''
-
-    def calc_torrent_details(self):
-        if not self.download_info or not self.download_info.torrent:
-            return ''
-
-        details = _(
-            "S: %(seeders)s  |  "
-            "L: %(leechers)s  |  "
-            "UR: %(up_rate)s  |  "
-            "UT: %(up_total)s  |  "
-            "DR: %(down_rate)s  |  "
-            "DT: %(down_total)s  |  "
-            "R: %(ratio).2f",
-            {"seeders": self.seeders,
-             "leechers": self.leechers,
-             "up_rate": self.up_rate,
-             "up_total": self.up_total,
-             "down_rate": self.down_rate,
-             "down_total": self.down_total,
-             "ratio": self.up_down_ratio})
-        return details
-
-class DownloadInfo(object):
-    """Tracks the download state of an item.
-
-    :param downloaded_size: bytes downloaded
-    :param rate: current download rate, in bytes per second
-    :param state: one of ``downloading``, ``uploading``, ``finished``,
-                  ``failed`` or ``paused``.  ``uploading`` is for
-                  torrents only.  It means that we've finished
-                  downloading the torrent and are now seeding it.
-    :param eta: Estimated seconds before the download is finished
-    :param startup_activity: The current stage of starting up
-    :param finished: True if the item has finished downloading
-    :param torrent: Is this a Torrent download?
-    """
-    def __init__(self, downloader):
-        self.downloaded_size = downloader.get_current_size()
-        self.total_size = downloader.get_total_size()
-        self.rate = downloader.get_rate()
-        self.state = downloader.get_state()
-        self.startup_activity = downloader.get_startup_activity()
-        self.finished = downloader.is_finished()
-        self.torrent = (downloader.get_type() == 'bittorrent')
-        if self.state == 'failed':
-            self.reason_failed = downloader.get_reason_failed()
-            self.short_reason_failed = downloader.get_short_reason_failed()
-        else:
-            self.reason_failed = u""
-            self.short_reason_failed = u""
-        self.eta = downloader.get_eta()
-
-class PendingDownloadInfo(DownloadInfo):
-    """DownloadInfo object for pending downloads (downloads queued,
-    but not started because we've reached some limit)
-    """
-    def __init__(self):
-        self.downloaded_size = 0
-        self.rate = 0
-        self.state = 'pending'
-        self.startup_activity = _('queued for download')
-        self.finished = False
-        self.torrent = False
-        self.reason_failed = u""
-        self.short_reason_failed = u""
-        self.eta = 0
-
 class WatchedFolderInfo(object):
     """Tracks the state of a watched folder.
     
@@ -1419,39 +1211,63 @@ class TabsChanged(FrontendMessage):
     '(%d added, %d changed, %d removed)>') % (self.type,
     len(self.added), len(self.changed), len(self.removed))
 
-class ItemList(FrontendMessage):
-    """Sends the frontend the initial list of items for a feed
+class ItemChanges(FrontendMessage):
+    """Sent to the frontend when items change
 
-    :param type: type of object being tracked (same as in TrackItems)
-    :param id: id of the object being tracked (same as in TrackItems)
-    :param items: list of ItemInfo objects
+    :attribute added: set ids for added items
+    :attribute changed: set ids for changed items
+    :attribute removed: set ids for removed items
+    :attribute changed_columns: set columns that were changed (the union of
+    changes for all items)
+    :attribute dlstats_changed: Did we get new download stats?
+    :attribute playlists_changed: Did items get added/removed from playlists?
     """
-    def __init__(self, typ, id_, item_infos):
-        self.type = typ
-        self.id = id_
-        self.items = item_infos
+    def __init__(self, added, changed, removed, changed_columns,
+                 dlstats_changed, playlists_changed):
+        self.added = frozenset(added)
+        self.changed = frozenset(changed)
+        self.removed = frozenset(removed)
+        self.changed_columns = frozenset(changed_columns)
+        self.dlstats_changed = dlstats_changed
+        self.playlists_changed = playlists_changed
 
-class ItemsChanged(FrontendMessage):
-    """Informs the frontend that the items in a feed have changed.
+class DeviceItemChanges(FrontendMessage):
+    """Sent to the frontend when items change on a device
 
-    :param type: type of object being tracked (same as in TrackItems)
-    :param id: id of the object being tracked (same as in TrackItems)
-    :param added: list containing an ItemInfo object for each added item.
-                  The order will be the order they were added.
-    :param changed: set containing an ItemInfo for each changed item.
-    :param removed: set containing ids for each item that was removed
+    :attribute device_id: id for the device
+    :attribute added: set ids for added items
+    :attribute changed: set ids for changed items
+    :attribute removed: set ids for removed items
+    :attribute changed_columns: set columns that were changed (the union of
+    changes for all items)
     """
-    def __init__(self, typ, id_, added, changed, removed):
-        self.type = typ
-        self.id = id_
+    def __init__(self, device_id, added, changed, removed, changed_columns):
+        self.device_id = device_id
         self.added = added
         self.changed = changed
         self.removed = removed
+        self.changed_columns = changed_columns
 
-    def __str__(self):
-        return ('<miro.messages.ItemsChanged %s:%s '
-    '(%d added, %d changed, %d removed)>') % (self.type, self.id,
-    len(self.added), len(self.changed), len(self.removed))
+class SharingItemChanges(FrontendMessage):
+    """Sent to the frontend when items change on a share
+
+    :attribute share_id: id for the share
+    :attribute added: set ids for added items
+    :attribute changed: set ids for changed items
+    :attribute removed: set ids for removed items
+    :attribute changed_columns: set columns that were changed (the union of
+    changes for all items)
+    :attribute changed_playlists: True if the any playlists have been changed
+    had their contents changed.
+    """
+    def __init__(self, share_id, added, changed, removed, changed_columns,
+                 changed_playlists):
+        self.share_id = share_id
+        self.added = added
+        self.changed = changed
+        self.removed = removed
+        self.changed_columns = changed_columns
+        self.changed_playlists = changed_playlists
 
 class WatchedFolderList(FrontendMessage):
     """Sends the frontend the initial list of watched folders.
@@ -1521,6 +1337,18 @@ class UnwatchedCountChanged(FrontendMessage):
     def __init__(self, count):
         self.count = count
 
+class ConverterList(FrontendMessage):
+    """Sends the list of converters to the frontend
+
+    :attribute converters: list of converter groups.  Each group will contain
+    a sublist of (identifier, name) tuples.
+    """
+    def __init__(self, converter_list):
+        self.converters = []
+        for name, converters in converter_list:
+            self.converters.append([(info.identifier, info.name)
+                                    for info in converters])
+
 class ConversionTaskInfo(object):
     """Tracks the state of an conversion task.
 
@@ -1551,10 +1379,11 @@ class ConversionTaskInfo(object):
         self.log_path = task.log_path
         self.progress = task.progress
         self.error = task.error
-        self.item_name = task.item_info.name
+        self.item_name = task.item_info.title
         self.item_thumbnail = task.item_info.thumbnail
         self.eta = task.get_eta()
         self.target = task.get_display_name()
+        self.output_size_guess = task.get_output_size_guess()
 
 class ConversionTasksList(FrontendMessage):
     """Send the current list of running and pending conversion tasks to the 
@@ -1600,28 +1429,26 @@ class ConversionTaskChanged(FrontendMessage):
 
 class SharingInfo(object):
     """Tracks the state of an extent share."""
-    def __init__(self, share_id, tracker_id, name, host, port, parent_id=None,
-                 playlist_id=None, podcast=False, has_children=False):
-        # We need to create a unique identifier for indexing.  Fortunately
-        # this may be non-numeric.  We just combine the name, host, port
-        # as our index.
-        self.id = share_id
-        self.tracker_id = tracker_id
-        self.name = name
-        self.host = host
-        self.port = port
+    def __init__(self, share):
+        self.id = 'sharing-%s' % (share.id,)
+        self.share_id = share.id
+        self.sqlite_path = share.db_path
+        self.name = share.name
+        self.host = share.host
+        self.port = share.port
         self.share_available = False
         self.stale_callback = None
         self.mount = False
-        self.podcast = podcast
         self.is_updating = False
+
+class SharingPlaylistInfo(object):
+    """Tracks the state a playlist on a share."""
+    def __init__(self, share_id, name, playlist_id, podcast):
+        self.share_id = share_id
+        self.id = u'sharing-%s-%s' % (share_id, playlist_id)
+        self.name = name
+        self.podcast = podcast
         self.playlist_id = playlist_id
-        if parent_id is not None:
-            self.is_folder = has_children
-            self.parent_id = parent_id
-        else:
-            self.parent_id = None
-            self.is_folder = True
 
 class SharingEject(BackendMessage):
     """Tells the backend that the user has requested the share be disconnected.
@@ -1632,13 +1459,18 @@ class SharingEject(BackendMessage):
 class DeviceInfo(object):
     """Tracks the state of an attached device.
     """
-    def __init__(self, id_, device_info, mount, database, size, remaining):
+    def __init__(self, id_, device_info, mount, sqlite_path, database, db_info,
+                 metadata_manager, size, remaining, read_only):
         self.id = id_
         self.mount = mount
+        self.sqlite_path = sqlite_path
         self.database = database
+        self.db_info = db_info
+        self.metadata_manager = metadata_manager
         self.size = size
         self.remaining = remaining
         self.info = device_info
+        self.read_only = read_only
         self.name = database.get('settings', {}).get('name', device_info.name)
 
     def __repr__(self):
@@ -1653,9 +1485,10 @@ class DeviceChanged(FrontendMessage):
 class CurrentSyncInformation(FrontendMessage):
     """Informs the frontend of what the current sync would look like.
     """
-    def __init__(self, device, count):
+    def __init__(self, device, count, size):
         self.device = device
         self.count = count
+        self.size = size
 
 class DeviceSyncChanged(FrontendMessage):
     """Informs the frontend that the status of a device sync has changed.  This
@@ -1674,8 +1507,8 @@ class MessageToUser(FrontendMessage):
         self.title = title
         self.desc = desc
 
-class PlayMovie(FrontendMessage):
-    """Starts playing a specific movie.
+class PlayMovies(FrontendMessage):
+    """Play a list of files
     """
     def __init__(self, item_infos):
         self.item_infos = item_infos
@@ -1732,12 +1565,14 @@ class DisplayInfo(object):
         self.key = key
         if display is not None:
             self.selected_view = display.selected_view
-            self.active_filters = display.active_filters
             self.shuffle = display.shuffle
             self.repeat = display.repeat
             self.selection = display.selection
             self.sort_state = display.sort_state
             self.last_played_item_id = display.last_played_item_id
+            # shallow-copy attributes that store lists, dicts, and sets so
+            # that changing the database object doesn't change the DisplayInfo
+            self.active_filters = copy.copy(display.active_filters)
         else:
             self.selected_view = None
             self.active_filters = None
@@ -1746,14 +1581,6 @@ class DisplayInfo(object):
             self.selection = None
             self.sort_state = None
             self.last_played_item_id = None
-        if display is not None and display.list_view_columns is not None:
-            self.list_view_columns = display.list_view_columns[:]
-        else:
-            self.list_view_columns = None
-        if display is not None and display.list_view_widths is not None:
-            self.list_view_widths = display.list_view_widths.copy()
-        else:
-            self.list_view_widths = None
 
 class GlobalInfo(object):
     """Contains the properties that are global to the widgets frontend
@@ -1770,8 +1597,14 @@ class ViewInfo(object):
         self.key = key
         if view is not None:
             self.scroll_position = view.scroll_position
+            # shallow-copy attributes that store lists, dicts, and sets so
+            # that changing the database object doesn't change the DisplayInfo
+            self.columns_enabled = copy.copy(view.columns_enabled)
+            self.column_widths = copy.copy(view.column_widths)
         else:
             self.scroll_position = None
+            self.columns_enabled = None
+            self.column_widths = None
 
 class OpenInExternalBrowser(FrontendMessage):
     """Opens the specified url in an external browser.
@@ -1804,8 +1637,19 @@ class FeedlessDownloadStarted(FrontendMessage):
     pass
 
 class MetadataProgressUpdate(FrontendMessage):
-    def __init__(self, target, remaining, eta, total):
+    def __init__(self, target, finished, finished_local, eta, total):
         self.target = target
-        self.remaining = remaining
+        self.finished = finished
+        self.finished_local = finished_local
         self.eta = eta
         self.total = total
+
+class SetNetLookupEnabledFinished(FrontendMessage):
+    """The backend has processed the SetNetLookupEnabled message."""
+    pass
+
+class NetLookupCounts(FrontendMessage):
+    """Update the frontend on how many items we're running net lookups for."""
+    def __init__(self, net_lookup_count, total_count):
+        self.net_lookup_count = net_lookup_count
+        self.total_count = total_count
