@@ -82,7 +82,7 @@ class PlaylistMixin:
         """Add a new item to end of the playlist.  """
         self.MapClass.add_item_id(self.id, item_id)
         item = models.Item.get_by_id(item_id)
-        item.save(always_signal=True)
+        item.playlists_changed(added=True)
 
         folder = self.get_folder()
         if folder is not None:
@@ -101,7 +101,7 @@ class PlaylistMixin:
             folder.remove_id(item_id)
         if signal_change:
             item = models.Item.get_by_id(item_id)
-            item.signal_change(needs_save=False)
+            item.playlists_changed()
 
     def add_item(self, item):
         """Add an item to the end of the playlist"""
@@ -125,6 +125,7 @@ class PlaylistMixin:
                     (self.id, item_id)).get_singleton()
             map_.position = i
             map_.signal_change()
+            models.Item.playlist_reordered()
 
 class SavedPlaylist(database.DDBObject, PlaylistMixin):
     """An ordered list of videos that the user has saved.
