@@ -62,6 +62,14 @@ class Pref:
         return self.key != other.key
 
 # These are normal user preferences.
+DONATE_PAYMENT_URL_TEMPLATE = Pref(key='donatePaymentURLTemplate', default='http://www.getmiro.com/give/?s=m%d', platformSpecific=False)
+DONATE_URL_TEMPLATE         = Pref(key='donateURLTemplate',     default='http://getmiro.com/give/m%s/', platformSpecific=False)
+DONATE_ASK1                 = Pref(key='donateAsk1',            default=10,    platformSpecific=False)
+DONATE_ASK2                 = Pref(key='donateAsk2',            default=50,    platformSpecific=False)
+DONATE_ASK3                 = Pref(key='donateAsk3',            default=100,   platformSpecific=False)
+DONATE_NOTHANKS             = Pref(key='donateNoThanks',        default=0,     platformSpecific=False)
+LAST_DONATE_TIME            = Pref(key='lastDonateTime',        default=0,     platformSpecific=False)
+DONATE_COUNTER              = Pref(key='donateCounter',         default=10,    platformSpecific=False)
 MAIN_WINDOW_FRAME           = Pref(key='mainWindowFrame',       default=None,  platformSpecific=False)
 LEFT_VIEW_SIZE              = Pref(key='leftViewSize',          default=None,  platformSpecific=False)
 RIGHT_VIEW_SIZE             = Pref(key='rightViewSize',         default=None,  platformSpecific=False)
@@ -85,7 +93,9 @@ BT_MAX_PORT                 = Pref(key='BitTorrentMaxPort',     default=8600,  p
 UPLOAD_RATIO                = Pref(key='uploadRatio',           default=2.0,   platformSpecific=False)
 LIMIT_UPLOAD_RATIO          = Pref(key='limitUploadRatio',      default=False, platformSpecific=False)
 STARTUP_TASKS_DONE          = Pref(key='startupTasksDone',      default=False, platformSpecific=False)
-SINGLE_VIDEO_PLAYBACK_MODE  = Pref(key='singleVideoPlaybackMode', default=False, platformSpecific=False)
+CONTINUOUS_VIDEO_PLAYBACK_MODE  = Pref(key='continuousVideoPlaybackMode', default=True, platformSpecific=False)
+CONTINUOUS_MUSIC_PLAYBACK_MODE  = Pref(key='continuousMusicPlaybackMode', default=True, platformSpecific=False)
+CONTINUOUS_PODCAST_PLAYBACK_MODE  = Pref(key='continuousPodcastPlaybackMode', default=True, platformSpecific=False)
 PLAY_DETACHED               = Pref(key='detachedPlaybackMode',  default=False, platformSpecific=False)
 DETACHED_WINDOW_FRAME       = Pref(key='detachedWindowFrame',   default=None,  platformSpecific=False)
 RESUME_VIDEOS_MODE          = Pref(key='resumeVideosMode',      default=True,  platformSpecific=False)
@@ -116,11 +126,16 @@ SHARE_WARN_ON_QUIT          = Pref(key='ShareWarnOnQuit',       default=True, pl
 SHARE_VIDEO                 = Pref(key='ShareVideo',            default=True, platformSpecific=False)
 SHARE_AUDIO                 = Pref(key='ShareAudio',            default=True, platformSpecific=False)
 SHARE_FEED                  = Pref(key='ShareFeed',             default=True, platformSpecific=False)
-MUSIC_TAB_CLICKED           = Pref(key='musicTabClicked',       default=False, platformSpecific=False)
+# the musicTabClicked key was used before miro 5.0.  It's been changed because
+# we want to pop up the dialog for users who ran 4.0.x and let them know about
+# internet lookups
+MUSIC_TAB_CLICKED           = Pref(key='musicTabClicked2',       default=False, platformSpecific=False)
 SHOW_PODCASTS_IN_VIDEO      = Pref(key='showPodcastsInVideo', default=True, platformSpecific=False)
 SHOW_PODCASTS_IN_MUSIC      = Pref(key='showPodcastsInMusic', default=False, platformSpecific=False)
 REMEMBER_LAST_DISPLAY       = Pref(key='rememberLastDisplay', default=False, platformSpecific=False)
 PODCASTS_DEFAULT_VIEW       = Pref(key='podcastsDefaultView', default=0, platformSpecific=False)
+# metadata
+LAST_RETRY_NET_LOOKUP       = Pref(key='lastRetryNetLookup', default=0, platformSpecific=False)
 # This doesn't need to be defined on the platform, but it can be overridden there if the platform wants to.
 SHOW_ERROR_DIALOG           = Pref(key='showErrorDialog',       default=True,  platformSpecific=True)
 
@@ -149,11 +164,11 @@ default_share = get_from_environ('DTV_SHARE_URL',
                                  u'http://www.miroguide.com/share')
 default_autoupdate = get_from_environ(
     'DTV_AUTOUPDATE_URL',
-    u'http://www.participatoryculture.org/democracy-appcast.xml')
+    u'http://miro-updates.participatoryculture.org/democracy-appcast.xml')
 
 default_autoupdate_beta = get_from_environ(
     'DTV_AUTOUPDATE_BETA_URL',
-    u'http://www.participatoryculture.org/democracy-appcast-beta.xml')
+    u'http://miro-updates.participatoryculture.org/democracy-appcast-beta.xml')
 
 
 CHANNEL_GUIDE_URL = Pref(key='ChannelGuideURL', default=default_guide,
@@ -192,6 +207,9 @@ TRANSLATE_URL     = Pref(key='TranslateURL', default=u"http://develop.participat
                          platformSpecific=False)
 PLANET_URL        = Pref(key='PlanetURL', default=u"http://planet.getmiro.com/",
                          platformSpecific=False)
+# TODO: should be set to False by default
+NET_LOOKUP_BY_DEFAULT  = Pref(key='UseInternetLookupForNew', default=False,
+                              platformSpecific=False)
 
 # These can be safely ignored on platforms without minimize to tray
 MINIMIZE_TO_TRAY = \
@@ -219,6 +237,8 @@ LOG_PATHNAME = \
     Pref(key='LogPathname',      default=None, platformSpecific=True)
 DOWNLOADER_LOG_PATHNAME = \
     Pref(key='DownloaderLogPathname', default=None, platformSpecific=True)
+HELPER_LOG_PATHNAME = \
+    Pref(key='HelperLogPathname', default=None, platformSpecific=True)
 GETTEXT_PATHNAME = \
     Pref(key='GetTextPathname', default=None, platformSpecific=True)
 ENABLED_EXTENSIONS = \
