@@ -31,7 +31,7 @@
 
 In general, frontends should do the following to handle startup.
 FIXME
-    - (optional) call startup.install_movies_gone_handler()
+    - (optional) call startup.install_movies_directory_gone_handler()
     - Call startup.initialize()
     - Wait for either the 'startup-success', or 'startup-failure' signal
 """
@@ -45,6 +45,7 @@ import traceback
 import threading
 import time
 
+from miro import api
 from miro import app
 from miro import autodler
 from miro import autoupdate
@@ -56,6 +57,7 @@ from miro import databaselog
 from miro import databaseupgrade
 from miro import dbupgradeprogress
 from miro import dialogs
+from miro import donate
 from miro import downloader
 from miro import eventloop
 from miro import fileutil
@@ -65,35 +67,31 @@ from miro import httpclient
 from miro import iconcache
 from miro import item
 from miro import itemsource
-from miro import iteminfocache
 from miro import feed
 from miro import folder
 from miro import messages
 from miro import messagehandler
-from miro import metadataprogress
 from miro import models
-from miro import moviedata
 from miro import playlist
 from miro import prefs
 import miro.plat.resources
-from miro.plat.utils import setup_logging
-from miro.plat import config as platformcfg
+from miro.plat.utils import setup_logging, filename_to_unicode
 from miro import tabs
 from miro import theme
+from miro import threadcheck
 from miro import util
 from miro import searchengines
 from miro import storedatabase
 from miro import conversions
 from miro import devices
 from miro import sharing
-from miro import transcode
 from miro import workerprocess
 from miro.plat import devicetracker
 
 DEBUG_DB_MEM_USAGE = False
 mem_usage_test_event = threading.Event()
 
-class StartupError(Exception):
+class StartupError(StandardError):
     def __init__(self, summary, description):
         self.summary = summary
         self.description = description
@@ -111,9 +109,7 @@ def startup_function(func):
             else:
                 m = messages.FrontendQuit()
             m.send_to_frontend()
-        except (SystemExit, KeyboardInterrupt):
-            raise
-        except Exception, exc:
+        except StandardError, exc:
             # we do this so that we only kick up the database error
             # if it's a database-related exception AND the app has a
             # db attribute
@@ -165,31 +161,8 @@ def startup_function(func):
                 m.send_to_frontend()
     return wrapped
 
-def _movies_directory_gone_handler(callback):
-    """Default _movies_directory_gone_handler.  The frontend should
-    override this using the ``install_movies_directory_gone_handler``
-    function.
-    """
-    logging.error("Movies directory is gone -- no handler installed!")
-    eventloop.add_urgent_call(callback, "continuing startup")
-
-def install_movies_directory_gone_handler(callback):
-    global _movies_directory_gone_handler
-    _movies_directory_gone_handler = callback
-
-def _first_time_handler(callback):
-    """Default _first_time_handler.  The frontend should override this
-    using the ``install_first_time_handler`` function.
-    """
-    logging.error("First time -- no handler installed.")
-    eventloop.add_urgent_call(callback, "continuing startup")
-
-def install_first_time_handler(callback):
-    global _first_time_handler
-    _first_time_handler = callback
-
 def setup_global_feed(url, *args, **kwargs):
-    view = feed.Feed.make_view('origURL=?', (url,))
+    view = feed.Feed.make_view('orig_url=?', (url,))
     view_count = view.count()
     if view_count == 0:
         logging.info("Spawning global feed %s", url)
@@ -215,7 +188,8 @@ def initialize(themeName):
             app.debugmode = False
 
     # this is platform specific
-    setup_logging()
+    setup_logging(app.config.get(prefs.LOG_PATHNAME),
+                 main_process=True)
     # this is portable general
     util.setup_logging()
     app.controller = controller.Controller()
@@ -242,7 +216,7 @@ def startup():
     logging.info("Builder:    %s", app.config.get(prefs.BUILD_MACHINE))
     logging.info("Build Time: %s", app.config.get(prefs.BUILD_TIME))
     logging.info("Debugmode:  %s", app.debugmode)
-    eventloop.connect('thread-started', finish_startup)
+    eventloop.connect('thread-started', startup_for_frontend)
     logging.info("Reading HTTP Password list")
     httpauth.init()
     httpauth.restore_from_file()
@@ -264,8 +238,13 @@ def load_extensions():
     app.extension_manager.load_extensions()
 
 @startup_function
-def finish_startup(obj, thread):
-    database.set_thread(thread)
+def startup_for_frontend(obj, thread):
+    """Run the startup code needed to get the frontend started
+
+    This function should be kept as small as possible to ensure good startup
+    times.
+    """
+    threadcheck.set_eventloop_thread(thread)
     logging.info("Installing deleted file checker...")
     item.setup_deleted_checker()
     logging.info("Restoring database...")
@@ -288,6 +267,7 @@ def finish_startup(obj, thread):
     except storedatabase.UpgradeError:
         raise StartupError(None, None)
     database.initialize()
+    downloader.reset_download_stats()
     end = time.time()
     logging.timing("Database upgrade time: %.3f", end - start)
     if app.db.startup_version != app.db.current_version:
@@ -299,19 +279,11 @@ def finish_startup(obj, thread):
         util.db_mem_usage_test()
         mem_usage_test_event.set()
 
-    # MetadataProgressUpdater needs to be installed before ItemInfoCache,
-    # since ItemInfoCache may create items if it uses failsafe mode
-    app.metadata_progress_updater = metadataprogress.MetadataProgressUpdater()
-    app.item_info_cache = iteminfocache.ItemInfoCache()
-    app.item_info_cache.load()
     dbupgradeprogress.upgrade_end()
 
-    logging.info("Loading video converters...")
-    conversions.conversion_manager.startup()
-    app.device_manager = devices.DeviceManager()
-    app.device_tracker = devicetracker.DeviceTracker()
+    app.startup_timer.log_time("after db upgrade")
 
-    searchengines.create_engines()
+    app.icon_cache_updater = iconcache.IconCacheUpdater()
     setup_global_feeds()
     # call fix_database_inconsistencies() ASAP after the manual feed is set up
     fix_database_inconsistencies()
@@ -320,15 +292,13 @@ def finish_startup(obj, thread):
     logging.info("setup theme...")
     setup_theme()
     install_message_handler()
-    itemsource.setup_handlers()
-    downloader.init_controller()
 
-    # Call this late, after the message handlers have been installed.
-    app.sharing_tracker = sharing.SharingTracker()
     app.sharing_manager = sharing.SharingManager()
-    app.transcode_manager = transcode.TranscodeManager()
+    app.download_state_manager = downloader.DownloadStateManager()
+    item.setup_change_tracker()
+    item.setup_metadata_manager()
 
-    eventloop.add_urgent_call(check_firsttime, "check first time")
+    _startup_checker.run_checks()
 
 def fix_database_inconsistencies():
     item.fix_non_container_parents()
@@ -336,101 +306,214 @@ def fix_database_inconsistencies():
     playlist.fix_missing_item_ids()
     folder.fix_playlist_missing_item_ids()
 
-@startup_function
-def check_firsttime():
-    """Run the first time wizard if need be.
+class StartupChecker(object):
+    """Handles various checks at startup.
+
+    This class handles the first-time startup check and the movies directory
+    gone check.
+
+    This code is a bit weird because of the interplay between the frontend and
+    the backend.  The checks run in the backend, but if they fail then the
+    frontend needs to prompt the user to ask them what to do.  Also, neither
+    side is totally started up at this point.
     """
-    callback = lambda: eventloop.add_urgent_call(check_movies_gone,
-                                                 "check movies gone")
-    if is_first_time():
-        logging.info("First time run -- calling handler.")
-        _first_time_handler(callback)
-        return
+    def run_checks(self):
+        self.check_firsttime()
 
-    eventloop.add_urgent_call(check_movies_gone, "check movies gone")
-
-@startup_function
-def check_movies_gone():
-    """Checks to see if the movies directory is gone.
-    """
-
-    # callback is what the frontend will call if the user asks us to continue
-    callback = lambda: eventloop.add_urgent_call(fix_movies_gone,
-                                               "fix movies gone")
-
-    movies_dir = fileutil.expand_filename(app.config.get(
-        prefs.MOVIES_DIRECTORY))
-
-    # if the directory doesn't exist, create it.
-    if (not os.path.exists(movies_dir) and
-            should_create_movies_directory(movies_dir)):
-        try:
-            fileutil.makedirs(movies_dir)
-        except OSError:
-            logging.info(
-                "Movies directory can't be created -- calling handler")
-            # FIXME - this isn't technically correct, but it's probably
-            # close enough that a user can fix the issue and Miro can
-            # run happily.
-            _movies_directory_gone_handler(callback)
+    @startup_function
+    def check_firsttime(self):
+        """Run the first time wizard if need be.
+        """
+        callback = lambda: eventloop.add_urgent_call(self.check_movies_gone,
+                                                     "check movies gone")
+        if is_first_time():
+            logging.info("First time run -- calling handler.")
+            self.first_time_handler(callback)
             return
 
-    # make sure the directory is writeable
-    if not os.access(movies_dir, os.W_OK):
-        _movies_directory_gone_handler(callback)
-        return
+        self.check_movies_gone()
 
-    # make sure that the directory is populated if we've downloaded stuff to
-    # it
-    if is_movies_directory_gone():
-        logging.info("Movies directory is gone -- calling handler.")
-        _movies_directory_gone_handler(callback)
-        return
+    def first_time_handler(callback):
+        """Default handler for first-time startup
 
-    eventloop.add_urgent_call(finish_backend_startup, "reconnect downloaders")
+        install_first_time_handler() replaces this method with the
+        frontend-specific one.
+        """
+        logging.error("First time -- no handler installed.")
+        eventloop.add_urgent_call(callback, "continuing startup")
 
-@startup_function
-def fix_movies_gone():
-    app.config.set(prefs.MOVIES_DIRECTORY, platformcfg.get(
-        prefs.MOVIES_DIRECTORY))
-    eventloop.add_urgent_call(finish_backend_startup, "reconnect downloaders")
+    @startup_function
+    def check_movies_gone(self, check_unmounted=True):
+        """Checks to see if the movies directory is gone.
+        """
 
-@startup_function
-def finish_backend_startup():
-    """Last bit of startup required before we load the frontend.  """
-    # Uncomment the next line to test startup error handling
-    # raise StartupError("Test Error", "Startup Failed")
-    reconnect_downloaders()
-    guide.download_guides()
-    feed.remove_orphaned_feed_impls()
-    messages.StartupSuccess().send_to_frontend()
+        movies_dir = fileutil.expand_filename(app.config.get(
+            prefs.MOVIES_DIRECTORY))
+        movies_dir = filename_to_unicode(movies_dir)
+
+        # if the directory doesn't exist, create it.
+        if (not os.path.exists(movies_dir) and
+                should_create_movies_directory(movies_dir)):
+            try:
+                fileutil.makedirs(movies_dir)
+            except OSError:
+                logging.info("Movies directory can't be created -- calling handler")
+                # FIXME - this isn't technically correct, but it's probably
+                # close enough that a user can fix the issue and Miro can
+                # run happily.
+                msg = _("Permissions error: %(appname)s couldn't "
+                        "create the folder.",
+                        {"appname": app.config.get(prefs.SHORT_APP_NAME)})
+                self.movies_directory_gone_handler(msg, movies_dir)
+                return
+
+        # make sure the directory is writeable
+        if not os.access(movies_dir, os.W_OK):
+            logging.info("Can't write to movies directory -- calling handler")
+            msg = _("Permissions error: %(appname)s can't "
+                    "write to the folder.",
+                    {"appname": app.config.get(prefs.SHORT_APP_NAME)})
+            self.movies_directory_gone_handler(msg, movies_dir)
+            return
+
+        # make sure that the directory is populated if we've downloaded stuff to
+        # it
+        if check_unmounted and check_movies_directory_unmounted():
+            logging.info("Movies directory is gone -- calling handler.")
+            msg = _("The folder contains no files: "
+                    "is it on a drive that's disconnected?")
+            self.movies_directory_gone_handler(msg, movies_dir,
+                                               allow_continue=True)
+            return
+
+        self.all_checks_done()
+
+    def movies_directory_gone_handler(self, message, movies_dir,
+                                      allow_continue=False):
+        """Default movies_directory_gone_handler.
+
+        This method simply quits when the movies directory is gone.
+        install_movies_directory_gone_handler() replaces this method with the
+        frontend-specific one.
+
+        present them with the following options:
+            - quit
+            - change movies directory
+            - continue with current directory (if allow_continue is True)
+
+        After the user picks, the frontend should call either
+        app.controller.shutdown() or startup.fix_movies_gone()
+        """
+        logging.error("Movies directory is gone -- no handler installed!")
+        app.controller.shutdown()
+
+    @startup_function
+    def fix_movies_gone(self, new_movies_directory):
+        """Called by the movies directory gone handler to fix the issue.
+
+        :param new_movies_directory: new path for the movies directory, or
+        None if we should continue with the current directory.
+        """
+        if new_movies_directory is not None:
+            app.config.set(prefs.MOVIES_DIRECTORY, new_movies_directory)
+        # do another check to make sure the selected directory works.  Here we
+        # skip the unmounted check, since it's not exact and the user is
+        # giving us a directory.
+        self.check_movies_gone(check_unmounted=False)
+
+    def all_checks_done(self):
+        # Uncomment the next line to test startup error handling
+        # raise StartupError("Test Error", "Startup Failed")
+        app.startup_timer.log_time("sending StartupSuccess()")
+        messages.StartupSuccess().send_to_frontend()
+
+_startup_checker = StartupChecker()
+
+def install_movies_directory_gone_handler(callback):
+    """Install a function to handle the movies directory being gone
+
+    The frontend should call this method and pass it a callback to handle this
+    situation.  The signature is (message, movies_dir, allow_continue=False).
+    The callback should present the user with the following options:
+        - quit
+        - change movies directory
+        - continue with current directory (if allow_continue is True)
+
+    After the user picks, the callback should call either
+    app.controller.shutdown() or startup.fix_movies_gone()
+    """
+    _startup_checker.movies_directory_gone_handler = callback
+
+def install_first_time_handler(callback):
+    """Install a function to handle first-time startup
+
+    If the frontend wants, it can pass a callback that shows a dialog to the
+    user on first-time startup.  The function will be passed a single argument
+    which is a callback function to call once the dialog completes.
+    """
+    _startup_checker.first_time_handler = callback
+
+def fix_movies_gone(new_movies_directory):
+    """Called by the movies directory gone handler to fix the issue.
+
+    :param new_movies_directory: new path for the movies directory, or None if
+    we should continue with the current directory.
+    """
+    eventloop.add_urgent_call(_startup_checker.fix_movies_gone,
+                              "fix movies gone",
+                              args=(new_movies_directory,))
 
 @eventloop.idle_iterator
 def on_frontend_started():
     """Perform startup actions that should happen after the frontend is
     already up and running.
+
+    This function happens using an idle iterator.  Before/after code that
+    could take a while to run, we yield to other eventloop callbacks.
     """
+    conversions.conversion_manager.startup()
+
+    app.sharing_tracker = sharing.SharingTracker()
+    app.sharing_manager.startup()
+    app.sharing_tracker.start_tracking()
+
+    app.device_manager = devices.DeviceManager()
+    app.device_tracker = devicetracker.DeviceTracker()
+    app.device_tracker.start_tracking()
+
+    reconnect_downloaders()
+    guide.download_guides()
+    feed.remove_orphaned_feed_impls()
+
+    app.download_state_manager.init_controller()
+    itemsource.setup_handlers()
+    if app.frontend_name == 'widgets':
+        app.donate_manager = donate.DonateManager()
+    else:
+        logging.warn("frontend is %s, not starting DonateManager()",
+                     app.frontend_name)
+
     logging.info("Starting auto downloader...")
     autodler.start_downloader()
+    app.icon_cache_updater.start_updates()
     yield None
     feed.expire_items()
     yield None
-    moviedata.movie_data_updater.start_thread()
-    yield None
     commandline.startup()
     yield None
     autoupdate.check_for_updates()
     yield None
+    app.local_metadata_manager.schedule_retry_net_lookup()
     # Delay running high CPU/IO operations for a bit
-    eventloop.add_timeout(5, downloader.startup_downloader,
+    eventloop.add_timeout(5, app.download_state_manager.startup_downloader,
             "start downloader daemon")
     eventloop.add_timeout(10, workerprocess.startup,
             "start worker process")
     eventloop.add_timeout(20, item.start_deleted_checker,
             "start checking deleted items")
     eventloop.add_timeout(30, feed.start_updates, "start feed updates")
-    eventloop.add_timeout(60, item.update_incomplete_movie_data,
-            "update movie data")
+    eventloop.add_timeout(60, item.update_incomplete_metadata,
+            "update metadata data")
     eventloop.add_timeout(90, clear_icon_cache_orphans, "clear orphans")
 
 def setup_global_feeds():
@@ -470,16 +553,16 @@ def mark_first_time():
 
 def should_create_movies_directory(path):
     """Figure out if we should create the movies directory if it's missing."""
-    if sys.platform == 'darwin' and path.startswith("/Volumes/"):
-        # Hack to fix #17826.  Don't try to create new directories in the
-        # mount points on OS X.
-        return False
-    return True
+    # We should only do this if the directory is the default directory.  This
+    # avoids trying to create files on unmonted filesystems (#17826)
+    return path == app.config.get_platform_default(prefs.MOVIES_DIRECTORY)
 
-def is_movies_directory_gone():
-    """Checks to see if the MOVIES_DIRECTORY exists.
+def check_movies_directory_unmounted():
+    """Checks to see MOVIES_DIRECTORY has been unmounted.
 
-    Returns True if yes, False if no.
+    Our hueristic is to check if there are any files in the directory.  If
+    it's totally empty, and we think that we should have a downloaded file in
+    it, then we return True.
     """
     movies_dir = fileutil.expand_filename(app.config.get(
         prefs.MOVIES_DIRECTORY))
@@ -594,6 +677,7 @@ def reconnect_downloaders():
         downloader_.remove()
     manualItems = item.Item.feed_view(feed.Feed.get_manual_feed().get_id())
     for item_ in manualItems:
-        if item_.downloader is None and item_.__class__ == item.Item:
+        if (item_.__class__ == item.Item and not item_.has_downloader() and
+          not item_.pending_manual_download):
             logging.warn("removing cancelled external torrent: %s", item_)
             item_.remove()
