@@ -203,68 +203,6 @@ Instead, astronauts have other options.</description>
         self.assertEqual(len(items), 1)
         my_feed.remove()
 
-class MultiFeedExpireTest(FeedTestCase):
-    def write_files(self, subfeed_count, feed_item_count):
-        all_urls = []
-        self.filenames = []
-
-        content = self.make_feed_content(feed_item_count)
-        for i in xrange(subfeed_count):
-            filename = self.make_temp_path()
-            open(filename, 'wb').write(content)
-            all_urls.append(u"file://%s" % filename)
-            self.filenames.append(filename)
-
-        self.url = u'dtv:multi:' + ','.join(all_urls) + "," + 'testquery'
-
-    def rewrite_files(self, feed_item_count):
-        content = self.make_feed_content(feed_item_count)
-        for filename in self.filenames:
-            open(filename, 'wb').write(content)
-
-    def make_feed_content(self, entry_count):
-        # make a feed with a new item and parse it
-        items = []
-        counter = 0
-
-        items.append("""<?xml version="1.0"?>
-<rss version="2.0">
-   <channel>
-      <title>Downhill Battle Pics</title>
-      <link>http://downhillbattle.org/</link>
-      <description>Downhill Battle is a non-profit organization working to \
-support participatory culture and build a fairer music industry.</description>
-      <pubDate>Wed, 16 Mar 2005 12:03:42 EST</pubDate>
-""")
-
-        for x in range(entry_count):
-            counter += 1
-            items.append("""\
-<item>
- <title>Bumper Sticker</title>
- <guid>guid-%s</guid>
- <enclosure url="http://downhillbattle.org/key/gallery/%s.mpg" />
- <description>I'm a musician and I support filesharing.</description>
-</item>
-""" % (counter, counter))
-
-        items.append("""
-   </channel>
-</rss>""")
-        return "".join(items)
-
-    def test_multi_feed_expire(self):
-        # test what happens when a RSSMultiFeed has feeds that
-        # reference the same item, and they are truncated at the same
-        # time (#11756)
-
-        self.write_files(5, 10) # 5 feeds containing 10 identical items
-        self.feed = self.make_feed()
-        app.config.set(prefs.TRUNCATE_CHANNEL_AFTER_X_ITEMS, 4)
-        app.config.set(prefs.MAX_OLD_ITEMS_DEFAULT, 5)
-        self.rewrite_files(1) # now only 5 items in each feed
-        self.update_feed(self.feed)
-
 class EnclosureFeedTestCase(FeedTestCase):
     def setUp(self):
         FeedTestCase.setUp(self)
@@ -523,12 +461,5 @@ by-nd/1.0</creativeCommons:license>
         self.save_then_restore_db()
         self.assertEquals(self.item.get_rss_id(), None)
 
-    def test_change_title(self):
-        entry = self.parsed_feed.entries[0]
-        self.item.set_title(u"new title")
-        self.save_then_restore_db()
-        self.assertEquals(self.item.get_title(), "new title")
-
-
 if __name__ == "__main__":
     unittest.main()
