@@ -1,33 +1,47 @@
+import datetime
+import contextlib
 import os
 import logging
+import random
 import unittest
 import tempfile
 import threading
 import shutil
 import functools
 
+from miro import api
 from miro import app
 from miro import config
+from miro import data
 from miro import database
+from miro import devices
 from miro import eventloop
+from miro import extensionmanager
 from miro import feed
-from miro import feedparserutil
 from miro import downloader
 from miro import httpauth
 from miro import httpclient
+from miro import iconcache
 from miro import item
-from miro import iteminfocache
-from miro import moviedata
+from miro import itemsource
+from miro import messages
 from miro import util
 from miro import prefs
+from miro import schema
 from miro import searchengines
+from miro import sharing
 from miro import signals
 from miro import storedatabase
+from miro import threadcheck
 from time import sleep
 from miro import models
 from miro import workerprocess
+from miro.data import itemtrack
+from miro.fileobject import FilenameType
 
+from miro.test import mock
 from miro.test import testhttpserver
+from miro.test import testobjects
 
 util.setup_logging()
 
@@ -46,6 +60,14 @@ util.setup_logging()
 
 import sys
 
+class MatchAny(object):
+    """Object that matches anything.
+
+    Useful for creating a wildcard when calling Mock.assert_called_with().
+    """
+    def __eq__(self, other):
+        return True
+
 VALID_PLATFORMS = ['linux', 'win32', 'osx']
 PLATFORM_MAP = {
     'osx': 'osx',
@@ -136,7 +158,7 @@ def skip_for_platforms(*platforms):
     else:
         return identity
 
-class HadToStopEventLoop(Exception):
+class HadToStopEventLoop(StandardError):
     pass
 
 class DummyMainFrame:
@@ -180,6 +202,8 @@ class DummyController:
     def failed_soft(self, when, details, with_exception=False):
         # FIXME: should have some way to make this turn into an exception
         if not self.failed_soft_okay:
+            print "failed_soft called in DummyController"
+            print details
             raise AssertionError("failed_soft called in DummyController")
         self.failed_soft_count += 1
 
@@ -195,7 +219,7 @@ def uses_httpclient(fun):
         # if there's already a curl_manager, then this is probably
         # being called in a nested context, so this iteration is not
         # in charge of starting and stopping the httpclient
-        if httpclient.curl_manager:
+        if not isinstance(httpclient.curl_manager, mock.Mock):
             return fun(*args, **kwargs)
 
         httpclient.start_thread()
@@ -214,20 +238,28 @@ def decorate_all_tests(class_dict, bases, decorator):
             if name.startswith("test"):
                 class_dict[name] = decorator(getattr(cls, name))
 
+class UnexpectedLogError(Exception):
+    # Note: this purposely doesn't subclass StandardError, since we don't want
+    # code to catch this one
+    pass
+
 class LogFilter(logging.Filter):
     """Log filter that turns logging messages into exceptions."""
 
     def __init__(self):
-        self.exception_level = logging.CRITICAL
+        self.exception_level = logging.WARN
         self.records = []
+        self.raised_error = False
 
     def set_exception_level(self, level):
         """Set the min logging level where we should throw an exception"""
         self.exception_level = level
 
     def filter(self, record):
-        if record.levelno >= self.exception_level:
-            raise AssertionError("Unexpected logging: %s" % record)
+        if record.levelno >= self.exception_level and not self.raised_error:
+            self.raised_error = True
+            raise UnexpectedLogError("Unexpected logging: %s" %
+                                     logging.Formatter().format(record))
         else:
             self.records.append(record)
             return False
@@ -242,17 +274,6 @@ class LogFilter(logging.Filter):
         for rec in self.records:
             assert rec.levelno == level
 
-class FakeMetadataProgressUpdater(object):
-    def __init__(self):
-        self.paths_processed = set()
-
-    def path_processed(self, path):
-        self.paths_processed.add(path)
-
-    def will_process_path(self, path):
-        # we should test this, but for now it's just a stub
-        pass
-
 class MiroTestCase(unittest.TestCase):
     def setUp(self):
         self.setup_log_filter()
@@ -262,26 +283,35 @@ class MiroTestCase(unittest.TestCase):
         self.setup_downloader_log()
         models.initialize()
         app.in_unit_tests = True
+        app.device_manager = devices.DeviceManager()
+        models.Item._path_count_tracker.reset()
+        testobjects.test_started(self)
         # Tweak Item to allow us to make up fake paths for FileItems
         models.Item._allow_nonexistent_paths = True
         # setup the deleted file checker
         item.setup_deleted_checker()
         item.start_deleted_checker()
-        # setup movie data stuff
-        self.metadata_progress_updater = FakeMetadataProgressUpdater()
-        app.metadata_progress_updater = self.metadata_progress_updater
-        moviedata.movie_data_updater = moviedata.MovieDataUpdater()
         # Skip worker proccess for feedparser
         feed._RUN_FEED_PARSER_INLINE = True
+        signals.system.connect('new-dialog', self.handle_new_dialog)
         # reload config and initialize it to temprary
         config.load_temporary()
+        self.setup_config_watcher()
         self.platform = app.config.get(prefs.APP_PLATFORM)
-        database.set_thread(threading.currentThread())
-        database.setup_managers()
+        self.set_temp_support_directory()
+        # setup icon_cache_updater
+        app.icon_cache_updater = iconcache.IconCacheUpdater()
+        # for the unittests, both the database code and any UI code should run
+        # in the main thread.
+        threadcheck.set_eventloop_thread(threading.currentThread())
+        threadcheck.set_ui_thread(threading.currentThread())
         self.raise_db_load_errors = True
         app.db = None
+        self.allow_db_upgrade_error_dialog = False
         self.reload_database()
-        self.setup_new_item_info_cache()
+        self.setup_dummy_message_handlers()
+        self.setup_dummy_curl_manager()
+        item.setup_metadata_manager(self.tempdir)
         searchengines._engines = [
             searchengines.SearchEngineInfo(u"all", u"Search All", u"", -1)
             ]
@@ -295,23 +325,55 @@ class MiroTestCase(unittest.TestCase):
         httpauth.init()
         # reset any logging records from our setUp call()
         self.log_filter.reset_records()
+        # create an extension manager that searches our tempdir for extensions
+        # NOTE: this doesn't actually load any extensions, since the directory
+        # is currently empty.  If you want to use the ExtensionManager you
+        # need to put a .miroext file in the tempdir then call
+        # app.extension_manager.load_extension()
+        app.extension_manager = extensionmanager.ExtensionManager(
+                [self.tempdir], [])
+        # Create a download state object (but don't start the downloader
+        # for the individual test unless necessary.  In this case we override
+        # the class to run the downloader).
+        app.download_state_manager = downloader.DownloadStateManager()
+        self.mock_dldaemon = mock.Mock()
+        downloader.RemoteDownloader.dldaemon = self.mock_dldaemon
+        self.mock_patchers = []
+
+    def setup_config_watcher(self):
+        app.backend_config_watcher = config.ConfigWatcher(
+            lambda func, *args: func(*args))
+
+    def set_temp_support_directory(self):
+        self.sandbox_support_directory = os.path.join(self.tempdir, 'support')
+        if not os.path.exists(self.sandbox_support_directory):
+            os.makedirs(self.sandbox_support_directory)
+        app.config.set(prefs.SUPPORT_DIRECTORY, self.sandbox_support_directory)
 
     def on_windows(self):
         return self.platform == "windows"
 
     def tearDown(self):
+        testobjects.test_stopped(self)
+        for patcher in self.mock_patchers:
+            patcher.stop()
+        self.destroy_connection_pools()
         # shutdown workerprocess if we started it for some reason.
         workerprocess.shutdown()
+        workerprocess._subprocess_manager = \
+                workerprocess.WorkerSubprocessManager()
+        workerprocess._miro_task_queue.reset()
         self.reset_log_filter()
         signals.system.disconnect_all()
         util.chatter = True
         self.stop_http_server()
-        del app.metadata_progress_updater
+
+        # unload extensions
+        self.unload_extensions()
 
         # Remove any leftover database
         app.db.close()
         app.db = None
-        database.setup_managers()
 
         # Remove anything that may have been accidentally queued up
         eventloop._eventloop = eventloop.EventLoop()
@@ -319,6 +381,57 @@ class MiroTestCase(unittest.TestCase):
         # Remove tempdir
         shutil.rmtree(self.tempdir, onerror=self._on_rmtree_error)
 
+    def destroy_connection_pools(self):
+        connection_pools = app.connection_pools
+        if connection_pools is not None:
+            for pool in connection_pools.get_all_pools():
+                pool.destroy()
+            app.connection_pools = None
+
+    def handle_new_dialog(self, obj, dialog):
+        """Handle the new-dialog signal
+
+        Subclasses must implement this if they expect to see a dialog.
+        """
+        raise AssertionError("Unexpected dialog: %s" % dialog)
+
+    def patch_function(self, function_name, new_function):
+        """Use Mock to replace an existing function for a single test.
+
+        function_name should be in the form "full.module.name.object".  For
+        example "miro.startup.startup"
+
+        This can also be used on a class object in order to return a different
+        object, if we only use class objects as factory functions.
+
+        :param function_name: name of the function to patch
+        :param new_function: function object to replace it with
+        :returns: Mock object used to patch
+        """
+        mock_object = self.patch_for_test(function_name)
+        mock_object.side_effect = new_function
+        return mock_object
+
+    def patch_for_test(self, object_name, autospec=True):
+        """Use Mock to replace a function/class/object with a mock object.
+
+        We will unpatch the object during teardown
+
+        :param object_name: name of the object to patch
+        :param autospec: autospec parameter to pass to mock.patch.  Generally
+        this should be True, except for class methods.
+        :returns: Mock object used to patch
+        """
+        patcher = mock.patch(object_name, autospec=autospec)
+        mock_object = patcher.start()
+        self.mock_patchers.append(patcher)
+        return mock_object
+
+    def unload_extensions(self):
+        for ext in app.extension_manager.extensions:
+            if ext.loaded:
+                app.extension_manager.unload_extension(ext)
+
     def setup_log_filter(self):
         """Make a LogFilter that will turn loggings into exceptions."""
         logger = logging.getLogger()
@@ -328,6 +441,15 @@ class MiroTestCase(unittest.TestCase):
         self.log_filter = LogFilter()
         logger.addFilter(self.log_filter)
 
+    @contextlib.contextmanager
+    def allow_warnings(self):
+        """Context manager to allow log warnings go through without triggering
+        a unittest error
+        """
+        self.log_filter.set_exception_level(logging.CRITICAL)
+        yield
+        self.log_filter.set_exception_level(logging.WARN)
+
     def reset_log_filter(self):
         logger = logging.getLogger()
         for old_filter in logger.filters:
@@ -336,6 +458,9 @@ class MiroTestCase(unittest.TestCase):
         # tearDown call.
         logger.setLevel(logging.ERROR)
 
+    def log_messages(self):
+        return '\n'.join(m.getMessage() for m in self.log_filter.records)
+
     def _on_rmtree_error(self, func, path, excinfo):
         global FILES_TO_CLEAN_UP
         FILES_TO_CLEAN_UP.append(path)
@@ -384,9 +509,26 @@ class MiroTestCase(unittest.TestCase):
             self.httpserver.stop()
             self.httpserver = None
 
-    def setup_new_item_info_cache(self):
-        app.item_info_cache = iteminfocache.ItemInfoCache()
-        app.item_info_cache.load()
+    def setup_dummy_message_handlers(self):
+        messages.FrontendMessage.handler = mock.Mock()
+        messages.BackendMessage.handler = mock.Mock()
+
+    def setup_dummy_curl_manager(self):
+        httpclient.curl_manager = mock.Mock()
+
+    def get_backend_messages(self, reset_mock=True):
+        msg_list = [args[0] for args, kwargs in
+                messages.BackendMessage.handler.handle.call_args_list]
+        if reset_mock:
+            messages.BackendMessage.handler.handle.reset_mock()
+        return msg_list
+
+    def get_frontend_messages(self, reset_mock=True):
+        msg_list = [args[0] for args, kwargs in
+                messages.FrontendMessage.handler.handle.call_args_list]
+        if reset_mock:
+            messages.FrontendMessage.handler.handle.reset_mock()
+        return msg_list
 
     def reset_failed_soft_count(self):
         app.controller.failed_soft_count = 0
@@ -394,22 +536,45 @@ class MiroTestCase(unittest.TestCase):
     def check_failed_soft_count(self, count):
         self.assertEquals(app.controller.failed_soft_count, count)
 
-    def reload_database(self, path=':memory:', schema_version=None,
-                        object_schemas=None, upgrade=True):
+    def reload_database(self, path=':memory:', upgrade=True, **kwargs):
         self.shutdown_database()
-        self.setup_new_database(path, schema_version, object_schemas)
+        self.setup_new_database(path, **kwargs)
         if upgrade:
-            app.db.upgrade_database()
-            database.update_last_id()
+            if self.allow_db_upgrade_error_dialog:
+                # this means that exceptions in the upgrade will be sent to a
+                # dialog box.  Be careful with this, if you don't handle the
+                # dialog, then the unit tests will hang.
+                app.db.upgrade_database(context='main')
+            else:
+                # normal case: use _upgrade_database() because we want
+                # exceptions to keep propagating
+                app.db._upgrade_database(context='main')
+        item.setup_change_tracker()
+        database.initialize()
+
+    def init_data_package(self):
+        """Initialize the data package
+
+        The data package is used by the frontend to get data.
+
+        Note: Since data uses a different connection than the backend system
+        (storedatabase and friends), we need to create an on-disk database.
+        """
+        self.db_path = self.make_temp_path(".sqlite")
+        if os.path.exists(self.db_path):
+            os.unlink(self.db_path)
+        self.reload_database(FilenameType(self.db_path))
+        data.init(self.db_path)
+        # use a mock objects for the database error handler
+        app.db_error_handler = mock.Mock()
 
     def clear_ddb_object_cache(self):
         app.db._ids_loaded = set()
         app.db._object_map = {}
+        app.db.cache = storedatabase.DatabaseObjectCache()
 
-    def setup_new_database(self, path, schema_version, object_schemas):
-        app.db = storedatabase.LiveStorage(path,
-                                           schema_version=schema_version,
-                                           object_schemas=object_schemas)
+    def setup_new_database(self, path, **kwargs):
+        app.db = storedatabase.LiveStorage(path, **kwargs)
         app.db.raise_load_errors = self.raise_db_load_errors
 
     def allow_db_load_errors(self, allow):
@@ -424,8 +589,9 @@ class MiroTestCase(unittest.TestCase):
 
     def reload_object(self, obj):
         # force an object to be reloaded from the databas.
-        del app.db._object_map[obj.id]
-        app.db._ids_loaded.remove(obj.id)
+        key = (obj.id, app.db.table_name(obj.__class__))
+        del app.db._object_map[key]
+        app.db._ids_loaded.remove(key)
         return obj.__class__.get_by_id(obj.id)
 
     def handle_error(self, obj, report):
@@ -446,8 +612,22 @@ class MiroTestCase(unittest.TestCase):
         self.assertSameSet(dict1.keys(), dict2.keys())
         for k in dict1:
             if not dict1[k] == dict2[k]:
-                raise AssertionError("Values differ for key %s: %s -- %s",
-                        k, dict1[k], dict2[k])
+                raise AssertionError("Values differ for key %r: %r -- %r" %
+                        (k, dict1[k], dict2[k]))
+
+    def assertClose(self, value1, value2, tolerance=0.1):
+        """Assert that 2 values are near each other.
+
+        :param value1: value to compare
+        :param value2: value to compare
+        :param tolerance: how different the two can be
+        """
+
+        difference = abs(value1 - value2)
+        relative_difference = difference / max(abs(value1), abs(value2))
+        if relative_difference > tolerance:
+            raise AssertionError("Difference too big: %s, %s" % (value1,
+                                                                 value2))
 
 class EventLoopTest(MiroTestCase):
     def setUp(self):
@@ -494,6 +674,19 @@ class EventLoopTest(MiroTestCase):
         finally:
             eventloop.thread_pool_quit()
 
+    def run_idles_for_this_loop(self):
+        idle_queue = eventloop._eventloop.idle_queue
+        urgent_queue = eventloop._eventloop.urgent_queue
+        while idle_queue.has_pending_idle() or urgent_queue.has_pending_idle():
+            if urgent_queue.has_pending_idle():
+                urgent_queue.process_idles()
+            if idle_queue.has_pending_idle():
+                idle_queue.process_next_idle()
+        # make sure that idles scheduled for the next loop run as well, but
+        # don't do this inside the while loop.
+        eventloop._eventloop._add_idles_for_next_loop()
+
+
     def run_pending_timeouts(self):
         scheduler = eventloop._eventloop.scheduler
         while scheduler.has_pending_timeout():
@@ -534,3 +727,49 @@ class DownloaderTestCase(EventLoopTest):
         downloader.shutdown_downloader(eventloop.shutdown)
         self.runEventLoop()
         EventLoopTest.tearDown(self)
+
+def dynamic_test(expected_cases=None):
+    """Class decorator for tests that use external test cases. This creates a
+    separate test method for each case; this makes it easier to debug tests that
+    go awry, makes it easier to see test progress from the command line (lots of
+    little tests rather than one big test), and increases the test count
+    appropriately::
+
+        class ExampleDynamicTest(object):
+            @classmethod
+            def generate_tests(cls):
+                # Should return an iterable of test cases. Each test case is an
+                # iterable of arguments to pass to the dynamic_test_case implementation.
+                #
+                # Test names will be created by stripping non-alphanumeric chars out of
+                # the value of the first arg in each test case, so the first arg should be
+                # a string that can be used to identify the test uniquely.
+                raise NotImplementedError
+
+            def dynamic_test_case(self, *args):
+                # This will be run once for each value produced by setup_tests. The
+                # iterables of values returned by generate_tests will be passed as
+                # arguments.
+                raise NotImplementedError
+    """
+
+    def _generate_closure(cls, args):
+        return lambda self: cls.dynamic_test_case(self, *args)
+
+    def wrap_class(cls):
+        generated_cases = 0
+        for test_args in cls.generate_tests():
+            test_name = ''.join(x for x in
+                                test_args[0].encode('ascii', 'ignore')
+                                if x.isalnum())
+            setattr(cls, 'test_%s_dyn' % test_name, _generate_closure(cls, test_args))
+            generated_cases += 1
+
+        if expected_cases is not None:
+            assert generated_cases == expected_cases, (
+                    "generated test count %d not equal to expected count %d for"
+                    " %s; if you have just added a test, update expected_cases"
+                    " (in the dynamic_test args); if not, there are test cases"
+                    " missing" % (generated_cases, expected_cases, cls.__name__))
+        return cls
+    return wrap_class
