@@ -1,9 +1,12 @@
 import functools
 import os
+import logging
 import pycurl
 import pickle
 from cStringIO import StringIO
 
+from miro import app
+from miro import prefs
 from miro import dialogs
 from miro import eventloop
 from miro import httpauth
@@ -483,21 +486,23 @@ class HTTPClientTest(HTTPClientTestBase):
 class HTTPAuthTest(HTTPClientTestBase):
     def setUp(self):
         HTTPClientTestBase.setUp(self)
-        self.callback_handle = None
         self.setup_cancel()
         self.dialogs_seen = 0
         self.dialog_callback = None
+        self.answer = None
 
     def setup_answer(self, username, password, button=dialogs.BUTTON_OK):
-        def handler(obj, dialog):
-            self.dialogs_seen += 1
-            if self.dialog_callback:
-                self.dialog_callback()
-            dialog.run_callback(button, unicode(username),
-                    unicode(password))
-        if self.callback_handle:
-            signals.system.disconnect(self.callback_handle)
-        self.callback_handle = signals.system.connect('new-dialog', handler)
+        self.answer = (unicode(username), unicode(password), button)
+
+    def handle_new_dialog(self, obj, dialog):
+        if self.answer is None:
+            HTTPClientTestBase.handle_new_dialog(self, obj, dialog)
+
+        self.dialogs_seen += 1
+        if self.dialog_callback:
+            self.dialog_callback()
+        username, password, button = self.answer
+        dialog.run_callback(button, username, password)
 
     def setup_cancel(self):
         self.setup_answer('', '', dialogs.BUTTON_CANCEL)
@@ -606,13 +611,17 @@ class HTTPAuthTest(HTTPClientTestBase):
         self.check_auth_canceled()
 
         self.setup_answer("wronguser", "wrongpass")
-        self.grab_url(self.httpserver.build_url('digest-protected/index.txt'))
+        with self.allow_warnings():
+            self.grab_url(
+                self.httpserver.build_url('digest-protected/index.txt'))
         self.check_auth_errback_called()
 
     @uses_httpclient
     def test_digest_auth_correct(self):
         self.setup_answer("user", "password")
-        self.grab_url(self.httpserver.build_url('digest-protected/index.txt'))
+        with self.allow_warnings():
+            self.grab_url(
+                self.httpserver.build_url('digest-protected/index.txt'))
         self.assertEquals(self.dialogs_seen, 1)
 
     @uses_httpclient
@@ -629,20 +638,29 @@ class HTTPAuthTest(HTTPClientTestBase):
     @uses_httpclient
     def test_digest_auth_memory(self):
         self.setup_answer("user", "password")
-        self.grab_url(self.httpserver.build_url('digest-protected/index.txt'))
+        with self.allow_warnings():
+            self.grab_url(
+                self.httpserver.build_url('digest-protected/index.txt'))
         self.assertEquals(self.dialogs_seen, 1)
         # We shouldn't see another dialog for the same URL
-        self.grab_url(self.httpserver.build_url('digest-protected/index.txt'))
+        with self.allow_warnings():
+            self.grab_url(
+                self.httpserver.build_url('digest-protected/index.txt'))
         self.assertEquals(self.dialogs_seen, 1)
         # ditto for ones in the same directory
-        self.grab_url(self.httpserver.build_url('digest-protected/index2.txt'))
+        with self.allow_warnings():
+            self.grab_url(
+                self.httpserver.build_url('digest-protected/index2.txt'))
         self.assertEquals(self.dialogs_seen, 1)
         # Even for ones in a subdirectory
-        self.grab_url(self.httpserver.build_url(
-                'digest-protected/foo/index2.txt'))
+        with self.allow_warnings():
+            self.grab_url(
+                self.httpserver.build_url('digest-protected/foo/index2.txt'))
         self.assertEquals(self.dialogs_seen, 1)
         # Or even for ones outside the directory (only true for digest auth)
-        self.grab_url(self.httpserver.build_url('digest-protected2/index.txt'))
+        with self.allow_warnings():
+            self.grab_url(
+                self.httpserver.build_url('digest-protected2/index.txt'))
         self.assertEquals(self.dialogs_seen, 1)
 
     @uses_httpclient
@@ -714,7 +732,8 @@ class HTTPAuthTest(HTTPClientTestBase):
         self.expecting_errback = True
 
         # test when schemes that aren't "basic" or "digest"
-        self.grab_url(self.httpserver.build_url('invalid-auth/badscheme'))
+        with self.allow_warnings():
+            self.grab_url(self.httpserver.build_url('invalid-auth/badscheme'))
         self.assert_(isinstance(self.grab_url_error,
             httpclient.AuthorizationFailed))
         # when we get invalid auth headers, we shouldn't pop up dialogs for
@@ -722,19 +741,22 @@ class HTTPAuthTest(HTTPClientTestBase):
         self.assertEquals(self.dialogs_seen, 0)
 
         # test auth header without realm
-        self.grab_url(self.httpserver.build_url('invalid-auth/norealm'))
+        with self.allow_warnings():
+            self.grab_url(self.httpserver.build_url('invalid-auth/norealm'))
         self.assert_(isinstance(self.grab_url_error,
             httpclient.AuthorizationFailed))
         self.assertEquals(self.dialogs_seen, 0)
 
         # test completely garbled auth header
-        self.grab_url(self.httpserver.build_url('invalid-auth/garbled'))
+        with self.allow_warnings():
+            self.grab_url(self.httpserver.build_url('invalid-auth/garbled'))
         self.assert_(isinstance(self.grab_url_error,
             httpclient.AuthorizationFailed))
         self.assertEquals(self.dialogs_seen, 0)
 
         # test auth header with no data
-        self.grab_url(self.httpserver.build_url('invalid-auth/'))
+        with self.allow_warnings():
+            self.grab_url(self.httpserver.build_url('invalid-auth/'))
         self.assert_(isinstance(self.grab_url_error,
             httpclient.AuthorizationFailed))
         self.assertEquals(self.dialogs_seen, 0)
@@ -748,10 +770,9 @@ class HTTPAuthBackendTest(EventLoopTest):
 
     def setUp(self):
         self.dialogs_seen = 0
-        signals.system.connect('new-dialog', self.handle_dialog)
         EventLoopTest.setUp(self)
 
-    def handle_dialog(self, obj, dialog):
+    def handle_new_dialog(self, obj, dialog):
         self.dialogs_seen += 1
         dialog.run_callback(dialogs.BUTTON_OK, u'user', u'password')
 
@@ -978,8 +999,15 @@ class NetworkErrorTest(HTTPClientTestBase):
     def test_connect_error(self):
         self.grab_url('http://255.255.255.255/')
         self.check_errback_called()
-        self.assert_(isinstance(self.grab_url_error,
-            httpclient.ConnectionError))
+        # The http proxy always replies with something, (probably a 40x or
+        # 50x message) so we should not check against this as it will
+        # be unreliable when the proxy is active.
+        if app.config.get(prefs.HTTP_PROXY_ACTIVE):
+            logging.info('Proxy active: skipping specific connection error '
+                         'check')
+        else:
+            self.assert_(isinstance(self.grab_url_error,
+                httpclient.ConnectionError))
 
     @uses_httpclient
     def test_closed_connection_error(self):
@@ -991,16 +1019,22 @@ class NetworkErrorTest(HTTPClientTestBase):
             httpclient.ServerClosedConnection))
 
     @uses_httpclient
-    def test_404_error(self):
+    def test_404_error(self, write_file=None):
         self.expecting_errback = True
         url = self.httpserver.build_url('badfile.txt')
-        self.grab_url(url)
+        self.grab_url(url, write_file=write_file)
         self.assert_(isinstance(self.grab_url_error,
             httpclient.UnexpectedStatusCode))
         # FIXME: It'd be nice if we could check a HTTP code rather than a
         # static message.
         self.assertEquals(self.grab_url_error.friendlyDescription,
                 _("File not found"))
+        if write_file:
+            self.assertEquals(open(write_file).read(), '')
+
+    def test_error_nofile(self):
+        write_file = self.make_temp_path(".txt")
+        self.test_404_error(write_file=write_file)
 
     @uses_mock_httpclient
     def test_bad_domain_name(self):
@@ -1029,7 +1063,8 @@ class NetworkErrorTest(HTTPClientTestBase):
         options = httpclient.TransferOptions("http://example.com/")
         bogus_transfer = httpclient.CurlTransfer(options,
                 self.grab_url_callback, self.grab_url_errback)
-        bogus_transfer.on_error(123456, BogusLibcurlHandle())
+        with self.allow_warnings():
+            bogus_transfer.on_error(123456, BogusLibcurlHandle())
         self.runPendingIdles()
 
         # Check that we saw a NetworkError and that the description strings
@@ -1038,5 +1073,22 @@ class NetworkErrorTest(HTTPClientTestBase):
         self.assert_(isinstance(self.grab_url_error,
             httpclient.NetworkError))
         self.assert_(isinstance(self.grab_url_error.longDescription, unicode))
-        self.assert_(isinstance(self.grab_url_error.friendlyDescription,
-            unicode))
+        self.assert_(isinstance(self.grab_url_error.friendlyDescription, unicode))
+
+class LimitProtocolTest(HTTPClientTestBase):
+    #test that we limit the protocols to HTTP and HTTPS
+    @uses_httpclient
+    def test_url_scheme(self):
+        self.expecting_errback = True
+        self.grab_url('ftp://ben@example.com/')
+        self.check_errback_called()
+        self.assert_(isinstance(self.grab_url_error, httpclient.MalformedURL))
+
+    @uses_httpclient
+    def test_redirect(self):
+        self.expecting_errback = True
+        self.httpserver.custom_redirect_url('ftp://ben@example.com/')
+        self.grab_url(self.httpserver.build_url('custom-redirect'))
+        self.check_errback_called()
+        self.assert_(isinstance(self.grab_url_error,
+                                httpclient.InvalidRedirect))
