@@ -1,11 +1,16 @@
 from datetime import datetime
 import os
 import unittest
+import string
+import random
 import time
 
+import sqlite3
+
 from miro import app
 from miro import database
 from miro import databaseupgrade
+from miro import devices
 from miro import dialogs
 from miro import downloader
 from miro import item
@@ -23,11 +28,13 @@ from miro import storedatabase
 from miro.plat import resources
 from miro.plat.utils import PlatformFilenameType
 
-from miro.test.framework import EventLoopTest, skip_for_platforms
+from miro.test import mock
+from miro.test.framework import (MiroTestCase, EventLoopTest,
+                                 skip_for_platforms, MatchAny)
 from miro.schema import (SchemaString, SchemaInt, SchemaFloat,
                          SchemaReprContainer, SchemaList, SchemaDict,
                          SchemaBool, SchemaFilename,
-                         SchemaBinary)
+                         SchemaBinary, SchemaStringSet)
 
 # create a dummy object schema
 class Human(database.DDBObject):
@@ -44,6 +51,7 @@ class Human(database.DDBObject):
             self.high_scores = high_scores
         self.stuff = stuff
         self.id_code = None
+        self.favorite_colors = set([u'red', u'blue'])
 
     def add_friend(self, friend):
         self.friends.append(friend)
@@ -85,6 +93,7 @@ class HumanSchema(schema.ObjectSchema):
         ('high_scores', SchemaDict(SchemaString(), SchemaInt())),
         ('stuff', SchemaReprContainer(noneOk=True)),
         ('id_code', SchemaBinary(noneOk=True)),
+        ('favorite_colors', SchemaStringSet(delimiter='@')),
     ]
 
     @staticmethod
@@ -145,7 +154,6 @@ class StoreDatabaseTest(EventLoopTest):
 
     def tearDown(self):
         # need to close the db before removing it from disk
-        from miro import app
         app.db.close()
         self.remove_database()
         corrupt_path = os.path.join(os.path.dirname(self.save_path),
@@ -158,7 +166,7 @@ class StoreDatabaseTest(EventLoopTest):
 class EmptyDBTest(StoreDatabaseTest):
     def test_open_empty_db(self):
         self.reload_test_database()
-        app.db.cursor.execute("SELECT name FROM sqlite_master "
+        app.db.cursor.execute("SELECT name FROM main.sqlite_master "
                 "WHERE type='table'")
         for row in app.db.cursor.fetchall():
             table = row[0]
@@ -181,33 +189,52 @@ class DBUpgradeTest(StoreDatabaseTest):
             pass
         StoreDatabaseTest.tearDown(self)
 
+    def load_fresh_database(self):
+        self.remove_database()
+        self.reload_database()
+        self.db = app.db
+
+    def load_upgraded_database(self):
+        shutil.copy(resources.path("testdata/olddatabase.v79"),
+                    self.save_path2)
+        self.reload_database(self.save_path2)
+        self.db = app.db
+
     @skip_for_platforms('win32')
     def test_indexes_same(self):
         # this fails on windows because it's using a non-Windows
         # database
-        self.remove_database()
-        self.reload_database()
-        app.db.cursor.execute("SELECT name FROM sqlite_master "
-                              "WHERE type='index'")
-        blank_db_indexes = set(app.db.cursor)
-        shutil.copy(resources.path("testdata/olddatabase.v79"),
-                    self.save_path2)
-        self.reload_database(self.save_path2)
-        app.db.cursor.execute("SELECT name FROM sqlite_master "
-                              "WHERE type='index'")
-        upgraded_db_indexes = set(app.db.cursor)
+        self.load_fresh_database()
+        self.db.cursor.execute("SELECT name FROM main.sqlite_master "
+                               "WHERE type='index'")
+        blank_db_indexes = set(self.db.cursor)
+        self.load_upgraded_database()
+        self.db.cursor.execute("SELECT name FROM main.sqlite_master "
+                               "WHERE type='index'")
+        upgraded_db_indexes = set(self.db.cursor)
+        self.assertEquals(upgraded_db_indexes, blank_db_indexes)
+
+    @skip_for_platforms('win32')
+    def test_triggers_same(self):
+        # this fails on windows because it's using a non-Windows
+        # database
+        self.load_fresh_database()
+        self.db.cursor.execute("SELECT name, sql FROM main.sqlite_master "
+                               "WHERE type='trigger'")
+        blank_db_indexes = set(self.db.cursor)
+        self.load_upgraded_database()
+        self.db.cursor.execute("SELECT name, sql FROM main.sqlite_master "
+                               "WHERE type='trigger'")
+        upgraded_db_indexes = set(self.db.cursor)
         self.assertEquals(upgraded_db_indexes, blank_db_indexes)
 
     @skip_for_platforms('win32')
     def test_schema_same(self):
         # this fails on windows because it's using a non-Windows
         # database
-        self.remove_database()
-        self.reload_database()
+        self.load_fresh_database()
         blank_column_types = self._get_column_types()
-        shutil.copy(resources.path("testdata/olddatabase.v79"),
-                    self.save_path2)
-        self.reload_database(self.save_path2)
+        self.load_upgraded_database()
         upgraded_column_types = self._get_column_types()
         self.assertEquals(set(blank_column_types.keys()),
                           set(upgraded_column_types.keys()))
@@ -219,14 +246,38 @@ class DBUpgradeTest(StoreDatabaseTest):
                                      (table_name, diff))
 
     def _get_column_types(self):
-        app.db.cursor.execute("SELECT name FROM sqlite_master "
+        self.db.cursor.execute("SELECT name FROM main.sqlite_master "
                               "WHERE type='table'")
         rv = {}
-        for table_name in app.db.cursor.fetchall():
-            app.db.cursor.execute('pragma table_info(%s)' % table_name)
-            rv[table_name] = set((r[1], r[2].lower()) for r in app.db.cursor)
+        for table_name in [r[0] for r in self.db.cursor.fetchall()]:
+            self.db.cursor.execute('pragma table_info(%s)' % table_name)
+            rv[table_name] = set((r[1], r[2].lower()) for r in self.db.cursor)
         return rv
 
+class DeviceDBUpgradeTest(DBUpgradeTest):
+    def setUp(self):
+        StoreDatabaseTest.setUp(self)
+        self.save_path2 = self.make_temp_path()
+
+    def tearDown(self):
+        try:
+            os.unlink(self.save_path2)
+        except OSError:
+            pass
+        StoreDatabaseTest.tearDown(self)
+
+    def load_fresh_database(self):
+        device_mount = self.make_temp_dir_path()
+        os.makedirs(os.path.join(device_mount, '.miro'))
+        self.db = devices.load_sqlite_database(device_mount, 1024)
+
+    def load_upgraded_database(self):
+        device_mount = self.make_temp_dir_path()
+        os.makedirs(os.path.join(device_mount, '.miro'))
+        shutil.copyfile(resources.path('testdata/5.x-device-database.sqlite'),
+                        os.path.join(device_mount, '.miro', 'sqlite'))
+        self.db = devices.load_sqlite_database(device_mount, 1024)
+
 class FakeSchemaTest(StoreDatabaseTest):
     OBJECT_SCHEMAS = test_object_schemas
 
@@ -243,6 +294,13 @@ class FakeSchemaTest(StoreDatabaseTest):
         databaseupgrade._upgrade_overide[2] = upgrade2
 
 class DiskTest(FakeSchemaTest):
+    def setUp(self):
+        FakeSchemaTest.setUp(self)
+        # should we handle upgrade error dialogs by clicking "start fresh"
+        self.handle_upgrade_error_dialogs = False
+        # should we handle database corrupt message boxes by clicking "OK?"
+        self.handle_corruption_dialogs = False
+
     def check_database(self):
         obj_map = {}
         for klass in (PCFProgramer, RestorableHuman, Human):
@@ -323,6 +381,7 @@ class DiskTest(FakeSchemaTest):
         self.reload_test_database()
         self.check_database()
         # check deleting the different class
+        im_special = self.reload_object(im_special)
         im_special.remove()
         self.db.pop()
         self.reload_test_database()
@@ -336,55 +395,74 @@ class DiskTest(FakeSchemaTest):
         self.setup_new_database(self.save_path, schema_version=0,
                 object_schemas=self.OBJECT_SCHEMAS)
         app.db.upgrade_database()
+        database.initialize()
         self.check_database()
 
     def test_upgrade(self):
         self.reload_test_database(version=1)
         new_lee = Human.get_by_id(self.lee.id)
         self.assertEquals(new_lee.name, 'new name')
+        # check that we created a backup file
+        backup_path = os.path.join(os.path.dirname(self.save_path),
+                                   'dbbackups', 'sqlitedb_backup_0')
+        if not os.path.exists(backup_path):
+            raise AssertionError("database backup doesn't exist")
+        # check that the backup has the old data
+        backup_conn = sqlite3.connect(backup_path)
+        cursor = backup_conn.execute("SELECT name FROM human WHERE id=?",
+                                     (self.lee.id,))
+        backup_lee_name = cursor.fetchone()[0]
+        self.assertEquals(backup_lee_name, 'lee')
 
     def test_restore_with_newer_version(self):
         self.reload_test_database(version=1)
         self.assertRaises(databaseupgrade.DatabaseTooNewError,
                 self.reload_test_database, version=0)
 
-    def test_last_id(self):
-        correct_last_id = database.DDBObject.lastID
-        database.DDBObject.lastID = 0
+    def test_make_new_id(self):
+        # Check that when we reload the database, the id counter stays the
+        # same
+        org_id = app.db_info.make_new_id()
+        # reload the database 
         self.reload_test_database()
-        self.assert_(database.DDBObject.lastID > 0)
-        self.assertEquals(database.DDBObject.lastID, correct_last_id)
+        self.assertEquals(app.db_info.make_new_id(), org_id)
 
     def check_reload_error(self, **reload_args):
         corrupt_path = os.path.join(os.path.dirname(self.save_path),
                                     'corrupt_database')
         self.assert_(not os.path.exists(corrupt_path))
         self.allow_db_load_errors(True)
-        self.reload_test_database(**reload_args)
+        with self.allow_warnings():
+            self.reload_test_database(**reload_args)
         self.allow_db_load_errors(False)
         self.assert_(os.path.exists(corrupt_path))
 
-    def handle_corrupt_db_dialogs(self, upgrade, corruption):
+    def handle_dialogs(self, upgrade, corruption):
         """Handle the dialogs that we pop up when we notice database errors.
 
         :param upgrade: handle upgrade dialogs by clicking "start fresh"
         :param corruption: handle database corrupt message boxes
         """
-        def dialog_handler(obj, dialog):
-            if upgrade and (dialogs.BUTTON_START_FRESH in dialog.buttons):
-                # handle database upgrade dialog
-                dialog.run_callback(dialogs.BUTTON_START_FRESH)
-            elif corruption and isinstance(dialog, dialogs.MessageBoxDialog):
-                # handle the load error dialog
-                dialog.run_callback(dialogs.BUTTON_OK)
-            else:
-                raise AssertionError("Don't know how to handle dialog: %s",
-                        dialog)
-        signals.system.connect('new-dialog', dialog_handler)
+        self.allow_db_upgrade_error_dialog = True
+        self.handle_upgrade_error_dialogs = upgrade
+        self.handle_corruption_dialogs = corruption
+
+    def handle_new_dialog(self, obj, dialog):
+        if (self.handle_upgrade_error_dialogs and 
+            (dialogs.BUTTON_START_FRESH in dialog.buttons)):
+            # handle database upgrade dialog
+            dialog.run_callback(dialogs.BUTTON_START_FRESH)
+        elif (self.handle_corruption_dialogs and
+              isinstance(dialog, dialogs.MessageBoxDialog)):
+            # handle the load error dialog
+            dialog.run_callback(dialogs.BUTTON_OK)
+        else:
+            return FakeSchemaTest.handle_new_dialog(self, obj, dialog)
 
     def test_upgrade_error(self):
-        self.handle_corrupt_db_dialogs(upgrade=True, corruption=False)
-        self.check_reload_error(version=2)
+        self.handle_dialogs(upgrade=True, corruption=False)
+        with self.allow_warnings():
+            self.check_reload_error(version=2)
 
     def test_corrupt_database(self):
         app.db.close()
@@ -392,12 +470,12 @@ class DiskTest(FakeSchemaTest):
         # depending on the SQLite version, we will notice the error when we
         # issup the PRAGMA journal_mode command, or when we do the upgrades.
         # Handle the dialogs for both.
-        self.handle_corrupt_db_dialogs(upgrade=True, corruption=True)
+        self.handle_dialogs(upgrade=True, corruption=True)
         self.check_reload_error()
 
     def test_database_data_error(self):
         app.db.cursor.execute("DROP TABLE human")
-        self.handle_corrupt_db_dialogs(upgrade=False, corruption=True)
+        self.handle_dialogs(upgrade=False, corruption=True)
         self.check_reload_error()
 
     def test_bulk_insert(self):
@@ -491,7 +569,8 @@ class ValidationTest(FakeSchemaTest):
         obj.signal_change()
 
     def assert_object_invalid(self, obj):
-        self.assertRaises(schema.ValidationError, obj.signal_change)
+        with self.allow_warnings():
+            self.assertRaises(schema.ValidationError, obj.signal_change)
 
     def test_none_values(self):
         self.lee.age = None
@@ -539,7 +618,8 @@ class CorruptReprTest(FakeSchemaTest):
     def test_repr_failure(self):
         app.db.cursor.execute("UPDATE human SET stuff='{baddata' "
                               "WHERE name='lee'")
-        restored_lee = self.reload_object(self.lee)
+        with self.allow_warnings():
+            restored_lee = self.reload_object(self.lee)
         self.assertEqual(restored_lee.stuff, 'testing123')
         app.db.cursor.execute("SELECT stuff from human WHERE name='lee'")
         row = app.db.cursor.fetchone()
@@ -548,21 +628,25 @@ class CorruptReprTest(FakeSchemaTest):
     def test_repr_failure_no_handler(self):
         app.db.cursor.execute("UPDATE pcf_programmer SET stuff='{baddata' "
                               "WHERE name='ben'")
-        self.assertRaises(SyntaxError, self.reload_object, self.ben)
+        with self.allow_warnings():
+            self.assertRaises(SyntaxError, self.reload_object, self.ben)
 
 class ConverterTest(StoreDatabaseTest):
     def test_convert_repr(self):
         converter = storedatabase.SQLiteConverter()
+        # _repr_to_sql ignores the schema_item parameter, so we can just pass
+        # in None
+        schema_item = None
 
         test1 = """{'updated_parsed': (2009, 6, 5, 1, 30, 0, 4, 156, 0)}"""
-        val = converter._convert_repr(test1)
+        val = converter._repr_from_sql(test1, schema_item)
         self.assertEquals(val, {"updated_parsed":
                                 (2009, 6, 5, 1, 30, 0, 4, 156, 0)})
 
         test2 = """{'updated_parsed': time.struct_time(tm_year=2009, \
 tm_mon=6, tm_mday=5, tm_hour=1, tm_min=30, tm_sec=0, tm_wday=4, tm_yday=156, \
 tm_isdst=0)}"""
-        val = converter._convert_repr(test2)
+        val = converter._repr_from_sql(test2, schema_item)
         self.assertEquals(val, {"updated_parsed":
                                 (2009, 6, 5, 1, 30, 0, 4, 156, 0)})
 
@@ -579,10 +663,11 @@ class CorruptDDBObjectReprTest(StoreDatabaseTest):
         self.tab_order = tabs.TabOrder(u'channel')
         self.guide = guide.ChannelGuide(u'http://example.com/')
         self.theme_hist = theme.ThemeHistory()
-        self.display_state = widgetstate.DisplayState((u'testtype', u'testid'))
+        self.view_state = widgetstate.ViewState((u'testtype', u'testid', 0))
 
     def check_fixed_value(self, obj, column_name, value, disk_value=None):
-        obj = self.reload_object(obj)
+        with self.allow_warnings():
+            obj = self.reload_object(obj)
         self.assertEquals(getattr(obj, column_name), value)
         # make sure the values stored on disk are correct as well
         if disk_value is None:
@@ -598,14 +683,6 @@ class CorruptDDBObjectReprTest(StoreDatabaseTest):
                 schema_item, disk_value)
         self.assertEqual(row[0], sql_value)
 
-    def test_corrupt_status(self):
-        app.db.cursor.execute("UPDATE remote_downloader "
-                "SET status='{baddata' WHERE id=?", (self.downloader.id,))
-        # setup_restored sets some values for status, so we will have
-        # more than an empty dict
-        self.check_fixed_value(self.downloader, 'status',
-                {'rate': 0, 'upRate': 0, 'eta': 0}, disk_value={})
-
     def test_corrupt_etag(self):
         app.db.cursor.execute("UPDATE saved_search_feed_impl "
                 "SET etag='{baddata' WHERE ufeed_id=?", (self.feed.id,))
@@ -619,7 +696,8 @@ class CorruptDDBObjectReprTest(StoreDatabaseTest):
     def test_corrupt_tab_ids(self):
         app.db.cursor.execute("UPDATE taborder_order "
                 "SET tab_ids='[1, 2; 3 ]' WHERE id=?", (self.tab_order.id,))
-        reloaded = self.reload_object(self.tab_order)
+        with self.allow_warnings():
+            reloaded = self.reload_object(self.tab_order)
         self.check_fixed_value(reloaded, 'tab_ids', [])
         # check that restore_tab_list() re-adds the tab ids
         reloaded.restore_tab_list()
@@ -636,17 +714,12 @@ class CorruptDDBObjectReprTest(StoreDatabaseTest):
                               (self.theme_hist.id,))
         self.check_fixed_value(self.theme_hist, 'pastThemes', [])
 
-    def test_corrupt_display_state(self):
-        app.db.cursor.execute("UPDATE display_state SET "
-                "active_filters=?, selected_view=?, "
-                "list_view_columns=?, list_view_widths=? WHERE id=?",
-                (5, 1, '{baddata', '{baddata', self.display_state.id))
-        self.check_fixed_value(self.display_state, 'list_view_columns', None)
-        self.check_fixed_value(self.display_state, 'list_view_widths', None)
-        # check that fields with valid values were salvaged
-        reloaded = self.reload_object(self.display_state)
-        self.assertEquals(reloaded.active_filters, 5)
-        self.assertEquals(reloaded.selected_view, 1)
+    def test_corrupt_view_state(self):
+        app.db.cursor.execute("UPDATE view_state SET "
+                "columns_enabled=?, column_widths=? WHERE id=?",
+                ('{baddata', '{baddata', self.view_state.id))
+        self.check_fixed_value(self.view_state, 'columns_enabled', None)
+        self.check_fixed_value(self.view_state, 'column_widths', None)
 
     def test_corrupt_link_history(self):
         # TODO: should test ScraperFeedIpml.linkHistory, but it's not so easy
@@ -682,7 +755,8 @@ class BadTabOrderTest(StoreDatabaseTest):
 
     def test_extra_tab_ids(self):
         self.screw_with_tab_order(self.f1.id, self.f2.id, self.folder.id, 123)
-        self.check_order(self.f1.id, self.f2.id, self.folder.id)
+        with self.allow_warnings():
+            self.check_order(self.f1.id, self.f2.id, self.folder.id)
 
     def test_order_wrong(self):
         self.f1.set_folder(self.folder)
@@ -692,5 +766,138 @@ class BadTabOrderTest(StoreDatabaseTest):
         self.screw_with_tab_order(self.f1.id, self.f2.id, self.folder.id)
         self.check_order(self.f2.id, self.folder.id, self.f1.id)
 
+class PreallocateTest(MiroTestCase):
+    def check_preallocate_size(self, path, preallocate_size):
+        disk_size = os.stat(path).st_size
+        # allow some variance for the disk size, we just need to be in the
+        # ballpark
+        self.assertClose(disk_size, preallocate_size)
+
+    def test_preallocate(self):
+        # test preallocating space
+        preallocate = 512 * 1024 # 512KB
+        path = os.path.join(self.tempdir, 'testdb')
+        storage = storedatabase.LiveStorage(path, preallocate=preallocate)
+        # check while open
+        self.check_preallocate_size(path, preallocate)
+        # check that it remains that big after close
+        storage.close()
+        self.check_preallocate_size(path, preallocate)
+
+class TemporaryModeTest(MiroTestCase):
+    # test getting an error when opening a new database and using an
+    # in-memory database to work around it
+
+    def setUp(self):
+        MiroTestCase.setUp(self)
+        self.save_path = os.path.join(self.tempdir, 'test-db')
+        # set up an error handler that tells LiveStorage to use temporary
+        # storage if it fails to open a new database
+        use_temp = storedatabase.LiveStorageErrorHandler.ACTION_USE_TEMPORARY
+        self.error_handler = mock.Mock()
+        self.error_handler.handle_open_error.return_value = use_temp
+
+        self.row_data = []
+
+        self.mock_add_timeout = mock.Mock()
+        self.patch_function('miro.eventloop.add_timeout',
+                            self.mock_add_timeout)
+
+        self.real_sqlite3_connect = sqlite3.connect
+        self.patch_function('sqlite3.connect', self.mock_sqlite3_connect)
+
+    def force_temporary_database(self):
+        """Open a new database and force it to fail.
+
+        After this we should be using an in-memory database and trying to save
+        it to disk every so often.
+        """
+
+        self.force_next_connect_to_fail = True
+        with self.allow_warnings():
+            self.reload_database(self.save_path,
+                                 object_schemas=test_object_schemas,
+                                 error_handler=self.error_handler)
+
+    def test_error_handler_called(self):
+        # Test that our error handler was called when storedatabase could'nt
+        # open the database
+        handle_open_error = self.error_handler.handle_open_error
+        self.force_temporary_database()
+        handle_open_error.assert_called_once_with()
+
+    def test_use_memory(self):
+        # Test that we use an in-memory database after failing to open a real
+        # one
+        self.force_temporary_database()
+        self.assertEquals(self.last_connect_path, ':memory:')
+        self.assert_(not os.path.exists(self.save_path))
+
+    def test_try_save_scheduling(self):
+        # test that we call add_timeout to schedule trying to save to the
+        # database
+        self.force_temporary_database()
+        delay = 300
+        self.mock_add_timeout.assert_called_once_with(
+            delay, app.db._try_save_temp_to_disk, MatchAny())
+        # Make the timeout run and fail.  Check that we schedule another try
+        self.mock_add_timeout.reset_mock()
+        self.force_next_connect_to_fail = True
+        with self.allow_warnings():
+            app.db._try_save_temp_to_disk()
+        self.mock_add_timeout.assert_called_once_with(
+            delay, app.db._try_save_temp_to_disk, MatchAny())
+        # make the timeout succeed.  Check that we don't schedule anymore
+        self.mock_add_timeout.reset_mock()
+        app.db._try_save_temp_to_disk()
+        self.assertEquals(self.mock_add_timeout.called, False)
+
+    def add_data(self, row_count):
+        for i in range(row_count):
+            age = 40
+            meters_tall = 2.5
+            name = u"Name-%s" % i
+            password = u"x" * i
+            Human(name, age, meters_tall, [], password=password)
+            self.row_data.append((name, age, meters_tall, password))
+
+    def check_data(self):
+        view = Human.make_view(order_by="name")
+        self.row_data.sort() # name is the 1st column, so this sorts by name
+        self.assertEquals(view.count(), len(self.row_data))
+        for i, h in enumerate(view):
+            self.assertEquals(h.name, self.row_data[i][0])
+            self.assertEquals(h.age, self.row_data[i][1])
+            self.assertEquals(h.meters_tall, self.row_data[i][2])
+            self.assertEquals(h.stuff, {'password': self.row_data[i][3]})
+
+    def test_data(self):
+        # test storing data in the temp database
+        self.force_temporary_database()
+        # add a bunch of fake data to the database
+        self.add_data(100)
+        # make the database save to its real path
+        with self.allow_warnings():
+            app.db._try_save_temp_to_disk()
+        # check that the data got saved to disk
+        self.assertEquals(self.last_connect_path, self.save_path)
+        self.assert_(os.path.exists(self.save_path))
+        # test that the data is correct immediately after change_path
+        self.check_data()
+        # add some more data
+        self.add_data(100)
+        # re-open the database and check that everything is correct
+        self.reload_database(self.save_path, object_schemas=test_object_schemas)
+        self.check_data()
+
+    def mock_sqlite3_connect(self, path, *args, **kwargs):
+        """Force the next call to sqlite3.connect to raise an exception.  """
+        if self.force_next_connect_to_fail:
+            self.force_next_connect_to_fail = False
+            raise sqlite3.OperationalError()
+        else:
+            self.last_connect_path = path
+            return self.real_sqlite3_connect(path, *args, **kwargs)
+
 if __name__ == '__main__':
     unittest.main()
