@@ -1,16 +1,22 @@
 # coding=latin-1
 # The above comment is required, because it includes non-latin characters
 # as an inline string in the source, we need to have this here as per PEP 263.
+import itertools
 import os
 import tempfile
+import time
+import signal
 import shutil
 import unittest
 import sys
+import zipfile
 
 from miro.test.framework import skip_for_platforms, MiroTestCase
 from miro import download_utils
 from miro import util
+from miro import buildutils
 from miro.fileobject import FilenameType
+from miro.plat.utils import unicode_to_filename
 
 # We're going to override this so we can guarantee that if the order
 # changes later that it doesn't really affect us.
@@ -33,6 +39,19 @@ class FakeStream:
     def flush(self):
         pass
 
+class MockCache(util.Cache):
+    """
+    MockCache is used to test the Cache object.  The new values are a tuple of
+    the value passed in and a counter value, incremented each time a new value
+    is made.
+    """
+    def __init__(self, size):
+        util.Cache.__init__(self, size)
+        self.value_counter = itertools.count()
+
+    def create_new_value(self, val, invalidator=None):
+        return (val, self.value_counter.next())
+
 class AutoFlushingStreamTest(unittest.TestCase):
     def setUp(self):
         unittest.TestCase.setUp(self)
@@ -174,6 +193,16 @@ class UtilTest(unittest.TestCase):
         # input, handleerror, expected output if handlerror is None,
         # then it isn't passed in as an argument
 
+        class GoodStringObject(object):
+            """Object whose __str__ method returns an ASCII string."""
+            def __str__(self):
+                return "abc"
+
+        class BadStringObject(object):
+            """Object whose __str__ method returns a non-ASCII string."""
+            def __str__(self):
+                return "abc\xe4"
+
         for i, h, o in [
             ( "", None, ""),
             ( "abc", None, "abc"),
@@ -181,7 +210,13 @@ class UtilTest(unittest.TestCase):
             ( 5.5, None, "5.5"),
             ( u"abc", None, "abc"),
             ( u"abc\xe4", None, "abc&#228;"),
-            ( u"abc\xe4", "replace", "abc?")
+            ( u"abc\xe4", "replace", "abc?"),
+            # test that bytestrings are converted to plain ASCII
+            ( "abc", None, "abc"),
+            ( "abc\xe4", None, "abc?"),
+            # test that objects are converted to plain ASCII
+            ( GoodStringObject(), None, "abc"),
+            ( BadStringObject(), None, "abc?"),
             ]:
 
             if h == None:
@@ -458,7 +493,7 @@ class DownloadUtilsTest(MiroTestCase):
     def test_next_free_filename_generators(self):
         # try path without extension
         path = "/foo/.bar/test"
-        generator = download_utils.next_free_filename_candidates(path)
+        generator = util.next_free_filename_candidates(path)
         # first candidate should just be the file itself
         self.assertEquals(generator.next(), "/foo/.bar/test")
         # next candidate should just be the file with .X added to it
@@ -467,7 +502,7 @@ class DownloadUtilsTest(MiroTestCase):
 
         # try path with extension
         path = "/foo/.bar/test.jpg"
-        generator = download_utils.next_free_filename_candidates(path)
+        generator = util.next_free_filename_candidates(path)
         # first candidate should just be the file itself
         self.assertEquals(generator.next(), "/foo/.bar/test.jpg")
         # next candidate should just be the file with .X added before the
@@ -476,7 +511,7 @@ class DownloadUtilsTest(MiroTestCase):
         self.assertEquals(generator.next(), "/foo/.bar/test.2.jpg")
 
         # test that if we call it too many times, we get an exception
-        generator = download_utils.next_free_filename_candidates(path)
+        generator = util.next_free_filename_candidates(path)
         for x in xrange(100000):
             try:
                 generator.next()
@@ -492,7 +527,7 @@ class DownloadUtilsTest(MiroTestCase):
 
     def test_next_free_directory_generators(self):
         path = "/foo/.bar/test"
-        generator = download_utils.next_free_directory_candidates(path)
+        generator = util.next_free_directory_candidates(path)
         # first candidate should just be the file itself
         self.assertEquals(generator.next(), "/foo/.bar/test")
         # next candidate should just be the file with .X added to it
@@ -500,7 +535,7 @@ class DownloadUtilsTest(MiroTestCase):
         self.assertEquals(generator.next(), "/foo/.bar/test.2")
 
         # test that if we call it too many times, we get an exception
-        generator = download_utils.next_free_directory_candidates(path)
+        generator = util.next_free_directory_candidates(path)
         for x in xrange(100000):
             try:
                 generator.next()
@@ -523,14 +558,14 @@ class DownloadUtilsTest(MiroTestCase):
 
         path1 = os.path.join(self.tempdir, 'foo')
         # test we find the a nonexistent file
-        returned_path, fp = download_utils.next_free_filename(path1)
+        returned_path, fp = util.next_free_filename(path1)
         self.assertEquals(returned_path, os.path.join(self.tempdir, 'foo.3'))
         # test that we create the file
         self.assert_(os.path.exists(returned_path))
 
         # try with an extension
         path2 = os.path.join(self.tempdir, 'bar.jpg')
-        returned_path, fp = download_utils.next_free_filename(path2)
+        returned_path, fp = util.next_free_filename(path2)
         self.assertEquals(returned_path, os.path.join(self.tempdir,
             'bar.2.jpg'))
         self.assert_(os.path.exists(returned_path))
@@ -543,7 +578,7 @@ class DownloadUtilsTest(MiroTestCase):
 
         path = os.path.join(self.tempdir, 'foo')
         # test we find the a nonexistent file
-        returned_path = download_utils.next_free_directory(path)
+        returned_path = util.next_free_directory(path)
         self.assertEquals(returned_path, os.path.join(self.tempdir, 'foo.3'))
         # test that we don't create the directory
         self.assert_(not os.path.exists(returned_path))
@@ -571,7 +606,7 @@ E = F
 """.strip().replace("S", " "))
         f.close()
 
-        cfg = util.read_simple_config_file(fn)
+        cfg = buildutils.read_simple_config_file(fn)
         self.assertEquals(cfg["a"], "b")
         self.assertEquals(cfg["c"], "d   ")
         self.assertEquals(cfg["E"], "F")
@@ -583,9 +618,9 @@ E = F
         cfg = {"a": "b",
                "c": "d",
                "E": "F   "}
-        util.write_simple_config_file(fn, cfg)
+        buildutils.write_simple_config_file(fn, cfg)
 
-        cfg2 = util.read_simple_config_file(fn)
+        cfg2 = buildutils.read_simple_config_file(fn)
         self.assertEquals(cfg2["a"], cfg["a"])
         self.assertEquals(cfg2["c"], cfg["c"])
         self.assertEquals(cfg2["E"], cfg["E"])
@@ -796,15 +831,25 @@ class TestCopySubtitleFile(unittest.TestCase):
 class TestNameSortKey(unittest.TestCase):
     def test_simple(self):
         for testcase in ((None, 'ZZZZZZZZZZZZZ'),
-                         (u'', [u'']),
-                         (u'a', [u'a']),
-                         (u'a1a', [u'a', 1.0, u'a']),
-                         (u'Episode_100', [u'episode_', 100.0, u'']),
-                         (u'episode_1', [u'episode_', 1.0, u''])
+                         (u'', (u'',)),
+                         (u'a', (u'a',)),
+                         (u'a1a', (u'a', 1.0, u'a')),
+                         (u'Episode_100', (u'episode_', 100.0, u'')),
+                         (u'episode_1', (u'episode_', 1.0, u''))
                          ):
             self.assertEquals(util.name_sort_key(testcase[0]),
                               testcase[1])
 
+    def test_hashable(self):
+        for testcase in (None,
+                         u'',
+                         u'a',
+                         u'a1a',
+                         u'Episode_100',
+                         u'episode_1',
+                         ):
+            hash(util.name_sort_key(testcase))
+
     def test_sorting(self):
         for inlist, outlist in (
             ([], []),
@@ -881,3 +926,198 @@ class TestGatherMediaFiles(unittest.TestCase):
         self.verify_results()
         self.add_file('test.ogv', True)
         self.verify_results()
+
+class TestBackupSupportDir(MiroTestCase):
+    # Test backing up the support directory
+    def setUp(self):
+        MiroTestCase.setUp(self)
+        self.support_dir = self.make_temp_dir_path()
+        self.correct_files = []
+        self.skip_dirs = []
+        self.setup_support_dir()
+
+    def setup_support_dir(self):
+        """Add objects to our fake support directory that we want around for
+        every test.
+        """
+
+        # add log files
+        self.add_file_to_support_dir('miro.log')
+        self.add_file_to_support_dir('miro-downloader.log')
+        for i in range(1, 5):
+            self.add_file_to_support_dir('miro.log.%s' % i)
+            self.add_file_to_support_dir('miro-downloader.log.%s' % i)
+        # add database files
+        self.add_file_to_support_dir('sqlitedb')
+        self.add_file_to_support_dir('sqlitedb-journal')
+        self.add_file_to_support_dir('dbbackups/sqlitedb_backup_165')
+        self.add_file_to_support_dir('dbbackups/sqlitedb_backup_170')
+        self.add_file_to_support_dir('dbbackups/sqlitedb_backup_183')
+        # add other files
+        self.add_skip_dir('icon-cache')
+        self.add_skip_dir('cover-art')
+        self.add_file_to_support_dir('httpauth', should_skip=True)
+        self.add_file_to_support_dir('preferences.bin', should_skip=True)
+        for i in range(5):
+            self.add_file_to_support_dir('cover-art/Album-%s' % i,
+                                         should_skip=True)
+            self.add_file_to_support_dir('icon-cache/icon-%s' % i,
+                                         should_skip=True)
+            self.add_file_to_support_dir('crashes/crash-report-%i' % i)
+
+    def add_skip_dir(self, skip_dir):
+        self.skip_dirs.append(os.path.join(self.support_dir, skip_dir))
+
+    def add_file_to_support_dir(self, path, archive_name=None,
+                                should_skip=False, contents='FAKE DATA'):
+        if archive_name is None:
+            archive_name = path
+        full_path = os.path.join(self.support_dir, path)
+        directory = os.path.dirname(full_path)
+        if not os.path.exists(directory):
+            os.makedirs(directory)
+        open(full_path, "wt").write(contents)
+        if not should_skip:
+            self.correct_files.append(archive_name)
+
+    def check_backup(self):
+        backup = util.SupportDirBackup(self.support_dir, self.skip_dirs,
+                                       max_size=1000000)
+        archive = zipfile.ZipFile(backup.fileobj(), 'r')
+        errors = archive.testzip()
+        self.assertTrue(errors is None, "Errors in the zip file: %s" % errors)
+        self.assertSameSet(archive.namelist(), self.correct_files)
+
+    def test_backup(self):
+        self.check_backup()
+
+    def test_extendend_chars(self):
+        filename = unicode_to_filename(u'\ufffdxtended Chars')
+        self.add_file_to_support_dir(filename, 'xtended Chars')
+        self.check_backup()
+
+    def test_size_limit(self):
+        # create 200 kb worth of data
+        large_data = " " * 200000
+        # add a bunch of those files
+        for i in xrange(10):
+            self.add_file_to_support_dir('big-file-%s' % i,
+                                         contents=large_data)
+        # check that we don't max an archive file too much bigger than our max
+        # size
+        max_size = 1000000 # 1MB
+        with self.allow_warnings():
+            backup = util.SupportDirBackup(self.support_dir, self.skip_dirs,
+                                           max_size=max_size)
+        filesize = os.stat(backup.backupfile).st_size
+        self.assertTrue(filesize <= 1100000,
+                        "Backup file too big.  filesize: %s max_size: %s" %
+                        (filesize, max_size))
+
+
+class MtimeInvalidatorTestCase(MiroTestCase):
+
+    def test_valid(self):
+        filename = os.path.join(self.tempdir, 'mtime_test')
+        file(filename, 'w').write('foo')
+        invalidator = util.mtime_invalidator(filename)
+        self.assertFalse(invalidator(None))
+
+    def test_invalid(self):
+        filename = os.path.join(self.tempdir, 'mtime_test_future')
+        file(filename, 'w').write('foo')
+        invalidator = util.mtime_invalidator(filename)
+        mtime = os.stat(filename).st_mtime
+        # pretend the file was modified in the future
+        os.utime(filename, (mtime + 10, mtime + 10))
+        self.assertTrue(invalidator(None))
+
+    def test_doesnotexist(self):
+        filename = os.path.join(self.tempdir,
+                                'mtime_test_doesnotexist')
+        invalidator = util.mtime_invalidator(filename)
+        self.assertTrue(invalidator(None))
+
+    def test_disappears(self):
+        filename = os.path.join(self.tempdir,
+                                'mtime_test_disappears')
+        file(filename, 'w').write('foo')
+        invalidator = util.mtime_invalidator(filename)
+        self.assertFalse(invalidator(None))
+        os.unlink(filename)
+        self.assertTrue(invalidator(None))
+
+class CacheTestCase(MiroTestCase):
+
+    def setUp(self):
+        MiroTestCase.setUp(self)
+        self.cache = MockCache(2)
+
+    def test_set_get(self):
+        self.cache.set(1, 1)
+        self.assertEquals(self.cache.get(1), 1)
+
+    def test_create_new_value_get(self):
+        self.assertEquals(self.cache.get(1), (1, 0))
+        self.assertEquals(self.cache.get(3), (3, 1))
+
+    def test_remove(self):
+        self.cache.set(1, 1)
+        self.cache.remove(1)
+        self.assertFalse(1 in self.cache.keys())
+
+    def test_lru(self):
+        self.cache.get(1)
+        self.cache.get(2)
+        self.cache.get(3)
+        # 1 has expired out
+        self.assertEquals(set(self.cache.keys()), set((2, 3)))
+
+    def test_invalidator_set(self):
+        def invalidator(key):
+            return True
+        self.cache.set(1, 1, invalidator=invalidator)
+        # previous value is now invalid, get a new one
+        self.assertEquals(self.cache.get(1), (1, 0))
+
+    def test_invalidator_get(self):
+        def invalidator(key):
+            return True
+        self.assertEquals(self.cache.get(1, invalidator=invalidator),
+                          (1, 0))
+        # previous value was invalid, get a new one
+        self.assertEquals(self.cache.get(1, invalidator=invalidator),
+                          (1, 1))
+
+
+class AlarmTestCase(MiroTestCase):
+    @staticmethod
+    def _long_function():
+        time.sleep(1.5)
+        return True
+
+    def _wrapped_function(self, set_signal=True):
+        with util.alarm(1, set_signal=set_signal):
+            return self._long_function()
+
+    if hasattr(signal, 'SIGALRM'):
+        def test_alarm_works(self):
+            self.assertRaises(IOError, self._wrapped_function)
+
+        def test_context_manager__True(self):
+            with util.alarm(1) as result:
+                self.assertTrue(result)
+
+    def test_alarm_noop(self):
+        self.assertTrue(self._wrapped_function(set_signal=False))
+
+    def test_context_manager__False(self):
+        with util.alarm(0, set_signal=False) as result:
+            self.assertFalse(result)
+
+class NamedTupleTest(MiroTestCase):
+    def test_namedtuple(self):
+        MyClass = util.namedtuple("MyClass", "a b c", "My Docstring")
+        m = MyClass(1,2,3)
+        self.assertEquals(m.__doc__, "My Docstring")
+        self.assertEquals((m.a, m.b, m.c), (1,2,3))
