@@ -164,7 +164,6 @@ class ThemeHistory(DDBObject):
             (u"http://revision3.com/lifehacker/feed/MP4-hd30", False),
             (u"http://feeds.thisamericanlife.org/talpodcast", False),
             (u"http://feeds.themoth.org/themothpodcast", False),
-            (u"http://feeds.feedburner.com/VodoPromotedWorks", False),
             ]
 
         for default in default_feeds:
@@ -182,10 +181,11 @@ class ThemeHistory(DDBObject):
 
         default_guides = [
             (u"http://www.youtube.com", u"YouTube", False),
+            (u"http://www.archive.org", u"Internet Archive", False),
             (u"http://www.hulu.com/", u"Hulu", False),
             (u"http://video.pbs.org/", u"PBS", False),
-            (u"http://www.youtorrent.com/", u"YouTorrent", False),
             (u"http://www.clearbits.net/", u"ClearBits", False),
+            (u"http://www.amara.org/", u"Amara", False),
             (u'http://www.amazon.com/b?_encoding=UTF8&site-redirect=&'
              'node=163856011&tag=pcultureorg-20&linkCode=ur2&camp=1789&'
              'creative=9325', u"Amazon MP3 Store", True),
@@ -194,6 +194,7 @@ class ThemeHistory(DDBObject):
              u"%26node%3D2350149011&tag=pcultureorg-20&linkCode=ur2&camp="
              u"1789&creative=9325", u"Amazon Android Store", True),
             (u"http://market.android.com/", u"Google Android Store", True),
+            (u"http://www.kqzyfj.com/click-5294129-10364534", u"eMusic", True)
             ]
 
         if app.debugmode:
