@@ -33,26 +33,42 @@ This module contains self-contained utility functions.  It shouldn't import
 any other Miro modules.
 """
 
-import sys
+from hashlib import sha1 as sha
+from StringIO import StringIO
+import cgi
+import collections
+import contextlib
 import itertools
+import logging
 import os
 import random
 import re
 import shutil
-import unicodedata
-try:
-    from hashlib import sha1 as sha
-except ImportError:
-    from sha import sha
-import string
-import urllib
 import socket
-import logging
-from miro import filetypes
-import traceback
+import string
+import signal
+import struct
 import subprocess
-from StringIO import StringIO
-from clock import clock
+import sys
+import tempfile
+import traceback
+import unicodedata
+import urllib
+import zipfile
+
+from miro.clock import clock
+from miro import filetypes
+from miro.plat.popen import Popen
+
+# Do NOT import libtorrent up here.  libtorrent.so is compiled with
+# @executable_path-relative path dependency for the Python library
+# within the Python framework.  This makes Miro.app invoke properly,
+# but you cannot invoke the python command line interpreter (like
+# we do with the metadata extractor) because the python executable
+# has a different path to the Miro binary in Miro.app, and so when
+# libtorrent tries to load Python shared library it fails.
+#
+# See bz:18370.
 
 # Should we print out warning messages.  Turn off in the unit tests.
 chatter = True
@@ -69,6 +85,15 @@ PREFERRED_TYPES_ORDER = dict((typ, i) for i, typ in
 
 MAX_TORRENT_SIZE = 500 * (2**10) # 500k
 
+def bitness():
+    return struct.calcsize('L') * 8
+
+def bits_32():
+    return bitness() == 32
+
+def bits_64():
+    return bitness() == 64
+
 def get_nice_stack():
     """Get a stack trace that's a easier to read that the full one."""
     stack = traceback.extract_stack()
@@ -104,84 +129,6 @@ def get_nice_stack():
     stack = [i for i in stack if 'trap_call' not in i]
     return stack
 
-
-CONFIG_LINE_RE = re.compile(r"^([^ ]+) *= *([^\r\n]*)[\r\n]*$")
-
-def read_simple_config_file(path):
-    """Parse a configuration file in a very simple format and return contents
-    as a dict.
-
-    Each line is either whitespace or "Key = Value".  Whitespace is ignored
-    at the beginning of Value, but the remainder of the line is taken
-    literally, including any whitespace.
-
-    Note: There is no way to put a newline in a value.
-    """
-    ret = {}
-
-    filep = open(path, "rt")
-    for line in filep.readlines():
-        # Skip blank lines
-        if not line.strip():
-            continue
-
-        # Otherwise it'd better be a configuration setting
-        match = CONFIG_LINE_RE.match(line)
-        if not match:
-            print ("WARNING: %s: ignored bad configuration directive '%s'" %
-                   (path, line))
-            continue
-
-        key = match.group(1)
-        value = match.group(2)
-        if key in ret:
-            print "WARNING: %s: ignored duplicate directive '%s'" % (path,
-                                                                     line)
-            continue
-
-        ret[key] = value
-
-    return ret
-
-def write_simple_config_file(path, data):
-    """Given a dict, write a configuration file in the format that
-    read_simple_config_file reads.
-    """
-    filep = open(path, "wt")
-
-    for k, v in data.iteritems():
-        filep.write("%s = %s\n" % (k, v))
-
-    filep.close()
-
-def query_revision():
-    """Called at build-time to ask git for the revision of this
-    checkout.
-
-    Returns the (url, revision) on success and None on failure.
-    """
-    url = "unknown"
-    revision = "unknown"
-    try:
-        proc = subprocess.Popen(["git", "config", "--list"],
-                             stdout=subprocess.PIPE)
-        info = proc.stdout.read().splitlines()
-        proc.stdout.close()
-        origline = "remote.origin.url"
-        info = [m for m in info if m.startswith(origline)]
-        if info:
-            url = info[0][len(origline)+1:].strip()
-
-        proc = subprocess.Popen(["git", "rev-parse", "HEAD"],
-                             stdout=subprocess.PIPE)
-        info = proc.stdout.read()
-        proc.stdout.close()
-        revision = info[0:8]
-        return (url, revision)
-    except StandardError, exc:
-        print "Exception thrown when querying revision: %s" % exc
-    return (url, revision)
-
 class AutoFlushingStream:
     """Converts a stream to an auto-flushing one.  It behaves in
     exactly the same way, except all write() calls are automatically
@@ -295,23 +242,43 @@ def get_torrent_info_hash(path):
         # file is too large, bailout.  (see #12301)
         raise ValueError("%s is not a valid torrent" % path)
 
-    import libtorrent as lt
     f = open(path, 'rb')
     try:
+        import libtorrent
         data = f.read(MAX_TORRENT_SIZE)
         if not data or data[0] != 'd':
             # File doesn't start with 'd', bailout  (see #12301)
             raise ValueError("%s is not a valid torrent" % path)
-        metainfo = lt.bdecode(data)
+        metainfo = libtorrent.bdecode(data)
         try:
             infohash = metainfo['info']
         except StandardError:
             raise ValueError("%s is not a valid torrent" % path)
-        infohash = sha(lt.bencode(infohash)).digest()
+        infohash = sha(libtorrent.bencode(infohash)).digest()
         return infohash
     finally:
         f.close()
 
+def get_name_from_torrent_metadata(metadata):
+    """Get the name of a torrent
+
+    metadata must be the contents of a torrent file.
+
+    :returns: torrent name unicode string
+    :raises ValueError: metadata was not formatted properly
+    """
+    import libtorrent
+    metadata_dict = libtorrent.bdecode(metadata)
+    if metadata_dict is None:
+        raise ValueError("metadata is not bencoded")
+    try:
+        return metadata_dict['info']['name'].decode('utf-8')
+    except KeyError, e:
+        raise ValueError("key missing when reading metadata: %s (%s)", e,
+                         metadata_dict)
+    except UnicodeError:
+        raise ValueError("torrent name is not valid utf-8")
+
 def gather_media_files(path):
     """Gather media files on the disk in a directory tree.
     This is used by the first time startup dialog.
@@ -422,9 +389,14 @@ def copy_subtitle_file(sub_path, video_path):
         sub_basename = video_basename_root + sub_ext
     dest_path = os.path.join(os.path.dirname(video_path), sub_basename)
     if sub_path != dest_path:
-        if os.path.exists(dest_path):
-            os.remove(dest_path)
-        shutil.copyfile(sub_path, dest_path)
+        try:
+            if os.path.exists(dest_path):
+                os.remove(dest_path)
+            shutil.copyfile(sub_path, dest_path)
+        except EnvironmentError:
+            logging.exception('unable to remove existing subtitle file '
+                              'or copy subtitle file')
+            dest_path = ''
     return dest_path
 
 def format_size_for_user(nbytes, zero_string="", with_decimals=True,
@@ -517,9 +489,12 @@ def setup_logging():
     """Adds TIMING and JSALERT logging levels.
     """
     logging.addLevelName(15, "STACK TRACE")
-    logging.stacktrace = lambda msg, *args, **kargs: logging.log(
-        15, "%s\n%s" % ("".join(traceback.format_stack()), msg),
-        *args, **kargs)
+    def stacktrace(msg, *args, **kwargs):
+        msg = "%s\n" + msg
+        stack = "".join(traceback.format_stack())
+        args = (stack,) + args
+        logging.log( 15, msg, *args, **kwargs)
+    logging.stacktrace = stacktrace
 
     logging.addLevelName(25, "TIMING")
     logging.timing = lambda msg, *args, **kargs: logging.log(25, msg,
@@ -588,8 +563,8 @@ def returns_file(func):
     return a filename, file object
     """
     def check_func(*args, **kwargs):
+        result = func(*args, **kwargs)
         try:
-            result = func(*args, **kwargs)
             filename, fileobj = result
             if result is not None and type(fileobj) != file:
                 raise ValueError('returns_file: not a valid file object')
@@ -623,7 +598,7 @@ def unicodify(data):
         data = data.decode('ascii', 'replace')
     return data
 
-def stringify(unicode_str, handleerror="xmlcharrefreplace"):
+def stringify(stringobj, handleerror="xmlcharrefreplace"):
     """Takes a possibly unicode string and converts it to a string
     string.  This is required for some logging especially where the
     things being logged are filenames which can be Unicode in the
@@ -640,11 +615,15 @@ def stringify(unicode_str, handleerror="xmlcharrefreplace"):
 
        This is not the inverse of unicodify!
     """
-    if isinstance(unicode_str, unicode):
-        return unicode_str.encode("ascii", handleerror)
-    if not isinstance(unicode_str, str):
-        return str(unicode_str)
-    return unicode_str
+    if isinstance(stringobj, unicode):
+        return stringobj.encode("ascii", handleerror)
+    if isinstance(stringobj, str):
+        # make sure bytestrings are ASCII
+         return stringobj.decode('ascii', 'replace').encode('ascii',
+                                                            'replace')
+    else:
+        # convert objects to strings, then ensure they are ASCII
+        return stringify(str(stringobj))
 
 def quote_unicode_url(url):
     """Quote international characters contained in a URL according to
@@ -659,23 +638,6 @@ def quote_unicode_url(url):
             quoted_chars.append(c)
     return u''.join(quoted_chars)
 
-def no_console_startupinfo():
-    """Returns the startupinfo argument for subprocess.Popen so that
-    we don't open a console window.  On platforms other than windows,
-    this is just None.  On windows, it's some win32 silliness.
-    """
-    if subprocess.mswindows:
-        startupinfo = subprocess.STARTUPINFO()
-        # XXX temporary: STARTF_USESHOWWINDOW is in a different location
-        # as of Python 2.6.6
-        try:
-            startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
-        except AttributeError:
-            startupinfo.dwFlags |= subprocess._subprocess.STARTF_USESHOWWINDOW
-        return startupinfo
-    else:
-        return None
-
 def call_command(*args, **kwargs):
     """Call an external command.  If the command doesn't exit with
     status 0, or if it outputs to stderr, an exception will be raised.
@@ -688,16 +650,18 @@ def call_command(*args, **kwargs):
         then this returns (retcode, stdout, stderr).  This implies
         ignore_stderr is True, so you don't need to explicitly state
         that, too.
+    :param env: dict.  Environment to pass to subprocess.Popen
     """
     ignore_stderr = kwargs.pop('ignore_stderr', False)
     return_everything = kwargs.pop('return_everything', False)
+    env = kwargs.pop('env', None)
 
     if kwargs:
         raise TypeError('extra keyword arguments: %s' % kwargs)
 
-    pipe = subprocess.Popen(args, stdout=subprocess.PIPE,
-            stdin=subprocess.PIPE, stderr=subprocess.PIPE,
-            startupinfo=no_console_startupinfo())
+    pipe = Popen(args, stdout=subprocess.PIPE,
+                 stdin=subprocess.PIPE, stderr=subprocess.PIPE,
+                 env=env)
     stdout, stderr = pipe.communicate()
     if return_everything:
         return (pipe.returncode, stdout, stderr)
@@ -1062,7 +1026,7 @@ def is_magnet_uri(uri):
     """ Returns true if this is a magnet link
         which can be handled by Miro.
     """
-    return MAGNET_MATCH_RE.match(uri) and info_hash_from_magnet(uri)
+    return bool(MAGNET_MATCH_RE.match(uri) and info_hash_from_magnet(uri))
 
 MAGNET_INFO_HASH_MATCH = re.compile(r'(?<=btih:)[a-zA-Z0-9]+')
 
@@ -1075,6 +1039,19 @@ def info_hash_from_magnet(uri):
     else:
        return None
 
+def title_from_magnet(uri):
+    """Get the title from a magnet URI, or None if there is not one
+    """
+    try:
+        query = uri[uri.find('?')+1:]
+        query_parsed = cgi.parse_qs(query)
+        if 'dn' in query_parsed:
+            return query_parsed['dn'][0]
+        else:
+            return None
+    except StandardError:
+        logging.warn("Error parsing title from magnet URI", exc_info=True)
+
 def _strip_accents(text):
     nfkd_form = unicodedata.normalize('NFKD', unicode(text))
     return u"".join([c for c in nfkd_form if not unicodedata.combining(c)])
@@ -1104,10 +1081,10 @@ def name_sort_key(text):
         return "ZZZZZZZZZZZZZ"
     text = text.lower()
     if text.startswith("a "):
-        text = text[2:]
+        text = text[2:] + ', a'
     elif text.startswith("the "):
-        text = text[4:]
-    return [_trynum(c) for c in NUM_RE.split(text)]
+        text = text[4:] + ', the'
+    return tuple(_trynum(c) for c in NUM_RE.split(text))
 
 LOWER_TRANSLATE = string.maketrans(string.ascii_uppercase,
                                    string.ascii_lowercase)
@@ -1135,27 +1112,65 @@ class DebuggingTimer:
     def log_total_time(self):
         logging.timing("total time: %0.3f", clock() - self.start_time)
 
+def mtime_invalidator(path):
+    """
+    Returns a function which returns True if the mtime of path is greater than
+    it was when this function was initially called.  Useful as an invalidator
+    for Cache.
+    """
+    path = os.path.abspath(path)
+    try:
+        mtime = os.stat(path).st_mtime
+    except EnvironmentError:
+        # if the file doesn't exist or has a problem when we start, the cache
+        # will always be invalid
+        return lambda x: True
+
+    def invalidator(key):
+        try:
+            return os.stat(path).st_mtime > mtime
+        except EnvironmentError:
+            # if the file disappears, the cache is also invalid
+            return True
+
+    return invalidator
+
 class Cache(object):
     def __init__(self, size):
         self.size = size
         self.dict = {}
         self.counter = itertools.count()
         self.access_times = {}
+        self.invalidators = {}
 
-    def get(self, key):
+    def get(self, key, invalidator=None):
         if key in self.dict:
-            self.access_times[key] = self.counter.next()
-            return self.dict[key]
-        else:
-            value = self.create_new_value(key)
-            self.set(key, value)
-            return value
+            existing_invalidator = self.invalidators[key]
+            if (existing_invalidator is None or
+                not existing_invalidator(key)):
+                self.access_times[key] = self.counter.next()
+                return self.dict[key]
 
-    def set(self, key, value):
+        value = self.create_new_value(key, invalidator=invalidator)
+        self.set(key, value, invalidator=invalidator)
+        return value
+
+    def set(self, key, value, invalidator=None):
         if len(self.dict) == self.size:
             self.shrink_size()
         self.access_times[key] = self.counter.next()
         self.dict[key] = value
+        self.invalidators[key] = invalidator
+
+    def remove(self, key):
+        if key in self.dict:
+            del self.dict[key]
+            del self.access_times[key]
+        if key in self.invalidators:
+            del self.invalidators[key]
+
+    def keys(self):
+        return self.dict.iterkeys()
 
     def shrink_size(self):
         # shrink by LRU
@@ -1163,12 +1178,232 @@ class Cache(object):
         to_sort.sort(key=lambda m: m[1])
         new_dict = {}
         new_access_times = {}
+        new_invalidators = {}
         latest_times = to_sort[len(self.dict) // 2:]
         for (key, time) in latest_times:
             new_dict[key] = self.dict[key]
+            new_invalidators[key] = self.invalidators[key]
             new_access_times[key] = time
         self.dict = new_dict
         self.access_times = new_access_times
 
-    def create_new_value(self, val):
+    def create_new_value(self, val, invalidator=None):
         raise NotImplementedError()
+
+def all_subclasses(cls):
+    """Find all subclasses of a given new-style class.
+
+    This method also returns sub-subclasses, etc.
+    """
+    for subclass in cls.__subclasses__():
+        yield subclass
+        for sub_subclass in all_subclasses(subclass):
+            yield sub_subclass
+
+def import_module(module_name):
+    """Import a module and return it.
+
+    This function works like __import__, except it returns the last module
+    named, rather than the first.  If you import 'foo.bar', __import__ will
+    return foo, but import_module will return bar.
+    """
+    mod = __import__(module_name)
+    parts = module_name.split('.')
+    for part in parts[1:]:
+        mod = getattr(mod, part)
+    return mod
+
+def make_file_url(path):
+    """Get a file:// URL for a file path."""
+    if isinstance(path, unicode):
+        path = path.encode('utf-8')
+    path_part = urllib.pathname2url(os.path.abspath(path))
+    # On windows pathname2url adds a leading "///" to absolute paths.  This is
+    # pretty weird and annoying, but easy to fix
+    path_part = re.sub(r'^/+', '', path_part)
+    # Always return str.  Pathname2url() returns a str and from that point
+    # there are no unicode to infect us for a unicode type upgrade.
+    return 'file:///' + path_part
+
+def split_values_for_sqlite(value_list):
+    """Split a list of values into chunks that SQL can handle.
+
+    The cursor.execute() method can only handle 999 values at once, this
+    method splits long lists into chunks where each chunk has is safe to feed
+    to sqlite.
+    """
+    CHUNK_SIZE = 990 # use 990 just to be on the safe side.
+    for start in xrange(0, len(value_list), CHUNK_SIZE):
+        yield value_list[start:start+CHUNK_SIZE]
+
+
+class SupportDirBackup(object):
+    """Backup the support directory to send in a crash report."""
+    def __init__(self, support_dir, skip_dirs, max_size):
+        logging.info("Attempting to back up support directory: %r",
+                     support_dir)
+        backupfile_start = os.path.join(tempfile.gettempdir(),
+                                        'miro-support-backup.zip')
+        self.backupfile, fp = next_free_filename(backupfile_start)
+        archive = zipfile.ZipFile(fp, "w")
+        self.skip_dirs = [os.path.normpath(d) for d in skip_dirs]
+
+        total_size = 0
+        for root, directories, files in os.walk(support_dir):
+            self.filter_directories(root, directories)
+            relativeroot = os.path.relpath(root, support_dir)
+            for fn in files:
+                if self.should_skip_file(root, fn):
+                    continue
+                path = os.path.join(root, fn)
+                if relativeroot != '.':
+                    relpath = os.path.join(relativeroot, fn)
+                else:
+                    relpath = fn
+                relpath = self.ensure_ascii_filename(relpath)
+                archive.write(path, relpath)
+                total_size += archive.getinfo(relpath).compress_size
+                if total_size >= max_size:
+                    break
+            if total_size >= max_size:
+                logging.warn("Support directory backup too big.  "
+                             "Quitting after %s bytes", total_size)
+                break
+        archive.close()
+        logging.info("Support directory backed up to %s (%d bytes)",
+                     self.backupfile, os.path.getsize(self.backupfile))
+
+    def filter_directories(self, root, directories):
+        """Remove directories from the list that os.walk() passes us."""
+        filtered = [d for d in directories
+                    if not self.should_skip_directory(os.path.join(root, d))]
+        # os.walk() wants us to change directories in-place
+        directories[:] = filtered
+
+    def should_skip_directory(self, directory):
+        for skip_dir in self.skip_dirs:
+            if directory.startswith(skip_dir):
+                return True
+        return False
+
+    def should_skip_file(self, directory, filename):
+        if os.path.islink(os.path.join(directory, filename)):
+            return True
+        if filename == 'httpauth':
+            # don't send http passwords over the internet
+            return True
+        if filename == 'preferences.bin':
+            # On windows, don't send the config file.  Other
+            # platforms don't handle config the same way, so we
+            # don't need to worry about them
+            return True
+        return False
+
+    def ensure_ascii_filename(self, relpath):
+        """Ensure that a path we are about to archive is ASCII."""
+
+        # NOTE: zipfiles in general, and especially the python zipfile module
+        # don't seem to support them well.  The only filenames we should be
+        # sending are ASCII anyways, so let's just use a hack here to force
+        # things.  See the "zipfile and unicode filenames" thread here:
+        # http://mail.python.org/pipermail/python-dev/2007-June/thread.html
+        if isinstance(relpath, unicode):
+            return relpath.encode('ascii', 'ignore')
+        else:
+            return relpath.decode('ascii', 'ignore').encode('ascii', 'ignore')
+
+    def fileobj(self):
+        """Get a file object for the archive file."""
+        return open(self.backupfile, "rb")
+
+def next_free_filename_candidates(path):
+    """Generates candidate names for next_free_filename."""
+
+    # try unmodified path first
+    yield path
+    # add stuff to the filename to try to make it unique
+
+    dirname, filename = os.path.split(path)
+    if not filename:
+        raise ValueError("%s is a directory name" % path)
+    basename, ext = os.path.splitext(filename)
+    count = 1
+    while True:
+        filename = "%s.%s%s" % (basename, count, ext)
+        yield os.path.join(dirname, filename)
+        count += 1
+        if count > 1000:
+            raise ValueError("Can't find available filename for %s" % path)
+
+@returns_file
+def next_free_filename(name):
+    """Finds a filename that's unused and similar the the file we want
+    to download and returns an open file handle to it.
+    """ 
+    check_f(name)
+    mask = os.O_CREAT | os.O_EXCL | os.O_RDWR
+    # On Windows we need to pass in O_BINARY, fdopen() even with 'b' 
+    # specified is not sufficient.
+    if sys.platform == 'win32':
+        mask |= os.O_BINARY
+
+    candidates = next_free_filename_candidates(name)
+    while True:
+        # Try with the name supplied.
+        newname = candidates.next()
+        try:
+            fd = os.open(newname, mask)
+            fp = os.fdopen(fd, 'wb')
+            return newname, fp
+        except OSError:
+            continue
+    return (newname, fp)
+
+def next_free_directory_candidates(name):
+    """Generates candidate names for next_free_directory."""
+    yield name
+    count = 1
+    while True:
+        yield "%s.%s" % (name, count)
+        count += 1
+        if count > 1000:
+            raise ValueError("Can't find available directory for %s" % name)
+
+@returns_filename
+def next_free_directory(name):
+    """Finds a unused directory name using name as a base.
+
+    This method doesn't create the directory, it just finds an an-used one.
+    """
+    candidates = next_free_directory_candidates(name)
+    while True:
+        candidate = candidates.next()
+        if not os.path.exists(candidate):
+            return candidate
+
+@contextlib.contextmanager
+def alarm(timeout, set_signal=True):
+    def alarm_handler(signum, frame):
+        raise IOError('timeout after %i seconds' % timeout)
+    if set_signal:
+        set_signal = hasattr(signal, 'SIGALRM')
+    if set_signal:
+        signal.signal(signal.SIGALRM, alarm_handler)
+        signal.alarm(timeout)
+    yield set_signal
+    if set_signal:
+        signal.alarm(0)
+
+def supports_alarm():
+    return hasattr(signal, 'SIGALRM')
+
+def namedtuple(class_name, fields, docstring=None):
+    """Version of collections.namedtuple that adds docstring support."""
+    # make the base class using the standard namedtuple
+    nt = collections.namedtuple(class_name + "Tuple", fields)
+    # make a subclass that adds the docstring and doesn't add a per-instance
+    # dict.
+    dct = { '__slots__': () }
+    if docstring:
+        dct['__doc__'] = docstring
+    return type(class_name, (nt,), dct)
