@@ -30,5 +30,7 @@ yum install \
    pywebkitgtk \
    gstreamer-python \
    python-mutagen \
-   ffmpeg \
-   ffmpeg2theora
+   boost-devel \
+   taglib-devel \
+   zlib-devel \
+   ffmpeg
