@@ -68,6 +68,7 @@ from miro.plat.frontends.widgets.threads import call_on_ui_thread
 from miro.plat.associate import associate_protocols
 
 from miro.frontends.widgets.gtk.widgetset import Rect
+from miro.frontends.widgets.gtk import gtkmenus
 from miro.frontends.widgets.gtk import webkitgtkhacks
 from miro.frontends.widgets.gtk import gtkdirectorywatch
 
@@ -139,23 +140,17 @@ def run_application():
 
 class LinuxApplication(Application):
     def run(self):
+        self.log_initial_info()
         gobject.set_application_name(app.config.get(prefs.SHORT_APP_NAME))
         os.environ["PULSE_PROP_media.role"] = "video"
 
-        gtk.gdk.threads_init()
+        gobject.threads_init()
         self._setup_webkit()
         associate_protocols(self._get_command())
         gtkdirectorywatch.GTKDirectoryWatcher.install()
+        self.menubar = gtkmenus.MainWindowMenuBar()
+        renderers.init_renderer()
         self.startup()
-
-        logging.info("Linux version:     %s %s %s",
-                     platform.system(),
-                     platform.release(),
-                     platform.machine())
-        logging.info("Python version:    %s", sys.version)
-        logging.info("Gtk+ version:      %s", gtk.gtk_version)
-        logging.info("PyGObject version: %s", gtk.ver)
-        logging.info("PyGtk version:     %s", gtk.pygtk_version)
         langs = ("LANGUAGE", "LC_ALL", "LC_MESSAGES", "LANG")
         langs = [(l, os.environ.get(l)) for l in langs if os.environ.get(l)]
         logging.info("Language:          %s", langs)
@@ -171,13 +166,18 @@ class LinuxApplication(Application):
             logging.info("pycurl:            %s", pycurl.version)
         except ImportError:
             logging.exception("pycurl won't load")
-        renderers.init_renderer()
         try:
             gtk.main()
         except (KeyboardInterrupt, SystemExit):
             self.do_quit()
         app.controller.on_shutdown()
 
+    def log_initial_info(self):
+        logging.info("Python version:    %s", sys.version)
+        logging.info("Gtk+ version:      %s", gtk.gtk_version)
+        logging.info("PyGObject version: %s", gtk.ver)
+        logging.info("PyGtk version:     %s", gtk.pygtk_version)
+
     def _setup_webkit(self):
         cookie_path = get_cookie_path()
         webkitgtkhacks.setup_cookie_storage(cookie_path)
@@ -385,7 +385,9 @@ class LinuxApplication(Application):
             return
 
         notification = pynotify.Notification(title, body)
-        if attach_trayicon and app.config.get(options.SHOW_TRAYICON):
+        if (hasattr(notification, 'attach_to_status_icon') and
+            attach_trayicon and
+            app.config.get(options.SHOW_TRAYICON)):
             notification.attach_to_status_icon(self.trayicon)
         if timeout:
             notification.set_timeout(timeout)
