@@ -55,9 +55,10 @@ from distutils.errors import DistutilsOptionError
 from distutils.util import change_root
 from glob import glob
 from string import Template
-import distutils.command.install_data
-import distutils.command.build
+from distutils.command.build import build
+from distutils.command.install_data import install_data
 import os
+import platform
 import pwd
 import subprocess
 import time
@@ -88,11 +89,11 @@ def get_root_dir():
     return root_dir
 
 root_dir = get_root_dir()
+git_root_dir = os.path.dirname(root_dir)
 portable_dir = os.path.join(root_dir, 'lib')
 portable_frontend_dir = os.path.join(portable_dir, 'frontends')
 portable_xpcom_dir = os.path.join(portable_frontend_dir, 'widgets', 'gtk',
                                   'xpcom')
-infolist_dir = os.path.join(portable_frontend_dir, 'widgets', 'infolist')
 dl_daemon_dir = os.path.join(portable_dir, 'dl_daemon')
 test_dir = os.path.join(portable_dir, 'test')
 resource_dir = os.path.join(root_dir, 'resources')
@@ -102,6 +103,9 @@ platform_package_dir = os.path.join(platform_dir, 'plat')
 platform_widgets_dir = os.path.join(platform_package_dir, 'frontends',
                                     'widgets')
 platform_extensions_dir = os.path.join(platform_dir, 'extensions')
+echoprint_dir = os.path.join(platform_dir, 'contrib', 'echoprint-codegen')
+echoprint_src_dir = os.path.join(echoprint_dir, 'src')
+enmfp_path = os.path.join(platform_dir, 'contrib', 'enmfp-codegen')
 
 # insert the root_dir to the beginning of sys.path so that we can
 # pick up portable and other packages
@@ -116,9 +120,9 @@ import plat
 sys.modules['miro'].plat = plat
 
 # little hack to get the version from the current app.config.template
-from miro import util
+from miro import buildutils
 app_config = os.path.join(resource_dir, 'app.config.template')
-appVersion = util.read_simple_config_file(app_config)['appVersion']
+appVersion = buildutils.read_simple_config_file(app_config)['appVersion']
 
 # RPM hack
 if 'bdist_rpm' in sys.argv:
@@ -199,12 +203,13 @@ def parse_pkg_config(command, components, options_dict=None):
         options_dict = {
             'include_dirs': [],
             'library_dirs': [],
-            'runtime_dirs': [],
             'libraries': [],
             'extra_compile_args': []
         }
+    
     commandLine = "%s --cflags --libs %s" % (command, components)
     output = get_command_output(commandLine).strip()
+    
     for comp in output.split():
         prefix, rest = comp[:2], comp[2:]
         if prefix == '-I':
@@ -213,12 +218,15 @@ def parse_pkg_config(command, components, options_dict=None):
             options_dict['library_dirs'].append(rest)
         elif prefix == '-l':
             options_dict['libraries'].append(rest)
+            
+            # Silence warning on `GtkItemFactoryCallback` declaration.
+            if rest.startswith('gtk-'):
+                options_dict['extra_compile_args'].append('-Wno-strict-prototypes')
         else:
             options_dict['extra_compile_args'].append(comp)
 
     commandLine = "%s --variable=libdir %s" % (command, components)
     output = get_command_output(commandLine).strip()
-    options_dict['runtime_dirs'].append(output)
 
     return options_dict
 
@@ -253,6 +261,25 @@ pygtkhacks_ext = \
             'pygobject-2.0 gtk+-2.0 glib-2.0 gthread-2.0')
     )
 
+namecollation_ext = \
+    Extension("miro.data.namecollation",
+        [os.path.join(portable_dir, 'data', 'namecollation.cpp')],
+        libraries=['sqlite3'],
+    )
+
+fixedliststore_dir = os.path.join(portable_frontend_dir, 'widgets', 'gtk',
+                                  'fixedliststore')
+fixedliststore_ext = \
+    Extension("miro.frontends.widgets.gtk.fixedliststore",
+          [
+              os.path.join(fixedliststore_dir, 'fixed-list-store.c'),
+              os.path.join(fixedliststore_dir, 'fixed-list-store-module.c'),
+              os.path.join(fixedliststore_dir, 'fixed-list-store-wrapper.c'),
+          ],
+          **parse_pkg_config('pkg-config',
+                             'pygobject-2.0 gtk+-2.0 glib-2.0 gthread-2.0')
+    )
+
 webkitgtkhacks_ext = \
     Extension("miro.frontends.widgets.gtk.webkitgtkhacks",
         [os.path.join(portable_frontend_dir, 'widgets', 'gtk',
@@ -261,22 +288,6 @@ webkitgtkhacks_ext = \
             'gtk+-2.0 webkit-1.0')
     )
 
-infolist_options = parse_pkg_config('pkg-config',
-            'pygobject-2.0 pygtk-2.0 gtk+-2.0 glib-2.0 ')
-infolist_options['include_dirs'].append(infolist_dir)
-infolist_options['include_dirs'].append(os.path.join(infolist_dir, 'gtk'))
-
-infolist_ext = \
-    Extension("miro.infolist",
-        [
-            os.path.join(infolist_dir, 'infolist-nodelist.c'),
-            os.path.join(infolist_dir, 'infolist.pyx'),
-            os.path.join(infolist_dir, 'gtk', 'infolist-gtk.c'),
-        ],
-        **infolist_options
-    )
-
-
 #### Build the data_files list ####
 def listfiles(path):
     all_files = [f for f in glob(os.path.join(path, '*')) if os.path.isfile(f)
@@ -292,22 +303,17 @@ files = [f for f in listfiles(resource_dir) \
 data_files.append(('/usr/share/miro/resources/', files))
 
 # handle the sub directories.
-for dir in ('searchengines',
-        'images',
-        'testdata',
-        'conversions',
-        'devices',
-        os.path.join('testdata', 'conversions'),
-        os.path.join('testdata', 'conversions', 'output'),
-        os.path.join('testdata', 'feedparsertests', 'feeds'),
-        os.path.join('testdata', 'feedparsertests', 'output'),
-        os.path.join('testdata', 'stripperdata'),
-        os.path.join('testdata', 'httpserver'),
-        os.path.join('testdata', 'metadata'),
-        os.path.join('testdata', 'locale', 'fr', 'LC_MESSAGES')):
+for dir in ('searchengines', 'images', 'conversions', 'devices'):
     source_dir = os.path.join(resource_dir, dir)
     dest_dir = os.path.join('/usr/share/miro/resources/', dir)
     data_files.append((dest_dir, listfiles(source_dir)))
+for path, dirs, files in os.walk(os.path.join(resource_dir, 'testdata')):
+    relpath = os.path.relpath(path, resource_dir)
+    dest_dir = os.path.join('/usr/share/miro/resources/', relpath)
+    files = [os.path.join(resource_dir, relpath, f) for f in files]
+    data_files.append((dest_dir, files))
+
+data_files.append((dest_dir, [os.path.join(resource_dir, 'donate.html')]))
 
 # add core extension files
 for root, dirs, files in os.walk(extensions_dir):
@@ -364,19 +370,19 @@ if not "clean" in sys.argv:
 
 
 #### Our specialized install_data command ####
-class install_data(distutils.command.install_data.install_data):
-    """install_data extends to default implementation so that it automatically
-    installs app.config from app.config.template.
+class miro_install_data(install_data):
+    """miro_install_data extends to default implementation so that it
+    automatically installs app.config from app.config.template.
     """
 
     def install_app_config(self):
         source = os.path.join(resource_dir, 'app.config.template')
         dest = '/usr/share/miro/resources/app.config'
 
-        config_file = util.read_simple_config_file(source)
+        config_file = buildutils.read_simple_config_file(source)
         print "Trying to figure out the git revision...."
         if config_file["appVersion"].endswith("git"):
-            revision = util.query_revision()
+            revision = buildutils.query_revision()
             if revision is None:
                 revision = "unknown"
                 revisionurl = "unknown"
@@ -421,10 +427,10 @@ class install_data(distutils.command.install_data.install_data):
             self.outfiles.append(dest)
 
     def run(self):
-        distutils.command.install_data.install_data.run(self)
+        install_data.run(self)
         self.install_app_config()
 
-class build(distutils.command.build.build):
+class miro_build(build):
     def build_segmenter(self):
         segmenter_src = os.path.join(platform_dir, 'miro-segmenter.c')
         cc = ccompiler.new_compiler()
@@ -436,16 +442,35 @@ class build(distutils.command.build.build):
         cc.add_include_dir('/usr/include/ffmpeg')
         output_dir = os.path.join(self.build_base, 'miro-segmenter')
         segmenter_objs = cc.compile([segmenter_src],
-                                    output_dir=output_dir)
+                                    output_dir=output_dir,
+                                    extra_preargs=(os.environ.get("CFLAGS") or "").split())
         cc.link_executable(segmenter_objs, 'miro-segmenter',
                            output_dir=output_dir)
         segmenter_exe = os.path.join(output_dir, 'miro-segmenter')
         self.distribution.scripts.append(segmenter_exe)
 
+    def build_echoprint_codegen(self):
+        subprocess.check_call('/usr/bin/make', cwd=echoprint_src_dir)
+        src_binary = os.path.join(echoprint_dir, 'echoprint-codegen')
+        dest_dir = os.path.join(self.build_base, 'echoprint-codegen')
+        dest_binary = os.path.join(dest_dir, 'echoprint-codegen')
+        self.mkpath(dest_dir)
+        if os.path.exists(dest_binary) and not self.dry_run:
+            os.unlink(dest_binary)
+        self.move_file(src_binary, dest_binary)
+        self.distribution.scripts.append(dest_binary)
+
+    def build_enmfp_codegen(self):
+        filename = 'codegen.Linux-%s' % platform.machine()
+        for path in glob(os.path.join(enmfp_path, filename)):
+            self.distribution.scripts.append(path)
+
     def run(self):
         self.build_segmenter()
-        distutils.command.build.build.run(self)
-        
+        self.build_echoprint_codegen()
+        self.build_enmfp_codegen()
+        build.run(self)
+
 class test_system(Command):
     description = ("Allows you to test configurations without compiling "
                    "or running.")
@@ -541,8 +566,9 @@ ext_modules = []
 ext_modules.append(ngrams_ext)
 ext_modules.append(xlib_ext)
 ext_modules.append(pygtkhacks_ext)
+ext_modules.append(namecollation_ext)
+ext_modules.append(fixedliststore_ext)
 ext_modules.append(webkitgtkhacks_ext)
-ext_modules.append(infolist_ext)
 
 script_files += [os.path.join(platform_dir, 'miro'),
                  os.path.join(platform_dir, 'miro.real')
@@ -561,6 +587,7 @@ setup(name='miro',
     packages=[
         'miro',
         'miro.libdaap',
+        'miro.data',
         'miro.dl_daemon',
         'miro.test',
         'miro.dl_daemon.private',
@@ -569,6 +596,7 @@ setup(name='miro',
         'miro.frontends.profilewidgets',
         'miro.frontends.shell',
         'miro.frontends.widgets',
+        'miro.frontends.widgets.gst',
         'miro.frontends.widgets.gtk',
         'miro.plat',
         'miro.plat.frontends',
@@ -583,8 +611,8 @@ setup(name='miro',
     cmdclass={
         'test_system': test_system,
         'build_ext': build_ext,
-        'build': build,
-        'install_data': install_data,
+        'build': miro_build,
+        'install_data': miro_install_data,
         'install_theme': install_theme,
         'clean': clean,
     }
