@@ -37,10 +37,10 @@ if [ ! -d "miro-binary-kit-osx-${BKIT_VERSION}" ]; then
     exit 1
 fi
 
-if [[ $OS_VERSION == "9" ]] || [[ $OS_VERSION == "10" ]]; then
-    TARGET_OS_VERSION=10.5
+if [[ $OS_VERSION == "10" ]] || [[ $OS_VERSION == "11" || $OS_VERSION == "12" ]]; then
+    TARGET_OS_VERSION=10.6
 else
-    echo "Building and running Miro is only supported on Mac OS X 10.5 and 10.6."
+    echo "Building and running Miro is only supported on Mac OS X 10.{6,7,8}."
     exit 1
 fi
 
@@ -53,7 +53,7 @@ if [ ! -d $SBOX_DIR ]; then
     exit 1
 fi
 
-PYTHON_VERSION=2.6
+PYTHON_VERSION=2.7
 PYTHON_ROOT=$SBOX_DIR/Frameworks/Python.framework/Versions/$PYTHON_VERSION
 PYTHON=$PYTHON_ROOT/bin/python$PYTHON_VERSION
 
