@@ -44,11 +44,9 @@ from miro import prefs
 from miro import downloader
 from miro import messages
 from miro import eventloop
-from miro import moviedata
 from miro import signals
 from miro import httpclient
 from miro import commandline
-from miro.frontends.widgets import menus
 from miro.frontends.widgets.application import Application
 from miro.plat import growl
 from miro.plat import bundle
@@ -56,8 +54,8 @@ from miro.plat import _growlImage
 from miro.plat import migrateappname
 from miro.plat.utils import ensureDownloadDaemonIsTerminated, filename_type_to_os_filename, os_filename_to_filename_type
 from miro.plat.utils import begin_thread_loop, finish_thread_loop
-from miro.plat.frontends.widgets import (widgetupdates, quicktime, osxmenus,
-        sparkleupdater, threads)
+from miro.plat.frontends.widgets import (quicktime, osxmenus, sparkleupdater,
+                                         threads)
 from miro.plat.frontends.widgets.rect import Rect
 from miro.gtcache import gettext as _
 
@@ -117,7 +115,6 @@ class OSXApplication(Application, signals.SignalEmitter):
         # singleton
         miroapp = MiroApplication.sharedApplication()
         miroapp.setDelegate_(self.app_controller)
-        app.size_request_manager = widgetupdates.SizeRequestManager()
         NSApplicationMain(sys.argv)        
 
     def connect_to_signals(self):
@@ -126,8 +123,6 @@ class OSXApplication(Application, signals.SignalEmitter):
         eventloop.connect('thread-did-start', self.endLoop)
         eventloop.connect('begin-loop', self.beginLoop)
         eventloop.connect('end-loop', self.endLoop)
-        moviedata.movie_data_updater.connect('begin-loop', self.beginLoop)
-        moviedata.movie_data_updater.connect('end-loop', self.endLoop)
         httpclient.register_on_start(
             lambda cm: cm.connect('begin-loop', self.beginLoop))
         httpclient.register_on_start(
@@ -135,11 +130,8 @@ class OSXApplication(Application, signals.SignalEmitter):
 
     def startup_ui(self):
         migrateappname.migrateVideos('Democracy', 'Miro')
-        osxmenus.populate_menu()
+        self.menubar = osxmenus.MenuBar()
         Application.startup_ui(self)
-        app.menu_manager.connect('enabled-changed', osxmenus.on_menu_change)
-        app.playback_manager.connect('did-start-playing', osxmenus.on_playback_change)
-        app.playback_manager.connect('did-stop', osxmenus.on_playback_change)
         # add the Amazon cookie to Safari
         storage = NSHTTPCookieStorage.sharedHTTPCookieStorage()
         if 'dmusic_download_manager_enabled' not in [
@@ -271,7 +263,11 @@ class OSXApplication(Application, signals.SignalEmitter):
             pass
         else:
             try:
-                appIcon.compositeToPoint_operation_((0,0), NSCompositeSourceOver)
+                appIcon.drawAtPoint_fromRect_operation_fraction_(
+                  (0, 0),
+                  NSZeroRect,
+                  NSCompositeSourceOver,
+                  1.0)
                 if self.unwatched_count > 0:
                     digits = len(str(self.unwatched_count))
                     badge = nil
@@ -285,7 +281,11 @@ class OSXApplication(Application, signals.SignalEmitter):
                         appIconSize = appIcon.size()
                         badgeSize = badge.size()
                         badgeLoc = (appIconSize.width - badgeSize.width, appIconSize.height - badgeSize.height)
-                        badge.compositeToPoint_operation_(badgeLoc, NSCompositeSourceOver)
+                        badge.drawAtPoint_fromRect_operation_fraction_(
+                          badgeLoc,
+                          NSZeroRect,
+                          NSCompositeSourceOver,
+                          1.0)
                         badgeLabel = NSString.stringWithString_(u'%d' % self.unwatched_count)
                         badgeLabelFont = NSFont.boldSystemFontOfSize_(24)
                         badgeLabelColor = NSColor.whiteColor()
@@ -460,44 +460,9 @@ class AppController(NSObject):
 
         eventloop.add_idle(lambda: commandline.parse_command_line_args([url]), "Open URL")
 
-    def validateUserInterfaceItem_(self, menuitem):
-        action = menuitem.representedObject()
-        item = menus.osx_menu_structure.get(action)
+    def handleMenuActivate_(self, menu_item):
+        osxmenus.handle_menu_activate(menu_item)
 
-        label = item.label
-        if action == 'PlayPauseItem':
-            label = item.state_labels.get(app.menu_manager.play_pause_state,
-                                          item.label)
-        else:
-            for state, actions in app.menu_manager.states.items():
-                if action in actions:
-                    label = item.state_labels.get(state, item.label)
-                    break
-        menuitem.setTitleWithMnemonic_(label.replace("_", "&"))
-
-        group_names = item.groups
-        for group_name in group_names:
-            if group_name in app.menu_manager.enabled_groups:
-                return True
-        return False
-
-    def handleMenuItem_(self, sender):
-        action = sender.representedObject()
-        if action == "PresentActualSize":
-            self.present_movie('natural-size')
-        elif action == "PresentDoubleSize":
-            self.present_movie('double-size')
-        elif action == "PresentHalfSize":
-            self.present_movie('half-size')
-        elif action == "ShowMain":
-            app.widgetapp.window.nswindow.makeKeyAndOrderFront_(sender)
-        else:
-            handler = menus.lookup_handler(action)
-            if handler is not None:
-                handler()
-            else:
-                logging.warn("No handler for %s" % action)
-    
     def present_movie(self, mode):
         # Make sure that the video window is not minimized and is the key window.
         # (based on a patch by Geoffrey Lee, see #13545 and #13546)
