@@ -35,6 +35,7 @@ from objc import YES, NO, nil
 
 from miro import app
 from miro import signals
+from miro import threadcheck
 from miro.plat.frontends.widgets import wrappermap
 from miro.plat.frontends.widgets.viewport import Viewport, BorrowedViewport
 
@@ -53,9 +54,12 @@ class Widget(signals.SignalEmitter):
     CREATES_VIEW = True 
 
     def __init__(self):
-        signals.SignalEmitter.__init__(self, 'size-request-changed',
-                'size-allocated', 'key-press', 'focus-out')
-        self.create_signal('place-in-scroller')
+        threadcheck.confirm_ui_thread()
+        signals.SignalEmitter.__init__(self)
+        self.create_signal('key-press')
+        self.create_signal('focus-out')
+        self.create_signal('size-request-changed', okay_to_nest=True)
+        self.create_signal('size-allocated', okay_to_nest=True)
         self.viewport = None
         self.parent_is_scroller = False
         self.manual_size_request = None
@@ -70,10 +74,6 @@ class Widget(signals.SignalEmitter):
         self.manual_size_request = (width, height)
         self.invalidate_size_request()
 
-    def clear_size_request_cache(self):
-        while app.size_request_manager.widgets_to_request:
-            app.size_request_manager._run_requests()
-
     def get_size_request(self):
         if self.manual_size_request:
             width, height = self.manual_size_request
@@ -92,11 +92,8 @@ class Widget(signals.SignalEmitter):
             return self.cached_size_request
 
     def invalidate_size_request(self):
-        app.size_request_manager.add_widget(self)
-
-    def do_invalidate_size_request(self):
         """Recalculate the size request for this widget."""
-        old_size_request = self.cached_size_request
+        old_size_request = self.get_size_request()
         self.cached_size_request = None
         self.emit('size-request-changed', old_size_request)
 
@@ -106,6 +103,18 @@ class Widget(signals.SignalEmitter):
         """
         raise NotImplementedError()
 
+    def _debug_size_request(self, nesting_level=0):
+        """Debug size request calculations.
+
+        This method recursively prints out the size request for each widget.
+        """
+        request = self.calc_size_request()
+        width = int(request[0])
+        height = int(request[1])
+        indent = '    ' * nesting_level
+        me = str(self.__class__).split('.')[-1]
+        print '%s%s: %sx%s' % (indent, me, width, height)
+
     def place(self, rect, containing_view):
         """Place this widget on a view.  """
         if self.viewport is None:
@@ -242,12 +251,11 @@ class Container(Widget):
 
     def children_changed(self):
         """Invoked when the set of children for this widget changes."""
-        self.do_invalidate_size_request()
+        self.invalidate_size_request()
 
-    def do_invalidate_size_request(self):
-        Widget.do_invalidate_size_request(self)
-        if self.viewport:
-            self.place_children()
+    def place(self, rect, containing_view):
+        Widget.place(self, rect, containing_view)
+        self.place_children()
 
     def viewport_created(self):
         self.place_children()
@@ -263,6 +271,11 @@ class Container(Widget):
         """Layout our child widgets.  Must be implemented by subclasses."""
         raise NotImplementedError()
 
+    def _debug_size_request(self, nesting_level=0):
+        for child in self.children:
+            child._debug_size_request(nesting_level+1)
+        Widget._debug_size_request(self, nesting_level)
+
 class Bin(Container):
     """Container that only has one child widget."""
 
