@@ -41,25 +41,51 @@ Check out the GTK tutorial for more info.
 """
 
 import itertools
+import struct
 
 from AppKit import *
 from Foundation import *
-from objc import YES, NO, nil, signature, loadBundle
+from objc import (YES, NO, nil, signature, loadBundle,
+                  _C_NSInteger, _C_NSUInteger, _C_NSBOOL)
 
 from miro.plat.frontends.widgets import tableview
 from miro.plat.frontends.widgets import wrappermap
 from miro.plat.frontends.widgets.base import Container, Bin, FlippedView
-from miro.util import Matrix
+from miro.util import Matrix, bitness
 
-try:
-    from AppKit import NSScrollerKnobStyleLight
-except ImportError:
-    # NSScrollerKnobStyleLight is only defined in recent x code versions.
-    NSScrollerKnobStyleLight = 2
+# These don't seem to be in pyobjc's AppKit (yet)
+NSScrollerKnobStyleDefault = 0
+NSScrollerKnobStyleDark = 1
+NSScrollerKnobStyleLight = 2
+
+NSScrollerStyleLegacy = 0
+NSScrollerStyleOverlay = 1
 
 rbSplitViewBundlePath = '%s/RBSplitView.framework' % NSBundle.mainBundle().privateFrameworksPath()
 loadBundle('RBSplitView', globals(), bundle_path=rbSplitViewBundlePath)
 
+_nspointsignature = dict()
+_nssizesignature = dict()
+_nsrectsignature = dict()
+_cgfloatsignature = dict()
+
+_nspointsignature[32] = '{_NSPoint=ff}'
+_nspointsignature[64] = '{CGPoint=dd}'
+
+_nssizesignature[32] = '{_NSSize=ff}'
+_nssizesignature[64] = '{CGSize=dd}'
+
+_nsrectsignature[32] = ''.join(['{_NSRect=', _nspointsignature[32],
+                               _nssizesignature[32], '}'])
+_nsrectsignature[64] = ''.join(['{CGRect=', _nspointsignature[64],
+                               _nssizesignature[64], '}'])
+
+_cgfloatsignature[32] = 'f'
+_cgfloatsignature[64] = 'd'
+
+# ILP32/LP64
+_bitness = bitness()
+
 def _extra_space_iter(extra_length, count):
     """Utility function to allocate extra space left over in containers."""
     if count == 0:
@@ -331,32 +357,37 @@ class SplitterDelegate(NSObject):
         self.disabledColor = NSColor.colorWithDeviceWhite_alpha_(135.0/255.0, 1.0)
         return self
 
-    @signature("{_NSRect={_NSPoint=ff}{_NSSize=ff}}@:@{_NSRect={_NSPoint=ff}{_NSSize=ff}}i")
+    @signature("%s@:@%s%s" % (_nsrectsignature[_bitness],
+                              _nsrectsignature[_bitness], _C_NSUInteger))
     def splitView_cursorRect_forDivider_(self, sender, rect, divider):
         if divider == 0:
             rect.origin.x -= 3
             rect.size.width += 6
         return rect
 
-    @signature("i@:@{_NSPoint=ff}@")
+    @signature("%s@:@%s@" % (_C_NSInteger, _nspointsignature[_bitness]))
     def splitView_dividerForPoint_inSubview_(self, sender, point, subview):
         left_width = self.splitter.left_view.bounds().size.width
         if (subview.identifier() == 'left' and point.x >= left_width-3) or (subview.identifier() == 'right' and point.x-left_width <= 3):
             return 0
         return NSNotFound
 
-    @signature("v@:@@{_NSRect={_NSPoint=ff}{_NSSize=ff}}{_NSRect={_NSPoint=ff}{_NSSize=ff}}")
+    @signature("v@:@@%s%s" % (_nsrectsignature[_bitness],
+                              _nsrectsignature[_bitness]))
     def splitView_changedFrameOfSubview_from_to_(self, sender, subview, before, after):
         if subview.identifier() == 'left':
             self.splitter.place_left_children()
         else:
             self.splitter.place_right_children()
 
-    @signature("v@:@ff")
+    @signature("v@:@%s%s" % (_cgfloatsignature[_bitness],
+                             _cgfloatsignature[_bitness]))
     def splitView_wasResizedFrom_to_(self, sender, before, after):
         sender.adjustSubviewsExcepting_(self.splitter.left_view);
 
-    @signature("{_NSRect={_NSPoint=ff}{_NSSize=ff}}@:@{_NSRect={_NSPoint=ff}{_NSSize=ff}}@@{_NSRect={_NSPoint=ff}{_NSSize=ff}}")
+    @signature("%s@:@%s@@%s" % (_nsrectsignature[_bitness],
+                                _nsrectsignature[_bitness],
+                                _nsrectsignature[_bitness]))
     def splitView_willDrawDividerInRect_betweenView_andView_withProposedRect_(self, sender, dividerRect, leading, trailing, imageRect):
         if self.splitter.view.window().isMainWindow():
             self.normalColor.set()
@@ -365,7 +396,7 @@ class SplitterDelegate(NSObject):
         NSRectFill(dividerRect)
         return NSZeroRect
 
-    @signature("i@:@i@@i")
+    @signature("%s@:@%s@@%s" % (_C_NSBOOL, _C_NSUInteger, _C_NSBOOL))
     def splitView_shouldResizeWindowForDivider_betweenView_andView_willGrow_(self, sender, divider, leading, trailing, grow):
         return (NSApp().currentEvent().modifierFlags() & NSAlternateKeyMask != 0)
 
@@ -564,12 +595,16 @@ class Table(Container):
                 size_array[p] += iter.next()
 
     def place_children(self):
+        # This method depepnds on us calling _calc_dimensions() in
+        # calc_size_request().  Ensure that this happens.
+        if self.cached_size_request is None:
+            self.get_size_request()
         column_positions = [0]
         for width in self.column_widths[:-1]:
-            column_positions.append(width + column_positions[-1])
+            column_positions.append(width + column_positions[-1] + self.column_spacing)
         row_positions = [0]
         for height in self.row_heights[:-1]:
-            row_positions.append(height + row_positions[-1])
+            row_positions.append(height + row_positions[-1] + self.row_spacing)
 
         my_x= self.viewport.placement.origin.x
         my_y = self.viewport.placement.origin.y
@@ -640,11 +675,22 @@ class Table(Container):
             for mem in self._cells:
                 if mem: mem.disable()
 
+class MiroScrollView(NSScrollView):
+    def tile(self):
+        NSScrollView.tile(self)
+        # tile is called when we need to layout our child view and scrollers.
+        # This probably means that we've either hidden or shown a scrollbar so
+        # call invalidate_size_request to ensure that things get re-layed out
+        # correctly.  (#see 13842)
+        wrapper = wrappermap.wrapper(self)
+        if wrapper is not None:
+            wrapper.invalidate_size_request()
+
 class Scroller(Bin):
     """See https://develop.participatoryculture.org/index.php/WidgetAPI for a description of the API for this class."""
     def __init__(self, horizontal, vertical):
         Bin.__init__(self)
-        self.view = NSScrollView.alloc().init()
+        self.view = MiroScrollView.alloc().init()
         self.view.setAutohidesScrollers_(YES)
         self.view.setHasHorizontalScroller_(horizontal)
         self.view.setHasVerticalScroller_(vertical)
@@ -662,13 +708,13 @@ class Scroller(Bin):
         self.view.setBorderType_(NSBezelBorder)
 
     def viewport_repositioned(self):
-        # If the window is resized, this translates to a viewport_repositoned()
-        # event.  So, do whatever super requires of us, then place the 
-        # chilren to work out if we need a scrollbar, then get the new size,
-        # then replace the children (which now takes into account of scrollbar
-        # size.)
+        # If the window is resized, this translates to a
+        # viewport_repositioned() event.  Instead of calling
+        # place_children() one, which is what our suporclass does, we need
+        # some extra logic here.  place the chilren to work out if we need a
+        # scrollbar, then get the new size, then replace the children (which
+        # now takes into account of scrollbar size.)
         super(Scroller, self).viewport_repositioned()
-        self.place_children()
         self.cached_size_request = self.calc_size_request()
         self.place_children()
 
@@ -701,14 +747,18 @@ class Scroller(Bin):
     def calc_size_request(self):
         if self.child:
             width = height = 0
+            try:
+                legacy = self.view.scrollerStyle() == NSScrollerStyleLegacy
+            except AttributeError:
+                legacy = True
             if not self.view.hasHorizontalScroller():
                 width = self.child.get_size_request()[0]
             if not self.view.hasVerticalScroller():
                 height = self.child.get_size_request()[1]
-            # Add a little room for the scrollbars
-            if self.view.hasHorizontalScroller():
+            # Add a little room for the scrollbars (if necessary)
+            if legacy and self.view.hasHorizontalScroller():
                 height += NSScroller.scrollerWidth()
-            if self.view.hasVerticalScroller():
+            if legacy and self.view.hasVerticalScroller():
                 width += NSScroller.scrollerWidth()
             return width, height
         else:
@@ -732,7 +782,6 @@ class Scroller(Bin):
             self.document_view.setFrame_(frame)
             self.document_view.setNeedsDisplay_(YES)
         self.view.setNeedsDisplay_(YES)
-        self.child.emit('place-in-scroller')
 
 class ExpanderView(FlippedView):
     def init(self):
