@@ -38,6 +38,7 @@ from miro import messages
 from miro.gtcache import gettext as _
 from miro.frontends.widgets.widgetconst import MAX_VOLUME
 from miro.plat import resources
+from miro.plat.utils import filename_to_unicode
 from miro.plat.frontends.widgets import threads
 from miro.plat.frontends.widgets import drawing
 from miro.plat.frontends.widgets import osxmenus
@@ -115,12 +116,14 @@ class OverlayPalette (NSWindowController):
 
         app.playback_manager.connect('will-play', self.video_will_play)
         app.playback_manager.connect('will-pause', self.video_will_pause)
+        app.playback_manager.connect('playing-info-changed',
+                                     self.on_info_changed)
 
         return self
 
     def awakeFromNib(self):
         image_path = resources.path('images/subtitles_down.png')
-        self.subtitlesButton.setImage_(NSImage.alloc().initWithContentsOfFile_(image_path))
+        self.subtitlesButton.setImage_(NSImage.alloc().initWithContentsOfFile_(filename_to_unicode(image_path)))
         self.subtitlesLabel.setTitleWithMnemonic_(_("Subtitles"))
         self.fsLabel.setTitleWithMnemonic_(_("Fullscreen"))
         self.shareButton.setImage_(getOverlayButtonImage(self.shareButton.bounds().size))
@@ -158,7 +161,7 @@ class OverlayPalette (NSWindowController):
         if item_info.remote:
             self.deleteButton.setEnabled_(False)
         self.renderer = renderer
-        self.titleLabel.setStringValue_(item_info.name)
+        self.titleLabel.setStringValue_(item_info.title)
         try:
             self.feedLabel.setStringValue_(widgetutil.get_feed_info(item_info.feed_id).name)
         except:
@@ -171,12 +174,11 @@ class OverlayPalette (NSWindowController):
             self.suspendAutoHiding()
             self.reveal(video_window)
 
-    def on_items_changed(self, changed):
-        for item_info in changed:
-            if item_info.id == self.item_info.id:
-                self.keepButton.setEnabled_(item_info.can_be_saved)
-                self.shareButton.setEnabled_(item_info.has_shareable_url)
-                self.update_(nil)
+    def on_playing_info_changed(self, new_info):
+        print 'PLAYING INFO CHANGED: ', new_info.title
+        self.keepButton.setEnabled_(new_info.can_be_saved)
+        self.shareButton.setEnabled_(new_info.has_shareable_url)
+        self.update_(nil)
                 
     def enter_fullscreen(self, videoWindow):
         self.in_fullscreen = True
@@ -191,11 +193,10 @@ class OverlayPalette (NSWindowController):
             self.adjustContent(videoWindow, True)
 
     def showSubtitlesMenu_(self, sender):
-        menu = NSMenu.alloc().init()
-        menu.setAutoenablesItems_(NO)
-        subtitles_tracks = app.playback_manager.player.get_subtitle_tracks()
-        osxmenus.populate_subtitles_menu(menu, subtitles_tracks)
-        NSMenu.popUpContextMenu_withEvent_forView_(menu, NSApp().currentEvent(), self.window().contentView())
+        subtitles_menu = app.widgetapp.menubar.find('SubtitlesMenu')
+        NSMenu.popUpContextMenu_withEvent_forView_(subtitles_menu._menu,
+                                                   NSApp().currentEvent(),
+                                                   self.window().contentView())
     
     def getHorizontalPosition(self, videoWindow, width):
         parentFrame = videoWindow.frame()
@@ -212,7 +213,7 @@ class OverlayPalette (NSWindowController):
             self.popInOutButton.setHidden_(YES)
             self.popInOutLabel.setHidden_(YES)
             image_path = resources.path('images/fullscreen_exit.png')
-            self.fsButton.setImage_(NSImage.alloc().initWithContentsOfFile_(image_path))
+            self.fsButton.setImage_(NSImage.alloc().initWithContentsOfFile_(filename_to_unicode(image_path)))
         else:
             if app.playback_manager.detached_window is None:
                 image_path = resources.path('images/popout.png')
@@ -220,12 +221,12 @@ class OverlayPalette (NSWindowController):
             else:
                 image_path = resources.path('images/popin.png')
                 label = _('Pop In')
-            self.popInOutButton.setImage_(NSImage.alloc().initWithContentsOfFile_(image_path))
+            self.popInOutButton.setImage_(NSImage.alloc().initWithContentsOfFile_(filename_to_unicode(image_path)))
             self.popInOutButton.setHidden_(NO)
             self.popInOutLabel.setHidden_(NO)
             self.popInOutLabel.setStringValue_(label)
             image_path = resources.path('images/fullscreen_enter.png')
-            self.fsButton.setImage_(NSImage.alloc().initWithContentsOfFile_(image_path))
+            self.fsButton.setImage_(NSImage.alloc().initWithContentsOfFile_(filename_to_unicode(image_path)))
 
         newFrame = self.window().frame() 
         if videoWindow.is_fullscreen or app.playback_manager.detached_window is not None: 
@@ -425,6 +426,10 @@ class OverlayPalette (NSWindowController):
         self.playPauseButton.setImage_(NSImage.imageNamed_(u'fs-button-play'))
         self.playPauseButton.setAlternateImage_(NSImage.imageNamed_(u'fs-button-play-alt'))
 
+    def on_info_changed(self, obj, item_info):
+        self.item_info = item_info
+        self.on_playing_info_changed(item_info)
+
     def remove(self):
         threads.warn_if_not_on_main_thread('OverlayPalette.remove')
         self.suspendAutoHiding()
@@ -527,7 +532,8 @@ class Slider (NSView):
 
     def drawCursor(self):
         x = self.getCursorPosition()
-        self.cursor.compositeToPoint_operation_((abs(x)+0.5, 0), NSCompositeSourceOver)
+        self.cursor.drawAtPoint_fromRect_operation_fraction_(
+          (abs(x)+0.5, 0), NSZeroRect, NSCompositeSourceOver, 1.0)
 
     def getCursorPosition(self):
         return (self.bounds().size.width - self.cursor.size().width) * self.value
