@@ -39,7 +39,9 @@ from miro import app
 from miro import prefs
 from miro import player
 from miro import iso639
+from miro.gtcache import gettext as _
 from miro.plat import utils
+from miro.plat import qttimeutils
 from miro.plat import bundle
 from miro.plat import qtcomp
 from miro.plat import script_codes
@@ -47,14 +49,6 @@ from miro.plat.frontends.widgets import threads
 from miro.plat.frontends.widgets.helpers import NotificationForwarder
 from miro.util import copy_subtitle_file
 
-###############################################################################
-
-qt_framework = pathForFramework("QuickTime.framework")
-qt_bundle = NSBundle.bundleWithPath_(qt_framework)
-loadBundleFunctions(qt_bundle, globals(), (('GetMediaLanguage', 's^^{}'),))
-
-###############################################################################
-
 def register_components():
     bundlePath = bundle.getBundlePath()
     componentsDirectoryPath = os.path.join(bundlePath, 'Contents', 'Components')
@@ -129,27 +123,6 @@ def warm_up():
     else:
         warmup_handler.handleInitialLoadStateForMovie_(warmup_movie)
 
-###############################################################################
-
-def qttime2secs(qttime):
-    timeScale = qttimescale(qttime)
-    if timeScale == 0:
-        return 0.0
-    timeValue = qttimevalue(qttime)
-    return timeValue / float(timeScale)
-
-def qttimescale(qttime):
-    if isinstance(qttime, tuple):
-        return qttime[1]
-    else:
-        return qttime.timeScale
-
-def qttimevalue(qttime):
-    if isinstance(qttime, tuple):
-        return qttime[0]
-    else:
-        return qttime.timeValue
-
 ###############################################################################
 # The QTMediaTypeSubtitle and QTMediaTypeClosedCaption media types are only
 # available in OS X 10.6 so we emulate them if they aren't defined.
@@ -189,7 +162,7 @@ class Player(player.Player):
     def set_item(self, item_info, callback, errback, force_subtitles=False):
         threads.warn_if_not_on_main_thread('quicktime.Player.set_item')
         self.reset()
-        qtmovie = self.get_movie_from_file(item_info.video_path)
+        qtmovie = self.get_movie_from_file(item_info.filename)
         self.callback = callback
         self.errback = errback
         self.force_subtitles = force_subtitles
@@ -235,17 +208,49 @@ class Player(player.Player):
             return None
         return qtmovie
 
+    def get_audio_tracks(self):
+        tracks = list()
+        if not self.movie:
+            return tracks
+        for i, track in enumerate(
+          self.movie.tracksOfMediaType_(QTMediaTypeSound)):
+            track_id = track.attributeForKey_(QTTrackIDAttribute)
+            # To avoid crappy names encoded into files, use "Track %d (xxx)
+            display_name = track.attributeForKey_(QTTrackDisplayNameAttribute)
+            name = _("Track %(track)d (%(name)s)",
+                {"track": i + 1, 
+                 "name": display_name
+                })
+            tracks.append((track_id, name))
+        return tracks
+
+    def get_enabled_audio_track(self):
+        if not self.movie:
+            return None
+        for track in self.movie.tracksOfMediaType_(QTMediaTypeSound):
+            if track.attributeForKey_(QTTrackEnabledAttribute) == 1:
+                return track.attributeForKey_(QTTrackIDAttribute)
+
+    def set_audio_track(self, new_track_id):
+        if not self.movie:
+            return
+        for track in self.movie.tracksOfMediaType_(QTMediaTypeSound):
+            track_id = track.attributeForKey_(QTTrackIDAttribute)
+            # In theory, you could have multiple enabled audio tracks, playing
+            # at the same time but that'd be a bit silly?
+            track.setEnabled_(new_track_id == track_id)
+
     def setup_subtitles(self, force_subtitles):
         if app.config.get(prefs.ENABLE_SUBTITLES) or force_subtitles:
             enabled_tracks = self.get_all_enabled_subtitle_tracks()
             if len(enabled_tracks) == 0:
                 tracks = self.get_subtitle_tracks()
                 if len(tracks) > 0:
-                    self.enable_subtitle_track(tracks[0])
+                    self.set_subtitle_track(tracks[0][1])
             elif len(enabled_tracks) > 1:
                 track_id = enabled_tracks[-1].attributeForKey_(QTTrackIDAttribute)
                 self.disable_subtitles()
-                self.enable_subtitle_track(track_id)
+                self.set_subtitle_track(track_id)
         else:
             self.disable_subtitles()
 
@@ -255,24 +260,20 @@ class Player(player.Player):
             for track in self.movie.tracks():
                 if self.is_subtitle_track(track):
                     media = track.media().quickTimeMedia()
-                    lang = GetMediaLanguage(media)
-                    display_name = track.attributeForKey_(QTTrackDisplayNameAttribute)
-                    if lang == 32767:    # 32764 = langUndefined
-                        name = display_name
-                    else:
-                        lang_code = script_codes.map_to_two_letters_code(lang)
-                        lang_info = iso639.find(lang_code)
-                        if lang_info is None:
-                            name = display_name
-                        else:
-                            name = lang_info["name"]
-                    if name != display_name:
-                        name = "%s (%s)" % (name, display_name)
-                    is_enabled = track.attributeForKey_(QTTrackEnabledAttribute) == 1
+                    name = track.attributeForKey_(QTTrackDisplayNameAttribute)
                     track_id = track.attributeForKey_(QTTrackIDAttribute)
-                    tracks.append((track_id, name, is_enabled))
+                    tracks.append((track_id, name))
         return tracks
 
+    def get_enabled_subtitle_track(self):
+        if self.movie is None:
+            return None
+        for track in self.movie.tracks():
+            if (self.is_subtitle_track(track) and
+                track.attributeForKey_(QTTrackEnabledAttribute) == 1):
+                return track.attributeForKey_(QTTrackIDAttribute)
+        return None
+
     def _find_track(self, key, value):
         if self.movie is not None:
             for track in self.movie.tracks():
@@ -290,8 +291,10 @@ class Player(player.Player):
                         tracks.append(track)
         return tracks
 
-    def get_enabled_subtitle_track(self):
-        return self._find_track(QTTrackEnabledAttribute, 1)
+    def unset_subtitle_track(self):
+        track = self._find_track(QTTrackEnabledAttribute, 1)
+        if track is not None:
+            track.setAttribute_forKey_(0, QTTrackEnabledAttribute)
 
     def get_all_enabled_subtitle_tracks(self):
         return self._find_all_tracks(QTTrackEnabledAttribute, 1)
@@ -307,13 +310,14 @@ class Player(player.Player):
         media_type = track.attributeForKey_(QTTrackMediaTypeAttribute)
         return (layer == -1 and media_type == QTMediaTypeVideo) or media_type in [QTMediaTypeSubtitle, QTMediaTypeClosedCaption]
 
-    def enable_subtitle_track(self, track_id):
-        current = self.get_enabled_subtitle_track()
-        if current is not None:
-            current.setAttribute_forKey_(0, QTTrackEnabledAttribute)
+    def set_subtitle_track(self, track_id):
+        self.unset_subtitle_track()
+        if track_id is None:
+            return
         to_enable = self.get_subtitle_track_by_id(track_id)
         if to_enable is not None:
             to_enable.setAttribute_forKey_(1, QTTrackEnabledAttribute)
+        app.menu_manager.update_menus('playback-changed')
 
     def disable_subtitles(self):
         tracks = self.get_all_enabled_subtitle_tracks()
@@ -326,7 +330,7 @@ class Player(player.Player):
             handle_successful_select()
         def handle_err():
             app.playback_manager.stop()
-        copy_subtitle_file(sub_path, self.item_info.video_path)
+        copy_subtitle_file(sub_path, self.item_info.filename)
         self.set_item(self.item_info, handle_ok, handle_err, True)
 
     def select_subtitle_encoding(self, encoding):
@@ -340,7 +344,7 @@ class Player(player.Player):
     def get_elapsed_playback_time(self):
         if self.movie:
             qttime = self.movie.currentTime()
-            return utils.qttime2secs(qttime)
+            return qttimeutils.qttime2secs(qttime)
         else:
             return 0
 
@@ -348,21 +352,11 @@ class Player(player.Player):
         if self.movie is None:
             return 0
         qttime = self.movie.duration()
-        return utils.qttime2secs(qttime)
-
-    def skip_forward(self):
-        current = self.get_elapsed_playback_time()
-        duration = self.get_total_playback_time()
-        pos = min(duration, current + 30.0)
-        self.seek_to(pos / duration)
-
-    def skip_backward(self):
-        current = self.get_elapsed_playback_time()
-        duration = self.get_total_playback_time()
-        pos = max(0, current - 15.0)
-        self.seek_to(pos / duration)
+        return qttimeutils.qttime2secs(qttime)
 
     def seek_to(self, position):
+        if not self.movie:
+            return
         qttime = self.movie.duration()
         if isinstance(qttime, tuple):
             qttime = (qttime[0] * position, qttime[1], qttime[2])
@@ -370,12 +364,9 @@ class Player(player.Player):
             qttime.timeValue = qttime.timeValue * position
         self.movie.setCurrentTime_(qttime)
 
-    def play_from_time(self, resume_time=0):
-        self.seek_to(resume_time / self.get_total_playback_time())
-        self.play()
-
     def set_playback_rate(self, rate):
-        self.movie.setRate_(rate)
+        if self.movie:
+            self.movie.setRate_(rate)
 
     def movie_load_state_changed(self, disconnect=True):
         callback = self.callback
@@ -408,6 +399,7 @@ class Player(player.Player):
                 self.movie_notifications.disconnect(
                     QTMovieLoadStateDidChangeNotification)
             self.setup_subtitles(force_subtitles)
+            tracks = self.movie.tracks()
             threads.call_on_ui_thread(callback)
         else:
             raise ValueError('Unknown QTMovieLoadStateAttribute value')
