@@ -56,7 +56,6 @@ class Image(object):
         if self.width * self.height == 0:
             raise ValueError('Image has invalid size: (%d, %d)' % (
                     self.width, self.height))
-        self.nsimage.setFlipped_(YES)
 
     def resize(self, width, height):
         return ResizedImage(self, width, height)
@@ -130,11 +129,14 @@ class NSImageDisplay(NSView):
     def drawRect_(self, dest_rect):
         if self.image is not None:
             source_rect = self.calculateSourceRectFromDestRect_(dest_rect)
-            NSGraphicsContext.currentContext().setShouldAntialias_(YES)
-            NSGraphicsContext.currentContext().setImageInterpolation_(
-                NSImageInterpolationHigh)
+            context = NSGraphicsContext.currentContext()
+            context.setShouldAntialias_(YES)
+            context.setImageInterpolation_(NSImageInterpolationHigh)
+            context.saveGraphicsState()
+            drawing.flip_context(self.bounds().size.height)
             self.image.nsimage.drawInRect_fromRect_operation_fraction_(
                 dest_rect, source_rect, NSCompositeSourceOver, 1.0)
+            context.restoreGraphicsState()
         if self.border:
             context = drawing.DrawingContext(self, self.bounds(), dest_rect)
             context.style = drawing.DrawingStyle()
@@ -189,10 +191,10 @@ class ImageDisplay(Widget):
 
 class ClickableImageButton(ImageDisplay):
     def __init__(self, image_path, max_width=None, max_height=None):
-        ImageDisplay.__init__(self)
-        self.set_border(True)
         self.max_width = max_width
         self.max_height = max_height
+        ImageDisplay.__init__(self)
+        self.set_border(True)
         self.image = None
         self._width, self._height = None, None
         if image_path:
@@ -210,12 +212,19 @@ class ClickableImageButton(ImageDisplay):
         else:
             return ImageDisplay.calc_size_request(self)
 
+class MiroImageView(NSImageView):
+    def viewWillMoveToWindow_(self, aWindow):
+        self.setAnimates_(not aWindow == nil)
+
 class AnimatedImageDisplay(Widget):
     def __init__(self, path):
         Widget.__init__(self)
-        self.nsimage = NSImage.alloc().initByReferencingFile_(filename_to_unicode(path))
-        self.view = NSImageView.alloc().init()
+        self.nsimage = NSImage.alloc().initByReferencingFile_(
+          filename_to_unicode(path))
+        self.view = MiroImageView.alloc().init()
         self.view.setImage_(self.nsimage)
+        # enabled when viewWillMoveToWindow:aWindow invoked
+        self.view.setAnimates_(NO)
 
     def calc_size_request(self):
         return self.nsimage.size().width, self.nsimage.size().height
