@@ -31,8 +31,6 @@
 associated classes.
 """
 
-from __future__ import with_statement # for python2.5
-
 import math
 import logging
 from contextlib import contextmanager
@@ -63,6 +61,10 @@ EXPANDER_PADDING = 6
 HEADER_HEIGHT = 17
 CUSTOM_HEADER_HEIGHT = 25
 
+def iter_range(ns_range):
+    """Iterate over an NSRange object"""
+    return xrange(ns_range.location, ns_range.location + ns_range.length)
+
 Rect = namedtuple('Rect', 'x y width height') 
 def NSRectToRect(nsrect):
     origin, size = nsrect.origin, nsrect.size
@@ -140,6 +142,18 @@ class HotspotTracker(object):
                     self.row)
             self.tableview.setNeedsDisplayInRect_(cell_frame)
 
+def _calc_interior_frame(total_frame, tableview):
+    """Calculate the inner cell area for a table cell.
+
+    We tell cocoa that the intercell spacing is (0, 0) and instead handle the
+    spacing ourselves.  This method calculates the area that a cell should
+    render to, given the total spacing.
+    """
+    return NSMakeRect(total_frame.origin.x + tableview.column_spacing // 2,
+            total_frame.origin.y + tableview.row_spacing // 2,
+            total_frame.size.width - tableview.column_spacing,
+            total_frame.size.height - tableview.row_spacing)
+
 class MiroTableCell(NSTextFieldCell):
     def init(self):
         return super(MiroTableCell, self).initTextCell_('')
@@ -159,7 +173,11 @@ class MiroTableCell(NSTextFieldCell):
             # OS X calls setObjectValue_('') on intialization
             NSCell.setObjectValue_(self, value_dict)
 
-class MiroTableInfoListTextCell(MiroTableCell):
+    def drawInteriorWithFrame_inView_(self, frame, view):
+        return NSTextFieldCell.drawInteriorWithFrame_inView_(self,
+                _calc_interior_frame(frame, view), view)
+
+class MiroTableItemListTextCell(MiroTableCell):
     def initWithAttrGetter_(self, attr_getter):
         self = self.init()
         self.setWraps_(NO)
@@ -168,6 +186,8 @@ class MiroTableInfoListTextCell(MiroTableCell):
         return self
 
     def drawWithFrame_inView_(self, frame, view):
+        # adjust frame based on the cell spacing
+        frame = _calc_interior_frame(frame, view)
         if (self.isHighlighted() and frame is not None and
             (view.isDescendantOf_(view.window().firstResponder()) or
              view.gradientHighlight) and view.window().isMainWindow()):
@@ -188,7 +208,7 @@ class MiroTableInfoListTextCell(MiroTableCell):
 
     def setObjectValue_(self, value):
         if isinstance(value, tuple):
-            info, attrs = value
+            info, attrs, group_info = value
             cell_text = self.attr_getter(info)
             NSCell.setObjectValue_(self, cell_text)
         else:
@@ -206,6 +226,10 @@ class MiroTableImageCell(NSImageCell):
     def setObjectValue_(self, value_dict):
         NSImageCell.setObjectValue_(self, value_dict['image'])
 
+    def drawInteriorWithFrame_inView_(self, frame, view):
+        return NSImageCell.drawInteriorWithFrame_inView_(self,
+                _calc_interior_frame(frame, view), view)
+
 class MiroCheckboxCell(NSButtonCell):
     def init(self):
         self = super(MiroCheckboxCell, self).init()
@@ -244,7 +268,13 @@ class MiroCheckboxCell(NSButtonCell):
         return NSButtonCell.stopTracking_at_inView_mouseIsUp_(self, lastPoint,
                 at, tableview, mouseIsUp)
 
+    def drawInteriorWithFrame_inView_(self, frame, view):
+        return NSButtonCell.drawInteriorWithFrame_inView_(self,
+                _calc_interior_frame(frame, view), view)
+
 class CellRendererBase(object):
+    DRAW_BACKGROUND = True
+
     def set_index(self, index):
         self.index = index
     
@@ -353,6 +383,15 @@ class CustomTableCell(NSCell):
 
     def drawInteriorWithFrame_inView_(self, frame, view):
         NSGraphicsContext.currentContext().saveGraphicsState()
+        if not self.wrapper.IGNORE_PADDING:
+            # adjust frame based on the cell spacing. We also have to adjust
+            # the hover position to account for the new frame
+            original_frame = frame
+            frame = _calc_interior_frame(frame, view)
+            hover_adjustment = (frame.origin.x - original_frame.origin.x,
+                                frame.origin.y - original_frame.origin.y)
+        else:
+            hover_adjustment = (0, 0)
         if self.wrapper.outline_column:
             pad_left = EXPANDER_PADDING
         else:
@@ -365,6 +404,9 @@ class CustomTableCell(NSCell):
         self.set_wrapper_data()
         column = self.wrapper.get_index()
         hover_pos = view.get_hover(self.row, column)
+        if hover_pos is not None:
+            hover_pos = [hover_pos[0] - hover_adjustment[0],
+                         hover_pos[1] - hover_adjustment[1]]
         self.wrapper.render(context, self.layout_manager, self.isHighlighted(),
                 self.hotspot, hover_pos)
         NSGraphicsContext.currentContext().restoreGraphicsState()
@@ -378,6 +420,8 @@ class CustomTableCell(NSCell):
 class CustomCellRenderer(CellRendererBase):
     CellClass = CustomTableCell
 
+    IGNORE_PADDING = False
+
     def __init__(self):
         self.outline_column = False
         self.index = None
@@ -398,33 +442,34 @@ class CustomCellRenderer(CellRendererBase):
     def hotspot_test(self, style, layout, x, y, width, height):
         return None
     
-class InfoListTableCell(CustomTableCell):
+class ItemListTableCell(CustomTableCell):
     def set_wrapper_data(self):
-        self.wrapper.info, self.wrapper.attrs = self.object_value
+        self.wrapper.info, self.wrapper.attrs, self.wrapper.group_info = \
+                self.object_value
 
-class InfoListRenderer(CustomCellRenderer):
-    CellClass = InfoListTableCell
+class ItemListRenderer(CustomCellRenderer):
+    CellClass = ItemListTableCell
 
     def hotspot_test(self, style, layout, x, y, width, height):
         return None
 
-class InfoListRendererText(CellRenderer):
+class ItemListRendererText(CellRenderer):
     def build_cell(self):
-        cell = MiroTableInfoListTextCell.alloc()
+        cell = MiroTableItemListTextCell.alloc()
         return cell.initWithAttrGetter_(self.get_value)
 
 def calc_row_height(view, model_row):
-    row_height = 0
+    max_height = 0
     model = view.dataSource().model
     for column in view.tableColumns():
         cell = column.dataCell()
         data = model.get_column_data(model_row, column)
         cell.setObjectValue_(data)
         cell_height = cell.calcHeight_(view)
-        row_height = max(row_height, cell_height)
-    if row_height == 0:
-        row_height = 12
-    return row_height
+        max_height = max(max_height, cell_height)
+    if max_height == 0:
+        max_height = 12
+    return max_height + view.row_spacing
 
 class TableViewDelegate(NSObject):
     def tableView_willDisplayCell_forTableColumn_row_(self, view, cell,
@@ -515,8 +560,14 @@ class TableViewCommon(object):
         self.setFocusRingType_(NSFocusRingTypeNone)
         self.handled_last_mouse_down = False
         self.gradientHighlight = False
-        self._column_wrappers = []
         self.tracking_area = None
+        self.group_lines_enabled = False
+        self.group_line_width = 1
+        self.group_line_color = (0, 0, 0, 1.0)
+        # we handle cell spacing manually
+        self.setIntercellSpacing_(NSSize(0, 0))
+        self.column_spacing = 3
+        self.row_spacing = 2
         return self
 
     def updateTrackingAreas(self):
@@ -538,7 +589,6 @@ class TableViewCommon(object):
     def addTableColumn_(self, column):
         index = len(self.tableColumns())
         column.set_index(index)
-        self._column_wrappers.append(column)
         self.column_index_map[column._column] = index
         self.SuperClass.addTableColumn_(self, column._column)
 
@@ -548,9 +598,6 @@ class TableViewCommon(object):
         for key, index in self.column_index_map.items():
             if index > removed:
                 self.column_index_map[key] -= 1
-        for column in self._column_wrappers[:]:
-            if column._column == column:
-                del self._column_wrappers[column]
 
     def moveColumn_toColumn_(self, src, dest):
         # Need to switch the TableColumn objects too
@@ -579,10 +626,16 @@ class TableViewCommon(object):
         widget = wrappermap.wrapper(self)
         window = widget.get_window()
         if window and window.is_active():
-            start_color = (0.588, 0.717, 0.843)
-            end_color = (0.416, 0.568, 0.713)
-            top_line_color = (0.416, 0.569, 0.714, 1.0)
-            bottom_line_color = (0.416, 0.569, 0.714, 1.0)
+            if focused:
+                start_color = (0.588, 0.717, 0.843)
+                end_color = (0.416, 0.568, 0.713)
+                top_line_color = (0.416, 0.569, 0.714, 1.0)
+                bottom_line_color = (0.416, 0.569, 0.714, 1.0)
+            else:
+                start_color = (168 / 255.0, 188 / 255.0, 208 / 255.0)
+                end_color = (129 / 255.0, 152 / 255.0, 176 / 255.0)
+                top_line_color = (129 / 255.0, 152 / 255.0, 175 / 255.0, 1.0)
+                bottom_line_color = (0.416, 0.569, 0.714, 1.0)
         else:
             start_color = (0.675, 0.722, 0.765)
             end_color = (0.592, 0.659, 0.710)
@@ -615,6 +668,98 @@ class TableViewCommon(object):
         gradient.draw()
         context.restoreGraphicsState()
 
+    def drawBackgroundInClipRect_(self, clip_rect):
+        # save our graphics state, since we are about to modify the clip path
+        graphics_context = NSGraphicsContext.currentContext()
+        graphics_context.saveGraphicsState()
+        # create a NSBezierPath that contains the rects of the columns with
+        # DRAW_BACKGROUND True.
+        clip_path = NSBezierPath.bezierPath()
+        number_of_columns = len(self.tableColumns())
+        for col_index in iter_range(self.columnsInRect_(clip_rect)):
+            column = wrappermap.wrapper(self.tableColumns()[col_index])
+            column_rect = None
+            if column.renderer.DRAW_BACKGROUND:
+                # We should draw the background for this column, add it's rect
+                # to our clip rect.
+                column_rect = self.rectOfColumn_(col_index)
+                clip_path.appendBezierPathWithRect_(column_rect)
+            else:
+                # We shouldn't draw the background for this column.  Don't add
+                # anything to the clip rect, but do draw the area before the
+                # first row and after the last row.
+                self.drawBackgroundOutsideContent_clipRect_(col_index,
+                        clip_rect)
+            if col_index == number_of_columns - 1: # last column
+                if not column_rect:
+                    column_rect = self.rectOfColumn_(col_index)
+                column_right = column_rect.origin.x + column_rect.size.width
+                clip_right = clip_rect.origin.x + clip_rect.size.width
+                if column_right < clip_right:
+                    # there's space to the right, so add that to the clip_rect
+                    remaining = clip_right - column_right
+                    left_rect = NSMakeRect(column_right, clip_rect.origin.y,
+                                           remaining, clip_rect.size.height)
+                    clip_path.appendBezierPathWithRect_(left_rect)
+        # clip to that path
+        clip_path.addClip()
+        # do the default drawing
+        self.SuperClass.drawBackgroundInClipRect_(self, clip_rect)
+        # restore graphics state
+        graphics_context.restoreGraphicsState()
+
+    def drawBackgroundOutsideContent_clipRect_(self, index, clip_rect):
+        """Draw our background outside the rows with content
+
+        We call this for cells with DRAW_BACKGROUND set to False.  For those,
+        we let the cell draw their own background, but we still need to draw
+        the background before the first cell and after the last cell.
+        """
+
+        self.backgroundColor().set()
+
+        total_rect = NSIntersectionRect(self.rectOfColumn_(index), clip_rect)
+
+        if self.numberOfRows() == 0:
+            # if no rows are selected, draw the background over everything
+            NSRectFill(total_rect)
+            return
+
+        # fill the area above the first row
+        first_row_rect = self.rectOfRow_(0)
+        if first_row_rect.origin.y > total_rect.origin.y:
+            height = first_row_rect.origin.y - total_rect.origin.y
+            NSRectFill(NSMakeRect(total_rect.origin.x, total_rect.origin.y,
+                    total_rect.size.width, height))
+
+        # fill the area below the last row
+        last_row_rect = self.rectOfRow_(self.numberOfRows()-1)
+        if NSMaxY(last_row_rect) < NSMaxY(total_rect):
+            y = NSMaxY(last_row_rect) + 1
+            height = NSMaxY(total_rect) - NSMaxY(last_row_rect)
+            NSRectFill(NSMakeRect(total_rect.origin.x, y,
+                    total_rect.size.width, height))
+
+    def drawRow_clipRect_(self, row, clip_rect):
+        self.SuperClass.drawRow_clipRect_(self, row, clip_rect)
+        if self.group_lines_enabled:
+            self.drawGroupLine_(row)
+
+    def drawGroupLine_(self, row):
+        model = wrappermap.wrapper(self).model
+        if (not isinstance(model, tablemodel.ItemListModel) or
+                model.item_list.get_grouping() is None):
+            return
+
+        info, attrs, group_info = model[row]
+        if group_info[0] == group_info[1] - 1:
+            rect = self.rectOfRow_(row)
+            rect.origin.y = NSMaxY(rect) - self.group_line_width
+            rect.size.height = self.group_line_width
+            NSColor.colorWithDeviceRed_green_blue_alpha_(
+                    *self.group_line_color).set()
+            NSRectFill(rect)
+
     def canDragRowsWithIndexes_atPoint_(self, indexes, point):
         return YES
 
@@ -717,9 +862,24 @@ class TableViewCommon(object):
     def handleContextMenu_(self, event):
         self.window().makeFirstResponder_(self)
         point = self.convertPoint_fromView_(event.locationInWindow(), nil)
+        column = self.columnAtPoint_(point)
         row = self.rowAtPoint_(point)
+        if self.group_lines_enabled and column == 0:
+            self.selectAllItemsInGroupForRow_(row)
         self.popup_context_menu(row, event)
 
+    def selectAllItemsInGroupForRow_(self, row):
+        wrapper = wrappermap.wrapper(self)
+        model = wrapper.model
+        if (not isinstance(model, tablemodel.ItemListModel) or
+                model.item_list.get_grouping() is None):
+            return
+
+        info, attrs, group_info = model[row]
+        select_range = NSMakeRange(row - group_info[0], group_info[1])
+        index_set = NSIndexSet.indexSetWithIndexesInRange_(select_range)
+        self.selectRowIndexes_byExtendingSelection_(index_set, NO)
+
     def popup_context_menu(self, row, event):
         selection = self.selectedRowIndexes()
         if row != -1 and not selection.containsIndex_(row):
@@ -769,7 +929,9 @@ class TableColumn(signals.SignalEmitter):
     def __init__(self, title, renderer, header=None, **attrs):
         signals.SignalEmitter.__init__(self)
         self.create_signal('clicked')
-        self._column = NSTableColumn.alloc().initWithIdentifier_(attrs)
+        self._column = MiroTableColumn.alloc().initWithIdentifier_(title)
+        self._column.set_attrs(attrs)
+        wrappermap.add(self._column, self)
         header_cell = MiroTableHeaderCell.alloc().init()
         self.custom_header = False
         if header:
@@ -783,6 +945,7 @@ class TableColumn(signals.SignalEmitter):
         self.sort_order_ascending = True
         self.sort_indicator_visible = False
         self.do_horizontal_padding = True
+        self.min_width = self.max_width = None
         renderer.setDataCell_(self._column)
 
     def set_do_horizontal_padding(self, horizontal_padding):
@@ -795,10 +958,10 @@ class TableColumn(signals.SignalEmitter):
             self._column.headerCell().setAlignment_(NSLeftTextAlignment)
 
     def set_min_width(self, width):
-        self._column.setMinWidth_(width)
+        self.min_width = width
 
     def set_max_width(self, width):
-        self._column.setMaxWidth_(width)
+        self.max_width = width
 
     def set_width(self, width):
         self._column.setWidth_(width)
@@ -830,6 +993,13 @@ class TableColumn(signals.SignalEmitter):
         self.index = index
         self.renderer.set_index(index)
 
+class MiroTableColumn(NSTableColumn):
+    def set_attrs(self, attrs):
+        self._attrs = attrs
+
+    def attrs(self):
+        return self._attrs
+
 class MiroTableView(NSTableView):
     SuperClass = NSTableView
     for name, value in TableViewCommon.__dict__.items():
@@ -847,8 +1017,8 @@ class MiroTableHeaderView(NSTableHeaderView):
     def initWithFrame_(self, frame):
         # frame is not used
         self = super(MiroTableHeaderView, self).initWithFrame_(frame)
-        self.custom_header = False
         self.selected = None
+        self.custom_header = False
         return self
 
     def drawRect_(self, rect):
@@ -900,7 +1070,10 @@ class MiroTableHeaderCell(NSTableHeaderCell):
         context = DrawingContext(view, drawing_rect, drawing_rect)
         context.style = self.make_drawing_style(frame, view)
         self.layout_manager.reset()
-        self.button.draw(context, self.layout_manager)
+        columns = wrappermap.wrapper(view.tableView()).columns
+        header_cells = [c._column.headerCell() for c in columns]
+        background_only = not self in header_cells
+        self.button.draw(context, self.layout_manager, background_only)
         NSGraphicsContext.currentContext().restoreGraphicsState()
 
     def make_drawing_style(self, frame, view):
@@ -935,6 +1108,9 @@ class CocoaSelectionOwnerMixin(SelectionOwnerMixin):
             return None
         return self.model.iter_for_row(self.tableview, row)
 
+    def _get_selected_rows(self):
+        return [self.model[i] for i in self._get_selected_iters()]
+
     @property
     def num_rows_selected(self):
         return self.tableview.numberOfSelectedRows()
@@ -964,7 +1140,6 @@ class CocoaScrollbarOwnerMixin(ScrollbarOwnerMixin):
     """Manages a TableView's scroll position."""
     def __init__(self):
         ScrollbarOwnerMixin.__init__(self, _work_around_17153=True)
-        self.connect('place-in-scroller', self.on_place_in_scroller)
         self.scroll_position = (0, 0)
         self.clipview_notifications = None
         self._position_set = False
@@ -992,11 +1167,6 @@ class CocoaScrollbarOwnerMixin(ScrollbarOwnerMixin):
         rect = NSMakeRect(scroll_to[0], scroll_to[1], size[0], size[1])
         self.tableview.scrollRectToVisible_(rect)
 
-    def on_place_in_scroller(self, scroller):
-        # workaround for 17153.1
-        if not self._position_set:
-            self._set_scroll_position(self.scroll_position)
-
     @property
     def _manually_scrolled(self):
         """Return whether the view has been scrolled explicitly by the user
@@ -1077,7 +1247,7 @@ class TableView(CocoaSelectionOwnerMixin, CocoaScrollbarOwnerMixin, Widget):
 
     draws_selection = True
 
-    def __init__(self, model):
+    def __init__(self, model, custom_headers=False):
         Widget.__init__(self)
         CocoaSelectionOwnerMixin.__init__(self)
         CocoaScrollbarOwnerMixin.__init__(self)
@@ -1089,9 +1259,9 @@ class TableView(CocoaSelectionOwnerMixin, CocoaScrollbarOwnerMixin, Widget):
         self.columns = []
         self.drag_source = self.drag_dest = None
         self.context_menu_callback = None
-        if isinstance(model, tablemodel.InfoListModel):
+        if isinstance(model, tablemodel.ItemListModel):
             self.tableview = MiroTableView.alloc().init()
-            self.data_source = tablemodel.MiroInfoListDataSource.alloc()
+            self.data_source = tablemodel.MiroItemListDataSource.alloc()
         elif self.is_tree():
             self.create_signal('row-expanded')
             self.create_signal('row-collapsed')
@@ -1100,6 +1270,8 @@ class TableView(CocoaSelectionOwnerMixin, CocoaScrollbarOwnerMixin, Widget):
         else:
             self.tableview = MiroTableView.alloc().init()
             self.data_source = tablemodel.MiroTableViewDataSource.alloc()
+        types = (tablemodel.MIRO_DND_ITEM_LOCAL,)
+        self.tableview.registerForDraggedTypes_(types)
         self.view = self.tableview
         self.data_source.initWithModel_(self.model)
         self.tableview.setDataSource_(self.data_source)
@@ -1112,17 +1284,38 @@ class TableView(CocoaSelectionOwnerMixin, CocoaScrollbarOwnerMixin, Widget):
         self.header_view = MiroTableHeaderView.alloc().initWithFrame_(
             NSMakeRect(0, 0, 0, 0))
         self.tableview.setCornerView_(None)
-        self.header_view.custom_header = False
-        self.custom_header = 0
+        self.custom_header = False
         self.header_height = HEADER_HEIGHT
         self.set_show_headers(True)
         self.notifications = NotificationForwarder.create(self.tableview)
-        self.model.connect_weak('row-changed', self.on_row_change)
-        self.model.connect_weak('structure-will-change',
-                self.on_model_structure_change)
+        self.model_signal_ids = [
+            self.model.connect_weak('row-changed', self.on_row_change),
+            self.model.connect_weak('structure-will-change',
+                    self.on_model_structure_change),
+        ]
         self.iters_to_update = []
         self.height_changed = self.reload_needed = False
+        self.old_selection = None
         self._resizing = False
+        if custom_headers:
+            self._enable_custom_headers()
+
+    def unset_model(self):
+        for signal_id in self.model_signal_ids:
+            self.model.disconnect(signal_id)
+        self.model = None
+        self.tableview.setDataSource_(None)
+        self.data_source = None
+
+    def _enable_custom_headers(self):
+        self.custom_header = True
+        self.header_height = CUSTOM_HEADER_HEIGHT
+        self.header_view.custom_header = True
+        self.tableview.setCornerView_(SorterPadding.alloc().init())
+
+    def enable_album_view_focus_hack(self):
+        # this only matters on GTK
+        pass
 
     def focus(self):
         if self.tableview.window() is not None:
@@ -1140,9 +1333,14 @@ class TableView(CocoaSelectionOwnerMixin, CocoaScrollbarOwnerMixin, Widget):
             self.tableview.hotspot_tracker.update_hit()
 
     def on_model_structure_change(self, model):
-        self.reload_needed = True
+        self.will_need_reload()
         self.cancel_hotspot_track()
 
+    def will_need_reload(self):
+        if not self.reload_needed:
+            self.reload_needed = True
+            self.old_selection = self._get_selected_rows()
+
     def cancel_hotspot_track(self):
         if self.tableview.hotspot_tracker is not None:
             self.tableview.hotspot_tracker.redraw_cell()
@@ -1152,13 +1350,13 @@ class TableView(CocoaSelectionOwnerMixin, CocoaScrollbarOwnerMixin, Widget):
         self.invalidate_size_request()
         item = notification.userInfo()['NSObject']
         iter_ = self.model.iter_for_item[item]
-        self.emit('row-expanded', iter_, self.model.get_path(iter_))
+        self.emit('row-expanded', iter_)
 
     def on_collapsed(self, notification):
         self.invalidate_size_request()
         item = notification.userInfo()['NSObject']
         iter_ = self.model.iter_for_item[item]
-        self.emit('row-collapsed', iter_, self.model.get_path(iter_))
+        self.emit('row-collapsed', iter_)
 
     def on_column_resize(self, notification):
         if self.auto_resizing or self._resizing:
@@ -1193,11 +1391,24 @@ class TableView(CocoaSelectionOwnerMixin, CocoaScrollbarOwnerMixin, Widget):
         return self.tableview.isItemExpanded_(iter.value())
 
     def calc_size_request(self):
-        self.tableview.tile()
-        height = self.tableview.frame().size.height
+        return self.calc_width(), self.calc_height()
+
+    def calc_height(self):
         if self._show_headers:
-            height += self.header_height
-        return self.calc_width(), height
+            height = self.header_height
+        else:
+            height = 0
+        if self.tableview.numberOfRows() == 0:
+            # No rows in the table, NSTableView uses 4px of height in this
+            # case.
+            return height + 4
+        if self.fixed_height:
+            height += self.tableview.rowHeight() * self.tableview.numberOfRows()
+        else:
+            for i in xrange(self.tableview.numberOfRows()):
+                it = self.model.iter_for_row(self.tableview, i)
+                height += calc_row_height(self.tableview, self.model[it])
+        return height
 
     def viewport_repositioned(self):
         self._do_layout()
@@ -1278,7 +1489,7 @@ class TableView(CocoaSelectionOwnerMixin, CocoaScrollbarOwnerMixin, Widget):
         if self.auto_resize:
             # Table auto-resizes, we can shrink to min-width for each column
             width = sum(column.minWidth() for column in columns)
-            width += self.tableview.intercellSpacing().width * self.column_count()
+            width += self.tableview.column_spacing * self.column_count()
         else:
             # Table doesn't auto-resize, the columns can't get smaller than
             # their current width
@@ -1289,7 +1500,7 @@ class TableView(CocoaSelectionOwnerMixin, CocoaScrollbarOwnerMixin, Widget):
         # stop our model from emitting signals, which is slow if we're
         # adding/removing/changing a bunch of rows.  Instead, just reload the
         # model afterwards.
-        self.reload_needed = True
+        self.will_need_reload()
         self.cancel_hotspot_track()
         self.model.freeze_signals()
 
@@ -1300,6 +1511,10 @@ class TableView(CocoaSelectionOwnerMixin, CocoaScrollbarOwnerMixin, Widget):
         size_changed = False
         if self.reload_needed:
             self.tableview.reloadData()
+            new_selection = self._get_selected_rows()
+            if new_selection != self.old_selection:
+                self.on_selection_changed(self.tableview)
+            self.old_selection = None
             size_changed = True
         elif self.iters_to_update:
             if self.fixed_height or not self.height_changed:
@@ -1324,7 +1539,7 @@ class TableView(CocoaSelectionOwnerMixin, CocoaScrollbarOwnerMixin, Widget):
                         # just ignore.
                         pass
                 self.tableview.noteHeightOfRowsWithIndexesChanged_(index_set)
-            size_changed = True
+                size_changed = True
         else:
             return
         if size_changed:
@@ -1336,18 +1551,16 @@ class TableView(CocoaSelectionOwnerMixin, CocoaScrollbarOwnerMixin, Widget):
         """If the table is width pixels big, how much width is available for
         the table's columns.
         """
-        spacing = self.tableview.intercellSpacing().width * self.column_count()
-        return width - spacing
+        # XXX this used to do some calculation with the spacing of each column,
+        # but it doesn't appear like we need it to be that complicated anymore
+        # (see #18273)
+        return width - 2
 
     def set_column_spacing(self, column_spacing):
-        spacing = self.tableview.intercellSpacing()
-        spacing.width = column_spacing
-        self.tableview.setIntercellSpacing_(spacing)
+        self.tableview.column_spacing = column_spacing
 
     def set_row_spacing(self, row_spacing):
-        spacing = self.tableview.intercellSpacing()
-        spacing.height = row_spacing
-        self.tableview.setIntercellSpacing_(spacing)
+        self.tableview.row_spacing = row_spacing
 
     def set_alternate_row_backgrounds(self, setting):
         self.tableview.setUsesAlternatingRowBackgroundColors_(setting)
@@ -1363,18 +1576,25 @@ class TableView(CocoaSelectionOwnerMixin, CocoaScrollbarOwnerMixin, Widget):
     def set_gradient_highlight(self, setting):
         self.tableview.gradientHighlight = setting
 
+    def set_group_lines_enabled(self, enabled):
+        self.tableview.group_lines_enabled = enabled
+        self.queue_redraw()
+
+    def set_group_line_style(self, color, width):
+        self.tableview.group_line_color = color + (1.0,)
+        self.tableview.group_line_width = width
+        self.queue_redraw()
+
     def get_tooltip(self, iter, column):
         return None
 
     def add_column(self, column):
-        if column.custom_header:
-            self.header_view.custom_header = True
-            if not self.custom_header:
-                self.tableview.setCornerView_(SorterPadding.alloc().init())
-            self.custom_header += 1
-            self.header_height = CUSTOM_HEADER_HEIGHT
+        if not self.custom_header == column.custom_header:
+            raise ValueError('Column header does not match type '
+                             'required by TableView')
         self.columns.append(column)
         self.tableview.addTableColumn_(column)
+        self._set_min_max_column_widths(column)
         if self.column_count() == 1 and self.is_tree():
             self.tableview.setOutlineTableColumn_(column._column)
             column.renderer.outline_column = True
@@ -1384,17 +1604,21 @@ class TableView(CocoaSelectionOwnerMixin, CocoaScrollbarOwnerMixin, Widget):
         self.invalidate_size_request()
         self.try_to_set_row_height()
 
+    def _set_min_max_column_widths(self, column):
+        if column.do_horizontal_padding:
+            spacing = self.tableview.column_spacing
+        else:
+            spacing = 0
+        if column.min_width > 0:
+            column._column.setMinWidth_(column.min_width + spacing)
+        if column.max_width > 0:
+            column._column.setMaxWidth_(column.max_width + spacing)
+
     def column_count(self):
         return len(self.tableview.tableColumns())
 
     def remove_column(self, index):
         column = self.columns.pop(index)
-        if column.custom_header:
-            self.custom_header -= 1
-        if self.custom_header == 0:
-            self.tableview.setCornerView_(None)
-            self.header_view.custom_header = False
-            self.header_height = HEADER_HEIGHT
         self.tableview.removeTableColumn_(column._column)
         self.invalidate_size_request()
 
@@ -1484,9 +1708,7 @@ class TableView(CocoaSelectionOwnerMixin, CocoaScrollbarOwnerMixin, Widget):
     def set_drag_dest(self, drag_dest):
         self.drag_dest = drag_dest
         if drag_dest is None:
-            self.tableview.unregisterDraggedTypes()
             self.data_source.setDragDest_(None)
         else:
             types = drag_dest.allowed_types()
-            self.tableview.registerForDraggedTypes_(types)
             self.data_source.setDragDest_(drag_dest)
