@@ -65,7 +65,9 @@ class MiroResponderInterceptor(NSResponder):
         We will give the wrapper for responder a chance to handle the event,
         then pass it along to responder.
         """
+        self = super(MiroResponderInterceptor, self).init()
         self.responder = responder
+        return self
 
     def keyDown_(self, event):
         if self.sendKeyDownToWrapper_(event):
@@ -83,8 +85,8 @@ class MiroResponderInterceptor(NSResponder):
 
         # Make a new MiroResponderInterceptor whose responder is the next
         # responder down the chain.
-        next_intercepter = MiroResponderInterceptor.alloc()
-        next_intercepter.initWithResponder_(self.responder.nextResponder())
+        next_intercepter = MiroResponderInterceptor.alloc().initWithResponder_(
+                           self.responder.nextResponder())
         # Install the interceptor
         self.responder.setNextResponder_(next_intercepter)
         # Send event along
@@ -118,8 +120,8 @@ class MiroWindow(NSWindow):
     def handleKeyDown_(self, event):
         if self.handle_tab_navigation(event):
             return
-        interceptor = MiroResponderInterceptor.alloc()
-        interceptor.initWithResponder_(self.firstResponder())
+        interceptor = MiroResponderInterceptor.alloc().initWithResponder_(
+                          self.firstResponder())
         interceptor.keyDown_(event)
 
     def handle_tab_navigation(self, event):
@@ -287,6 +289,7 @@ class Window(signals.SignalEmitter):
 
     def on_content_widget_size_request_change(self, widget, old_size):
         self.update_size_constraints()
+        self.place_child()
 
     def set_content_widget(self, widget):
         if self.content_widget:
@@ -371,7 +374,7 @@ class Dialog(DialogBase):
 
     def build_text(self):
         vbox = VBox(spacing=6)
-        if self.description:
+        if self.description is not None:
             description_label = Label(self.description, wrap=True)
             description_label.set_bold(True)
             description_label.set_size_request(360, -1)
@@ -425,11 +428,7 @@ class Dialog(DialogBase):
 
     def on_content_widget_size_request_change(self, widget, old_size):
         width, height = self.content_widget.get_size_request()
-        # It is possible the window is torn down between the size invalidate
-        # request and the actual size invalidation invocation.  So check
-        # to see if nswindow is there if not then do not do anything.
-        if self.window and (width, height) != old_size:
-            self.change_content_size(width, height)
+        self.change_content_size(width, height)
 
     def change_content_size(self, width, height):
         content_rect = self.window.contentRectForFrameRect_(
@@ -443,6 +442,9 @@ class Dialog(DialogBase):
         content_rect.size = NSSize(width, height)
         new_frame = self.window.frameRectForContentRect_(content_rect)
         self.window.setFrame_display_(new_frame, NO)
+        # Need to call place() again, since our window has changed size
+        contentView = self.window.contentView()
+        self.content_widget.place(contentView.frame(), contentView)
 
     def run(self):
         self.window = self.build_window()
@@ -692,7 +694,7 @@ class PreferenceToolbarDelegate(NSObject):
                                                                  flag):
         panel = self.panels[itemIdentifier]
         item = PreferenceItem.alloc().initWithItemIdentifier_(itemIdentifier)
-        item.setLabel_(panel[1])
+        item.setLabel_(unicode(panel[1]))
         item.setImage_(NSImage.imageNamed_(u"pref_tab_%s" % itemIdentifier))
         item.setAction_("switchPreferenceView:")
         item.setTarget_(self)
@@ -717,6 +719,35 @@ class DialogWindow(Window):
             mask |= NSMiniaturizableWindowMask
         return mask
 
+class DonateWindow(Window):
+    def __init__(self, title):
+        Window.__init__(self, title, Rect(0, 0, 640, 440))
+        self.panels = dict()
+        self.identifiers = list()
+        self.first_show = True
+        self.nswindow.setShowsToolbarButton_(NO)
+        self.nswindow.setReleasedWhenClosed_(NO)
+        self.app_notifications = NotificationForwarder.create(NSApp())
+        self.app_notifications.connect(self.on_app_quit, 
+            'NSApplicationWillTerminateNotification')
+
+    def destroy(self):
+        super(PreferencesWindow, self).destroy()
+        self.app_notifications.disconnect()
+
+    def get_style_mask(self):
+        return (NSTitledWindowMask | NSClosableWindowMask |
+                NSMiniaturizableWindowMask)
+ 
+    def show(self):
+        if self.first_show:
+            self.nswindow.center()
+            self.first_show = False
+        Window.show(self)
+
+    def on_app_quit(self, notification):
+        self.close()
+
 class PreferencesWindow(Window):
     def __init__(self, title):
         Window.__init__(self, title, Rect(0, 0, 640, 440))
