@@ -36,6 +36,7 @@ import sys
 import time
 import errno
 import signal
+import struct
 import subprocess
 import plistlib
 
@@ -45,13 +46,17 @@ from AppKit import *
 
 from miro import app
 from miro import prefs
-from miro.gtcache import gettext as _
+from miro.gtcache import gettext_lazy as _
 from miro.importmedia import import_itunes_path
-from miro.util import returns_unicode, returns_binary, check_u, check_b
+from miro.util import (returns_unicode,
+                       returns_binary, check_u, check_b,
+                       bits_32)
 from miro.plat.filenames import (PlatformFilenameType,
                                  os_filename_to_filename_type,
                                  filename_type_to_os_filename)
+from miro.plat import qt_extractor
 from miro.plat.frontends.widgets.threads import on_ui_thread
+from miro.plat.popen import Popen
 
 # We need to define samefile for the portable code.  Lucky for us, this is
 # very easy.
@@ -165,26 +170,25 @@ def initialize_locale():
     _locale_initialized = True
     del pool
 
-def setup_logging (in_downloader=False):
-    if in_downloader:
-        log_name = os.environ.get("DEMOCRACY_DOWNLOADER_LOG")
-        level = logging.INFO
+def setup_logging(pathname, main_process=False):
+    if app.debugmode:
+        level = logging.DEBUG
     else:
-        log_name = app.config.get(prefs.LOG_PATHNAME)
-        if app.config.get(prefs.APP_VERSION).endswith("git"):
-            level = logging.DEBUG
-        else:
-            level = logging.WARN
+        level = logging.INFO
 
-    logging.basicConfig(level=level, format='%(levelname)-8s %(message)s')
     rotater = logging.handlers.RotatingFileHandler(
-        log_name, mode="w", maxBytes=100000, backupCount=5)
+        pathname, mode="w", maxBytes=100000, backupCount=5)
     formatter = logging.Formatter('%(asctime)s %(levelname)-8s %(message)s')
     rotater.setFormatter(formatter)
     logging.getLogger('').addHandler(rotater)
     logging.getLogger('').setLevel(level)
     rotater.doRollover()
 
+    if main_process:
+        stdouthandler = logging.StreamHandler(sys.stdout)
+        stdouthandler.setFormatter(formatter)
+        logging.getLogger('').addHandler(stdouthandler)
+
 @returns_binary
 def utf8_to_filename(filename):
     if not isinstance(filename, str):
@@ -347,20 +351,20 @@ def _app_command_line():
     """Get the command line to lanch the Miro.app bundle. """
     exe = NSBundle.mainBundle().executablePath()
 
-    os_info = os.uname()
-    os_version = int(os_info[2].split('.')[0])
-    if os_version < 9:
-        return [exe]
+    # If we are running in 32 bit mode we want our spawned stuff to be
+    # 32-bit too.  Otherwise we running in the "best" mode so do nothing.
+    if bits_32():
+        return ['/usr/bin/arch', '-32', exe]
     else:
-        arch = subprocess.Popen("/usr/bin/arch", stdout=subprocess.PIPE).communicate()[0].strip()
-        return ['/usr/bin/arch', '-%s' % arch, exe]
+        return [exe]
 
 @on_ui_thread
 def launch_download_daemon(oldpid, env):
     kill_process(oldpid)
 
     env['DEMOCRACY_DOWNLOADER_LOG'] = app.config.get(prefs.DOWNLOADER_LOG_PATHNAME)
-    env['VERSIONER_PYTHON_PREFER_32_BIT'] = "yes"
+    if bits_32():
+        env['VERSIONER_PYTHON_PREFER_32_BIT'] = "yes"
     env["MIRO_APP_VERSION"] = app.config.get(prefs.APP_VERSION)
     env.update(os.environ)
 
@@ -445,16 +449,16 @@ def _get_cmd_line_and_env_for_script(script_name):
             env[k] = env[k].encode('utf-8')
     return ((py_exe_path, script_path), env)
 
-def movie_data_program_info(movie_path, thumbnail_path):
-    cmd_line, env = _get_cmd_line_and_env_for_script('qt_extractor.py')
-    return (cmd_line + (movie_path, thumbnail_path), env)
+def run_media_metadata_extractor(movie_path, thumbnail_path):
+    return qt_extractor.run(movie_path, thumbnail_path)
 
 def miro_helper_program_info():
     cmd_line = _app_command_line() + [u'--miro-helper']
     env = {
-            'VERSIONER_PYTHON_PREFER_32_BIT': 'yes',
             'MIRO_APP_VERSION': app.config.get(prefs.APP_VERSION)
     }
+    if bits_32():
+        env['VERSIONER_PYTHON_PREFER_32_BIT'] = 'yes'
     return cmd_line, env
 
 ###############################################################################
@@ -467,8 +471,11 @@ def get_segmenter_executable_path():
     return path.encode('utf-8')
 
 def get_transcode_video_options():
-    has_video_args = ['-vcodec', 'libx264', '-vpre', 'ipod320',
-                      '-vpre', 'superfast', '-threads', '0', '-s', '480x360',
+    has_video_args = ['-vcodec', 'libx264', '-coder', '0', '-bf', '0',
+                      '-flags2', '-wpred-dct8x8', '-level', '13',
+                      '-maxrate', '768000', '-bufsize', '3000000',
+                      '-wpredp', '0', '-preset', 'superfast',
+                      '-threads', '0', '-s', '480x360',
                       '-vbsf', 'h264_mp4toannexb']
     return has_video_args
 
@@ -494,45 +501,22 @@ def get_ffmpeg_executable_path():
 def customize_ffmpeg_parameters(default_parameters):
     return default_parameters
 
-def get_ffmpeg2theora_executable_path():
+def get_enmfp_executable_info():
     bundle_path = NSBundle.mainBundle().bundlePath()
-    # Unicode kludge.  This wouldn't be a problem once we switch to Python 3.
-    path = os.path.join(bundle_path, "Contents", "Helpers", "ffmpeg2theora")
-    return path.encode('utf-8')
-
-def customize_ffmpeg2theora_parameters(default_parameters):
-    return default_parameters
+    # XXX Unicode kludge.  This wouldn't be a problem once we switch to 
+    # Python 3.
+    helpers_dir = os.path.join(bundle_path, "Contents", "Helpers")
+    exe_path = os.path.join(helpers_dir, "codegen.Darwin").encode('utf-8')
+    env = os.environ.copy()
+    if 'PATH' in env:
+        env['PATH'] = ':'.join([helpers_dir.encode('utf-8'), env['PATH']])
+    else:
+        env['PATH'] = helpers_dir.encode('utf-8')
+    return {
+        'path': exe_path,
+        'env': env,
+    }
     
-###############################################################################
-
-def qttime2secs(qttime):
-    timeScale = qttimescale(qttime)
-    if timeScale == 0:
-        return 0.0
-    timeValue = qttimevalue(qttime)
-    return timeValue / float(timeScale)
-
-def qttimescale(qttime):
-    if isinstance(qttime, tuple):
-        return qttime[1]
-    else:
-        return qttime.timeScale
-
-def qttimevalue(qttime):
-    if isinstance(qttime, tuple):
-        return qttime[0]
-    else:
-        return qttime.timeValue
-
-def qttimevalue_set(qttime, value):
-    if isinstance(qttime, tuple):
-        return(value, qttime[1], qttime[2])
-    else:
-        qttime.timeValue = value
-        return qttime
-
-###############################################################################
-
 def get_logical_cpu_count():
     try:
         import multiprocessing
