@@ -63,8 +63,6 @@ PORTABLE_DIR = os.path.join(ROOT_DIR, 'lib')
 PLATFORM_DIR = os.path.join(ROOT_DIR, 'osx')
 PLATFORM_PACKAGE_DIR = os.path.join(PLATFORM_DIR, 'plat')
 
-INFOLIST_DIR = os.path.join(PORTABLE_DIR, 'frontends', 'widgets', 'infolist')
-
 PYTHON_VERSION = sys.version[0:3]
 
 SANDBOX_ROOT_DIR = os.path.normpath(os.path.normpath(os.path.join(ROOT_DIR, '..')))
@@ -97,7 +95,7 @@ sys.path.append(ROOT_DIR)
 import lib
 sys.modules['miro'] = lib
 
-from miro import util
+from miro import buildutils
 
 # =============================================================================
 # Utility function used to extract stuff from the binary kit
@@ -106,7 +104,15 @@ from miro import util
 def copy_binaries(source, target, binaries):
     mkpath(target)
     for mem in binaries:
-        copy_file(os.path.join(BKIT_DIR, source, mem), target, update=True)
+        src = os.path.join(BKIT_DIR, source, mem)
+        if os.path.islink(src):
+            dst = os.path.join(target, mem)
+            linkto = os.readlink(src)
+            if os.path.exists(dst):
+                os.remove(dst) 
+            os.symlink(linkto, dst)
+        else:
+            copy_file(src, target, update=True)
 
 def extract_binaries(source, target, force=True):
     if force and os.path.exists(target):
@@ -142,13 +148,14 @@ def extract_binaries(source, target, force=True):
 class Config (object):
     
     def __init__(self, path, themePath=None):
-        self.config = util.read_simple_config_file(path)
+        self.config = buildutils.read_simple_config_file(path)
         self.themeDir = os.path.join(ROOT_DIR, 'osx', 'build', 'theme')
         self.themeConfig = None
         if themePath is not None:
             self.extract_theme_content(themePath, self.themeDir)
             themeConfigPath = os.path.join(self.themeDir, "app.config")
-            self.themeConfig = util.read_simple_config_file(themeConfigPath)
+            self.themeConfig = buildutils.read_simple_config_file(
+                    themeConfigPath)
         elif os.path.exists(self.themeDir):
             shutil.rmtree(self.themeDir)
         
@@ -212,7 +219,9 @@ class MiroBuild (py2app):
         ("keep-tests",   "u", "keep the unit tests module"),
         ("make-dmg",     "d", "produce a disk image"),
         ("force-update", "f", "force resource update"),
-        ("theme=",       "t", "theme file to use")]
+        ("theme=",       "t", "theme file to use"),
+        ("sign=",        "s", "identity to sign app with"),
+    ]
 
     boolean_options = py2app.boolean_options + ["make-dmg", "force-update", "keep-tests"]
         
@@ -220,6 +229,7 @@ class MiroBuild (py2app):
         self.keep_tests = False
         self.make_dmg = False
         self.force_update = False
+        self.sign = None
         self.theme = None
         py2app.initialize_options(self)
         
@@ -231,7 +241,7 @@ class MiroBuild (py2app):
 
     def setup_config(self):
         # Get revision information.
-        revision = util.query_revision()
+        revision = buildutils.query_revision()
         if revision is None:
             revisionURL = 'unknown'
             revisionNum = '0000'
@@ -271,12 +281,13 @@ class MiroBuild (py2app):
         self.distribution.ext_modules.append(self.get_growl_ext())
         self.distribution.ext_modules.append(self.get_growl_image_ext())
         self.distribution.ext_modules.append(self.get_fasttypes_ext())
+        self.distribution.ext_modules.append(self.get_namecollation_ext())
         self.distribution.ext_modules.append(self.get_ngrams_ext())
-        self.distribution.ext_modules.append(self.get_infolist_ext())
 
         self.distribution.packages = [
             'miro',
             'miro.libdaap',
+            'miro.data',
             'miro.dl_daemon',
             'miro.dl_daemon.private',
             'miro.frontends',
@@ -304,7 +315,6 @@ class MiroBuild (py2app):
             os.path.join('Resources-Widgets', 'MainMenu.nib'),
             os.path.join('Resources-Widgets', 'OverlayPalette.nib'),
             os.path.join('Resources-Widgets', 'Credits.html'),
-            'qt_extractor.py',
             os.path.join(PORTABLE_DIR, 'miro_helper.py'),
         ]
 
@@ -318,6 +328,7 @@ class MiroBuild (py2app):
         self.fmwkRoot = os.path.join(self.bundleRoot, 'Frameworks')
         self.cmpntRoot = os.path.join(self.bundleRoot, 'Components')
         self.helpersRoot = os.path.join(self.bundleRoot, 'Helpers')
+        self.librariesRoot = os.path.join(self.bundleRoot, 'Lib')
         self.prsrcRoot = os.path.join(self.rsrcRoot, 'resources')
         self.extRoot = os.path.join(self.rsrcRoot, 'extensions')
 
@@ -350,22 +361,19 @@ class MiroBuild (py2app):
         return Extension("miro.fasttypes",
                 [os.path.join(PORTABLE_DIR, 'fasttypes.c')])
 
+    def get_namecollation_ext(self):
+        sqlite3_dir = os.path.join(BKIT_DIR, "sqlite3")
+        return Extension("miro.data.namecollation",
+                [os.path.join(PORTABLE_DIR, 'data', 'namecollation.cpp')],
+                library_dirs=[sqlite3_dir],
+                include_dirs=[sqlite3_dir],
+                libraries=['sqlite3'],
+            )
+
     def get_ngrams_ext(self):
         return Extension("miro.ngrams",
                 [os.path.join(PORTABLE_DIR, 'ngrams.c')])
 
-    def get_infolist_ext(self):
-        return Extension("miro.infolist",
-                [
-                    os.path.join(INFOLIST_DIR, 'infolist-nodelist.c'),
-                    os.path.join(INFOLIST_DIR, 'infolist.pyx'),
-                    os.path.join(INFOLIST_DIR, 'cocoa', 'infolist-cocoa.m'),
-                    ],
-                include_dirs = [
-                    INFOLIST_DIR,
-                    ],
-                )
-
     
     def fillTemplate(self, templatepath, outpath, **vars):
         s = open(templatepath, 'rt').read()
@@ -385,6 +393,8 @@ class MiroBuild (py2app):
         self.precompile_site_pyc()
         self.copy_quicktime_components()
         self.copy_conversion_helpers()
+        self.copy_codegen_helpers()
+        self.copy_libsqlite3()
         self.copy_ffmpeg_presets()
         self.copy_portable_resources()
         self.copy_extensions()
@@ -399,6 +409,9 @@ class MiroBuild (py2app):
         self.clean_up_incomplete_lproj()
         self.clean_up_unwanted_data()
 
+        if self.sign is not None:
+            self.sign_app()
+
         if self.make_dmg:
             self.make_disk_image()
     
@@ -486,8 +499,21 @@ class MiroBuild (py2app):
     def copy_conversion_helpers(self):
         print 'Copying video conversion helpers to application bundle'
         copy_binaries('segmenter/bin/', self.helpersRoot, ["miro-segmenter"])
-        copy_binaries('ffmpeg/bin/', self.helpersRoot, ["ffmpeg"])
-        copy_binaries('ffmpeg2theora/bin/', self.helpersRoot, ["ffmpeg2theora"])
+        ffmpeg_files = ["ffmpeg"]
+        lib_paths = glob(os.path.join(BKIT_DIR, "ffmpeg", "bin", "*.dylib"))
+        ffmpeg_files.extend(os.path.basename(p) for p in lib_paths)
+        copy_binaries('ffmpeg/bin/', self.helpersRoot, ffmpeg_files)
+
+    def copy_codegen_helpers(self):
+        print 'Copying ENMFP code generate to the application bundle'
+        copy_binaries('enmfp-codegen', self.helpersRoot, [
+            "codegen.Darwin",
+            "libtag.1.dylib",
+        ])
+
+    def copy_libsqlite3(self):
+        print 'Copying libsqlite3.dylib'
+        copy_binaries('sqlite3/', self.librariesRoot, ['libsqlite3.dylib'])
 
     def copy_ffmpeg_presets(self):
         preset_source_dir = os.path.join(BKIT_DIR, 'ffmpeg', 'presets')
@@ -504,7 +530,8 @@ class MiroBuild (py2app):
         if not os.path.exists(self.prsrcRoot):
             os.mkdir(self.prsrcRoot)
 
-        resources = ['searchengines', 'images', 'conversions', 'devices']
+        resources = ['donate.html', 'searchengines', 'images', 'conversions',
+                     'devices']
         if self.keep_tests:
             resources.append('testdata')
 
@@ -575,9 +602,12 @@ class MiroBuild (py2app):
                 print "    %s" % dest
     
     def precompile_site_pyc(self):
-        print "Pre-compiling site.py"
+        print "Pre-compiling python sources"
         import py_compile
         py_compile.compile(os.path.join(self.rsrcRoot, 'lib', 'python%s' % PYTHON_VERSION, 'site.py'))
+        print "Deleting python sources"
+        # These can go, since we have the pyc now
+        os.remove(os.path.join(self.rsrcRoot, 'lib', 'python%s' % PYTHON_VERSION, 'site.py'))
     
     def copy_theme_files(self):
         # Copy theme files to the application bundle
@@ -624,9 +654,7 @@ class MiroBuild (py2app):
                 shutil.rmtree(lproj)
         
     def clean_up_unwanted_data(self):
-        """docstring for clean_up_unwanted_data"""
-        pass
-        # Check that we haven't left some turds in the application bundle.
+        """Remove turds from the application bundle."""
         
         excluded_folders = ['.svn']
         if not self.keep_tests:
@@ -638,9 +666,12 @@ class MiroBuild (py2app):
                 if excluded in dirs:
                     dirs.remove(excluded)
                     wipeList.append(os.path.join(root, excluded))
-            for excluded in ('.DS_Store', 'info.nib', 'classes.nib'):
+            for excluded in ('.DS_Store', 'info.nib', 'classes.nib', 'site.py'):
                 if excluded in files:
                     wipeList.append(os.path.join(root, excluded))
+            for f in files:
+                if f.startswith("._"):
+                    wipeList.append(os.path.join(root, f))
         
         if len(wipeList) > 0:
             print "Wiping out unwanted data from the application bundle."
@@ -651,6 +682,37 @@ class MiroBuild (py2app):
                 else:
                     os.remove(item)
 
+    def sign_app(self):
+        self.sign_directory_contents('Components')
+        self.sign_directory_contents('Helpers')
+        self.sign_directory_contents('Frameworks', skip=['Python.framework'])
+        self.sign_component(os.path.join(self._get_app_root(), 'Contents',
+                                         'Frameworks', 'Python.framework',
+                                         'Versions', '2.7', 'bin', 'python'))
+        self.sign_component(os.path.join(self._get_app_root(), 'Contents',
+                                         'Frameworks', 'Python.framework',
+                                         'Versions', '2.7'))
+        self.sign_all_libraries()
+        self.sign_component(self._get_app_root())
+
+    def sign_directory_contents(self, directory, skip=None):
+        directory = os.path.join(self._get_app_root(), 'Contents', directory)
+        for f in os.listdir(directory):
+            if skip and f in skip:
+                continue
+            path = os.path.join(directory, f)
+            if not os.path.islink(path):
+                self.sign_component(path)
+
+    def sign_all_libraries(self):
+        for root, dirs, files in os.walk(self._get_app_root()):
+            for f in files:
+                if f.endswith(".so"):
+                    self.sign_component(os.path.join(root, f))
+
+    def sign_component(self, path):
+        subprocess.check_call(['codesign', '-fs', self.sign, path])
+
     def make_disk_image(self):
         print "Building disk image..."
 
