@@ -45,8 +45,6 @@ from miro.plat import proxyfind
 from miro.plat import resources
 from miro.plat import specialfolders
 
-proxy_info = proxyfind.get_proxy_info()
-
 def _get_support_directory():
     if u3info.u3_active:
         path = u3info.APP_DATA_PREFIX
@@ -149,6 +147,14 @@ def get(descriptor):
         return os.path.join(directory,
             ('%s-downloader.log' % app.configfile['shortAppName']))
 
+    elif descriptor == prefs.HELPER_LOG_PATHNAME:
+        if u3info.u3_active:
+            directory = u3info.app_data_path
+        else:
+            directory = tempfile.gettempdir()
+        return os.path.join(directory,
+            ('%s-helper.log' % app.configfile['shortAppName']))
+
     elif descriptor == prefs.RUN_AT_STARTUP:
         import logging
         # we use the legacy startup registry key, so legacy versions
@@ -185,25 +191,29 @@ def get(descriptor):
         return False
 
     elif descriptor == prefs.HTTP_PROXY_ACTIVE:
+        proxy_info = proxyfind.get_proxy_info()
         return proxy_info.host is not None
     elif descriptor == prefs.HTTP_PROXY_HOST:
+        proxy_info = proxyfind.get_proxy_info()
         return proxy_info.host
     elif descriptor == prefs.HTTP_PROXY_PORT:
+        proxy_info = proxyfind.get_proxy_info()
         return proxy_info.port
     elif descriptor == prefs.HTTP_PROXY_IGNORE_HOSTS:
+        proxy_info = proxyfind.get_proxy_info()
         return proxy_info.ignore_hosts
     elif descriptor == prefs.AUTOUPDATE_URL:
         if app.configfile.contains(descriptor.key):
             default = app.configfile.get(descriptor.key)
         else:
-            default = u'http://www.participatoryculture.org' \
+            default = u'http://miro-updates.participatoryculture.org' \
                        '/democracy-appcast-windows.xml'
         return prefs.get_from_environ('DTV_AUTOUPDATE_URL', default)
     elif descriptor == prefs.AUTOUPDATE_BETA_URL:
         if app.configfile.contains(descriptor.key):
             default = app.configfile.get(descriptor.key)
         else:
-            default = u'http://www.participatoryculture.org' \
+            default = u'http://miro-updates.participatoryculture.org' \
                        '/democracy-appcast-windows-beta.xml'
         return prefs.get_from_environ('DTV_AUTOUPDATE_BETA_URL', default)
     # Proxy authorization isn't suppored on windows, so the following
