@@ -39,21 +39,28 @@
 #include <Python.h>
 #include "nsCOMPtr.h"
 #include "nsComponentManagerUtils.h"
+#include "nsCWebBrowserPersist.h"
 #include "nsEmbedCID.h"
+#include "nsIChannel.h"
 #include "nsIDOMWindow.h"
 #include "nsIInterfaceRequestorUtils.h"
+#include "nsIIOService.h"
+#include "nsILocalFile.h"
+#include "nsIProperties.h"
 #include "nsISupportsImpl.h"
 #include "nsIWebBrowser.h"
 #include "nsIWebBrowserFocus.h"
-#include "docshell/nsDocShellCID.h"
-#include "docshell/nsIDocShellTreeItem.h"
-#include "docshell/nsIWebNavigationInfo.h"
-#include "necko/nsIURI.h"
-#include "xpcom/nsServiceManagerUtils.h"
+#include "nsIWebBrowserPersist.h"
+#include "nsDocShellCID.h"
+#include "nsIDocShellTreeItem.h"
+#include "nsIWebNavigationInfo.h"
+#include "nsIURI.h"
+#include "nsServiceManagerUtils.h"
+#include "nsNetCID.h"
+#include "nsXPCOMCID.h"
 
 #include "MiroBrowserEmbed.h"
 #include "xulrunnerbrowser.h"
-#include "FixFocus.h"
 
 #include <stdio.h>
 #include <string.h>
@@ -89,7 +96,6 @@ nsresult MiroBrowserEmbed::init(unsigned long parentWindow, int x,
     browserBaseWindow->SetVisibility(PR_TRUE);
     browserBaseWindow->SetEnabled(PR_TRUE);
 
-    install_focus_fixes((HWND)mWindow);
     rv = mWebBrowser->GetParentURIContentListener(
             getter_AddRefs(mParentContentListener));
     NS_ENSURE_SUCCESS(rv, rv);
@@ -115,8 +121,8 @@ nsresult MiroBrowserEmbed::disable()
 
     rv = browserBaseWindow->SetVisibility(PR_FALSE);
     NS_ENSURE_SUCCESS(rv, rv);
-    rv = browserBaseWindow->SetEnabled(PR_FALSE);
-    NS_ENSURE_SUCCESS(rv, rv);
+    browserBaseWindow->SetEnabled(PR_FALSE);
+    // If SetEnabled fails, we should still just return NS_OK.  See #18793.
     return NS_OK;
 }
 
@@ -167,6 +173,29 @@ nsresult MiroBrowserEmbed::loadURI(const char* uri)
     return NS_OK;
 }
 
+// Download a URI to a path
+nsresult MiroBrowserEmbed::downloadURI(const char* uri, const char* path)
+{
+  nsresult rv;
+  nsIURI *aURI;
+  nsCOMPtr<nsIIOService> io_service(do_GetService(NS_IOSERVICE_CONTRACTID));
+  io_service->NewURI(nsDependentCString(uri), nsnull, nsnull, &aURI);
+  if (!aURI) {
+    return NS_ERROR_FAILURE;
+  }
+  nsCOMPtr<nsIWebBrowserPersist> persist(do_CreateInstance(NS_WEBBROWSERPERSIST_CONTRACTID, &rv));
+  NS_ENSURE_SUCCESS(rv, rv);
+  nsCOMPtr<nsILocalFile> file(do_CreateInstance(NS_LOCAL_FILE_CONTRACTID, &rv));
+  NS_ENSURE_SUCCESS(rv, rv);
+  file->InitWithPath(NS_ConvertASCIItoUTF16(path));
+  persist->SetProgressListener(this);
+  persist->SetPersistFlags(nsIWebBrowserPersist::PERSIST_FLAGS_AUTODETECT_APPLY_CONVERSION|
+			   nsIWebBrowserPersist::PERSIST_FLAGS_CLEANUP_ON_FAILURE|
+			   nsIWebBrowserPersist::PERSIST_FLAGS_FORCE_ALLOW_COOKIES);
+  persist->SaveURI(aURI, nsnull, nsnull, nsnull, "", file);
+  return NS_OK;
+}
+
 nsresult MiroBrowserEmbed::getCurrentURI(char ** uri)
 {
     nsresult rv;
@@ -222,8 +251,9 @@ nsresult MiroBrowserEmbed::resize(int x, int y, int width, int height)
             PR_TRUE);
 }
 
-// Give the browser keyboard focus
-nsresult MiroBrowserEmbed::focus()
+// Activate the browser window.  This makes it take keyboard focus and display
+// the caret
+nsresult MiroBrowserEmbed::activate()
 {
     nsCOMPtr<nsIWebBrowserFocus> browserFocus(
             do_GetInterface(mWebBrowser));
@@ -231,6 +261,15 @@ nsresult MiroBrowserEmbed::focus()
     return browserFocus->Activate();
 }
 
+// Deactivate the browser window.
+nsresult MiroBrowserEmbed::deactivate()
+{
+    nsCOMPtr<nsIWebBrowserFocus> browserFocus(
+            do_GetInterface(mWebBrowser));
+    if(!browserFocus) return NS_ERROR_FAILURE;
+    return browserFocus->Deactivate();
+}
+
 // Set the focus callback.  This will be called when the user tabs through all
 // the elements in the browser and the next Widget should be given focus.
 void MiroBrowserEmbed::SetFocusCallback(focusCallback callback, void* data)
@@ -466,6 +505,7 @@ NS_IMETHODIMP MiroBrowserEmbed::GetSiteWindow(void * *aSiteWindow)
 NS_IMETHODIMP MiroBrowserEmbed::OnStartURIOpen(nsIURI *aURI, PRBool *_retval)
 {
     nsresult rv;
+    int should_load;
     nsCAutoString specString;
     *_retval = PR_FALSE;
 
@@ -476,9 +516,10 @@ NS_IMETHODIMP MiroBrowserEmbed::OnStartURIOpen(nsIURI *aURI, PRBool *_retval)
     // continue the load.  However, it seems like the opposite is actually the
     // case.
     if(mURICallback && is_enabled()) {
-        if(mURICallback((char*)specString.get(), mURICallbackData) == 0) {
-            *_retval = PR_TRUE;
-        }
+      should_load = mURICallback((char*)specString.get(), mURICallbackData);
+      if(should_load == 0) {
+	*_retval = PR_TRUE;
+      }
     }
     return NS_OK;
 }
@@ -548,12 +589,22 @@ NS_IMETHODIMP MiroBrowserEmbed::SetParentContentListener(nsIURIContentListener *
 /* void onStateChange (in nsIWebProgress aWebProgress, in nsIRequest aRequest, in unsigned long aStateFlags, in nsresult aStatus); */
 NS_IMETHODIMP MiroBrowserEmbed::OnStateChange(nsIWebProgress *aWebProgress, nsIRequest *aRequest, PRUint32 aStateFlags, nsresult aStatus)
 {
+    nsresult rv;
+    nsCAutoString specString;
     if((aStateFlags & nsIWebProgressListener::STATE_IS_NETWORK) &&
             mNetworkCallback && is_enabled()) {
+        if (aRequest) {
+	  nsIURI *aURI;
+	  ((nsIChannel*)aRequest)->GetOriginalURI(&aURI);
+          rv = aURI->GetSpec(specString);
+          NS_ENSURE_SUCCESS(rv, rv);
+	} else {
+          NS_NAMED_LITERAL_STRING(specString, "");
+        }
         if(aStateFlags & nsIWebProgressListener::STATE_START) {
-            mNetworkCallback(PR_TRUE, mNetworkCallbackData);
+	  mNetworkCallback(PR_TRUE, 0, (char*)specString.get(), mNetworkCallbackData);
         } else if(aStateFlags & nsIWebProgressListener::STATE_STOP) {
-            mNetworkCallback(PR_FALSE, mNetworkCallbackData);
+	  mNetworkCallback(PR_FALSE, aStatus, (char*)specString.get(), mNetworkCallbackData);
         }
     }
     return NS_OK;
