@@ -43,13 +43,15 @@ import sqlite3
 
 from miro import app
 from miro import prefs
-from miro.gtcache import gettext as _
+from miro.gtcache import gettext_lazy as _
 from miro.importmedia import import_itunes_path
 from miro.plat import config as plat_config
 from miro.plat import prelogger
 from miro.plat import proxyfind
 from miro.plat import resources
 from miro.plat import specialfolders
+from miro.plat.popen import Popen
+from miro.frontends.widgets.gst import gst_extractor
 from miro.util import returns_unicode, check_u
 from miro.util import AutoLoggingStream
 from miro import fileutil
@@ -187,53 +189,50 @@ class ApatheticRotatingFileHandler(RotatingFileHandler):
         pass
 
 FORMAT = "%(asctime)s %(levelname)-8s %(name)s: %(message)s"
-def setup_logging(in_downloader=False):
-    if in_downloader:
-        if 'MIRO_IN_UNIT_TESTS' in os.environ:
-            level = logging.WARN
-        elif os.environ.get('MIRO_DEBUGMODE', '') == True:
-            level = logging.DEBUG
-        else:
-            level = logging.INFO
-        logging.basicConfig(level=level,
-                            format='%(asctime)s %(levelname)-8s %(message)s',
-                            stream=sys.stderr)
-        pathname = os.environ.get("DEMOCRACY_DOWNLOADER_LOG")
-        if not pathname:
-            return
+def setup_logging(pathname, main_process=False):
+    if 'MIRO_IN_UNIT_TESTS' in os.environ:
+        level = logging.WARN
+    elif (os.environ.get('MIRO_DEBUGMODE', '') == True or
+            app.debugmode):
+        level = logging.DEBUG
     else:
-        if app.debugmode:
-            level = logging.DEBUG
-        else:
-            level = logging.INFO
-        logging.basicConfig(level=level,
-                            format=FORMAT)
-        pathname = app.config.get(prefs.LOG_PATHNAME)
+        level = logging.INFO
 
     rotater = ApatheticRotatingFileHandler(
         pathname, mode="a", maxBytes=100000, backupCount=5)
 
     formatter = logging.Formatter(FORMAT)
-    rotater.setLevel(level)
     rotater.setFormatter(formatter)
     logger = logging.getLogger('')
     logger.addHandler(rotater)
+    logger.setLevel(level)
     rotater.doRollover()
+
+    if main_process:
+        # If it was redirected (by py2exe) then reset stdout/stderr to the
+        # AutoLoggingStream.  Otherwise, we do actually want to output to
+        # console and so leave it untouched.   Just add a log handler for it.
+        # For release builds we don't care and just redirect everything for
+        # the console to the log.
+        if app.debugmode:
+            redirected = sys.stdout != sys.__stdout__
+            if redirected:
+                sys.stdout = AutoLoggingStream(logging.warn, '(from stdout) ')
+                sys.stderr = AutoLoggingStream(logging.error, '(from stderr) ')
+            else:
+                stdouthandler = logging.StreamHandler(sys.stdout)
+                stdouthandler.setFormatter(formatter)
+                logger.addHandler(stdouthandler)
+        else:
+            sys.stdout = AutoLoggingStream(logging.warn, '(from stdout) ')
+            sys.stderr = AutoLoggingStream(logging.error, '(from stderr) ')
+
     try:
         for record in prelogger.remove():
             logger.handle(record)
     except ValueError:
         logging.debug("No records from prelogger.")
 
-    if app.debugmode:
-        stdouthandler = logging.StreamHandler(sys.stdout)
-        stdouthandler.setLevel(level)
-        stdouthandler.setFormatter(formatter)
-        logger.addHandler(stdouthandler)
-    else:
-        sys.stdout = AutoLoggingStream(logging.warn, '(from stdout) ')
-        sys.stderr = AutoLoggingStream(logging.error, '(from stderr) ')
-
     logging.info("Logging set up to %s at level %s", pathname, level)
 
 @returns_unicode
@@ -337,59 +336,39 @@ def kill_process(pid):
 
 def launch_download_daemon(oldpid, env):
     kill_process(oldpid)
+
+    # FIXME - wtf is going on here between os.environ, env and
+    # environ?
     for key, value in env.items():
         os.environ[key] = value
 
     environ = os.environ.copy()
-    environ['DEMOCRACY_DOWNLOADER_LOG'] = app.config.get(prefs.DOWNLOADER_LOG_PATHNAME)
+    environ['DEMOCRACY_DOWNLOADER_LOG'] = app.config.get(
+        prefs.DOWNLOADER_LOG_PATHNAME)
     environ['MIRO_APP_VERSION'] = app.config.get(prefs.APP_VERSION)
     if hasattr(app, 'in_unit_tests'):
         environ['MIRO_IN_UNIT_TESTS'] = '1'
     environ.update(env)
 
-    # on windows, subprocess can ONLY accept strings (no unicode) in
-    # environment values.  at present this affects FFMPEG_DATADIR
-    # which doesn't matter in the downloader, so we remove it.
-    del environ["FFMPEG_DATADIR"]
-
-    # start the downloader.  We use the subprocess module to turn off
-    # the console.  One slightly awkward thing is that the current
-    # process might not have a valid stdin/stdout/stderr, so we create
-    # a pipe to it that we never actually use.
-
     # note that we use "Miro" instead of the app name here, so custom
     # versions will work
 
-    # note that the application filename has to be in double-quotes
-    # otherwise it kicks up "%1 is not a valid Win32 application"
-    # errors on some Windows machines.  Why it only happens on some is
-    # a mystery of the universe.  Bug #9274.
-    downloader_path = '"%s"' % os.path.join(resources.app_root(),
-                                           "Miro_Downloader.exe")
-    startupinfo = subprocess.STARTUPINFO()
-    # TEMPORARY: the STARTF_USESHOWWINDOW has been moved into
-    # subprocess._subprocess in Python 2.6.6 and beyond.
-    try:
-        startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
-    except AttributeError:
-        startupinfo.dwFlags |= subprocess._subprocess.STARTF_USESHOWWINDOW
-    subprocess.Popen(downloader_path, stdout=subprocess.PIPE,
-                     stderr=subprocess.PIPE,
-                     stdin=subprocess.PIPE,
-                     startupinfo=startupinfo,
-                     env=environ)
+    downloader_path = (os.path.join(resources.app_root(),
+                                   "Miro_Downloader.exe"),) 
+    return Popen(downloader_path, close_fds=True, env=environ)
 
 def exit_miro(return_code):
     """Python's sys.exit isn't sufficient in a Windows
     application. It's not clear why.
     """
-    ctypes.windll.kernel32.ExitProcess(return_code)
+    try:
+        # this kicks up an access violation error.  not clear why.
+        ctypes.windll.kernel32.ExitProcess(return_code)
+    except WindowsError:
+        pass
 
-def movie_data_program_info(movie_path, thumbnail_path):
-    exe_path = os.path.join(resources.app_root(), 'Miro_MovieData.exe')
-    cmd_line = (exe_path, movie_path, thumbnail_path)
-    env = None
-    return (cmd_line, env)
+def run_media_metadata_extractor(movie_path, thumbnail_path):
+    return gst_extractor.run(movie_path, thumbnail_path)
 
 def get_logical_cpu_count():
     try:
@@ -409,9 +388,18 @@ def get_logical_cpu_count():
 def get_segmenter_executable_path():
     return os.path.join(resources.app_root(), "miro-segmenter.exe")
 
+def get_enmfp_executable_info():
+    return {
+        'path': os.path.join(resources.app_root(), "codegen.windows.exe")
+    }
+
+
 def get_transcode_video_options():
-    has_video_args = ['-vcodec', 'libx264', '-vpre', 'ipod320',
-                      '-vpre', 'ultrafast', '-threads', '0', '-s', '480x360',
+    has_video_args = ['-vcodec', 'libx264', '-coder', '0', '-bf', '0',
+                      '-flags2', '-wpred-dct8x8', '-level', '13',
+                      '-maxrate', '768000', '-bufsize', '3000000',
+                      '-wpredp', '0', '-preset', 'ultrafast',
+                      '-threads', '0', '-s', '480x360',
                       '-vbsf', 'h264_mp4toannexb']
     return has_video_args
 
@@ -423,7 +411,9 @@ def get_transcode_audio_options():
     return has_audio_args
 
 def setup_ffmpeg_presets():
-    os.environ['FFMPEG_DATADIR'] = resources.app_root()
+    from miro.plat.specialfolders import get_short_path_name
+    os.environ['AVCONV_DATADIR'] = get_short_path_name(
+        resources.app_root())
 
 def get_ffmpeg_executable_path():
     return os.path.join(resources.app_root(), "ffmpeg.exe")
@@ -438,21 +428,7 @@ def customize_ffmpeg_parameters(params):
     # FIXME - assumes the last item is {output}.
     # look at last item output
     params[-1] = get_short_path_name(params[-1])
-    return params
 
-def get_ffmpeg2theora_executable_path():
-    return os.path.join(resources.app_root(), "ffmpeg2theora.exe")
-
-def customize_ffmpeg2theora_parameters(params):
-    from miro.plat.specialfolders import get_short_path_name
-    # look for -o output
-    ind = params.index("-o")
-    if ind != -1 and len(params) > ind + 2:
-        params[ind+1] = get_short_path_name(params[ind+1])
-
-    # FIXME - assumes the last item is {input}.
-    # look at last item input
-    params[-1] = get_short_path_name(params[-1])
     return params
 
 def begin_thread_loop(context_object):
@@ -509,7 +485,7 @@ def get_plat_media_player_name_path():
     return (_('iTunes'), import_itunes_path(itunes_path))
 
 def miro_helper_program_info():
-    cmd_line = (os.path.join(resources.app_root(), 'Miro_Helper.exe'))
+    cmd_line = (os.path.join(resources.app_root(), 'Miro_Helper.exe'),)
     env = None
     return (cmd_line, env)
 
