@@ -61,7 +61,14 @@ BINARY_KIT_VERSION = open("binary_kit_version").read().strip()
 # below.
 BINARY_KIT_ROOT = "miro-binary-kit-win-%s" % BINARY_KIT_VERSION
 
-if not os.path.exists or not os.path.isdir(BINARY_KIT_ROOT):
+if os.path.exists(os.path.join("miro-binary-kit-win", "binary-kit")):
+    # miro-binary-kit-win/binary-kit/ overrides the binary_kit_version
+    # stuff.  that makes it easier for devs to do dev things and
+    # put the miro-binary-kit-win repository in the tv/windows/
+    # directory.
+    BINARY_KIT_ROOT = os.path.join("miro-binary-kit-win", "binary-kit")
+
+elif not os.path.exists or not os.path.isdir(BINARY_KIT_ROOT):
     print "Binary kit %s is missing.  Run 'setup_binarykit.sh'." % BINARY_KIT_ROOT
     sys.exit(1)
 
@@ -69,21 +76,23 @@ ZLIB_INCLUDE_PATH = os.path.join(BINARY_KIT_ROOT, 'zlib', 'include')
 ZLIB_LIB_PATH = os.path.join(BINARY_KIT_ROOT, 'zlib', 'lib')
 ZLIB_RUNTIME_LIBRARY_PATH = os.path.join(BINARY_KIT_ROOT, 'zlib')
 
-OPENSSL_INCLUDE_PATH = os.path.join(BINARY_KIT_ROOT, 'openssl', 'include')
-OPENSSL_LIB_PATH = os.path.join(BINARY_KIT_ROOT, 'openssl', 'lib')
-OPENSSL_LIBRARIES = ['ssleay32', 'libeay32']
+OPENSSL_LIB_PATH = os.path.join(BINARY_KIT_ROOT, 'openssl')
 
-# GTK_ROOT_PATH = os.path.join(BINARY_KIT_ROOT, 'gtk+-bundle_2.20.0-20100406_win32')
-GTK_ROOT_PATH = os.path.join(BINARY_KIT_ROOT, 'gtk+-bundle_2.16.6-20091215_win32')
+import gtk
+GTK_ROOT_PATH = os.path.join(
+    os.path.dirname(gtk.__file__),
+    os.pardir,
+    "runtime")
+GTK_BIN_PATH = os.path.join(GTK_ROOT_PATH, 'bin')
 GTK_INCLUDE_PATH = os.path.join(GTK_ROOT_PATH, 'include')
 GTK_LIB_PATH = os.path.join(GTK_ROOT_PATH, 'lib')
-GTK_BIN_PATH = os.path.join(GTK_ROOT_PATH, 'bin')
 GTK_INCLUDE_DIRS = [
     os.path.join(GTK_INCLUDE_PATH, 'atk-1.0'),
     os.path.join(GTK_INCLUDE_PATH, 'gtk-2.0'),
     os.path.join(GTK_INCLUDE_PATH, 'glib-2.0'),
     os.path.join(GTK_INCLUDE_PATH, 'pango-1.0'),
     os.path.join(GTK_INCLUDE_PATH, 'cairo'),
+    os.path.join(GTK_INCLUDE_PATH, 'gdk-pixbuf-2.0'),
     os.path.join(GTK_LIB_PATH, 'glib-2.0', 'include'),
     os.path.join(GTK_LIB_PATH, 'gtk-2.0', 'include'),
 ]
@@ -95,18 +104,21 @@ PYGTK_INCLUDE_DIR = os.path.join(BINARY_KIT_ROOT, 'pygtk')
 XULRUNNER_SDK_PATH = os.path.join(BINARY_KIT_ROOT, 'xulrunner-sdk')
 XULRUNNER_SDK_BIN_PATH = os.path.join(XULRUNNER_SDK_PATH, 'bin')
 
-VLC_PATH = os.path.join(BINARY_KIT_ROOT, 'libvlc')
+SQLITE3_PATH = os.path.join(BINARY_KIT_ROOT, 'sqlite3')
+
+GSTREAMER_PATH = os.path.join(BINARY_KIT_ROOT, 'gstreamer')
 LIBTORRENT_PATH = os.path.join(BINARY_KIT_ROOT, 'libtorrent')
 MUTAGEN_PATH = os.path.join(BINARY_KIT_ROOT, 'mutagen-1.20')
 WINSPARKLE_PATH = os.path.join(BINARY_KIT_ROOT, 'winsparkle')
 
+ENMFP_PATH = os.path.join(BINARY_KIT_ROOT, 'enmfp-codegen')
 SEGMENTER_PATH = os.path.join(BINARY_KIT_ROOT, 'segmenter')
 FFMPEG_PATH = os.path.join(
-    BINARY_KIT_ROOT, 'ffmpeg', 'ffmpeg-r25766-swscale-r32562-mingw32-shared')
-FFMPEG2THEORA_PATH = os.path.join(BINARY_KIT_ROOT, 'ffmpeg2theora')
+    BINARY_KIT_ROOT, 'ffmpeg', 'libav-win32-20111229')
 
 VCREDIST71_PATH = os.path.join(BINARY_KIT_ROOT, 'vc71redist')
 VCREDIST90_PATH = os.path.join(BINARY_KIT_ROOT, 'vc90redist')
+VCREDIST100_PATH = os.path.join(BINARY_KIT_ROOT, 'vc100redist')
 
 # so py2exe can find DLLs
 sys.path.append(VCREDIST71_PATH)
@@ -152,7 +164,6 @@ platform_package_dir = os.path.join(platform_dir, 'plat')
 widgets_dir = os.path.join(platform_package_dir, 'frontends', 'widgets')
 portable_dir = os.path.join(root_dir, 'lib')
 portable_widgets_dir = os.path.join(portable_dir, 'frontends', 'widgets')
-infolist_dir = os.path.join(portable_widgets_dir, 'infolist')
 portable_xpcom_dir = os.path.join(portable_widgets_dir, 'gtk', 'xpcom')
 test_dir = os.path.join(root_dir, 'resources')
 resources_dir = os.path.join(root_dir, 'resources')
@@ -165,12 +176,15 @@ sys.path.insert(0, root_dir)
 import lib
 sys.modules['miro'] = lib
 
-from miro import util
+from miro import buildutils
 
 # put the libtorrent extension on the path, so py2exe can find the
 # .pyd file
 sys.path.insert(0, LIBTORRENT_PATH)
 sys.path.insert(0, MUTAGEN_PATH)
+sys.path.insert(0, os.path.join(GSTREAMER_PATH, 'lib', 'site-packages'))
+sys.path.insert(0, os.path.join(GSTREAMER_PATH, 'lib', 'site-packages', 'gst-0.10'))
+
 
 #### Extensions ####
 ngrams_ext = \
@@ -187,10 +201,45 @@ pygtkhacks_ext = Extension(
     library_dirs=[GTK_LIB_PATH],
     libraries=[
         'gtk-win32-2.0',
+        'gdk-win32-2.0',
         'pango-1.0',
         ]
     )
 
+namecollation_ext = \
+    Extension("miro.data.namecollation",
+        [os.path.join(portable_dir, 'data', 'namecollation.cpp')],
+        # borrow sqlite3.h from xulrunner
+        include_dirs=[os.path.join(XULRUNNER_SDK_PATH, 'include')],
+        library_dirs=[SQLITE3_PATH],
+        libraries=['sqlite3'],
+    )
+
+fixedliststore_dir = os.path.join(portable_widgets_dir, 'gtk', 'fixedliststore')
+fixedliststore_ext = Extension(
+        "miro.frontends.widgets.gtk.fixedliststore",
+        [
+            os.path.join(fixedliststore_dir, 'fixed-list-store.c'),
+            os.path.join(fixedliststore_dir, 'fixed-list-store-module.c'),
+            os.path.join(fixedliststore_dir, 'fixed-list-store-wrapper.c'),
+        ],
+        include_dirs=GTK_INCLUDE_DIRS + [PYGOBJECT_INCLUDE_DIR],
+        library_dirs=[GTK_LIB_PATH],
+        libraries=[
+            'gtk-win32-2.0',
+            'gdk-win32-2.0',
+            'glib-2.0',
+            'gobject-2.0',
+            'gthread-2.0',
+        ]
+    )
+
+embeddingwindow_ext = \
+    Extension("miro.plat.frontends.widgets.embeddingwindow",
+        [os.path.join(widgets_dir, 'embeddingwindow.c')],
+        libraries=[ 'user32', 'gdi32', 'comctl32'],
+    )
+
 xulrunnerbrowser_ext_dir = os.path.join(widgets_dir, 'XULRunnerBrowser')
 xulrunnerbrowser_ext = Extension(
     "miro.plat.frontends.widgets.xulrunnerbrowser",
@@ -205,9 +254,6 @@ xulrunnerbrowser_ext = Extension(
         ("XPCOM_GLUE", 1),
         ("PCF_USING_XULRUNNER19", 1),
         ],
-    extra_compile_args=[
-        "/Zc:wchar_t-"
-        ],
     library_dirs=[
         os.path.join(XULRUNNER_SDK_PATH, 'lib'),
         GTK_LIB_PATH,
@@ -226,44 +272,27 @@ xulrunnerbrowser_ext = Extension(
         os.path.join(xulrunnerbrowser_ext_dir, 'MiroBrowserEmbed.cpp'),
         os.path.join(xulrunnerbrowser_ext_dir, 'MiroWindowCreator.cpp'),
         os.path.join(xulrunnerbrowser_ext_dir, 'MiroDirectoryProvider.cpp'),
-        os.path.join(xulrunnerbrowser_ext_dir, 'FixFocus.cpp'),
         os.path.join(xulrunnerbrowser_ext_dir, 'Init.cpp'),
         ]
     )
 
-infolist_ext = \
-    Extension("miro.infolist",
-        [
-            os.path.join(infolist_dir, 'infolist-nodelist.c'),
-            os.path.join(infolist_dir, 'infolist.pyx'),
-            os.path.join(infolist_dir, 'gtk', 'infolist-gtk.c'),
-        ],
-        include_dirs=GTK_INCLUDE_DIRS + [
-            PYGOBJECT_INCLUDE_DIR,
-            PYGTK_INCLUDE_DIR,
-            infolist_dir,
-            os.path.join(infolist_dir, 'gtk'),
-        ],
-        library_dirs=[GTK_LIB_PATH],
-        libraries=[
-            'gobject-2.0',
-            'glib-2.0',
-            'gtk-win32-2.0',
-            'pango-1.0',
-            ],
-    )
-
 # Setting the path here allows py2exe to find the DLLS
 os.environ['PATH'] = ';'.join([
-        OPENSSL_LIB_PATH, ZLIB_RUNTIME_LIBRARY_PATH,
-        LIBTORRENT_PATH, GTK_BIN_PATH, os.environ['PATH']])
+        OPENSSL_LIB_PATH,
+        ZLIB_RUNTIME_LIBRARY_PATH,
+        LIBTORRENT_PATH,
+        GTK_LIB_PATH,
+        GTK_BIN_PATH,
+        os.environ['PATH']])
 
 # Private extension modules to build.
 ext_modules = [
     ngrams_ext,
     pygtkhacks_ext,
+    namecollation_ext,
+    fixedliststore_ext,
+    embeddingwindow_ext,
     xulrunnerbrowser_ext,
-    infolist_ext,
 ]
 
 def fill_template(templatepath, outpath, **vars):
@@ -275,39 +304,41 @@ def fill_template(templatepath, outpath, **vars):
 
 # Data files
 data_files = []
+
+# add VC10.0 DLLs for ENMFP
+data_files.extend(find_data_files('.', VCREDIST100_PATH))
+
+# xulrunner
 data_files.extend(find_data_files('xulrunner', XULRUNNER_SDK_BIN_PATH))
 
-image_loader_path = os.path.join('lib', 'gtk-2.0', '2.10.0', 'loaders')
+# Clearlooks-Visto theme files
+theme_name = 'Clearlooks-Visto'
+theme_files_path = os.path.join(BINARY_KIT_ROOT, 'gtk-clearlooks')
 theme_engine_path = os.path.join('lib', 'gtk-2.0', '2.10.0', 'engines')
-theme_path = os.path.join('share', 'themes')
-for path in (image_loader_path, theme_engine_path, theme_path):
-    src_path = os.path.join(GTK_ROOT_PATH, path)
-    data_files.extend(find_data_files(path, src_path))
+data_files.append((theme_engine_path, [os.path.join(theme_files_path, 'libclearlooks.dll')]))
+data_files.extend(find_data_files(os.path.join('share', 'themes', theme_name),
+                                  os.path.join(theme_files_path, theme_name)))
 
-data_files.append(('', iglob(os.path.join(GTK_BIN_PATH, '*.dll'))))
-data_files.extend(find_data_files(
-        'vlc-plugins', os.path.join(VLC_PATH, 'vlc-plugins')))
-data_files.append(('', [os.path.join(VLC_PATH, 'libvlc.dll')]))
-data_files.append(('', [os.path.join(VLC_PATH, 'libvlccore.dll')]))
+# gstreamer files
+data_files.append(('', iglob(os.path.join(GSTREAMER_PATH, 'lib', '*.dll'))))
+data_files.append(
+    ('gstreamer-0.10',
+     iglob(os.path.join(GSTREAMER_PATH, 'lib', 'gstreamer-0.10', '*.dll'))))
 data_files.append(('', [os.path.join(WINSPARKLE_PATH, 'WinSparkle.dll')]))
 data_files.append(('', [os.path.join(LIBTORRENT_PATH, 'libtorrent.pyd')]))
 data_files.append(('', [
             os.path.join(SEGMENTER_PATH, 'miro-segmenter.exe'),
-            os.path.join(FFMPEG_PATH, 'bin', 'ffmpeg.exe'),
-            os.path.join(FFMPEG_PATH, 'bin', 'avcodec-52.dll'),
-            os.path.join(FFMPEG_PATH, 'bin', 'avcore-0.dll'),
-            os.path.join(FFMPEG_PATH, 'bin', 'avdevice-52.dll'),
-            os.path.join(FFMPEG_PATH, 'bin', 'avfilter-1.dll'),
-            os.path.join(FFMPEG_PATH, 'bin', 'avformat-52.dll'),
-            os.path.join(FFMPEG_PATH, 'bin', 'avutil-50.dll'),
-            os.path.join(FFMPEG_PATH, 'bin', 'swscale-0.dll'),
-            os.path.join(FFMPEG2THEORA_PATH, 'bin', 'ffmpeg2theora.exe')]))
-data_files.append(('', glob(os.path.join(FFMPEG_PATH, 'presets', '*.ffpreset'))))
+            os.path.join(ENMFP_PATH, 'codegen.windows.exe'),
+            os.path.join(ENMFP_PATH, 'codegen.dll'),
+            os.path.join(FFMPEG_PATH, 'usr', 'bin', 'ffmpeg.exe')]))
+data_files.append(('', glob(os.path.join(FFMPEG_PATH, 'usr', 'bin', '*.dll'))))
+data_files.append(('', glob(os.path.join(FFMPEG_PATH, '..', 'presets', '*.ffpreset'))))
 data_files.append(('', glob(os.path.join(VCREDIST71_PATH, '*.dll'))))
 data_files.extend(find_data_files('Microsoft.VC90.CRT',
     os.path.join(VCREDIST90_PATH, 'Microsoft.VC90.CRT')))
 data_files.append((os.path.join('etc', 'gtk-2.0'), [
             os.path.join(os.path.dirname(__file__), 'gtkrc')]))
+data_files.append(('', iglob(os.path.join(GTK_BIN_PATH, '*.dll'))))
 
 # handle the resources subdirectories.
 for dir in ('searchengines', 'images', 'conversions', 'devices'):
@@ -316,8 +347,12 @@ for dir in ('searchengines', 'images', 'conversions', 'devices'):
     data_files.extend(find_data_files(dest_dir, source_dir))
 
 data_files.append(('resources', [os.path.join(root_dir, 'CREDITS')]))
+data_files.append(('resources', [os.path.join(resources_dir, 'donate.html')]))
 
 data_files.extend(find_data_files("extensions", extensions_dir))
+# use the sqlite3.dll from our binary kit.  It's a newer version that comes
+# with python.
+data_files.append(('', [os.path.join(SQLITE3_PATH, 'sqlite3.dll')]))
 
 locale_temp_dir = os.path.join(os.path.dirname(__file__), "build", "locale")
 
@@ -342,7 +377,7 @@ data_files.extend(find_data_files(os.path.join("resources", "locale"),
                                   locale_temp_dir))
 
 app_config = os.path.join(resources_dir, 'app.config.template')
-template_vars = util.read_simple_config_file(app_config)
+template_vars = buildutils.read_simple_config_file(app_config)
 
 # pixmap for the about dialog
 icon_path = os.path.join("icons", "hicolor", "128x128", "apps")
@@ -359,7 +394,7 @@ class install_data(distutils.command.install_data.install_data):
     def install_app_config(self):
         template = os.path.join(resources_dir, 'app.config.template')
         dest = os.path.join(self.install_dir, 'resources', 'app.config')
-        revision = util.query_revision()
+        revision = buildutils.query_revision()
         if revision is None:
             revision = "unknown"
             revisionurl = "unknown"
@@ -384,23 +419,9 @@ class install_data(distutils.command.install_data.install_data):
             BUILD_TIME=str(time.time()))
         self.outfiles.append(dest)
 
-    def install_gdk_pixbuf_loaders(self):
-        basename = os.path.join('etc', 'gtk-2.0', 'gdk-pixbuf.loaders')
-        source = os.path.join(GTK_ROOT_PATH, basename)
-        dest = os.path.join(self.install_dir, basename)
-        contents = open(source).read()
-        # Not sure why they have paths like this in the file, but we
-        # need to change them.
-        contents = contents.replace(
-            "c:/devel/target/9c384abfa28a3e070eb60fc2972f823b/", "")
-        self.mkpath(os.path.dirname(dest))
-        open(dest, 'wt').write(contents)
-        self.outfiles.append(dest)
-
     def run(self):
         distutils.command.install_data.install_data.run(self)
         self.install_app_config()
-        self.install_gdk_pixbuf_loaders()
 
 # We want to make sure we include msvcp90.dll in the dist directory.
 # Recipe taken from
@@ -430,6 +451,7 @@ class bdist_miro(Command):
         pass
 
     def run(self):
+        self.setup_gstreamer_bin_dir()
         self.run_command('py2exe')
         self.copy_ico()
         if self.test:
@@ -448,6 +470,55 @@ class bdist_miro(Command):
         self.copy_file("Miro.ico",
                        os.path.join(dist_dir, "%s.ico" % shortappname))
 
+    def setup_gstreamer_bin_dir(self):
+        """Setup a directory that contains gstreamer runtime DLLs
+
+        The issue here is that we want some DLLs from the gstreamer runtime
+        directory, but not the ones that are also included in GTK_BIN_PATH.
+        The GTK versions are newer and GTK's ABI is backward compatible but
+        not forward compatible.
+
+        Our strategy is to create a new directory inside the build directory
+        and copy over the files that we need, but not the GTK DLLs.
+        """
+        # figure out a directory to put the gstreamer dlls
+        bdist_dir = self.get_finalized_command('bdist').bdist_base
+        gstreamer_bin_dir = os.path.join(bdist_dir, 'gstreamer_bin')
+        # move them there
+        self.copy_gstreamer_dlls(gstreamer_bin_dir)
+        # add that directory to the PATH so py2exe finds DLLs that it needs
+        os.environ['PATH'] = ';'.join([gstreamer_bin_dir, os.environ['PATH']])
+        # add the DLLs to data_files as well, since some of them are linked
+        # dynamically
+        self.distribution.data_files.append(
+                ('', iglob(os.path.join(gstreamer_bin_dir, '*.dll'))))
+
+    def copy_gstreamer_dlls(self, dest_path):
+        """Copy gstreamer DLLs to a directory.
+
+        This method does the work for setup_gstreamer_bin_dir().  It copies
+        over DLLs from the gstreamer bin directory, unless they also exist in
+        GTK_BIN_PATH.
+        """
+        src_path = os.path.join(GSTREAMER_PATH, 'bin')
+        files_to_copy = []
+        for name in os.listdir(src_path):
+            if name == 'libgtkgl-2.0-1.dll':
+                # This one isn't in the GTK bin directory, but we still want
+                # to skiip it.  It's definitely not needed, and might get in
+                # the way.
+                continue
+            if os.path.exists(os.path.join(GTK_BIN_PATH, name)):
+                # This file is also in the GTK runtime directory.  We want to
+                # use the GTK version, so don't copy the file
+                continue
+            files_to_copy.append(name)
+
+        dir_util.create_tree(dest_path, files_to_copy)
+        for name in files_to_copy:
+            file_util.copy_file(os.path.join(src_path, name), dest_path,
+                    update=True)
+
 class runmiro(Command):
     description = "build Miro and start it up"
 
@@ -471,14 +542,12 @@ class bdist_nsis(Command):
 
     user_options = [
         ('generic', None, 'Build a generic installer instead of the Miro-branded installer.'),
-        ('nozugo', None, 'Do not include the silent Zugo toolbar installer.'),
         ('install-icon=', None, 'ICO file to use for the installer.'),
         ('install-image=', None, 'BMP file to use for the welcome/finish pages.')
         ]
 
     def initialize_options(self):
         self.generic = False
-        self.nozugo = False
         self.install_icon = None
         self.install_image = None
 
@@ -501,8 +570,7 @@ class bdist_nsis(Command):
         log.info("building installer")
 
         self.copy_file(os.path.join(platform_dir, 'Miro.nsi'), self.dist_dir)
-        if not self.nozugo:
-            self.copy_file(os.path.join(platform_dir, 'toolbar-silent.exe'), self.dist_dir)
+        self.copy_file(os.path.join(platform_dir, 'OpenInstallUtils.nsh'), self.dist_dir)
         self.copy_file(self.install_icon, self.dist_dir)
         self.copy_file(self.install_image, self.dist_dir)
 
@@ -517,8 +585,6 @@ class bdist_nsis(Command):
         nsis_vars['CONFIG_EXECUTABLE'] = "%s.exe" % template_vars['shortAppName']
         nsis_vars['CONFIG_DOWNLOADER_EXECUTABLE'] = "%s_Downloader.exe" % \
                 template_vars['shortAppName']
-        nsis_vars['CONFIG_MOVIE_DATA_EXECUTABLE'] = "%s_MovieData.exe" % \
-                template_vars['shortAppName']
         nsis_vars['CONFIG_HELPER_EXECUTABLE'] = "%s_Helper.exe" % \
                 template_vars['shortAppName']
         nsis_vars['CONFIG_ICON'] = "%s.ico" % template_vars['shortAppName']
@@ -526,23 +592,18 @@ class bdist_nsis(Command):
         nsis_vars['MIRO_INSTALL_ICON'] = self.install_icon
         nsis_vars['MIRO_INSTALL_IMAGE'] = self.install_image
         nsis_vars['CONFIG_BINARY_KIT'] = BINARY_KIT_ROOT
-        if not self.nozugo:
-            nsis_vars['MIROBAR_EXE'] = 'toolbar-silent.exe'
         if self.generic:
             nsis_vars['GENERIC_INSTALLER'] = '1'
-        nsis_vars['MOZILLA_INSTALLER'] = '1'
+
 
         output_file = '%s-%s'
         # One stage installer
         if self.generic:
             output_file = "%s-generic" % output_file
-        if self.nozugo:
-            output_file = "%s-nozugo" % output_file
 
         output_file = (output_file %
                        (template_vars['shortAppName'], template_vars['appVersion']))
         nsis_vars['CONFIG_OUTPUT_FILE'] = '%s.exe' % output_file
-        nsis_vars['CONFIG_TWOSTAGE'] = "No"
 
         nsis_args = ["/D%s=%s" % (k, v) for (k, v) in nsis_vars.iteritems()]
         nsis_args.append(os.path.join(self.dist_dir, "Miro.nsi"))
@@ -552,39 +613,6 @@ class bdist_nsis(Command):
         if subprocess.call([NSIS_PATH] + nsis_args) != 0:
             print "ERROR creating the 1 stage installer, quitting"
             return
-        
-        # Two stage installer
-        if self.generic:
-            output_file = '%s-%s-generic-twostage.exe'
-        else:
-            output_file = "%s-%s-twostage.exe"
-        output_file = (output_file %
-                       (template_vars['shortAppName'], template_vars['appVersion']))
-        nsis_vars['CONFIG_OUTPUT_FILE'] = output_file
-        nsis_vars['CONFIG_TWOSTAGE'] = "Yes"
-        nsis_vars.pop('MIROBAR_EXE', None)
-
-        nsis_args = ["/D%s=%s" % (k, v) for (k, v) in nsis_vars.iteritems()]
-        nsis_args.append(os.path.join(self.dist_dir, "Miro.nsi"))
-
-        if os.access(output_file, os.F_OK):
-            os.remove(output_file)
-        subprocess.call([NSIS_PATH] + nsis_args)
-
-        zip_path = os.path.join(self.dist_dir, "%s-Contents-%s.zip" %
-            (template_vars['shortAppName'], template_vars['appVersion']))
-        self.zipfile = zip.ZipFile(zip_path, 'w', zip.ZIP_DEFLATED)
-        self.add_file(nsis_vars['CONFIG_EXECUTABLE'])
-        self.add_file(nsis_vars['CONFIG_ICON'])
-        self.add_file(nsis_vars['CONFIG_MOVIE_DATA_EXECUTABLE'])
-        self.add_file(nsis_vars['CONFIG_HELPER_EXECUTABLE'])
-        self.add_glob("*.dll")
-
-        self.add_directory("defaults")
-        self.add_directory("resources")
-        self.add_directory("xulrunner")
-
-        self.zipfile.close()
 
     def add_glob(self, wildcard):
         wildcard = os.path.join(self.dist_dir, wildcard)
@@ -613,6 +641,23 @@ class bdist_nsis(Command):
             for name in files:
                 self.add_file(os.path.join(root, name))
 
+package_list=[
+    'miro',
+    'miro.libdaap',
+    'miro.data',
+    'miro.dl_daemon',
+    'miro.dl_daemon.private',
+    'miro.frontends',
+    'miro.frontends.widgets',
+    'miro.frontends.widgets.gst',
+    'miro.frontends.widgets.gtk',
+    'miro.test',
+    'miro.plat',
+    'miro.plat.renderers',
+    'miro.plat.frontends',
+    'miro.plat.frontends.widgets',
+]
+
 if __name__ == "__main__":
     setup(
         windows=[
@@ -634,11 +679,6 @@ if __name__ == "__main__":
                 'dest_base': 'mirodebug',
                 'icon_resources': [(0, "Miro.ico")],
                 },
-            {
-                'script': 'moviedata_util.py',
-                'dest_base': '%s_MovieData' % template_vars['shortAppName'],
-                'icon_resources': [(0, "Miro.ico")],
-                },
             {
                 'script': os.path.join(portable_dir, 'miro_helper.py'),
                 'dest_base': '%s_Helper' % template_vars['shortAppName'],
@@ -646,20 +686,7 @@ if __name__ == "__main__":
                 },
             ],
         ext_modules=ext_modules,
-        packages=[
-            'miro',
-            'miro.libdaap',
-            'miro.dl_daemon',
-            'miro.dl_daemon.private',
-            'miro.frontends',
-            'miro.frontends.widgets',
-            'miro.frontends.widgets.gtk',
-            'miro.test',
-            'miro.plat',
-            'miro.plat.renderers',
-            'miro.plat.frontends',
-            'miro.plat.frontends.widgets',
-            ],
+        packages=package_list,
         package_dir={
             'miro': portable_dir,
             'miro.plat': platform_package_dir,
@@ -674,13 +701,11 @@ if __name__ == "__main__":
             },
         options={
             'py2exe': {
-                'packages': [
-                    'encodings'
-                    ],
+                'packages': package_list + ['encodings'],
                 # don't include stray DLLs from Win2008
                 'dll_excludes': ['DNSAPI.DLL', 'MSIMG32.DLL'],
                 'includes': ('cairo, pango, pangocairo, atk, gobject, '
-                             'gio, libtorrent, mutagen'),
+                             'gio, libtorrent, mutagen, gst'),
                 },
             },
         )
